@@ -1,6 +1,6 @@
 #!/bin/sh
 #
-#	$Id: do_examples.sh,v 1.4 2004/04/29 04:15:26 pwessel Exp $
+#	$Id: do_examples.sh,v 1.5 2006/01/09 21:51:46 remko Exp $
 #
 # Bourne script to test all GMT examples (sh versions).
 # If one argument is given it is used as the bin path
@@ -49,7 +49,9 @@ do
 	if [ -e ex${dir}/job${dir}.sh ]; then
 		echo -n "Doing example ${dir}..."
 		cd ex${dir}
+		cp ../.gmtdefaults4.doc .gmtdefaults4
 		sh job${dir}.sh
+		rm -f .gmtdefaults4
 		cd ..
 		echo "done"
 	fi
