@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    $Id: blockmode.c,v 1.16 2004/04/25 09:10:45 pwessel Exp $
+ *    $Id: blockmode.c,v 1.36 2006/05/14 01:43:04 pwessel Exp $
  *
- *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -25,49 +25,51 @@
    Author: 	Walter H. F. Smith
    Date:	28 October, 1998
    Version:	1st version based on GMT3.1b blockmedian.c, but with changes
-   		to struct DATA and to sort comparison routines to facilitate
+   		to struct BLK_DATA and to sort comparison routines to facilitate
    		mode coding.
    Modified:	3.3.5: 10-JUL-2000 PW: Added -L option
    Version:	3.4 01-MAR-2001 by PW, Replace -N with -F, and added -C
    Version:	4 01-MAR-2003 by PW
+   Version:	4.1: 14-SEP-2005 by PW, Added enhanced -I
+   		4-APR-2006 by PW: Added -E for LMS scale, low, and high value
+			Also implemented size_t counters to be 64-bit compatible.
 */
 
+#define BLOCKMODE
+
 #include "gmt.h"
+#include "block_subs.h"
 
-struct DATA {
-	double	a[4];	/* a[0] = x, a[1] = y, a[2] = z, a[3] = w  */
-        int     i;
-}       *data;
+int main (int argc, char **argv)
+{
+	BOOLEAN	error = FALSE, nofile = TRUE, done = FALSE, first = TRUE, mode_xy;
 
-
-int	compare_x(const void *point_1, const void *point_2);
-int	compare_y(const void *point_1, const void *point_2);
-int	compare_index_z(const void *point_1, const void *point_2);
-double	weighted_mode (struct DATA *d, double wsum, int n, int k);
-
-main (int argc, char **argv)
-{	
-	
-	BOOLEAN	error, weighted, offset, report, nofile = TRUE, done = FALSE, first = TRUE;
-	BOOLEAN use_center = FALSE, go_quickly = FALSE, mode_xy = TRUE;
 	/* Default value for go_quickly = FALSE for backward compatibility with 3.0  */
-	
+
 	FILE *fp = NULL;
+
+	double	*in, out[4], wesn[4], i_n_in_cell, weight, *z_tmp = NULL;
+
+	int	i, j, ix, iy, fno, n_files = 0, n_args, n_req;
+	int	n_expected_fields, n_fields, n_out, w_col;
 	
-	double	west, east, south, north, delta_x, delta_y, del_off;
-	double	*in, out[4], idx, idy, i_n_in_cell, xy_off;
-	
-	int	i, j, n_x, n_y, ix, iy, index, first_in_cell, first_in_new_cell, fno, n_files = 0, n_args, n_req;
-	int	n_lost, n_read, n_pitched, n_cells_filled, n_alloc, n_expected_fields, n_fields, n_out;
-	int	n_in_cell;
-	
+	size_t index, first_in_cell, first_in_new_cell, n_lost, n_read;
+	size_t n_pitched, n_cells_filled, n_alloc, n_in_cell, nz, nz_alloc;
+
 	char	modifier, buffer[BUFSIZ], format[BUFSIZ];
 
-	argc = GMT_begin (argc, argv);
+	struct GRD_HEADER h;
+
+	struct BLK_DATA *data;
+	struct BLOCKMODE_CTRL *Ctrl;
 	
-	west = east = south = north = delta_x = delta_y = 0.0;
-	del_off = 0.5;
-	error = weighted = offset = report = FALSE;
+	double weighted_mode (struct BLK_DATA *d, double wsum, size_t n, int k);
+	
+	argc = GMT_begin (argc, argv);
+
+	Ctrl = (struct BLOCKMODE_CTRL *) New_Blockmode_Ctrl ();	/* Allocate and initialize a new control structure */
+
+	GMT_grd_init (&h, argc, argv, FALSE);
 
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '-') {
@@ -82,17 +84,23 @@ main (int argc, char **argv)
 				case 'b':
 				case 'f':
 				case '\0':
-                                      error += GMT_get_common_args (argv[i], &west, &east, &south, &north);
+                                      error += GMT_parse_common_options (argv[i], &h.x_min, &h.x_max, &h.y_min, &h.y_max);
                                       break;
                               
 				/* Supplemental parameters */
                               
 				case 'C':
-					mode_xy = FALSE;
-					use_center = TRUE;
+					Ctrl->C.active = TRUE;
+					break;
+				case 'E':
+					Ctrl->E.active = TRUE;		/* Extended report with standard deviation, min, and max in cols 4-6 */
 					break;
 				case 'I':
-					GMT_getinc (&argv[i][2], &delta_x, &delta_y);
+					Ctrl->I.active = TRUE;
+					if (GMT_getinc (&argv[i][2], &Ctrl->I.xinc, &Ctrl->I.yinc)) {
+						GMT_inc_syntax ('I', 1);
+						error = TRUE;
+					}
 					break;
 				case 'L':	/* Obsolete, but backward compatibility prevails [use -f instead] */
 					GMT_io.in_col_type[0] = GMT_io.out_col_type[0] = GMT_IS_LON;
@@ -101,24 +109,21 @@ main (int argc, char **argv)
 					break;
 				case 'N':	/* Backward compatible with 3.3.6 */
 				case 'F':
-					offset = TRUE;
+					Ctrl->F.active = TRUE;
 					break;
 				case 'Q':
-					go_quickly = TRUE;
+					Ctrl->Q.active = TRUE;
 					break;
 				case 'W':
-					if ( (modifier = argv[i][2]) == 'i' || modifier == 'I') {
-						weighted = TRUE;
-						report = FALSE;
-					}
-					else if (modifier == 'O' || modifier == 'o') {
-						report = TRUE;
-						weighted = FALSE;
-					}
+					Ctrl->W.active = TRUE;
+					if ( (modifier = argv[i][2]) == 'i' || modifier == 'I')
+						Ctrl->W.weighted[GMT_IN] = TRUE;
+					else if (modifier == 'O' || modifier == 'o')
+						Ctrl->W.weighted[GMT_OUT] = TRUE;
 					else
-						weighted = report = TRUE;
+						Ctrl->W.weighted[GMT_IN] = Ctrl->W.weighted[GMT_OUT] = TRUE;
 					break;
-					
+
 				default:
 					error = TRUE;
                                         GMT_default_error (argv[i][1]);
@@ -129,60 +134,62 @@ main (int argc, char **argv)
 			n_files++;
 	}
 
-	if (argc == 1 || GMT_quick) {
+	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "blockmode %s - Block averaging by mode estimates\n\n", GMT_VERSION);
-		fprintf (stderr, "usage: blockmode [infile(s)] -I<xinc[m]>[/<yinc>[m]] -R<west/east/south/north>\n");
-		fprintf (stderr, "\t[-C] [-F] [-H[<nrec>]] [-Q] [-V] [-W[i][o] ] [-:] [-bi[s][<n>]]\n\t[-bo[s][<n>]] [-f[i|o]<colinfo>]\n\n");
-		
-		if (GMT_quick) exit (EXIT_FAILURE);
-		
-		fprintf (stderr, "\t-I sets Increment of the grid; enter xinc, optionally xinc/yinc.\n");
-		fprintf (stderr, "\t   Default is yinc = xinc.  Append an m [or s] to xinc or yinc to indicate minutes [or seconds],\n");
-		fprintf (stderr, "\t   e.g., -I10m/5m grids longitude every 10 minutes, latitude every 5 minutes.\n");
+		fprintf (stderr, "usage: blockmode [infile(s)] %s %s\n", GMT_I_OPT, GMT_Rgeo_OPT);
+		fprintf (stderr, "\t[-C] [-E] [-F] [%s] [-Q] [-V] [-W[i][o] ] [%s] [%s]\n", GMT_H_OPT, GMT_t_OPT, GMT_b_OPT);
+		fprintf (stderr, "\t[%s]\n\n", GMT_f_OPT);
+
+		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
+
+		GMT_inc_syntax ('I', 0);
 		GMT_explain_option ('R');
 		fprintf (stderr, "\n\tOPTIONS:\n");
 		fprintf (stderr, "\t-C Output center of block and mode z-value  [Default is mode location (but see -Q)]\n");
+		fprintf (stderr, "\t-E Extend output with LMS scale (s), low (l), and high (h) value per block, i,e,\n");
+		fprintf (stderr, "\t   output (x,y,z,s,l,h[,w]) [Default outputs (x,y,z[,w]); see -W regarding w.\n");
 		fprintf (stderr, "\t-F Offsets registration so block edges are on gridlines (pixel reg.).  [Default: grid reg.]\n");
 		GMT_explain_option ('H');
 		fprintf (stderr, "\t-Q Quicker; get mode z and mean x,y.  [Default gets mode x, mode y, mode z.]\n");
 		GMT_explain_option ('V');
-		fprintf (stderr, "\t-W sets Weight options.  -WI reads Weighted Input (4 cols: x,y,z,w) but writes only (x,y,z) Output.\n");
-		fprintf (stderr, "\t   -WO reads unWeighted Input (3 cols: x,y,z) but reports sum (x,y,z,w) Output.\n");
+		fprintf (stderr, "\t-W sets Weight options.\n");
+		fprintf (stderr, "\t-  -Wi reads Weighted Input (4 cols: x,y,z,w) but writes only (x,y,z[,s,l,h]) Output.\n");
+		fprintf (stderr, "\t   -Wo reads unWeighted Input (3 cols: x,y,z) but reports sum (x,y,z[,s,l,h],w) Output.\n");
 		fprintf (stderr, "\t   -W with no modifier has both weighted Input and Output; Default is no weights used.\n");
 		GMT_explain_option (':');
 		GMT_explain_option ('i');
 		GMT_explain_option ('n');
-		fprintf (stderr, "\t   Default is 3 (or 4 if -W is set).\n");
+		fprintf (stderr, "\t   Default is 3 [6 with -E] columns (or 4 [7] if -W is set).\n");
 		GMT_explain_option ('o');
 		GMT_explain_option ('n');
 		GMT_explain_option ('f');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
-	
-	if (!mode_xy && go_quickly == 1) {
+
+	if (Ctrl->C.active && Ctrl->Q.active) {
 		fprintf (stderr, "%s: GMT WARNING:  -C overrides -Q\n", GMT_program);
-		go_quickly = 0;
+		Ctrl->Q.active = FALSE;
 	}
 	if (!project_info.region_supplied) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify -R option\n", GMT_program);
 		error++;
 	}
-	if (delta_x <= 0.0 || delta_y <= 0.0) {
+	if (Ctrl->I.xinc <= 0.0 || Ctrl->I.yinc <= 0.0) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -I option.  Must specify positive increment(s)\n", GMT_program);
 		error = TRUE;
 	}
-	if (GMT_io.binary[GMT_IN] && gmtdefs.io_header[GMT_IN]) {
+	if (GMT_io.binary[GMT_IN] && GMT_io.io_header[GMT_IN]) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data cannot have header -H\n", GMT_program);
 		error++;
 	}
-	n_req = (weighted) ? 4 : 3;
+	n_req = (Ctrl->W.weighted[GMT_IN]) ? 4 : 3;
 	if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] == 0) GMT_io.ncol[GMT_IN] = n_req;
 	if (GMT_io.binary[GMT_IN] && n_req > GMT_io.ncol[GMT_IN]) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR.  binary input data must have at least %d columns\n", GMT_program, n_req);
 		error++;
 	}
-	
+
 	if (error) exit (EXIT_FAILURE);
 
 	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
@@ -195,42 +202,38 @@ main (int argc, char **argv)
 #ifdef SET_IO_MODE
 	GMT_setmode (GMT_OUT);
 #endif
+	
+	mode_xy = !Ctrl->C.active;
+	h.x_inc = Ctrl->I.xinc;
+	h.y_inc = Ctrl->I.yinc;
+	h.node_offset = Ctrl->F.active;
+	GMT_RI_prepare (&h);	/* Ensure -R -I consistency and set nx, ny */
+	h.xy_off = 0.5 * h.node_offset;	/* Use to calculate mean location of block */
 
-	idx = 1.0 / delta_x;
-	idy = 1.0 / delta_y;
-	n_x = irint ((east - west) * idx) + 1;
-	n_y = irint ((north - south) * idy) + 1;
-	
-	if (offset) {
-		n_x--;
-		n_y--;
-		del_off = 0.0;
-	}
-	xy_off = 0.5 - del_off;
-	
-	
 	if (gmtdefs.verbose) {
 		sprintf (format, "%%s: W: %s E: %s S: %s N: %s nx: %%d ny: %%d\n", gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
-		fprintf (stderr, format, GMT_program, west, east, south, north, n_x, n_y);
+		fprintf (stderr, format, GMT_program, h.x_min, h.x_max, h.y_min, h.y_max, h.nx, h.ny);
 	}
 
 	n_read = n_pitched = 0;
         n_alloc = GMT_CHUNK;
-        data = (struct DATA *) GMT_memory (VNULL, (size_t)n_alloc, sizeof(struct DATA), GMT_program);
-	
+        data = (struct BLK_DATA *) GMT_memory (VNULL, n_alloc, sizeof(struct BLK_DATA), GMT_program);
+
+	GMT_set_xy_domain (wesn, &h);	/* May include some padding if gridline-registered */
+		
 	/* Read the input data  */
-	
-	n_expected_fields = (GMT_io.binary[GMT_IN]) ? GMT_io.ncol[GMT_IN] : 3 + weighted;
+
+	n_expected_fields = (GMT_io.binary[GMT_IN]) ? GMT_io.ncol[GMT_IN] : 3 + Ctrl->W.weighted[GMT_IN];
 
 	if (n_files > 0)
 		nofile = FALSE;
 	else
 		n_files = 1;
 	n_args = (argc > 1) ? argc : 2;
-	
+
 	for (fno = 1; !done && fno < n_args; fno++) {	/* Loop over input files, if any */
 		if (!nofile && argv[fno][0] == '-') continue;
-		
+
 		if (nofile) {	/* Just read standard input */
 			fp = GMT_stdin;
 			done = TRUE;
@@ -244,129 +247,143 @@ main (int argc, char **argv)
 		}
 
 		if (!nofile && gmtdefs.verbose) fprintf (stderr, "%s: Working on file %s\n", GMT_program, argv[fno]);
-		
-		if (gmtdefs.io_header[GMT_IN]) {
-			for (i = 0; i < gmtdefs.n_header_recs; i++) {
+
+		if (GMT_io.io_header[GMT_IN]) {
+			for (i = 0; i < GMT_io.n_header_recs; i++) {
 				GMT_fgets (buffer, BUFSIZ, fp);
 				buffer[strlen(buffer)-1] = 0;
-				if (first && gmtdefs.io_header[GMT_OUT]) (report && !(weighted)) ? fprintf (GMT_stdout, "%s weights\n", buffer) : fprintf (GMT_stdout, "%s\n", buffer);
+				if (first && GMT_io.io_header[GMT_OUT]) (Ctrl->W.weighted[GMT_OUT] && !(Ctrl->W.weighted[GMT_IN])) ? fprintf (GMT_stdout, "%s weights\n", buffer) : fprintf (GMT_stdout, "%s\n", buffer);
 			}
 			first = FALSE;
 		}
-				
+
 		while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
-	
+
 			if (GMT_io.status & GMT_IO_MISMATCH) {
-				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields,  n_expected_fields, n_read);
+				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %u\n", GMT_program, n_fields,  n_expected_fields, n_read);
 				exit (EXIT_FAILURE);
 			}
 
 			if (GMT_is_dnan (in[2])) continue;	/* Skip when z = NaN */
 
 			n_read++;
-		
-			ix = (int)floor(((in[0] - west) * idx) + del_off);
-			if ( ix < 0 || ix >= n_x ) continue;
-			iy = (int)floor(((in[1] - south) * idy) + del_off);
-			if ( iy < 0 || iy >= n_y ) continue;
-		
-			index = iy * n_x + ix;
-		
+
+			if (GMT_y_is_outside (in[1],  wesn[2], wesn[3])) continue;	/* Outside y-range */
+			if (GMT_x_is_outside (&in[0], wesn[0], wesn[1])) continue;	/* Outside x-range */
+
+			ix = GMT_x_to_i (in[0], h.x_min, h.x_inc, h.xy_off, h.nx);
+			if ( ix < 0 || ix >= h.nx ) continue;
+			iy = GMT_y_to_j (in[1], h.y_min, h.y_inc, h.xy_off, h.ny);
+			if ( iy < 0 || iy >= h.ny ) continue;
+
+			index = ((size_t)iy) * ((size_t)h.nx) + ((size_t)ix);		/* 64-bit safe 1-D index */
+
 			data[n_pitched].i = index;
 			if (mode_xy) {
-				data[n_pitched].a[0] = in[0];
-				data[n_pitched].a[1] = in[1];
+				data[n_pitched].a[BLK_X] = in[0];
+				data[n_pitched].a[BLK_Y] = in[1];
 			}
-			data[n_pitched].a[2] = in[2];
-			data[n_pitched].a[3] = (weighted) ? in[3] : 1.0;
+			data[n_pitched].a[BLK_Z] = in[2];
+			data[n_pitched].a[BLK_W] = (Ctrl->W.weighted[GMT_IN]) ? in[3] : 1.0;
 
 			n_pitched++;
 			if (n_pitched == n_alloc) {
 				n_alloc += GMT_CHUNK;
-				data = (struct DATA *) GMT_memory ((void *)data, (size_t)n_alloc, sizeof(struct DATA), GMT_program);
+				data = (struct BLK_DATA *) GMT_memory ((void *)data, n_alloc, sizeof(struct BLK_DATA), GMT_program);
 			}
 		}
 		if (fp != GMT_stdin) GMT_fclose(fp);
-		
-	}
-	
-	data = (struct DATA *) GMT_memory ((void *)data, (size_t)n_pitched, sizeof(struct DATA), GMT_program);
-	n_lost = n_read - n_pitched;
-	if (gmtdefs.verbose) fprintf(stderr,"%s: N read: %d N used: %d N outside_area: %d\n", GMT_program, 
-		n_read,n_pitched,n_lost);
 
+	}
+
+	data = (struct BLK_DATA *) GMT_memory ((void *)data, n_pitched, sizeof(struct BLK_DATA), GMT_program);
+	n_lost = n_read - n_pitched;
+	if (gmtdefs.verbose) fprintf(stderr,"%s: N read: %u N used: %u N outside_area: %u\n", GMT_program, n_read, n_pitched, n_lost);
 
 	/* Ready to go. */
-	
-	n_out = (report) ? 4 : 3;
-	
+
+	n_out = (Ctrl->W.weighted[GMT_OUT]) ? 4 : 3;
+	if (Ctrl->E.active) n_out += 3;
+	w_col = n_out - 1;	/* Weights always reported in last output column */
+	nz_alloc = GMT_CHUNK;
+	if (Ctrl->E.active) z_tmp = (double *) GMT_memory (VNULL, nz_alloc, sizeof (double), GMT_program);	/* tmp array for L1 scale calculation */
+
 	/* Sort on index and Z value */
-	
-	qsort((void *)data, (size_t)n_pitched, sizeof (struct DATA), compare_index_z);
-	
+
+	qsort((void *)data, n_pitched, sizeof (struct BLK_DATA), BLK_compare_index_z);
+
 	/* Find n_in_cell and write appropriate output  */
 
 	first_in_cell = 0;
 	n_cells_filled = 0;
 	while (first_in_cell < n_pitched) {
-		out[3] = data[first_in_cell].a[3];
-		if (use_center) {
-			j = data[first_in_cell].i / n_x;
-			i = data[first_in_cell].i % n_x;
-			out[0] = west  + (i + xy_off) * delta_x;
-			out[1] = south + (j + xy_off) * delta_y;
+		weight = data[first_in_cell].a[BLK_W];
+		if (Ctrl->E.active) z_tmp[0] = data[first_in_cell].a[BLK_Z];
+		nz = 1;
+		if (Ctrl->C.active) {
+			j = data[first_in_cell].i / h.nx;
+			i = data[first_in_cell].i % h.nx;
+			out[0] = GMT_i_to_x (i, h.x_min, h.x_max, h.x_inc, h.xy_off, h.nx);
+			out[1] = GMT_j_to_y (j, h.y_min, h.y_max, h.y_inc, h.xy_off, h.ny);
 		}
 		else {
-			out[0] = data[first_in_cell].a[0];
-			out[1] = data[first_in_cell].a[1];
+			out[0] = data[first_in_cell].a[BLK_X];
+			out[1] = data[first_in_cell].a[BLK_Y];
 		}
 		first_in_new_cell = first_in_cell + 1;
 		while ( (first_in_new_cell < n_pitched) && (data[first_in_new_cell].i == data[first_in_cell].i) ) {
-			out[3] += data[first_in_new_cell].a[3];	/* Summing up weights */
+			weight += data[first_in_new_cell].a[BLK_W];	/* Summing up weights */
 			if (mode_xy) {
-				out[0] += data[first_in_new_cell].a[0];
-				out[1] += data[first_in_new_cell].a[1];
+				out[0] += data[first_in_new_cell].a[BLK_X];
+				out[1] += data[first_in_new_cell].a[BLK_Y];
+			}
+			if (Ctrl->E.active) {	/* Must get a temporary copy of the sorted z array */
+				z_tmp[nz] = data[first_in_new_cell].a[BLK_Z];
+				nz++;
+				if (nz == nz_alloc) {
+					nz_alloc += GMT_CHUNK;
+					z_tmp = (double *) GMT_memory ((void *)z_tmp, nz_alloc, sizeof (double), GMT_program);
+				}
 			}
 			first_in_new_cell++;
 		}
 		n_in_cell = first_in_new_cell - first_in_cell;
-		if (n_in_cell > 2) {
-			/* data are already sorted on z; get z mode  */
-			out[2] = weighted_mode (&data[first_in_cell], out[3], n_in_cell, 2);
-			if (go_quickly) {
+		if (n_in_cell > 2) {	/* data are already sorted on z; get z mode  */
+			out[2] = weighted_mode (&data[first_in_cell], weight, n_in_cell, 2);
+			if (Ctrl->Q.active) {
 				i_n_in_cell = 1.0 / n_in_cell;
 				out[0] *= i_n_in_cell;
 				out[1] *= i_n_in_cell;
 			}
 			else if (mode_xy) {
-				qsort((void *)&data[first_in_cell], (size_t)n_in_cell, sizeof (struct DATA), compare_x);
-				out[0] = weighted_mode (&data[first_in_cell], out[3], n_in_cell, 0);
-				
-				qsort((void *)&data[first_in_cell], (size_t)n_in_cell, sizeof (struct DATA), compare_y);
-				out[1] = weighted_mode (&data[first_in_cell], out[3], n_in_cell, 1);
+				qsort((void *)&data[first_in_cell], n_in_cell, sizeof (struct BLK_DATA), BLK_compare_x);
+				out[0] = weighted_mode (&data[first_in_cell], weight, n_in_cell, 0);
+
+				qsort((void *)&data[first_in_cell], n_in_cell, sizeof (struct BLK_DATA), BLK_compare_y);
+				out[1] = weighted_mode (&data[first_in_cell], weight, n_in_cell, 1);
 			}
 		}
 		else if (n_in_cell == 2) {
-			if (data[first_in_cell].a[3] > data[first_in_cell+1].a[3]) {
-				out[2] = data[first_in_cell].a[2];
-				if (go_quickly) {
+			if (data[first_in_cell].a[BLK_W] > data[first_in_cell+1].a[BLK_W]) {
+				out[2] = data[first_in_cell].a[BLK_Z];
+				if (Ctrl->Q.active) {
 					out[0] *= 0.5;
 					out[1] *= 0.5;
 				}
 				else if (mode_xy) {
-					out[0] = data[first_in_cell].a[0];
-					out[1] = data[first_in_cell].a[1];
+					out[0] = data[first_in_cell].a[BLK_X];
+					out[1] = data[first_in_cell].a[BLK_Y];
 				}
 			}
-			else if (data[first_in_cell].a[3] < data[first_in_cell+1].a[3]) {
-				out[2] = data[first_in_cell+1].a[2];
-				if (go_quickly) {
+			else if (data[first_in_cell].a[BLK_W] < data[first_in_cell+1].a[BLK_W]) {
+				out[2] = data[first_in_cell+1].a[BLK_Z];
+				if (Ctrl->Q.active) {
 					out[0] *= 0.5;
 					out[1] *= 0.5;
 				}
 				else if (mode_xy) {
-					out[0] = data[first_in_cell+1].a[0];
-					out[1] = data[first_in_cell+1].a[1];
+					out[0] = data[first_in_cell+1].a[BLK_X];
+					out[1] = data[first_in_cell+1].a[BLK_Y];
 				}
 			}
 			else {
@@ -374,26 +391,46 @@ main (int argc, char **argv)
 					out[0] *= 0.5;
 					out[1] *= 0.5;
 				}
-				out[2] = 0.5 * (data[first_in_cell].a[2] + data[first_in_cell+1].a[2]);
+				out[2] = 0.5 * (data[first_in_cell].a[BLK_Z] + data[first_in_cell+1].a[BLK_Z]);
 			}
 		}
 		else
-			out[2] = data[first_in_cell].a[2];
+			out[2] = data[first_in_cell].a[BLK_Z];
+
+		if (Ctrl->E.active) {
+			out[4] = z_tmp[0];	/* Low value */
+			out[5] = z_tmp[nz-1];	/* High value */
+			/* Turn z_tmp into absolute deviations from the mode (out[2]) */
+			if (nz > 1) {
+				for (index = 0; index < nz; index++) z_tmp[index] = fabs (z_tmp[index] - out[2]);
+				qsort ((void *)z_tmp, nz, sizeof (double), GMT_comp_double_asc);
+				out[3] = (nz%2) ? z_tmp[nz/2] : 0.5 * (z_tmp[(nz-1)/2] + z_tmp[nz/2]);
+				out[3] *= 1.4826;	/* This will be LMS MAD-based scale */
+			}
+			else
+				out[3] = GMT_d_NaN;
+		}
+		if (Ctrl->W.weighted[GMT_OUT]) out[w_col] = weight;
 		
 		GMT_output (GMT_stdout, n_out, out);
-		
+
 		n_cells_filled++;
 		first_in_cell = first_in_new_cell;
 	}
 
-	if (gmtdefs.verbose) fprintf(stderr,"%s: N_cells_filled: %d\n", GMT_program, n_cells_filled);
+	if (gmtdefs.verbose) fprintf(stderr,"%s: N_cells_filled: %u\n", GMT_program, n_cells_filled);
 
 	GMT_free ((void *)data);
-	
+	if (Ctrl->E.active) GMT_free ((void *)z_tmp);
+
         GMT_end (argc, argv);
+
+	Free_Blockmode_Ctrl (Ctrl);	/* Deallocate control structure */
+
+	exit (EXIT_SUCCESS);
 }
 
-double	weighted_mode (struct DATA *d, double wsum, int n, int k)
+double	weighted_mode (struct BLK_DATA *d, double wsum, size_t n, int k)
 {
 	/* Estimate mode by finding a maximum in the estimated
 	   pdf of weighted data.  Estimate the pdf as the finite 
@@ -415,16 +452,16 @@ double	weighted_mode (struct DATA *d, double wsum, int n, int k)
 	   I assumed n > 2 when I wrote this.  */
 	   
 	double	top, topj, topi, bottomj, bottomi, pj, pi;
-	int	i, j, nh;
+	size_t	i, j, nh;
 
 	i = 0;
 	j = n - 1;
 	nh = n / 2;
 	top = wsum;
-	
+
 	while (j-i > nh) {
-		topi = top - d[i].a[3];
-		topj = top - d[j].a[3];
+		topi = top - d[i].a[BLK_W];
+		topj = top - d[j].a[BLK_W];
 		bottomi = d[j].a[k] - d[i+1].a[k];
 		bottomj = d[j-1].a[k] - d[i].a[k];
 
@@ -446,91 +483,13 @@ double	weighted_mode (struct DATA *d, double wsum, int n, int k)
 				top = topj;
 			}
 			else {
-				top -= (d[i].a[3] + d[j].a[3]);
+				top -= (d[i].a[BLK_W] + d[j].a[BLK_W]);
                                 i++;
                                 j--;
 			}
 		}
 	}
-	return(0.5*(d[j].a[k] + d[i].a[k]));
+	return (0.5*(d[j].a[k] + d[i].a[k]));
 }
 
-int	compare_index_z (const void *point_1, const void *point_2)
-{
-	int	index_1, index_2;
-	double	data_1, data_2;
-	struct DATA *p1, *p2;
-	
-	p1 = (struct DATA *)point_1;
-	p2 = (struct DATA *)point_2;
-	index_1 = p1->i;
-	index_2 = p2->i;
-
-	if (index_1 < index_2)
-		return (-1);
-	else if (index_1 > index_2)
-		return (1);
-	else {
-		data_1 = p1->a[2];
-		data_2 = p2->a[2];
-		if (data_1 < data_2)
-			return (-1);
-		else if (data_1 > data_2)
-			return (1);
-		else
-			return (0);
-	}
-}
-int	compare_x(const void *point_1, const void *point_2)
-{
-	int	index_1, index_2;
-	double	x_1, x_2;
-	struct DATA *p1, *p2;
-	
-	p1 = (struct DATA *)point_1;
-	p2 = (struct DATA *)point_2;
-	index_1 = p1->i;
-	index_2 = p2->i;
-
-	if (index_1 < index_2)
-		return (-1);
-	else if (index_1 > index_2)
-		return (1);
-	else {
-		x_1 = p1->a[0];
-		x_2 = p2->a[0];
-		if (x_1 < x_2)
-			return (-1);
-		else if (x_1 > x_2)
-			return (1);
-		else
-			return (0);
-	}
-}
-
-int	compare_y(const void *point_1, const void *point_2)
-{
-	int	index_1, index_2;
-	double	y_1, y_2;
-	struct DATA *p1, *p2;
-	
-	p1 = (struct DATA *)point_1;
-	p2 = (struct DATA *)point_2;
-	index_1 = p1->i;
-	index_2 = p2->i;
-
-	if (index_1 < index_2)
-		return (-1);
-	else if (index_1 > index_2)
-		return (1);
-	else {
-		y_1 = p1->a[1];
-		y_2 = p2->a[1];
-		if (y_1 < y_2)
-			return (-1);
-		else if (y_1 > y_2)
-			return (1);
-		else
-			return (0);
-	}
-}
+#include "block_subs.c"
