@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdraster.c,v 1.15 2004/05/24 20:47:25 pwessel Exp $
+ *	$Id: grdraster.c,v 1.31 2006/04/10 05:47:30 pwessel Exp $
  *
- *    Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
+ *    Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
  *    See README file for copying and redistribution conditions.
  *--------------------------------------------------------------------*/
 /* grdraster.c -- read a rasterfile and extract a region as a grdfile.
@@ -27,72 +27,72 @@
  *				  by the new configure script.
  *		05 May 2000   PW: Added option -Z [-bo[s]] to write xyz to stdout instead
  *		12 Jan 2001   PW: Dynamically allocate info structures; no longer hardwired.
+ *		27 Jun 2005   PW: Allow modifiers G(ographic) and C(artesian) in registration flag.
+ *		19 Sep 2005   PW: Allow optional H<bytes> at end of line for skipping headers.
  */
 
 #include "gmt.h"
 
-struct RASTER_INFO {
+struct GRDRASTER_INFO {
 	struct GRD_HEADER h;
 	int	id;		/* File number  */
 	int	nglobal;	/* If not 0, ras is global and i%nglobal makes it periodic  */
 	int	nanflag;
 	int	nanset;		/* True if raster uses nanflag to signal NaN  */
+	int	skip;		/* Skip this number of header bytes when opeing file  */
 	char	type;
 	BOOLEAN swap_me;	/* TRUE if data set need to be swapped */
 	BOOLEAN geo;		/* TRUE if we believe x/y is lon/lat, FALSE otherwise */
 };
 
+int main (int argc, char **argv)
+{
+	BOOLEAN xyz_out = FALSE;
+
 #if WORDS_BIGENDIAN == 0
 	char my_endian = 'L';	/* This machine is Little endian */
 #else
 	char my_endian = 'B';	/* This machine is Big endian */
 #endif
-
-static unsigned char maskset[8] = {128, 64, 32, 16, 8, 4, 2, 1};
-int load_rasinfo (struct RASTER_INFO **rasinfo);
-
-main (int argc, char **argv)
-{
-	struct RASTER_INFO myras;
-	struct GRD_HEADER h;
-	struct RASTER_INFO *rasinfo;
-
-	double	tol;
-	double	grdlatorigin, grdlonorigin, raslatorigin, raslonorigin;
-
-	int	i, j, k, ksize, iselect, nselected, imult, jmult, nrasters, ij_offset;
-	int	irasstart, jrasstart, n_nan, iras, jras, ij, ijras, jseek, nmask;
+	char	*grdfilename, *buffer = NULL, *r_opt = NULL, *tselect = CNULL, match[GRD_REMARK_LEN];
+	unsigned char *ubuffer = NULL;
+	static unsigned char maskset[8] = {128, 64, 32, 16, 8, 4, 2, 1};
+	
+	int	i, j, k, ksize = 0, iselect, nselected, imult, jmult, nrasters, ij_offset;
+	int	irasstart, jrasstart, n_nan, iras, jras, ij, ijras, jseek, nmask = 0;
 	int	error, incset, firstread;
 	
-	BOOLEAN xyz_out = FALSE;
-
-	float	*grd, *floatrasrow;
+	float	*grd = NULL, *floatrasrow = NULL;
 	
-	double *x, y, half, out[3];
+	double	tol;
+	double	grdlatorigin, grdlonorigin, raslatorigin, raslonorigin;
+	double *x = NULL, y, out[3];
+
+	struct GRDRASTER_INFO myras;
+	struct GRD_HEADER h;
+	struct GRDRASTER_INFO *rasinfo;
 
 	FILE *fp;
 
-	char	*grdfilename, *buffer, *r_opt;
-	unsigned char *ubuffer;
+	void convert_u_row (struct GRDRASTER_INFO ras, float *row, unsigned char *buffer);
+	void convert_c_row (struct GRDRASTER_INFO ras, float *row, char *buffer);
+	void convert_d_row (struct GRDRASTER_INFO ras, float *row, short unsigned int *buffer);
+	void convert_i_row (struct GRDRASTER_INFO ras, float *row, short int *buffer);
+	void convert_l_row (struct GRDRASTER_INFO ras, float *row, int *buffer);
+	int load_rasinfo (struct GRDRASTER_INFO **rasinfo, char endian);
 
-	void	convert_u_row(struct RASTER_INFO ras, float *row, unsigned char *buffer);
-	void	convert_c_row(struct RASTER_INFO ras, float *row, char *buffer);
-	void	convert_d_row(struct RASTER_INFO ras, float *row, short unsigned int *buffer);
-	void	convert_i_row(struct RASTER_INFO ras, float *row, short int *buffer);
-	void	convert_l_row(struct RASTER_INFO ras, float *row, int *buffer);
-
-	GMT_begin (argc, argv);
+	argc = GMT_begin (argc, argv);
 
 	nselected = 0;
 	error = incset = FALSE;
 	grdfilename = CNULL;
 	GMT_grd_init (&h, argc, argv, FALSE);
 
-	if ( !(nrasters = load_rasinfo(&rasinfo) ) ) {
+	if (!(nrasters = load_rasinfo(&rasinfo, my_endian))) {
 		fprintf(stderr, "%s:  ERROR reading grdraster.info file.\n", GMT_program);
 		exit (EXIT_FAILURE);
 	}
-	project_info.region_supplied = FALSE;	/* Because () sets it to TRUE by using GMT_get_common_args to process file regions */
+	project_info.region_supplied = FALSE;	/* Because () sets it to TRUE by using GMT_parse_common_options to process file regions */
 
 	for (i = 1; !error && i < argc; i++) {
 		if (argv[i][0] == '-') {
@@ -103,11 +103,11 @@ main (int argc, char **argv)
 				case 'J':
 				case 'V':
 				case '\0':
-					error += GMT_get_common_args (argv[i], &h.x_min, &h.x_max, &h.y_min, &h.y_max);
+					error += GMT_parse_common_options (argv[i], &h.x_min, &h.x_max, &h.y_min, &h.y_max);
 					break;
 				
 				case 'b':
-					error += GMT_io_selection (&argv[i][2]);
+					error += GMT_parse_b_option (&argv[i][2]);
 					break;
 				case 'G':
 					grdfilename = &argv[i][2];
@@ -123,16 +123,17 @@ main (int argc, char **argv)
 		}
 		else {
 			nselected++;
-			error += ( (sscanf(argv[i], "%d", &iselect)) != 1);
+			tselect = argv[i];
 		}
 	}
 
-	if (argc == 1 || GMT_quick) {
+	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "grdraster %s - Extract a region from a raster and save in a grdfile\n\n", GMT_VERSION);
-		fprintf (stderr, "usage: grdraster <file number> -R<west/east/south/north>[r] [-G<grdfilename>] [-I<dx>[m][/<dy>[m]]]\n");
-		fprintf (stderr, " [-bo[s]]\n");
+		fprintf (stderr, "usage: grdraster <file number>|<text> %s [-G<grdfilename>] [%s]\n", GMT_Rgeo_OPT, GMT_Id_OPT);
+		fprintf (stderr, " [%s]\n", GMT_bo_OPT);
 
-		fprintf (stderr, "\t<file number> (#) corresponds to one of these:\n\n");
+		fprintf (stderr, "\t<file number> (#) or <text> corresponds to one of the datasets listed.\n");
+		fprintf (stderr, "\t[<text> can be a unique substring of the description].\n\n");
 		fprintf (stderr, "#	Data Description	Unit	Coverage		Spacing	Registration\n");
 		fprintf (stderr, "------------------------------------------------------------------------------------\n");
 		for (i = 0; i < nrasters; i++) fprintf (stderr, "%s\n", rasinfo[i].h.command);
@@ -142,7 +143,7 @@ main (int argc, char **argv)
 #else
 		fprintf (stderr, "grdraster default binary byte order is Big-endian\n");
 #endif
-		if (GMT_quick) exit (EXIT_FAILURE);
+		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
 		fprintf (stderr, "\t-R specifies the west, east, south, and north edges of the area.\n");
 		fprintf (stderr, "\t   Use dd:mm format for regions given in degrees and minutes.\n");
@@ -180,19 +181,40 @@ main (int argc, char **argv)
 		error++;
 	}
 	else {
+		/* Check if given argument is an integer ID.  If so, assign iselect, else set it to -1 */
+		GMT_str_toupper (tselect);	/* Make it upper case - which wont affect integers */
+		for (j = i = 0; tselect[j] && i == 0; j++) if (!isdigit ((int)tselect[j])) i = 1;
+		iselect = (i == 0) ? atoi (tselect) : -1;
 		j = -1;
-		for (i = 0; i < nrasters; i++) {
-			if (rasinfo[i].id == iselect) {
-				if (j == -1)
-					j = i;
-				else {
-					fprintf (stderr, "%s ERROR:  Two rasters have the same file number in grdraster.info\n", GMT_program);
-					error++;
+		for (i = 0; !error && i < nrasters; i++) {
+			if (iselect != -1) {	/* We gave an integer ID */
+				if (rasinfo[i].id == iselect) {
+					if (j == -1)
+						j = i;
+					else {
+						fprintf (stderr, "%s ERROR:  At least two rasters have the same file number in grdraster.info\n", GMT_program);
+						error++;
+					}
+				}
+			}
+			else {	/* We gave a text snippet to match in command */
+				strcpy (match, rasinfo[i].h.command);
+				GMT_str_toupper (match);	/* Make it upper case  */
+				if (strstr (match, tselect)) {	/* Found a matching text string */
+					if (j == -1)
+						j = i;
+					else {
+						fprintf (stderr, "%s ERROR:  At least two rasters have the same text [%s] in grdraster.info\n", GMT_program, tselect);
+						error++;
+					}
 				}
 			}
 		}
 		if (j == -1) {
-			fprintf (stderr, "%s ERROR:  No raster with file number %d in grdraster.info\n", GMT_program, iselect);
+			if (iselect != -1)
+				fprintf (stderr, "%s ERROR:  No raster with file number %d in grdraster.info\n", GMT_program, iselect);
+			else
+				fprintf (stderr, "%s ERROR:  No raster with text %s in grdraster.info\n", GMT_program, tselect);
 			error++;
 		}
 		else {
@@ -296,11 +318,13 @@ main (int argc, char **argv)
 	h.node_offset = myras.h.node_offset;
 	h.z_min = DBL_MAX;
 	h.z_max = -DBL_MAX;
+	h.xy_off = 0.5 * h.node_offset;
+	myras.h.xy_off = 0.5 * myras.h.node_offset;
 
-	grdlatorigin = (h.node_offset) ? h.y_max - 0.5 * h.y_inc : h.y_max;
-	grdlonorigin = (h.node_offset) ? h.x_min + 0.5 * h.x_inc : h.x_min;
-	raslatorigin = (myras.h.node_offset) ? myras.h.y_max - 0.5*myras.h.y_inc : myras.h.y_max;
-	raslonorigin = (myras.h.node_offset) ? myras.h.x_min + 0.5*myras.h.x_inc : myras.h.x_min;
+	grdlatorigin = GMT_j_to_y (0, h.y_min, h.y_max, h.y_inc, h.xy_off, h.ny);
+	grdlonorigin = GMT_i_to_x (0, h.x_min, h.x_max, h.x_inc, h.xy_off, h.nx);
+	raslatorigin = GMT_j_to_y (0, myras.h.y_min, myras.h.y_max, myras.h.y_inc, myras.h.xy_off, myras.h.ny);
+	raslonorigin = GMT_i_to_x (0, myras.h.x_min, myras.h.x_max, myras.h.x_inc, myras.h.xy_off, myras.h.nx);
 	irasstart = irint( (grdlonorigin - raslonorigin) / myras.h.x_inc);
 	jrasstart = irint( (raslatorigin - grdlatorigin) / myras.h.y_inc);
 	if (myras.nglobal) while (irasstart < 0) irasstart += myras.nglobal;
@@ -310,8 +334,7 @@ main (int argc, char **argv)
 	if (xyz_out) {	/* Need just space for one row */
 		grd = (float *)GMT_memory(VNULL, (size_t)(h.nx), sizeof(float), GMT_program);
 		x = (double *)GMT_memory(VNULL, (size_t)(h.nx), sizeof(double), GMT_program);
-		half = (h.node_offset) ? 0.5 : 0.0;
-		for (i = 0; i < h.nx; i++) x[i] = h.x_min + (i + half) * h.x_inc;
+		for (i = 0; i < h.nx; i++) x[i] = GMT_i_to_x (i, h.x_min, h.x_max, h.x_inc, h.xy_off, h.nx);
 		ij_offset = 0;
 	} else {	/* Need entire grid */
 		grd = (float *)GMT_memory(VNULL, (size_t)(h.nx*h.ny), sizeof(float), GMT_program);
@@ -331,6 +354,7 @@ main (int argc, char **argv)
 				ksize = 1;
 				break;
 			case 'd':
+			case 'h':
 			case 'i':
 				ksize = 2;
 				break;
@@ -348,6 +372,10 @@ main (int argc, char **argv)
 		fprintf(stderr,"%s ERROR opening %s for read.\n", GMT_program, myras.h.remark);
 		exit (EXIT_FAILURE);
 	}
+	if (myras.skip && fseek (fp, (long) (myras.skip), SEEK_CUR) ) {
+		fprintf(stderr,"%s ERROR skipping %d bytes in %s.\n", GMT_program, myras.skip, myras.h.remark);
+		exit (EXIT_FAILURE);
+	}
 	if (gmtdefs.verbose) fprintf(stderr, "%s:  Reading from raster %s\n", GMT_program, myras.h.remark);
 	if (gmtdefs.verbose && myras.swap_me) fprintf (stderr, "%s:  Data from %s will be byte-swapped\n", GMT_program, myras.h.remark);
 
@@ -360,7 +388,7 @@ main (int argc, char **argv)
 			exit (EXIT_FAILURE);
 		}
 		for (j = 0, jras = jrasstart; j < h.ny; j++, jras += jmult) {
-			y = h.y_max - (j + half) * h.y_inc;
+			y = GMT_j_to_y (j, h.y_min, h.y_max, h.y_inc, h.xy_off, h.ny);
 			if (jras < 0 || jras > myras.h.ny) {
 				/* This entire row is outside the raster:  */
 				for (i = 0, ij = j * ij_offset; i < h.nx; i++, ij++) grd[ij] = GMT_f_NaN;
@@ -398,7 +426,7 @@ main (int argc, char **argv)
 	else {
 		firstread = TRUE;
 		for (j = 0, jras = jrasstart; j < h.ny; j++, jras += jmult) {
-			y = h.y_max - (j + half) * h.y_inc;
+			y = GMT_j_to_y (j, h.y_min, h.y_max, h.y_inc, h.xy_off, h.ny);
 			if (jras < 0 || jras > myras.h.ny) {
 				/* This entire row is outside the raster:  */
 				for (i = 0, ij = j * ij_offset; i < h.nx; i++, ij++) grd[ij] = GMT_f_NaN;
@@ -492,9 +520,11 @@ main (int argc, char **argv)
 	GMT_free ((void *)rasinfo);
 
 	GMT_end (argc, argv);
+	
+	exit (EXIT_SUCCESS);
 }
 
-void convert_u_row(struct RASTER_INFO ras, float *row, unsigned char *buffer)
+void convert_u_row(struct GRDRASTER_INFO ras, float *row, unsigned char *buffer)
 {
 	int	i, tempval;
 	for (i = 0; i < ras.h.nx; i++) {
@@ -511,7 +541,7 @@ void convert_u_row(struct RASTER_INFO ras, float *row, unsigned char *buffer)
 	return;
 }
 
-void convert_c_row(struct RASTER_INFO ras, float *row, char *buffer)
+void convert_c_row(struct GRDRASTER_INFO ras, float *row, char *buffer)
 {
 	int	i, tempval;
 	for (i = 0; i < ras.h.nx; i++) {
@@ -528,7 +558,7 @@ void convert_c_row(struct RASTER_INFO ras, float *row, char *buffer)
 	return;
 }
 
-void convert_d_row(struct RASTER_INFO ras, float *row, short unsigned int *buffer)
+void convert_d_row(struct GRDRASTER_INFO ras, float *row, short unsigned int *buffer)
 {
 	int	i, tempval;
 	for (i = 0; i < ras.h.nx; i++) {
@@ -547,7 +577,7 @@ void convert_d_row(struct RASTER_INFO ras, float *row, short unsigned int *buffe
 	return;
 }
 
-void convert_i_row(struct RASTER_INFO ras, float *row, short int *buffer)
+void convert_i_row(struct GRDRASTER_INFO ras, float *row, short int *buffer)
 {
 	int	i, tempval;
 	for (i = 0; i < ras.h.nx; i++) {
@@ -566,7 +596,7 @@ void convert_i_row(struct RASTER_INFO ras, float *row, short int *buffer)
 	return;
 }
 
-void convert_l_row(struct RASTER_INFO ras, float *row, int *buffer)
+void convert_l_row(struct GRDRASTER_INFO ras, float *row, int *buffer)
 {
 	int	i, tempval;
 	for (i = 0; i < ras.h.nx; i++) {
@@ -585,7 +615,7 @@ void convert_l_row(struct RASTER_INFO ras, float *row, int *buffer)
 	return;
 }
 
-int	load_rasinfo(struct RASTER_INFO **ras)
+int	load_rasinfo(struct GRDRASTER_INFO **ras, char endian)
 {
 	/* Read the file $(GMT_GRIDDIR)/grdraster.info
 		Store the i'th row of the file in rasinfo[i].h.command.
@@ -605,7 +635,7 @@ int	load_rasinfo(struct RASTER_INFO **ras)
 	double	global_lon, lon_tol;
 	char	buf[GRD_REMARK_LEN], *d, dir[GRD_REMARK_LEN];
 	FILE	*fp;
-	struct RASTER_INFO *rasinfo;
+	struct GRDRASTER_INFO *rasinfo;
 
 	if ((d = getenv ("GMT_GRIDDIR")) == CNULL)	/* Look in default location */
 		sprintf (dir, "%s%cshare%cdbase", GMTHOME, DIR_DELIM, DIR_DELIM);
@@ -620,7 +650,7 @@ int	load_rasinfo(struct RASTER_INFO **ras)
 	}
 
 	n_alloc = GMT_SMALL_CHUNK;
-	rasinfo = (struct RASTER_INFO *) GMT_memory (VNULL, n_alloc, sizeof (struct RASTER_INFO), GMT_program);
+	rasinfo = (struct GRDRASTER_INFO *) GMT_memory (VNULL, n_alloc, sizeof (struct GRDRASTER_INFO), GMT_program);
 	
 	while (fgets(rasinfo[nfound].h.command, GRD_COMMAND_LEN, fp) ) {
 		if (rasinfo[nfound].h.command[0] == '#') continue;
@@ -729,7 +759,7 @@ int	load_rasinfo(struct RASTER_INFO **ras)
 			GMT_io.in_col_type[1] = GMT_IS_FLOAT;
 		}
 			
-		if (GMT_get_common_args(buf, &rasinfo[nfound].h.x_min, &rasinfo[nfound].h.x_max, &rasinfo[nfound].h.y_min, &rasinfo[nfound].h.y_max) ) {
+		if (GMT_parse_common_options(buf, &rasinfo[nfound].h.x_min, &rasinfo[nfound].h.x_max, &rasinfo[nfound].h.y_min, &rasinfo[nfound].h.y_max) ) {
 			fprintf(stderr,"%s: error reading grdraster.info\n", GMT_program);
 			fprintf(stderr,"\t-R string conversion error.\n");
 			return(0);
@@ -769,6 +799,18 @@ int	load_rasinfo(struct RASTER_INFO **ras)
 			return(0);
 		}
 		rasinfo[nfound].h.node_offset = (rasinfo[nfound].h.command[i] == 'P') ? 1 : 0;
+		
+		/* Check if we have optional G (geographic) or C (Cartesian) that should override the auto test above */
+		
+		if (rasinfo[nfound].h.command[i+1] == 'G') {	/* Explicit geographic grid */
+			rasinfo[nfound].geo = TRUE;
+			i++;
+		}
+		else if (rasinfo[nfound].h.command[i+1] == 'C') {	/* Explicit Cartesian grid */
+			rasinfo[nfound].geo = FALSE;
+			i++;
+		}
+			
 		stop_point = i + 1;
 
 		/* Get type  */
@@ -899,27 +941,55 @@ int	load_rasinfo(struct RASTER_INFO **ras)
 			sprintf(rasinfo[nfound].h.remark, "%s%c%s", dir, DIR_DELIM, buf);
 		}
 
-		/* Decode SWAP flag, if present  */
+		/* Decode SWAP flag or SKIP command, if present  */
 
 		i = j + 1;
 		while (i < length && (rasinfo[nfound].h.command[i] == ' ' || rasinfo[nfound].h.command[i] == '\t') ) i++;
-		if (i < length) {	/* Swap flag set*/
+		if (i < length) {	/* Swap or skip flag set*/
 			switch (rasinfo[nfound].h.command[i]) {
 				case 'L':
 				case 'l':	/* Little endian byte order */
-					rasinfo[nfound].swap_me = (my_endian != 'L');	/* Must swap */
+					rasinfo[nfound].swap_me = (endian != 'L');	/* Must swap */
 					break;
 				case 'B':
 				case 'b':	/* Big endian byte order */
-					rasinfo[nfound].swap_me = (my_endian != 'B');	/* Must swap */
+					rasinfo[nfound].swap_me = (endian != 'B');	/* Must swap */
+					break;
+				case 'H':
+				case 'h':	/* Give header size for skipping */
+					rasinfo[nfound].skip = atoi (&rasinfo[nfound].h.command[i+1]);	/* Must skip header */
 					break;
 				default:
 					fprintf (stderr,"%s: error reading grdraster.info\n", GMT_program);
-					fprintf(stderr,"\tByte order conversion error.\n");
+					fprintf(stderr,"\tByte order or skip conversion error.\n");
 					return (0);
 			}
 		}
+		
+		j = i + 1;
+		while (j < length && (rasinfo[nfound].h.command[j] == ' ' || rasinfo[nfound].h.command[j] == '\t') ) j++;
 
+		if (j < length) {	/* Skip of swap flag following the first skip or Swap flag */
+			switch (rasinfo[nfound].h.command[j]) {
+				case 'L':
+				case 'l':	/* Little endian byte order */
+					rasinfo[nfound].swap_me = (endian != 'L');	/* Must swap */
+					break;
+				case 'B':
+				case 'b':	/* Big endian byte order */
+					rasinfo[nfound].swap_me = (endian != 'B');	/* Must swap */
+					break;
+				case 'H':
+				case 'h':	/* Give header size for skipping */
+					rasinfo[nfound].skip = atoi (&rasinfo[nfound].h.command[j+1]);	/* Must skip header */
+					break;
+				default:
+					fprintf (stderr,"%s: error reading grdraster.info\n", GMT_program);
+					fprintf(stderr,"\tByte order or skip conversion error.\n");
+					return (0);
+			}
+		}
+		
 		/* Get here when all is OK for this line:  */
 		global_lon = 360.0 - (1 - rasinfo[nfound].h.node_offset)*rasinfo[nfound].h.x_inc;
 		lon_tol = 0.01 * rasinfo[nfound].h.x_inc;
@@ -942,11 +1012,11 @@ int	load_rasinfo(struct RASTER_INFO **ras)
 		
 		if ((size_t)nfound == n_alloc) {
 			n_alloc += GMT_SMALL_CHUNK;
-			rasinfo = (struct RASTER_INFO *) GMT_memory ((void *)rasinfo, n_alloc, sizeof (struct RASTER_INFO), GMT_program);
+			rasinfo = (struct GRDRASTER_INFO *) GMT_memory ((void *)rasinfo, n_alloc, sizeof (struct GRDRASTER_INFO), GMT_program);
 		}
 	}
 	fclose (fp);
-	if (nfound > 0) rasinfo = (struct RASTER_INFO *) GMT_memory ((void *)rasinfo, nfound, sizeof (struct RASTER_INFO), GMT_program);
+	if (nfound > 0) rasinfo = (struct GRDRASTER_INFO *) GMT_memory ((void *)rasinfo, nfound, sizeof (struct GRDRASTER_INFO), GMT_program);
 	
 	*ras = rasinfo;
 	return (nfound);
