@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    $Id: fitcircle.c,v 1.3.4.6 2004/01/02 22:23:10 pwessel Exp $
+ *    $Id: fitcircle.c,v 1.25 2006/04/13 06:20:34 pwessel Exp $
  *
- *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -38,7 +38,7 @@
  *		between datum and mean; hence above average gives data far
  *		from mean larger weight in determining pole.  This is
  *		analogous to fitting line in plane, where data far from
- *		average abcissa have large leverage in determining slope.
+ *		average abscissa have large leverage in determining slope.
  * For L2:
  *	Create 3 x 3 matrix of sums of products of data vector elements.
  *	Find eigenvectors and eigenvalues of this matrix.
@@ -54,78 +54,98 @@
  *
  * Author:	W. H. F. Smith
  * Date:	16 September 1991.
- * Version:	3.4.3
+ * Version:	4
  *
  */
 
 #include "gmt.h"
 
-struct	DATA {
-	double	x[3];
-}	*data;
+struct FITCIRCLE_CTRL {	/* All control options for this program (except common args) */
+	/* active is TRUE if the option has been activated */
+	struct L {	/* -L[<n>] */
+		BOOLEAN active;
+		int norm;	/* 1, 2, or 3 (both) */
+	} L;
+	struct S {	/* -S */
+		BOOLEAN active;
+	} S;
+};
 
-main (int argc, char **argv)
+struct	FITCIRCLE_DATA {
+	double	x[3];
+};
+
+int main (int argc, char **argv)
 {
-	int	i, j, k, imin, imax, n_alloc, n_data, n, np, nrots, norm = -1, n_fields, n_expected_fields;
-	BOOLEAN	error = FALSE, greenwich = FALSE, find_small_circle = FALSE;
+	BOOLEAN	error = FALSE, greenwich = FALSE;
+
+	char	format[BUFSIZ];
+
+	int	i, j, k, imin, imax, n_alloc, n_data, n, np, nrots, n_fields, n_expected_fields;
 
 	double	lonsum, latsum, *in;
 	double	meanv[3], cross[3], cross_sum[3], gcpole[3], scpole[3];		/* Extra vectors  */
 	double	*a, *lambda, *v, *b, *z;	/* Matrix stuff */
-	double	get_small_circle(struct DATA *data, int ndata, double *center, double *gcpole, double *scpole, int norm, double *work), rad, *work;
-
-	char	buffer[BUFSIZ], format[BUFSIZ];
+	double	get_small_circle (struct FITCIRCLE_DATA *data, int ndata, double *center, double *gcpole, double *scpole, int norm, double *work);
+	double	rad, *work = VNULL;
 
 	FILE	*fp = NULL;
 
+	struct	FITCIRCLE_DATA *data;
+	struct FITCIRCLE_CTRL *Ctrl;
+
+	void *New_Fitcircle_Ctrl (), Free_Fitcircle_Ctrl (struct FITCIRCLE_CTRL *C);
+
 	argc = GMT_begin (argc, argv);
 
-        for (i = 1; i < argc; i++) {
-                if (argv[i][0] == '-') {
-                        switch (argv[i][1]) {
-			
-		
+ 	Ctrl = (struct FITCIRCLE_CTRL *) New_Fitcircle_Ctrl ();		/* Allocate and initialize defaults in a new control structure */
+
+	for (i = 1; i < argc; i++) {
+		if (argv[i][0] == '-') {
+			switch (argv[i][1]) {
+
+
 				/* Common parameters */
-			
+
 				case 'H':
 				case 'V':
 				case ':':
-				case '\0':
-					error += GMT_get_common_args (argv[i], 0, 0, 0, 0);
-					break;
-				
-				/* Supplemental parameters */
-				
   				case 'b':
-					error += GMT_io_selection (&argv[i][2]);
+  				case 'f':
+				case '\0':
+					error += GMT_parse_common_options (argv[i], 0, 0, 0, 0);
 					break;
 
+				/* Supplemental parameters */
+
 				case 'L':
-					norm = 3;
-                                        if (argv[i][2]) norm = atoi(&argv[i][2]);
-                                        break;
-                                case 'S':
-					find_small_circle = TRUE;
-                                        break;
-                                default:
-                                        error = TRUE;
-                                        GMT_default_error (argv[i][1]);
+					Ctrl->L.active = TRUE;
+					Ctrl->L.norm = 3;
+					if (argv[i][2]) Ctrl->L.norm = atoi(&argv[i][2]);
+					break;
+				case 'S':
+					Ctrl->S.active = TRUE;
                                         break;
+				default:
+					error = TRUE;
+					GMT_default_error (argv[i][1]);
+					break;
                         }
                 }
-                else {
-                        if ((fp = GMT_fopen(argv[i], GMT_io.r_mode)) == NULL) {
-                                fprintf (stderr, "%s: Could not open file %s\n", GMT_program, argv[i]);
-                                error = TRUE;
-                        }
-                }
-        }
+		else {
+			if ((fp = GMT_fopen(argv[i], GMT_io.r_mode)) == NULL) {
+				fprintf (stderr, "%s: Could not open file %s\n", GMT_program, argv[i]);
+				error = TRUE;
+			}
+		}
+	}
 
-	if (argc == 1 || GMT_quick) {
+	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "fitcircle %s - find best-fitting great circle to points on sphere\n\n", GMT_VERSION);
-		fprintf(stderr,"usage:  fitcircle [<input_file>] -L[<n>] [-H[<nrec>]] [-S] [-V] [-:] [-bi[s][<n>]]\n\n");
-		
-		if (GMT_quick) exit (EXIT_FAILURE);
+		fprintf(stderr,"usage:  fitcircle [<input_file>] -L[<n>] [%s] [-S] [-V] [%s]\n", GMT_H_OPT, GMT_t_OPT);
+		fprintf(stderr,"\t[%s] [%s]\n\n", GMT_bi_OPT, GMT_f_OPT);
+
+		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
 		fprintf(stderr,"\tReads from input_file or standard input\n");
 		fprintf(stderr,"\t-L specify norm as -L1 or -L2; or use -L or -L3 to give both.\n");
@@ -136,49 +156,50 @@ main (int argc, char **argv)
 		GMT_explain_option (':');
 		GMT_explain_option ('i');
 		GMT_explain_option ('n');
+		GMT_explain_option ('f');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
 
- 	if (norm < 1 || norm > 3) {
+ 	if (Ctrl->L.norm < 1 || Ctrl->L.norm > 3) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -L option:  Choose between 1, 2, or 3\n", GMT_program);
 		error++;
 	}
-	if (GMT_io.binary[0] && gmtdefs.io_header) {
+	if (GMT_io.binary[GMT_IN] && GMT_io.io_header[GMT_IN]) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data cannot have header -H\n", GMT_program);
 		error++;
 	}
-        if (GMT_io.binary[0] && GMT_io.ncol[0] == 0) GMT_io.ncol[0] = 2;
-        if (GMT_io.binary[0] && GMT_io.ncol[0] < 2) {
-                fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data (-bi) must haveat least 2 columns\n", GMT_program);
+        if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] == 0) GMT_io.ncol[GMT_IN] = 2;
+        if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] < 2) {
+                fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data (-bi) must have at least 2 columns\n", GMT_program);
 		error++;
 	}
 	if (error) exit (EXIT_FAILURE);
 
-	GMT_put_history (argc, argv);	/* Update .gmtcommands */
+	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
 
-	if (GMT_io.binary[0] && gmtdefs.verbose) {
+	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[0], type[GMT_io.single_precision[0]]);
+		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 
 	if (fp == NULL) {
 		fp = GMT_stdin;
 #ifdef SET_IO_MODE
-		GMT_setmode (0);
+		GMT_setmode (GMT_IN);
 #endif
 	}
 	n_alloc = GMT_CHUNK;
 	n_data = 0;
 	lonsum = latsum = 0.0;
-	sprintf (format, "%s\t%s", gmtdefs.d_format, gmtdefs.d_format);
-	
-	data = (struct DATA *) GMT_memory (VNULL, (size_t)n_alloc, sizeof(struct DATA), GMT_program);
 
-	if (gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) GMT_fgets (buffer, BUFSIZ, fp);
-		
-	n_expected_fields = (GMT_io.ncol[0]) ? GMT_io.ncol[0] : 2;
+	data = (struct FITCIRCLE_DATA *) GMT_memory (VNULL, (size_t)n_alloc, sizeof(struct FITCIRCLE_DATA), GMT_program);
+
+	if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (format, BUFSIZ, fp);
+
+	sprintf (format, "%s\t%s", gmtdefs.d_format, gmtdefs.d_format);
+	n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : 2;
 
 	while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
@@ -194,13 +215,13 @@ main (int argc, char **argv)
 
 		if (n_data == n_alloc) {
 			n_alloc += GMT_CHUNK;
-      			data = (struct DATA *) GMT_memory ((void *)data, (size_t)n_alloc, sizeof(struct DATA), GMT_program);
+      			data = (struct FITCIRCLE_DATA *) GMT_memory ((void *)data, (size_t)n_alloc, sizeof(struct FITCIRCLE_DATA), GMT_program);
 	        }
         }
         if (fp != GMT_stdin) GMT_fclose(fp);
         
-      	data = (struct DATA *) GMT_memory ((void *)data, (size_t)n_data, sizeof(struct DATA), GMT_program);
-	if (find_small_circle && (norm%2) ) work = (double *) GMT_memory (VNULL, (size_t)n_data, sizeof(double), GMT_program);
+      	data = (struct FITCIRCLE_DATA *) GMT_memory ((void *)data, (size_t)n_data, sizeof(struct FITCIRCLE_DATA), GMT_program);
+	if (Ctrl->S.active && (Ctrl->L.norm%2) ) work = (double *) GMT_memory (VNULL, (size_t)n_data, sizeof(double), GMT_program);
 
 	lonsum /= n_data;
 	latsum /= n_data;
@@ -221,7 +242,7 @@ main (int argc, char **argv)
 
 	GMT_normalize3v (meanv);
 
-	if (norm%2) {
+	if (Ctrl->L.norm%2) {
 		GMT_cart_to_geo (&latsum, &lonsum, meanv, TRUE);
 		if (greenwich && lonsum < 0.0) lonsum += 360.0;
 		fprintf (GMT_stdout, format, lonsum, latsum);
@@ -242,8 +263,8 @@ main (int argc, char **argv)
 			}
 		}
 		GMT_normalize3v (cross_sum);
-		if (find_small_circle) for (i = 0; i < 3; i++) gcpole[i] = cross_sum[i];
-		
+		if (Ctrl->S.active) for (i = 0; i < 3; i++) gcpole[i] = cross_sum[i];
+
 		GMT_cart_to_geo (&latsum, &lonsum, cross_sum, TRUE);
 		if (greenwich && lonsum < 0.0) lonsum += 360.0;
 		fprintf (GMT_stdout, format, lonsum, latsum);
@@ -253,7 +274,7 @@ main (int argc, char **argv)
 		if (greenwich && lonsum < 0.0) lonsum += 360.0;
 		fprintf (GMT_stdout, format, lonsum, latsum);
 		fprintf (GMT_stdout, "\tL1 S Hemisphere Great Circle Pole (Cross-Averaged)\n");
-		if (find_small_circle) {
+		if (Ctrl->S.active) {
 			if (gmtdefs.verbose) fprintf (stderr,"Fitting small circle using L1 norm.\n");
 			rad = get_small_circle (data, n_data, meanv, gcpole, scpole, 1, work);
 			if (rad >= 0.0) {
@@ -268,7 +289,7 @@ main (int argc, char **argv)
 	}
 
 	sprintf (format, "%s\t%s", gmtdefs.d_format, gmtdefs.d_format);
-	if (norm/2) {
+	if (Ctrl->L.norm/2) {
 
 		n = 3;
 		np = n;
@@ -297,13 +318,13 @@ main (int argc, char **argv)
 		if (greenwich && lonsum < 0.0) lonsum += 360.0;
 		fprintf (GMT_stdout, format, lonsum, latsum);
 		fprintf (GMT_stdout, "\tL2 Average Position (Eigenval Method)\n");
-		
+
 		if (v[imin*np+2] < 0.0)	/* Eigvec is in S Hemisphere  */
 			for (i = 0; i < 3; i++) gcpole[i] = -v[imin*np+i];
 		else
 			for (i = 0; i < 3; i++) gcpole[i] = v[imin*np+i];
-		
-		GMT_cart_to_geo (&latsum, &lonsum, gcpole, TRUE);		
+
+		GMT_cart_to_geo (&latsum, &lonsum, gcpole, TRUE);
 		if (greenwich && lonsum < 0.0) lonsum += 360.0;
 		fprintf (GMT_stdout, format, lonsum, latsum);
 		fprintf (GMT_stdout, "\tL2 N Hemisphere Great Circle Pole (Eigenval Method)\n");
@@ -318,7 +339,7 @@ main (int argc, char **argv)
 		GMT_free ((void *)b);
 		GMT_free ((void *)lambda);
 		GMT_free ((void *)a);
-		if (find_small_circle) {
+		if (Ctrl->S.active) {
 			if (gmtdefs.verbose) fprintf(stderr,"%s: Fitting small circle using L2 norm.\n", GMT_program);
 			rad = get_small_circle(data, n_data, meanv, gcpole, scpole, 2, work);
 			if (rad >= 0.0) {
@@ -332,12 +353,17 @@ main (int argc, char **argv)
 			}
 		}
 	}
-	if (find_small_circle && (norm%2) ) GMT_free ((void *)work);
+	if (Ctrl->S.active && (Ctrl->L.norm%2) ) GMT_free ((void *)work);
 	GMT_free ((void *)data);
+
+	Free_Fitcircle_Ctrl (Ctrl);	/* Deallocate control structure */
+	
 	GMT_end (argc, argv);
+	
+	exit (EXIT_SUCCESS);
 }
 
-double	get_small_circle (struct DATA *data, int ndata, double *center, double *gcpole, double *scpole, int norm, double *work)
+double	get_small_circle (struct FITCIRCLE_DATA *data, int ndata, double *center, double *gcpole, double *scpole, int norm, double *work)
 {
 
 	/* Find scpole, the pole to the best-fit small circle, 
@@ -348,7 +374,7 @@ double	get_small_circle (struct DATA *data, int ndata, double *center, double *g
 	int	i, j;
 	double	temppole[3], a[3], b[3], oldpole[3];
 	double	trypos, tryneg, afit, bfit, afactor, bfactor, fit, oldfit;
-	double	length_ab, length_aold, length_bold, circle_misfit(struct DATA *data, int ndata, double *pole, int norm, double *work, double *circle_distance), circle_distance;
+	double	length_ab, length_aold, length_bold, circle_misfit(struct FITCIRCLE_DATA *data, int ndata, double *pole, int norm, double *work, double *circle_distance), circle_distance;
 
 	/* First find out if solution is between center and gcpole,
 		or center and -gcpole:  */
@@ -447,7 +473,7 @@ double	get_small_circle (struct DATA *data, int ndata, double *center, double *g
 	return (R2D * circle_distance);
 }
 
-double	circle_misfit(struct DATA *data, int ndata, double *pole, int norm, double *work, double *circle_distance)
+double	circle_misfit(struct FITCIRCLE_DATA *data, int ndata, double *pole, int norm, double *work, double *circle_distance)
 {
 	/* Find the L(norm) misfit between a small circle through
 		center with pole pole.  Return misfit in radians.  */
@@ -488,3 +514,19 @@ double	circle_misfit(struct DATA *data, int ndata, double *pole, int norm, doubl
 	}
 	return (norm == 1) ? misfit : sqrt (misfit);
 }
+
+void *New_Fitcircle_Ctrl () {	/* Allocate and initialize a new control structure */
+	struct FITCIRCLE_CTRL *C;
+	
+	C = (struct FITCIRCLE_CTRL *) GMT_memory (VNULL, 1, sizeof (struct FITCIRCLE_CTRL), "New_Fitcircle_Ctrl");
+	
+	/* Initialize values whose defaults are not 0/FALSE/NULL */
+	
+	C->L.norm = -1;
+	
+	return ((void *)C);
+}
+
+void Free_Fitcircle_Ctrl (struct FITCIRCLE_CTRL *C) {	/* Deallocate control structure */
+	GMT_free ((void *)C);	
+}
