@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt_shore.c,v 1.9 2004/01/02 22:45:13 pwessel Exp $
+ *	$Id: gmt_shore.c,v 1.21 2006/04/10 04:43:31 pwessel Exp $
  *
- *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -56,7 +56,7 @@ void GMT_br_to_degree (struct GMT_BR *c, short int dx, short int dy, double *lon
 void shore_prepare_sides(struct GMT_SHORE *c, int dir);
 int GMT_shore_asc_sort (const void *a, const void *b);
 int GMT_shore_desc_sort(const void *a, const void *b);
-int check_nc_status (int status);
+BOOLEAN shore_getpathname (char *name, char *path);
 
 int GMT_set_resolution (char *res, char opt)
 {
@@ -98,11 +98,11 @@ int GMT_init_shore (char res, struct GMT_SHORE *c, double w, double e, double s,
 	short *stmp;
 	int *itmp;
 	size_t start[1], count[1];
-	char file[32], path[BUFSIZ];
+	char file[GMT_TEXT_LEN], path[BUFSIZ];
 	
 	sprintf (file, "binned_GSHHS_%c.cdf", res);
 	
-        if (!GMT_getpathname (file, path)) return (-1);	/* Failed to find file */
+        if (!shore_getpathname (file, path)) return (-1);	/* Failed to find file */
         
 	check_nc_status (nc_open (path, NC_NOWRITE,&c->cdfid));
                 
@@ -281,14 +281,14 @@ int GMT_init_br (char which, char res, struct GMT_BR *c, double w, double e, dou
 	short *stmp;
 	int *itmp;
 	size_t start[1], count[1];
-	char file[32], path[BUFSIZ];
+	char file[GMT_TEXT_LEN], path[BUFSIZ];
 	
 	if (which == 'r')
 		sprintf (file, "binned_river_%c.cdf", res);
 	else
 		sprintf (file, "binned_border_%c.cdf", res);
 	
-        if (!GMT_getpathname (file, path)) return (-1);	/* Failed to find file */
+        if (!shore_getpathname (file, path)) return (-1);	/* Failed to find file */
 
 	check_nc_status (nc_open (path, NC_NOWRITE, &c->cdfid));
         
@@ -434,7 +434,7 @@ void GMT_get_br_bin (int b, struct GMT_BR *c, int *level, int n_levels)
 	
 }
 
-int GMT_assemble_shore (struct GMT_SHORE *c, int dir, int first_level, BOOLEAN assemble, BOOLEAN shift, double west, double east, struct POL **pol)
+int GMT_assemble_shore (struct GMT_SHORE *c, int dir, int first_level, BOOLEAN assemble, BOOLEAN shift, double west, double east, struct GMT_GSHHS_POL **pol)
                 
                      
 /* assemble: TRUE if polygons is needed */
@@ -442,15 +442,15 @@ int GMT_assemble_shore (struct GMT_SHORE *c, int dir, int first_level, BOOLEAN a
 /* edge: Edge test for shifting */
 
 {
-	struct POL *p;
-	int start_side, next_side, id, P = 0, more, p_alloc, wet_or_dry, use_this_level, high_seg_level = MAX_LEVEL;
-	int n_alloc, cid, nid, add, first_pos, entry_pos, n, low_level, high_level, nseg_at_level[MAX_LEVEL+1];
+	struct GMT_GSHHS_POL *p;
+	int start_side, next_side, id, P = 0, more, p_alloc, wet_or_dry, use_this_level, high_seg_level = GMT_MAX_GSHHS_LEVEL;
+	int n_alloc, cid, nid, add, first_pos, entry_pos, n, low_level, high_level, nseg_at_level[GMT_MAX_GSHHS_LEVEL+1];
 	BOOLEAN completely_inside;
 	double *xtmp, *ytmp, plon, plat;
 	
 	if (!assemble) {	/* Easy, just need to scale all segments to degrees and return */
 	
-		p = (struct POL *) GMT_memory (VNULL, (size_t)c->ns, sizeof (struct POL), "GMT_assemble_shore");
+		p = (struct GMT_GSHHS_POL *) GMT_memory (VNULL, (size_t)c->ns, sizeof (struct GMT_GSHHS_POL), "GMT_assemble_shore");
 		
 		for (id = 0; id < c->ns; id++) {
 			p[id].lon = (double *) GMT_memory (VNULL, (size_t)c->seg[id].n, sizeof (double), "GMT_assemble_shore");
@@ -467,9 +467,9 @@ int GMT_assemble_shore (struct GMT_SHORE *c, int dir, int first_level, BOOLEAN a
 	
 	/* Check the consistency of node levels in case some features are dropped */
 	
-	memset ((void *)nseg_at_level, 0, (size_t)((MAX_LEVEL + 1) * sizeof (int)));
+	memset ((void *)nseg_at_level, 0, (size_t)((GMT_MAX_GSHHS_LEVEL + 1) * sizeof (int)));
 	for (id = 0; id < c->ns; id++) if (c->seg[id].entry != 4) nseg_at_level[c->seg[id].level]++;	/* Only count segments that crosses the bin */
-	for (n = 0; n <= MAX_LEVEL; n++) if (nseg_at_level[n]) high_seg_level = n;
+	for (n = 0; n <= GMT_MAX_GSHHS_LEVEL; n++) if (nseg_at_level[n]) high_seg_level = n;
 	
 	if (c->ns == 0) for (n = 0; n < 4; n++) high_seg_level = MIN (c->node_level[n], high_seg_level);	/* Initialize to lowest when there are no segments */
 	for (n = high_level = 0; n < 4; n++) {
@@ -489,9 +489,9 @@ int GMT_assemble_shore (struct GMT_SHORE *c, int dir, int first_level, BOOLEAN a
 	shore_prepare_sides (c, dir);
 	
 	p_alloc = (c->ns == 0) ? 1 : GMT_SMALL_CHUNK;
-	p = (struct POL *) GMT_memory (VNULL, (size_t)p_alloc, sizeof (struct POL), "GMT_assemble_shore");
+	p = (struct GMT_GSHHS_POL *) GMT_memory (VNULL, (size_t)p_alloc, sizeof (struct GMT_GSHHS_POL), "GMT_assemble_shore");
 	
-	low_level = MAX_LEVEL;
+	low_level = GMT_MAX_GSHHS_LEVEL;
 	
 	if (completely_inside && use_this_level) {	/* Must include path of this bin outline as first polygon */
 		p[0].n = GMT_graticule_path (&p[0].lon, &p[0].lat, dir, c->lon_corner[3], c->lon_corner[1], c->lat_corner[0], c->lat_corner[2]);
@@ -502,7 +502,7 @@ int GMT_assemble_shore (struct GMT_SHORE *c, int dir, int first_level, BOOLEAN a
 	
 	while (c->n_entries > 0) {	/* More segments to connect */
 	
-		low_level = MAX_LEVEL;
+		low_level = GMT_MAX_GSHHS_LEVEL;
 		start_side = 0;
 		id = GMT_shore_get_first_entry (c, dir, &start_side);
 		next_side = c->seg[id].exit;
@@ -566,7 +566,7 @@ int GMT_assemble_shore (struct GMT_SHORE *c, int dir, int first_level, BOOLEAN a
 		P++;
 		if (P == p_alloc) {
 			p_alloc += GMT_SMALL_CHUNK;
-			p = (struct POL *) GMT_memory ((void *)p, (size_t)p_alloc, sizeof (struct POL), "GMT_assemble_shore");
+			p = (struct GMT_GSHHS_POL *) GMT_memory ((void *)p, (size_t)p_alloc, sizeof (struct GMT_GSHHS_POL), "GMT_assemble_shore");
 		}
 		
 	}
@@ -584,13 +584,13 @@ int GMT_assemble_shore (struct GMT_SHORE *c, int dir, int first_level, BOOLEAN a
 		P++;
 		if (P == p_alloc) {
 			p_alloc += GMT_SMALL_CHUNK;
-			p = (struct POL *) GMT_memory ((void *)p, (size_t)p_alloc, sizeof (struct POL), "GMT_assemble_shore");
+			p = (struct GMT_GSHHS_POL *) GMT_memory ((void *)p, (size_t)p_alloc, sizeof (struct GMT_GSHHS_POL), "GMT_assemble_shore");
 		}
 	}
 	
 	GMT_shore_pau_sides (c);
 
-	if (c->ns > 0) p = (struct POL *) GMT_memory ((void *)p, (size_t)P, sizeof (struct POL), "GMT_assemble_shore");
+	if (c->ns > 0) p = (struct GMT_GSHHS_POL *) GMT_memory ((void *)p, (size_t)P, sizeof (struct GMT_GSHHS_POL), "GMT_assemble_shore");
 	
 	for (id = 0; id < P; id++) GMT_shore_path_shift2 (p[id].lon, p[id].lat, p[id].n, west, east, c->leftmost_bin);
 
@@ -598,14 +598,14 @@ int GMT_assemble_shore (struct GMT_SHORE *c, int dir, int first_level, BOOLEAN a
 	return (P);
 }
 		
-int GMT_assemble_br (struct GMT_BR *c, BOOLEAN shift, double edge, struct POL **pol)
+int GMT_assemble_br (struct GMT_BR *c, BOOLEAN shift, double edge, struct GMT_GSHHS_POL **pol)
 /* shift: TRUE if longitudes may have to be shifted */
 /* edge: Edge test for shifting */       
 {
-	struct POL *p;
+	struct GMT_GSHHS_POL *p;
 	int id;
 	
-	p = (struct POL *) GMT_memory (VNULL, (size_t)c->ns, sizeof (struct POL), "GMT_assemble_br");
+	p = (struct GMT_GSHHS_POL *) GMT_memory (VNULL, (size_t)c->ns, sizeof (struct GMT_GSHHS_POL), "GMT_assemble_br");
 	
 	for (id = 0; id < c->ns; id++) {
 		p[id].lon = (double *) GMT_memory (VNULL, (size_t)c->seg[id].n, sizeof (double), "GMT_assemble_br");
@@ -659,7 +659,7 @@ void GMT_br_cleanup (struct GMT_BR *c)
 	
 }
 
-int GMT_prep_polygons (struct POL **p_old, int np, BOOLEAN greenwich, BOOLEAN sample, double step, int anti_bin)
+int GMT_prep_polygons (struct GMT_GSHHS_POL **p_old, int np, BOOLEAN greenwich, BOOLEAN sample, double step, int anti_bin)
 {
 	/* This function will go through each of the polygons and determine
 	 * if the polygon is clipped by the map boundary, and if so if it
@@ -673,11 +673,13 @@ int GMT_prep_polygons (struct POL **p_old, int np, BOOLEAN greenwich, BOOLEAN sa
 	 * sample is TRUE if we need to resample the polygons to reduce point spacing
 	 * step is the new maximum point separation in degrees
 	 * anti_bin, if >= 0, indicates a possible problem bin at the antipole using -JE only
+	 * We also explicitly close all polygons if they are not so already.
 	 */
 
-	int k, np_new, n_use, n, start;
+	int k, np_new, n_use, n, start, n_alloc;
+	BOOLEAN close;
 	double *xtmp, *ytmp;
-	struct POL *p;
+	struct GMT_GSHHS_POL *p;
 
 	p = *p_old;
 
@@ -686,7 +688,7 @@ int GMT_prep_polygons (struct POL **p_old, int np, BOOLEAN greenwich, BOOLEAN sa
 	for (k = 0; k < np; k++) {
 			
 		if (sample) p[k].n = GMT_fix_up_path (&p[k].lon, &p[k].lat, p[k].n, greenwich, step);
-				
+		
 		/* Clip polygon against map boundary if necessary and return plot x,y in inches */
 				
 		if ((n = GMT_clip_to_map (p[k].lon, p[k].lat, p[k].n, &xtmp, &ytmp)) == 0) {	/* Completely outside */
@@ -703,23 +705,35 @@ int GMT_prep_polygons (struct POL **p_old, int np, BOOLEAN greenwich, BOOLEAN sa
 			GMT_n_plot = (*GMT_truncate) (xtmp, ytmp, n, start, -1);
 			n_use = GMT_compact_line (GMT_x_plot, GMT_y_plot, GMT_n_plot, FALSE, 0);
 			if (project_info.three_D) GMT_2D_to_3D (GMT_x_plot, GMT_y_plot, project_info.z_level, GMT_n_plot);
-			p[k].lon = (double *) GMT_memory ((void *)p[k].lon, (size_t)n_use, sizeof (double), GMT_program);
-			p[k].lat = (double *) GMT_memory ((void *)p[k].lat, (size_t)n_use, sizeof (double), GMT_program);
+			close = GMT_polygon_is_open (GMT_x_plot, GMT_y_plot, n_use);
+			n_alloc = (close) ? n_use + 1 : n_use;
+			p[k].lon = (double *) GMT_memory ((void *)p[k].lon, (size_t)n_alloc, sizeof (double), GMT_program);
+			p[k].lat = (double *) GMT_memory ((void *)p[k].lat, (size_t)n_alloc, sizeof (double), GMT_program);
 			memcpy ((void *)p[k].lon, (void *)GMT_x_plot, (size_t)(n_use * sizeof (double)));
 			memcpy ((void *)p[k].lat, (void *)GMT_y_plot, (size_t)(n_use * sizeof (double)));
-			p[k].n = n_use;
+			if (close) {	/* Must explicitly close the polygon */
+				p[k].lon[n_use] = p[k].lon[0];
+				p[k].lat[n_use] = p[k].lat[0];
+			}
+			p[k].n = n_alloc;
 								
 			/* Then truncate agains right border */
 						
 			GMT_n_plot = (*GMT_truncate) (xtmp, ytmp, n, start, +1);
 			n_use = GMT_compact_line (GMT_x_plot, GMT_y_plot, GMT_n_plot, FALSE, 0);
 			if (project_info.three_D) GMT_2D_to_3D (GMT_x_plot, GMT_y_plot, project_info.z_level, GMT_n_plot);
-			p = (struct POL *) GMT_memory ((void *)p, (size_t)(np_new + 1), sizeof (struct POL), GMT_program);
-			p[np_new].lon = (double *) GMT_memory (VNULL, (size_t)n_use, sizeof (double), GMT_program);
-			p[np_new].lat = (double *) GMT_memory (VNULL, (size_t)n_use, sizeof (double), GMT_program);
+			p = (struct GMT_GSHHS_POL *) GMT_memory ((void *)p, (size_t)(np_new + 1), sizeof (struct GMT_GSHHS_POL), GMT_program);
+			close = GMT_polygon_is_open (GMT_x_plot, GMT_y_plot, n_use);
+			n_alloc = (close) ? n_use + 1 : n_use;
+			p[np_new].lon = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
+			p[np_new].lat = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
 			memcpy ((void *)p[np_new].lon, (void *)GMT_x_plot, (size_t)(n_use * sizeof (double)));
 			memcpy ((void *)p[np_new].lat, (void *)GMT_y_plot, (size_t)(n_use * sizeof (double)));
-			p[np_new].n = n_use;
+			if (close) {	/* Must explicitly close the polygon */
+				p[np_new].lon[n_use] = p[np_new].lon[0];
+				p[np_new].lat[n_use] = p[np_new].lat[0];
+			}
+			p[np_new].n = n_alloc;
 			p[np_new].interior = p[k].interior;
 			p[np_new].level = p[k].level;
 			np_new++;
@@ -735,11 +749,17 @@ int GMT_prep_polygons (struct POL **p_old, int np, BOOLEAN greenwich, BOOLEAN sa
 			}
 					
 			else {
-				p[k].lon = (double *) GMT_memory ((void *)p[k].lon, (size_t)n_use, sizeof (double), GMT_program);
-				p[k].lat = (double *) GMT_memory ((void *)p[k].lat, (size_t)n_use, sizeof (double), GMT_program);
+				close = GMT_polygon_is_open (xtmp, ytmp, n_use);
+				n_alloc = (close) ? n_use + 1 : n_use;
+				p[k].lon = (double *) GMT_memory ((void *)p[k].lon, (size_t)n_alloc, sizeof (double), GMT_program);
+				p[k].lat = (double *) GMT_memory ((void *)p[k].lat, (size_t)n_alloc, sizeof (double), GMT_program);
 				memcpy ((void *)p[k].lon, (void *)xtmp, (size_t)(n_use * sizeof (double)));
 				memcpy ((void *)p[k].lat, (void *)ytmp, (size_t)(n_use * sizeof (double)));
-				p[k].n = n_use;
+				if (close) {	/* Must explicitly close the polygon */
+					p[k].lon[n_use] = p[k].lon[0];
+					p[k].lat[n_use] = p[k].lat[0];
+				}
+				p[k].n = n_alloc;
 			}
 		}
 				
@@ -786,12 +806,12 @@ int GMT_shore_get_next_entry (struct GMT_SHORE *c, int dir, int side, int id)
 {
 	/* Finds the next entry point on the given side that is further away
 	 * in the <dir> direction than previous point.  It removes the info
-	 * regarding the new entry from the SIDE structure */
+	 * regarding the new entry from the GSHHS_SIDE structure */
 	 
 	int k, pos, n;
 	
 	if (id < 0)
-		pos = (dir == 1) ? 0 : MAX_DELTA;
+		pos = (dir == 1) ? 0 : GSHHS_MAX_DELTA;
 	else {
 		n = c->seg[id].n - 1;
 		pos = GMT_shore_get_position (side, c->seg[id].dx[n], c->seg[id].dy[n]);
@@ -826,15 +846,15 @@ int GMT_shore_get_first_entry (struct GMT_SHORE *c, int dir, int *side)
 
 int GMT_shore_asc_sort (const void *a, const void *b)
 {
-	if (((struct SIDE *)a)->pos < ((struct SIDE *)b)->pos) return (-1);
-	if (((struct SIDE *)a)->pos > ((struct SIDE *)b)->pos) return (1);
+	if (((struct GSHHS_SIDE *)a)->pos < ((struct GSHHS_SIDE *)b)->pos) return (-1);
+	if (((struct GSHHS_SIDE *)a)->pos > ((struct GSHHS_SIDE *)b)->pos) return (1);
 	return (0);
 }
 
 int GMT_shore_desc_sort (const void *a, const void *b)
 {
-	if (((struct SIDE *)a)->pos < ((struct SIDE *)b)->pos) return (1);
-	if (((struct SIDE *)a)->pos > ((struct SIDE *)b)->pos) return (-1);
+	if (((struct GSHHS_SIDE *)a)->pos < ((struct GSHHS_SIDE *)b)->pos) return (1);
+	if (((struct GSHHS_SIDE *)a)->pos > ((struct GSHHS_SIDE *)b)->pos) return (-1);
 	return (0);
 }
 
@@ -844,7 +864,7 @@ void GMT_shore_pau_sides (struct GMT_SHORE *c)
 	for (i = 0; i < 4; i++) GMT_free ((void *)c->side[i]);
 }
 
-void GMT_free_polygons (struct POL *p, int n)
+void GMT_free_polygons (struct GMT_GSHHS_POL *p, int n)
 {
 	int k;
 	for (k = 0; k < n; k++) {
@@ -884,7 +904,7 @@ int GMT_shore_get_position (int side, short int x, short int y)
 {
 	/* Returns the position along the given side */
 	
-	return ((side%2) ? ((side == 1) ? (unsigned short)y : MAX_DELTA - (unsigned short)y) : ((side == 0) ? (unsigned short)x : MAX_DELTA - (unsigned short)x));
+	return ((side%2) ? ((side == 1) ? (unsigned short)y : GSHHS_MAX_DELTA - (unsigned short)y) : ((side == 0) ? (unsigned short)x : GSHHS_MAX_DELTA - (unsigned short)x));
 }
 
 void shore_prepare_sides (struct GMT_SHORE *c, int dir)
@@ -905,8 +925,8 @@ void shore_prepare_sides (struct GMT_SHORE *c, int dir)
 	for (s = 0; s < c->ns; s++) if (c->seg[s].entry < 4) c->nside[c->seg[s].entry]++;
 	
 	for (i = c->n_entries = 0; i < 4; i++) {	/* Allocate memory and add corners */
-		c->side[i] = (struct SIDE *) GMT_memory (VNULL, (size_t)c->nside[i], sizeof (struct SIDE), "shore_prepare_sides");
-		c->side[i][0].pos = (dir == 1) ? MAX_DELTA : 0;
+		c->side[i] = (struct GSHHS_SIDE *) GMT_memory (VNULL, (size_t)c->nside[i], sizeof (struct GSHHS_SIDE), "shore_prepare_sides");
+		c->side[i][0].pos = (dir == 1) ? GSHHS_MAX_DELTA : 0;
 		c->side[i][0].id = i - 4;
 		c->n_entries += c->nside[i] - 1;
 	}
@@ -921,9 +941,85 @@ void shore_prepare_sides (struct GMT_SHORE *c, int dir)
 	
 	for (i = 0; i < 4; i++)	{	/* sort on position */
 		if (dir == 1)
-			qsort ((void *)c->side[i], (size_t)c->nside[i], sizeof (struct SIDE), GMT_shore_asc_sort);
+			qsort ((void *)c->side[i], (size_t)c->nside[i], sizeof (struct GSHHS_SIDE), GMT_shore_asc_sort);
 		else
-			qsort ((void *)c->side[i], (size_t)c->nside[i], sizeof (struct SIDE), GMT_shore_desc_sort);
+			qsort ((void *)c->side[i], (size_t)c->nside[i], sizeof (struct GSHHS_SIDE), GMT_shore_desc_sort);
 	}
 }
 
+BOOLEAN shore_getpathname (char *name, char *path) {
+	/* Prepends the appropriate directory to the file name
+	 * and returns TRUE if file is readable. */
+	 
+	BOOLEAN found;
+	char dir[BUFSIZ];
+	FILE *fp;
+
+	/* First check the $GMTHOME/share/coast directory */
+
+	sprintf (path, "%s%cshare%ccoast%c%s", GMTHOME, DIR_DELIM, DIR_DELIM, DIR_DELIM, name);
+	if (!access (path, R_OK)) return (TRUE);	/* File exists and is readable, return with name */
+
+	/* File was not readable.  Now check if it exists */
+
+	if (!access (path, F_OK))  { /* Kicks in if file is there, meaning it has the wrong permissions */
+		fprintf (stderr, "%s: Error: GMT does not have permission to open %s!\n", GMT_program, path);
+		exit (EXIT_FAILURE);
+	}
+	/* Nothing in share/coast; do a backwards-compatible check in the $GMTHOME/share directory */
+
+	sprintf (path, "%s%cshare%c%s", GMTHOME, DIR_DELIM, DIR_DELIM, name);
+	if (!access (path, R_OK)) return (TRUE);	/* File exists and is readable, return with name */
+
+	/* File was not readable.  Now check if it exists */
+
+	if (!access (path, F_OK))  { /* Kicks in if file is there, meaning it has the wrong permissions */
+		fprintf (stderr, "%s: Error: GMT does not have permission to open %s!\n", GMT_program, path);
+		exit (EXIT_FAILURE);
+	}
+
+	/* File is not there.  Thus, we check if a coastline.conf file exists
+	 * It is not an error if we cannot find the named file, only if it is found
+	 * but cannot be read due to permission problems */
+
+	sprintf (dir, "%s%cshare%ccoastline.conf", GMTHOME, DIR_DELIM, DIR_DELIM);
+	if (!access (dir, F_OK))  { /* File exists... */
+		if (access (dir, R_OK)) {	/* ...but cannot be read */
+			fprintf (stderr, "%s: Error: GMT does not have permission to open %s!\n", GMT_program, dir);
+			exit (EXIT_FAILURE);
+		}
+	}
+	else {	/* There is no coastline.conf file to use; we're out of luck */
+		fprintf (stderr, "%s: Error: No configuration file %s available!\n", GMT_program, dir);
+		exit (EXIT_FAILURE);
+	}
+
+	/* We get here if coastline.conf exists - search among its directories for the named file */
+
+	if ((fp = fopen (dir, "r")) == NULL) {	/* This shouldn't be necessary, but cannot hurt */
+		fprintf (stderr, "%s: Error: Cannot open configuration file %s\n", GMT_program, dir);
+		exit (EXIT_FAILURE);
+	}
+
+	found = FALSE;
+	while (!found && fgets (dir, BUFSIZ, fp)) {	/* Loop over all input lines until found or done */
+		if (dir[0] == '#' || dir[0] == '\n') continue;	/* Comment or blank */
+
+		GMT_chop (dir);		/* Chop off LF or CR/LF */
+		sprintf (path, "%s%c%s", dir, DIR_DELIM, name);
+		if (!access (path, F_OK)) {	/* TRUE if file exists */
+			if (!access (path, R_OK)) {	/* TRUE if file is readable */
+				found = TRUE;
+			}
+			else {
+				fprintf (stderr, "%s: Error: GMT does not have permission to open %s!\n", GMT_program, path);
+				exit (EXIT_FAILURE);
+			}
+		}
+	}
+
+	fclose (fp);
+
+	return (found);
+}
+
