@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt_vector.c,v 1.5 2004/05/27 04:24:29 pwessel Exp $
+ *	$Id: gmt_vector.c,v 1.9 2006/03/22 05:20:11 pwessel Exp $
  *
- *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -22,10 +22,13 @@
  * Version:	3.4
  */
  
+#define GMT_WITH_NO_PS
 #include "gmt.h"
 
 #define MAX_SWEEPS 50
 
+double GMT_mag3v (double *a);
+
 int	GMT_jacobi (double *a, int *n, int *m, double *d, double *v, double *b, double *z, int *nrots) {
 /*
  *
