@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdcut.c,v 1.9 2004/01/02 22:45:13 pwessel Exp $
+ *	$Id: grdcut.c,v 1.21 2006/04/12 21:55:46 pwessel Exp $
  *
- *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -27,41 +27,54 @@
  
 #include "gmt.h"
 
-float	*grd;
+struct GRDCUT_CTRL {
+	struct G {	/* -G<output_grdfile> */
+		BOOLEAN active;
+		char *file;
+	} G;
+};
 
-main (int argc, char **argv)
+int main (int argc, char **argv)
 {
 	BOOLEAN	error = FALSE, global = FALSE;
 
 	int	i, nx_old, ny_old, nx_new, ny_new, one_or_zero;
 
+	char *grd_in, format[BUFSIZ];
+
+	float	*grd;
+
 	double	w_new = 0.0, e_new = 0.0, s_new = 0.0, n_new = 0.0;
 	double	w_old, e_old, s_old, n_old;
 
-	char *grd_in, *grd_out, format[BUFSIZ];
-
 	struct GRD_HEADER header, test_header;
+	struct GRDCUT_CTRL *Ctrl;
 
+	void *New_Grdcut_Ctrl (), Free_Grdcut_Ctrl (struct GRDCUT_CTRL *C);
+	
 	argc = GMT_begin (argc, argv);
-	
-	grd_in = grd_out = CNULL;
-	
+
+	Ctrl = (struct GRDCUT_CTRL *) New_Grdcut_Ctrl ();	/* Allocate and initialize a new control structure */
+
+	grd_in = CNULL;
+
 	/* Check and interpret the command line arguments */
-	
+
 	for (i =1; i < argc; i++) {
 		if (argv[i][0] == '-') {
 			switch(argv[i][1]) {
 				/* Common parameters */
-			
+
 				case 'R':
 				case 'V':
 				case 'f':
 				case '\0':
-					error += GMT_get_common_args (argv[i], &w_new, &e_new, &s_new, &n_new);
+					error += GMT_parse_common_options (argv[i], &w_new, &e_new, &s_new, &n_new);
 					break;
-					
+
 	 			case 'G':
-	 				grd_out = &argv[i][2];
+					Ctrl->G.active = TRUE;
+					Ctrl->G.file = strdup (&argv[i][2]);
 					break;
 				default:		/* Options not recognized */
 					error = TRUE;
@@ -72,13 +85,13 @@ main (int argc, char **argv)
 		else
 	 		grd_in = argv[i];
 	}
-	
-	if (argc == 1 || GMT_quick) {
+
+	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr,"grdcut %s - Extract subsets from grdfiles\n\n", GMT_VERSION);
-		fprintf (stderr, "usage: grdcut <input_grd> -G<output_grd> -R<west/east/south/north> [-V] [-f[i|o]<colinfo>]\n");
-		
-		if (GMT_quick) exit (EXIT_FAILURE);
-		
+		fprintf (stderr, "usage: grdcut <input_grd> -G<output_grd> %s [-V] [%s]\n", GMT_Rgeo_OPT, GMT_f_OPT);
+
+		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
+
 		fprintf (stderr, "\t<input_grd> is file to extract a subset from.\n");
 		fprintf (stderr, "\t-G specifies output grdfile\n");
 		GMT_explain_option ('R');
@@ -91,12 +104,12 @@ main (int argc, char **argv)
 	}
 
 	/* Check that the options selected make sense */
-	
+
 	if (!project_info.region_supplied) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify -R option\n", GMT_program);
 		error++;
 	}
-	if (!grd_out) {
+	if (!Ctrl->G.file) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -G option:  Must specify output file\n", GMT_program);
 		error++;
 	}
@@ -113,12 +126,22 @@ main (int argc, char **argv)
 		fprintf (stderr, "%s: Error opening file %s\n", GMT_program, grd_in);
 		exit (EXIT_FAILURE);
 	}
-		
+
 	if (s_new < header.y_min || s_new > header.y_max) error = TRUE;
 	if (n_new < header.y_min || n_new > header.y_max) error = TRUE;
 	global = (fabs (header.x_max - header.x_min) == 360.0);
-		
+
 	if ( !global && ((w_new < header.x_min) || (e_new > header.x_max)) ) error = TRUE;
+	if (GMT_io.in_col_type[GMT_X] == GMT_IS_LON) {
+		if (w_new < header.x_min && e_new < header.x_min) {
+			header.x_min -= 360.0;
+			header.x_max -= 360.0;
+		}
+		if (w_new > header.x_max && e_new > header.x_max) {
+			header.x_min += 360.0;
+			header.x_max += 360.0;
+		}
+	}
 	if (error) {
 		fprintf (stderr, "%s: Subset exceeds data domain!\n", GMT_program);
 		exit (EXIT_FAILURE);
@@ -129,22 +152,22 @@ main (int argc, char **argv)
 	test_header.x_min = w_new;	test_header.x_max = e_new;	test_header.x_inc = header.x_inc;
 	test_header.y_min = s_new;	test_header.y_max = n_new;	test_header.y_inc = header.y_inc;
 	GMT_grd_RI_verify (&test_header, 1);
-	
+
 	/* OK, so far so good. Check if new wesn differs from old wesn by integer dx/dy */
-	
-	if (GMT_minmaxinc_verify (header.x_min, w_new, header.x_inc, SMALL) == 1) {
+
+	if (GMT_minmaxinc_verify (header.x_min, w_new, header.x_inc, GMT_SMALL) == 1) {
 		fprintf (stderr, "%s: Old and new x_min do not differ by N * dx\n", GMT_program);
 		exit (EXIT_FAILURE);
 	}
-	if (GMT_minmaxinc_verify (e_new, header.x_max, header.x_inc, SMALL) == 1) {
+	if (GMT_minmaxinc_verify (e_new, header.x_max, header.x_inc, GMT_SMALL) == 1) {
 		fprintf (stderr, "%s: Old and new x_max do not differ by N * dx\n", GMT_program);
 		exit (EXIT_FAILURE);
 	}
-	if (GMT_minmaxinc_verify (header.y_min, s_new, header.y_inc, SMALL) == 1) {
+	if (GMT_minmaxinc_verify (header.y_min, s_new, header.y_inc, GMT_SMALL) == 1) {
 		fprintf (stderr, "%s: Old and new y_min do not differ by N * dy\n", GMT_program);
 		exit (EXIT_FAILURE);
 	}
-	if (GMT_minmaxinc_verify (n_new, header.y_max, header.y_inc, SMALL) == 1) {
+	if (GMT_minmaxinc_verify (n_new, header.y_max, header.y_inc, GMT_SMALL) == 1) {
 		fprintf (stderr, "%s: Old and new y_max do not differ by N * dy\n", GMT_program);
 		exit (EXIT_FAILURE);
 	}
@@ -157,7 +180,7 @@ main (int argc, char **argv)
 	one_or_zero = (header.node_offset) ? 0 : 1;
 	nx_new = irint ((e_new - w_new) / header.x_inc) + one_or_zero;
 	ny_new = irint ((n_new - s_new) / header.y_inc) + one_or_zero;
-			
+
 	grd = (float *) GMT_memory (VNULL, (size_t)(nx_new * ny_new), sizeof (float), GMT_program);
 	if (GMT_read_grd (grd_in, &header, grd, w_new, e_new, s_new, n_new, GMT_pad, FALSE)) {
 		fprintf (stderr, "%s: Error reading file %s\n", GMT_program, grd_in);
@@ -174,12 +197,31 @@ main (int argc, char **argv)
 		fprintf (stderr, format, w_new, e_new, s_new, n_new, header.x_inc, header.y_inc, nx_new, ny_new);
 	}
 
-	if (GMT_write_grd (grd_out, &header, grd, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE)) {
-		fprintf (stderr, "%s: Error writing file %s\n", GMT_program, grd_out);
+	if (GMT_write_grd (Ctrl->G.file, &header, grd, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE)) {
+		fprintf (stderr, "%s: Error writing file %s\n", GMT_program, Ctrl->G.file);
 		exit (EXIT_FAILURE);
 	}
-	
+
 	GMT_free ((void *) grd);
-	
+
+	Free_Grdcut_Ctrl (Ctrl);	/* Deallocate control structure */
+
 	GMT_end (argc, argv);
+
+	exit (EXIT_SUCCESS);
+}
+
+void *New_Grdcut_Ctrl () {	/* Allocate and initialize a new control structure */
+	struct GRDCUT_CTRL *C;
+	
+	C = (struct GRDCUT_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GRDCUT_CTRL), "New_Grdcut_Ctrl");
+	
+	/* Initialize values whose defaults are not 0/FALSE/NULL */
+			
+	return ((void *)C);
+}
+
+void Free_Grdcut_Ctrl (struct GRDCUT_CTRL *C) {	/* Deallocate control structure */
+	if (C->G.file) GMT_free ((void *)C->G.file);	
+	GMT_free ((void *)C);	
 }
