@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdfft.c,v 1.2.4.6 2004/01/02 22:23:11 pwessel Exp $
+ *	$Id: grdfft.c,v 1.22 2006/05/12 04:15:12 pwessel Exp $
  *
- *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -30,11 +30,65 @@
  *   Note to myself:  users want azimuthal power spectra, and
  *   cross-spectra between two grids.  --whfs
  *
- * Version:	3.4.3
+ * Version:	4
  */
 
 #include "gmt.h"
 
+struct GRDFFT_CTRL {
+	int n_op_count, n_par;
+	int *operation;
+	double	*par;
+
+	struct A {	/* -A<azimuth> */
+		BOOLEAN active;
+		double value;
+	} A;
+	struct C {	/* -C<zlevel> */
+		BOOLEAN active;
+		double value;
+	} C;
+	struct D {	/* -D[<scale>|g] */
+		BOOLEAN active;
+		double value;
+	} D;
+	struct E {	/* -E[x_or_y][w] */
+		BOOLEAN active;
+		int mode;
+	} E;
+	struct F {	/* -F[x_or_y]<lc>/<lp>/<hp>/<hc> */
+		BOOLEAN active;
+		int mode;
+		double lc, lp, hp, hc;
+	} F;
+	struct G {	/* -G<outfile> */
+		BOOLEAN active;
+		char *file;
+	} G;
+	struct I {	/* -I[<scale>|g] */
+		BOOLEAN active;
+		double value;
+	} I;
+	struct L {	/* -L */
+		BOOLEAN active;
+	} L;
+	struct M {	/* -M */
+		BOOLEAN active;
+	} M;
+	struct N {	/* -N<stuff> */
+		BOOLEAN active;
+		double value;
+	} N;
+	struct S {	/* -S<scale> */
+		BOOLEAN active;
+		double value;
+	} S;
+	struct T {	/* -T<te/rl/rm/rw/ri> */
+		BOOLEAN active;
+		double te, rhol, rhom, rhow, rhoi;
+	} T;
+};
+
 #ifndef FSIGNIF
 #define FSIGNIF 24
 #endif
@@ -61,7 +115,7 @@ double	data_var, data2_var, delta_kx, delta_ky;
 double	a[3];	/* Plane fitting coefficients  */
 double	mGal_at_45 = 980619.9203; /* Moritz's 1980 IGF value for gravity in mGal at 45 degrees latitude */
 
-char	*infile = NULL, *outfile = NULL;
+char	*infile = NULL;
 
 struct GRD_HEADER h;
 struct F_INFO {
@@ -88,27 +142,30 @@ struct FFT_SUGGESTION {
 float	*datac, *workc;
 double	scale_out = 1.0;
 
-int parse_f_string(char *c), read_data(int argc, char **argv), get_prime_factors(int n, int *f), get_non_symmetric_f(int *f, int n);
-void do_continuation(double *zlevel), do_azimuthal_derivative(double *azim), do_differentiate(double *par), do_integrate(double *par), do_isostasy(double *par);
-void do_filter(void), do_spectrum(double *par, int give_wavelength), remove_plane(void), taper_edges(void);
-
-main (int argc, char **argv)
+int main (int argc, char **argv)
 {
+	int op_count = 0, par_count = 0;
 
-	int	op_count = 0, n_op_count = 0, par_count = 0, n_par = 0, *operation = NULL;
-	BOOLEAN error = FALSE, stop, chose_spectrum = FALSE, give_wavelength;
-	double	*par = NULL;
+	BOOLEAN error = FALSE, stop, give_wavelength = FALSE;
+	struct GRDFFT_CTRL *Ctrl;
+	int parse_f_string (char *c), read_data(struct GRDFFT_CTRL *Ctrl, int argc, char **argv);
+	void do_continuation(double *zlevel), do_azimuthal_derivative(double *azim), do_differentiate(double *par), do_integrate(double *par), do_isostasy(double *par);
+	void do_filter(void), do_spectrum(double *par, int give_wavelength);
+
+	void *New_Grdfft_Ctrl (), Free_Grdfft_Ctrl (struct GRDFFT_CTRL *C);
+	
+	argc = GMT_begin (argc, argv);
+
+	Ctrl = (struct GRDFFT_CTRL *) New_Grdfft_Ctrl ();	/* Allocate and initialize a new control structure */
 
 	for (i = 0; i < 3; i++) {
-		f_info.lc[i] = f_info.lp[i] = -1.0;	/* Set negative, below valid freqency range  */
-		f_info.hp[i] = f_info.hc[i] = DBL_MAX;	/* Set huge positive, above valid freqency range  */
+		f_info.lc[i] = f_info.lp[i] = -1.0;	/* Set negative, below valid frequency range  */
+		f_info.hp[i] = f_info.hc[i] = DBL_MAX;	/* Set huge positive, above valid frequency range  */
 		f_info.ltaper[i] = f_info.htaper[i] = 0.0;	/* 1/width of taper, zero when not used  */
 		f_info.do_this[i] = FALSE;
 	}
 	f_info.set_already = FALSE;
 	
-	argc = GMT_begin (argc, argv);
-
         for (i = 1; i < argc; i++) {
                 if (argv[i][0] == '-') {
                         switch (argv[i][1]) {
@@ -117,47 +174,47 @@ main (int argc, char **argv)
 			
 				case 'V':
 				case '\0':
-					error += GMT_get_common_args (argv[i], 0, 0, 0, 0);
+					error += GMT_parse_common_options (argv[i], 0, 0, 0, 0);
 					break;
 				
 				/* Supplemental parameters */
 			
                                 case 'A':
-					n_op_count++;
-					operation = (int *) GMT_memory ((void *)operation, (size_t)n_op_count, sizeof(int), GMT_program);
-                                        operation[op_count] = AZIMUTHAL_DERIVATIVE;
+					Ctrl->n_op_count++;
+					Ctrl->operation = (int *) GMT_memory ((void *)Ctrl->operation, (size_t)Ctrl->n_op_count, sizeof(int), GMT_program);
+                                        Ctrl->operation[op_count] = AZIMUTHAL_DERIVATIVE;
 					op_count++;
-					n_par++;
-					par = (double *) GMT_memory ((void *)par, (size_t)n_par, sizeof(double), GMT_program);
-					if ((sscanf(&argv[i][2], "%lf", &par[par_count])) != 1) {
+					Ctrl->n_par++;
+					Ctrl->par = (double *) GMT_memory ((void *)Ctrl->par, (size_t)Ctrl->n_par, sizeof(double), GMT_program);
+					if ((sscanf(&argv[i][2], "%lf", &Ctrl->par[par_count])) != 1) {
 						fprintf (stderr, "%s: GMT SYNTAX ERROR -A option:  Cannot read azimuth\n", GMT_program);
 						error++;
 					}
 					par_count++;
                                         break;
                                 case 'C':
-					n_op_count++;
-					operation = (int *) GMT_memory ((void *)operation, (size_t)n_op_count, sizeof(int), GMT_program);
-                                        operation[op_count] = UP_DOWN_CONTINUE;
+					Ctrl->n_op_count++;
+					Ctrl->operation = (int *) GMT_memory ((void *)Ctrl->operation, (size_t)Ctrl->n_op_count, sizeof(int), GMT_program);
+                                        Ctrl->operation[op_count] = UP_DOWN_CONTINUE;
 					op_count++;
-					n_par++;
-					par = (double *) GMT_memory ((void *)par, (size_t)n_par, sizeof(double), GMT_program);
-					if ((sscanf(&argv[i][2], "%lf", &par[par_count])) != 1) {
+					Ctrl->n_par++;
+					Ctrl->par = (double *) GMT_memory ((void *)Ctrl->par, (size_t)Ctrl->n_par, sizeof(double), GMT_program);
+					if ((sscanf(&argv[i][2], "%lf", &Ctrl->par[par_count])) != 1) {
 						fprintf (stderr, "%s: GMT SYNTAX ERROR -C option:  Cannot read zlevel\n", GMT_program);
 						error++;
 					}
 					par_count++;
                                         break;
                                 case 'D':
-					n_op_count++;
-					operation = (int *) GMT_memory ((void *)operation, (size_t)n_op_count, sizeof(int), GMT_program);
-                                        operation[op_count] = DIFFERENTIATE;
+					Ctrl->n_op_count++;
+					Ctrl->operation = (int *) GMT_memory ((void *)Ctrl->operation, (size_t)Ctrl->n_op_count, sizeof(int), GMT_program);
+                                        Ctrl->operation[op_count] = DIFFERENTIATE;
 					op_count++;
-					n_par++;
-					par = (double *) GMT_memory ((void *)par, (size_t)n_par, sizeof(double), GMT_program);
+					Ctrl->n_par++;
+					Ctrl->par = (double *) GMT_memory ((void *)Ctrl->par, (size_t)Ctrl->n_par, sizeof(double), GMT_program);
                                         if (argv[i][2]) {
-						par[par_count] = (argv[i][2] == 'g' || argv[i][2] == 'G') ? mGal_at_45 : atof (&argv[i][2]);
-						if (par[par_count] == 0.0) {
+						Ctrl->par[par_count] = (argv[i][2] == 'g' || argv[i][2] == 'G') ? mGal_at_45 : atof (&argv[i][2]);
+						if (Ctrl->par[par_count] == 0.0) {
 							fprintf (stderr, "%s: GMT SYNTAX ERROR -D option:  scale must be nonzero\n", GMT_program);
 							error++;
 						}
@@ -165,32 +222,31 @@ main (int argc, char **argv)
 					par_count++;
                                         break;
                                 case 'E':
-					n_op_count++;
-					operation = (int *) GMT_memory ((void *)operation, (size_t)n_op_count, sizeof(int), GMT_program);
-                                        operation[op_count] = SPECTRUM;
+					Ctrl->n_op_count++;
+					Ctrl->operation = (int *) GMT_memory ((void *)Ctrl->operation, (size_t)Ctrl->n_op_count, sizeof(int), GMT_program);
+                                        Ctrl->operation[op_count] = SPECTRUM;
 					op_count++;
-					n_par++;
-					par = (double *) GMT_memory ((void *)par, (size_t)n_par, sizeof(double), GMT_program);
-					par[par_count] = 0.0;
-					give_wavelength = FALSE;
+					Ctrl->n_par++;
+					Ctrl->par = (double *) GMT_memory ((void *)Ctrl->par, (size_t)Ctrl->n_par, sizeof(double), GMT_program);
+					Ctrl->par[par_count] = 0.0;
 					j = 2;
 					while(argv[i][j]) {
  						if (argv[i][j] == 'x' || argv[i][j] == 'X')
-							par[par_count] = 1.0;
+							Ctrl->par[par_count] = 1.0;
 						else if (argv[i][j] == 'y' || argv[i][j] == 'Y')
-							par[par_count] = -1.0;
+							Ctrl->par[par_count] = -1.0;
 						else if (argv[i][j] == 'w' || argv[i][j] == 'W')
 							give_wavelength = TRUE;
 						j++;
 					}
 					par_count++;
-					chose_spectrum = TRUE;
+					Ctrl->E.active = TRUE;
                                         break;
                                 case 'F':
                                 	if (!(f_info.set_already)) {
-						n_op_count++;
-						operation = (int *) GMT_memory ((void *)operation, (size_t)n_op_count, sizeof(int), GMT_program);
-                                        	operation[op_count] = FILTER;
+						Ctrl->n_op_count++;
+						Ctrl->operation = (int *) GMT_memory ((void *)Ctrl->operation, (size_t)Ctrl->n_op_count, sizeof(int), GMT_program);
+                                        	Ctrl->operation[op_count] = FILTER;
                                         	op_count++;
                                         	f_info.set_already = TRUE;
                                         }
@@ -200,18 +256,18 @@ main (int argc, char **argv)
                                         }
                                         break;
                                 case 'G':
-                                        outfile = &argv[i][2];
+                                        Ctrl->G.file = strdup (&argv[i][2]);
                                         break;
                                 case 'I':
-					n_op_count++;
-					operation = (int *) GMT_memory ((void *)operation, (size_t)n_op_count, sizeof(int), GMT_program);
-                                        operation[op_count] = INTEGRATE;
+					Ctrl->n_op_count++;
+					Ctrl->operation = (int *) GMT_memory ((void *)Ctrl->operation, (size_t)Ctrl->n_op_count, sizeof(int), GMT_program);
+                                        Ctrl->operation[op_count] = INTEGRATE;
 					op_count++;
-					n_par++;
-					par = (double *) GMT_memory ((void *)par, (size_t)n_par, sizeof(double), GMT_program);
+					Ctrl->n_par++;
+					Ctrl->par = (double *) GMT_memory ((void *)Ctrl->par, (size_t)Ctrl->n_par, sizeof(double), GMT_program);
                                         if (argv[i][2]) {
-						par[par_count] = (argv[i][2] == 'g' || argv[i][2] == 'G') ? mGal_at_45 : atof (&argv[i][2]);
-						if (par[par_count] == 0.0) {
+						Ctrl->par[par_count] = (argv[i][2] == 'g' || argv[i][2] == 'G') ? mGal_at_45 : atof (&argv[i][2]);
+						if (Ctrl->par[par_count] == 0.0) {
 							fprintf (stderr, "%s: GMT SYNTAX ERROR -I option:  scale must be nonzero\n", GMT_program);
 							error++;
 						}
@@ -219,10 +275,10 @@ main (int argc, char **argv)
 					par_count++;
                                         break;
                                 case 'L':
-                                        leave_trend_alone = TRUE;
+                                        Ctrl->L.active = TRUE;
                                         break;
                                 case 'M':
-                                        map_units = TRUE;
+                                        Ctrl->M.active = TRUE;
                                         break;
                                 case 'N':
 					if (argv[i][2] == 'f' || argv[i][2] == 'F')
@@ -235,25 +291,26 @@ main (int argc, char **argv)
 					}
                                         break;
                                 case 'S':
+                                        Ctrl->S.active = TRUE;
                                         scale_out = (argv[i][2] == 'd' || argv[i][2] == 'D') ? 1.0e6: atof (&argv[i][2]);
                                         break;
                                 case 'T':
-					n_op_count++;
-					operation = (int *) GMT_memory ((void *)operation, (size_t)n_op_count, sizeof(int), GMT_program);
-                                        operation[op_count] = ISOSTASY;
+					Ctrl->n_op_count++;
+					Ctrl->operation = (int *) GMT_memory ((void *)Ctrl->operation, (size_t)Ctrl->n_op_count, sizeof(int), GMT_program);
+                                        Ctrl->operation[op_count] = ISOSTASY;
 					op_count++;
- 					n_par += 5;
-					par = (double *) GMT_memory ((void *)par, (size_t)n_par, sizeof(double), GMT_program);
-                                        n = sscanf (&argv[i][2], "%lf/%lf/%lf/%lf/%lf", &par[par_count],
-						&par[par_count+1], &par[par_count+2], &par[par_count+3], &par[par_count+4]);
-					for (j = 1, k = 0; j < 5; j++) if (par[par_count+j] < 0.0) k++;
+ 					Ctrl->n_par += 5;
+					Ctrl->par = (double *) GMT_memory ((void *)Ctrl->par, (size_t)Ctrl->n_par, sizeof(double), GMT_program);
+                                        n = sscanf (&argv[i][2], "%lf/%lf/%lf/%lf/%lf", &Ctrl->par[par_count],
+						&Ctrl->par[par_count+1], &Ctrl->par[par_count+2], &Ctrl->par[par_count+3], &Ctrl->par[par_count+4]);
+					for (j = 1, k = 0; j < 5; j++) if (Ctrl->par[par_count+j] < 0.0) k++;
 					if (n != 5 || k > 0) {
 						fprintf (stderr, "%s: GMT SYNTAX ERROR -T option.  Correct syntax:\n", GMT_program);
 						fprintf (stderr, "\t-T<te>/<rhol>/<rhom>/<rhow>/<rhoi>, all densities >= 0\n");
 						error++;
 					}
 					par_count += 5;
-                                        leave_trend_alone = TRUE;
+                                        Ctrl->L.active = TRUE;
                                         break;
                                default:
                                         error = TRUE;
@@ -265,13 +322,13 @@ main (int argc, char **argv)
                         infile = argv[i];
         }
 
-	if (argc == 1 || GMT_quick) {
-		fprintf (stderr, "grdfft %s - Perform mathematical operations on grdfiles in the frequency domain\n\n", GMT_VERSION);
+	if (argc == 1 || GMT_give_synopsis_and_exit) {
+		fprintf (stderr, "grdfft %s - Perform mathematical operations on grdfiles in the wavenumber (or frequency) domain\n\n", GMT_VERSION);
 		fprintf (stderr,"usage: grdfft <in_grdfile> [-G<out_grdfile>] [-A<azimuth>] [-C<zlevel>]\n");
 		fprintf (stderr,"\t[-D[<scale>|g]] [-E[x_or_y][w]] [-F[x_or_y]<lc>/<lp>/<hp>/<hc>] [-I[<scale>|g]] [-L] [-M]\n");
 		fprintf (stderr,"\t[-N<stuff>] [-S<scale>] [-T<te/rl/rm/rw/ri>] [-V]\n\n");
 		
-		if (GMT_quick) exit (EXIT_FAILURE);
+		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 		
 		fprintf (stderr,"\tin_grdfile is the input netCDF grdfile\n");
 		fprintf (stderr, "\tOPTIONS:\n");
@@ -284,7 +341,7 @@ main (int argc, char **argv)
 		fprintf (stderr,"\t-F Filter r [x] [y] freq according to four wavelengths <lc>/<lp>/<hp>/<hc>.\n");
 		fprintf (stderr,"\t   freq outside <lc>/<hc> are cut; inside <lp>/<hp> are passed, rest are tapered.\n");
 		fprintf (stderr,"\t   replace wavelength by - to skip, e.g.  -F-/-/500/100 is a low-pass filter.\n");
-		fprintf (stderr,"\t-I Integrate, i.e., devide by kr [ * scale].  Use -Ig to get m from mGal]\n");
+		fprintf (stderr,"\t-I Integrate, i.e., divide by kr [ * scale].  Use -Ig to get m from mGal]\n");
 		fprintf (stderr,"\t-L Leave trend alone.  Do not remove least squares plane from data.  [Default removes plane].\n");
 		fprintf (stderr,"\t-M Map units used.  Convert grid dimensions from degrees to meters.\n");
 		fprintf (stderr,"\t-N<stuff>  Choose or inquire about suitable grid dimensions for FFT.\n");
@@ -303,7 +360,7 @@ main (int argc, char **argv)
 		exit (EXIT_FAILURE);
 	}
 
-	if (!(n_op_count)) {
+	if (!(Ctrl->n_op_count)) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify at least one operation\n", GMT_program);
 		error++;
 	}
@@ -319,19 +376,19 @@ main (int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify input file\n", GMT_program);
 		error++;
 	}
-	if (chose_spectrum && outfile) {
+	if (Ctrl->E.active && Ctrl->G.file) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -E option:  -G ignored (stdout used for ASCII output)\n", GMT_program);
 		error++;
 	}
-	if (!chose_spectrum && !outfile) {
+	if (!Ctrl->E.active && !Ctrl->G.file) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -G option:  Must specify output file\n", GMT_program);
 		error++;
 	}
 	if (error) exit (EXIT_FAILURE);
 
-	GMT_put_history (argc, argv);	/* Update .gmtcommands */
+	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
 
-	if (read_data(argc, argv) ) {
+	if (read_data(Ctrl, argc, argv) ) {
 		fprintf (stderr,"%s: Fatal GMT_memory error.  Exiting.\n", GMT_program);
 		exit (EXIT_FAILURE);
 	}
@@ -349,31 +406,31 @@ main (int argc, char **argv)
 	
 	GMT_fourt (datac, narray, 2, -1, 1, workc);
 
-	for (op_count=0, par_count = 0; op_count < n_op_count; op_count++) {
-		switch(operation[op_count]) {
+	for (op_count = par_count = 0; op_count < Ctrl->n_op_count; op_count++) {
+		switch (Ctrl->operation[op_count]) {
 			case UP_DOWN_CONTINUE:
-				if (gmtdefs.verbose) ((par[par_count] < 0.0) ? fprintf (stderr, "downward continuation...") : fprintf (stderr, "upward continuation..."));
-				do_continuation (&par[par_count]);
+				if (gmtdefs.verbose) ((Ctrl->par[par_count] < 0.0) ? fprintf (stderr, "downward continuation...") : fprintf (stderr, "upward continuation..."));
+				do_continuation (&Ctrl->par[par_count]);
 				par_count++;
 				break;
 			case AZIMUTHAL_DERIVATIVE:
 				if (gmtdefs.verbose) fprintf (stderr, "azimuthal derivative...");
-				do_azimuthal_derivative (&par[par_count]);
+				do_azimuthal_derivative (&Ctrl->par[par_count]);
 				par_count++;
 				break;
 			case DIFFERENTIATE:
 				if (gmtdefs.verbose) fprintf (stderr, "differentiate...");
-				do_differentiate (&par[par_count]);
+				do_differentiate (&Ctrl->par[par_count]);
 				par_count++;
 				break;
 			case INTEGRATE:
 				if (gmtdefs.verbose) fprintf (stderr, "integrate...");
-				do_integrate (&par[par_count]);
+				do_integrate (&Ctrl->par[par_count]);
 				par_count++;
 				break;
 			case ISOSTASY:
 				if (gmtdefs.verbose) fprintf (stderr, "isostasy...");
-				do_isostasy (&par[par_count]);
+				do_isostasy (&Ctrl->par[par_count]);
 				par_count += 5;
 				break;
 			case FILTER:
@@ -382,13 +439,13 @@ main (int argc, char **argv)
 				break;
 			case SPECTRUM:
 				if (gmtdefs.verbose) fprintf (stderr, "spectrum...");
-				do_spectrum (&par[par_count], give_wavelength);
+				do_spectrum (&Ctrl->par[par_count], give_wavelength);
 				par_count += 2;
 				break;
 		}
 	}
 
-	if (outfile) {
+	if (Ctrl->G.file) {
 
 		if (gmtdefs.verbose) fprintf (stderr, "inverse FFT...");
 
@@ -399,8 +456,8 @@ main (int argc, char **argv)
 
 		/* The data are in the middle of the padded array:  */
 
-		if (GMT_write_grd (outfile, &h, datac, h.x_min, h.x_max, h.y_min, h.y_max, GMT_pad, TRUE)) {
-			fprintf (stderr, "%s: Error writing file %s\n", GMT_program, outfile);
+		if (GMT_write_grd (Ctrl->G.file, &h, datac, h.x_min, h.x_max, h.y_min, h.y_max, GMT_pad, TRUE)) {
+			fprintf (stderr, "%s: Error writing file %s\n", GMT_program, Ctrl->G.file);
 			exit (EXIT_FAILURE);
 		}
 	}
@@ -410,18 +467,21 @@ main (int argc, char **argv)
 
 	if (gmtdefs.verbose) fprintf (stderr, "Done\n");
 
+	Free_Grdfft_Ctrl (Ctrl);	/* Deallocate control structure */
+
 	GMT_end (argc, argv);
 
 	exit (EXIT_SUCCESS);
 }
 
-read_data(int argc, char **argv)
+int read_data (struct GRDFFT_CTRL *Ctrl, int argc, char **argv)
 {
 	int	worksize, factors[32];
 	double	tdummy, edummy;
 	void	suggest_fft(int nx, int ny, struct FFT_SUGGESTION *fft_sug, int do_print);
 	void	fourt_stats(int nx, int ny, int *f, double *r, int *s, double *t);
-
+	void remove_plane(void), taper_edges(void);
+	
 	if (GMT_read_grd_info (infile, &h)) {
 		fprintf (stderr, "%s: Error opening file %s\n", GMT_program, infile);
 		exit (EXIT_FAILURE);
@@ -492,17 +552,15 @@ read_data(int argc, char **argv)
 		return (-1);
 	}
 
-	if (!(leave_trend_alone) )remove_plane();
+	if (!(Ctrl->L.active) )remove_plane();
 	if (!(force_narray) )taper_edges();
 
 	delta_kx = 2 * M_PI / (nx2 * h.x_inc);
 	delta_ky = 2 * M_PI / (ny2 * h.y_inc);
-	if (map_units) {
+	if (Ctrl->M.active) {
 		/* Give delta_kx, delta_ky units of 2pi/meters  */
-		double tmp;
-		tmp = 2.0 * M_PI * gmtdefs.ellipse[N_ELLIPSOIDS-1].eq_radius / 360.0;	/* GRS-80 sphere m/degree */
-		delta_kx /= (tmp * cos(0.5 * (h.y_min + h.y_max) * D2R) );
-		delta_ky /= tmp;
+		delta_kx /= (project_info.M_PR_DEG * cosd (0.5 * (h.y_min + h.y_max)) );
+		delta_ky /= project_info.M_PR_DEG;
 	}
 
 	return(0);
@@ -559,7 +617,7 @@ void	remove_plane(void)
 	/* Rescale a1,a2 into user's units, in case useful later:  */
 	a[1] *= (2.0/(h.x_max - h.x_min));
 	a[2] *= (2.0/(h.y_max - h.y_min));
-	if(gmtdefs.verbose)fprintf (stderr,"%s: Plane removed.  Mean, S.D., Dx, Dy:  %.8lg\t%.8lg\t%.8lg\t%.8lg\n", GMT_program, a[0],data_var,a[1],a[2]);
+	if(gmtdefs.verbose)fprintf (stderr,"%s: Plane removed.  Mean, S.D., Dx, Dy:  %.8g\t%.8g\t%.8g\t%.8g\n", GMT_program, a[0],data_var,a[1],a[2]);
 }
 
 void	taper_edges (void)
@@ -675,7 +733,7 @@ void	do_differentiate (double *par)
 
 void	do_integrate (double *par)
 {
-	/* Integrate in frequency domain by deviding by kr [scale optional] */
+	/* Integrate in frequency domain by dividing by kr [scale optional] */
 	int	k;
 	double fact, scale;
 
@@ -789,7 +847,7 @@ double	f_wt(int k)
 			case 1:
 				freq = kx(k);
 				break;
-			case 2:
+			default:
 				freq = ky(k);
 				break;
 		}
@@ -810,9 +868,9 @@ double	f_wt(int k)
 
 int	parse_f_string(char *c)
 {
-	int	i, j, n_tokens, descending;
+	int	i, j, n_tokens, pos, descending;
 	double	fourvals[4];
-	char	line[256], *p;
+	char	line[GMT_LONG_TEXT], p[GMT_LONG_TEXT];
 	
 	strcpy(line, c);
 	i = j = 0;	/* j is Filter type:  r=0, x=1, y=2  */
@@ -828,9 +886,8 @@ int	parse_f_string(char *c)
 	
 	f_info.do_this[j] = TRUE;
 	
-	n_tokens = 0;
-	p = strtok(&line[i],"/");
-	while (p) {
+	n_tokens = pos = 0;
+	while ((GMT_strtok (&line[i], "/", &pos, p))) {
 		if (n_tokens > 3) {
 			fprintf(stderr,"%s: Too many slashes in -F.\n", GMT_program);
 			return(TRUE);
@@ -845,7 +902,6 @@ int	parse_f_string(char *c)
 			}
 		}
 		n_tokens++;
-		p = strtok(CNULL, "/");
 	}
 	
 	if (n_tokens != 4) {
@@ -903,7 +959,7 @@ void	do_spectrum(double *par, int give_wavelength)
 	int	k, nk, nused, ifreq;
 	double	delta_k, r_delta_k, freq, *power, eps_pow;
 	PFD	get_k;
-	char	format[64];
+	char	format[GMT_TEXT_LEN];
 	/*  Added by WHFS in version 3.001 to normalize power  :  */
 	double	powfactor;
 
@@ -1004,7 +1060,7 @@ void	fourt_stats(int nx, int ny, int *f, double *r, int *s, double *t)
 	int	n_factors, i, ntotal, sum2, sumnot2, nnot2;
 	int	nonsymx, nonsymy, nonsym, storage;
 	double	err_scale, sig_bits = FSIGNIF;
-
+	int get_prime_factors(int n, int *f), get_non_symmetric_f(int *f, int n);
 
 	/* Find workspace needed.  First find non_symmetric factors in nx, ny  */
 	n_factors = get_prime_factors(nx, f);
@@ -1076,7 +1132,7 @@ void	suggest_fft(int nx, int ny, struct FFT_SUGGESTION *fft_sug, int do_print)
 	fourt_stats(nx, ny, f, &given_err, &given_space, &given_time);
 	given_space += nx*ny;
 	given_space *= 8;
-	if (do_print) fprintf(stderr,"%s:  Data dimension\t%d %d\ttime factor %.8lg\trms error %.8le\tbytes %d\n", GMT_program,
+	if (do_print) fprintf(stderr,"%s:  Data dimension\t%d %d\ttime factor %.8g\trms error %.8e\tbytes %d\n", GMT_program,
 		nx, ny, given_time, given_err, given_space);
 
 	best_err = s_err = t_err = given_err;
@@ -1134,11 +1190,11 @@ void	suggest_fft(int nx, int ny, struct FFT_SUGGESTION *fft_sug, int do_print)
 	}
 
 	if (do_print) {
-		fprintf(stderr,"%s:  Highest speed\t%d %d\ttime factor %.8lg\trms error %.8le\tbytes %d\n", GMT_program,
+		fprintf(stderr,"%s:  Highest speed\t%d %d\ttime factor %.8g\trms error %.8e\tbytes %d\n", GMT_program,
 			nx_best_t, ny_best_t, best_time, t_err, t_space);
-		fprintf(stderr,"%s:  Most accurate\t%d %d\ttime factor %.8lg\trms error %.8le\tbytes %d\n", GMT_program,
+		fprintf(stderr,"%s:  Most accurate\t%d %d\ttime factor %.8g\trms error %.8e\tbytes %d\n", GMT_program,
 			nx_best_e, ny_best_e, e_time, best_err, e_space);
-		fprintf(stderr,"%s:  Least storage\t%d %d\ttime factor %.8lg\trms error %.8le\tbytes %d\n", GMT_program,
+		fprintf(stderr,"%s:  Least storage\t%d %d\ttime factor %.8g\trms error %.8e\tbytes %d\n", GMT_program,
 			nx_best_s, ny_best_s, s_time, s_err, best_space);
 	}
 	/* Fastest solution:  */
@@ -1282,3 +1338,20 @@ int	get_prime_factors(int n, int *f)
 	}
 	return (n_factors);
 }
+
+void *New_Grdfft_Ctrl () {	/* Allocate and initialize a new control structure */
+	struct GRDFFT_CTRL *C;
+	
+	C = (struct GRDFFT_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GRDFFT_CTRL), "New_Grdfft_Ctrl");
+	
+	/* Initialize values whose defaults are not 0/FALSE/NULL */
+
+	return ((void *)C);
+}
+
+void Free_Grdfft_Ctrl (struct GRDFFT_CTRL *C) {	/* Deallocate control structure */
+	if (C->operation) GMT_free ((void *)C->operation);	
+	if (C->par) GMT_free ((void *)C->par);	
+	if (C->G.file) GMT_free ((void *)C->G.file);	
+	GMT_free ((void *)C);	
+}
