@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdmask.c,v 1.17 2004/04/25 09:10:45 pwessel Exp $
+ *	$Id: grdmask.c,v 1.42 2006/05/14 23:31:46 pwessel Exp $
  *
- *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -35,43 +35,75 @@
  
 #include "gmt.h"
 
-float *data;
+#define GRDMASK_N_CLASSES	3	/* outside, on edge, and inside */
+#define GRDMASK_OUTSIDE		0
+#define GRDMASK_ONEDGE		1
+#define GRDMASK_INSIDE		2
 
-main (int argc, char **argv)
+struct GRDMASK_CTRL {
+	struct A {	/* -A[step] */
+		BOOLEAN active;
+		double step;
+	} A;
+	struct F {	/* -F */
+		BOOLEAN active;
+	} F;
+	struct G {	/* -G<maskfile> */
+		BOOLEAN active;
+		char *file;
+	} G;
+	struct I {	/* -Idx[/dy] */
+		BOOLEAN active;
+		double xinc, yinc;
+	} I;
+	struct N {	/* -N<maskvalues> */
+		BOOLEAN active;
+		double mask[GRDMASK_N_CLASSES];	/* values for each level */
+	} N;
+	struct S {	/* -S<radius>[m|c|k|K] */
+		BOOLEAN active;
+		double radius;
+		char unit;
+	} S;
+};
+
+int main (int argc, char **argv)
 {
+	BOOLEAN	error = FALSE, done, nofile = TRUE, greenwich, periodic = FALSE, resample = FALSE;
+
+	char line[BUFSIZ], ptr[BUFSIZ];
 
 	int	i, j, k, ij, nm, n_path, side, fno, n_files = 0, n_alloc = GMT_CHUNK, n_args;
-	int	one_or_zero, di, dj, i0, j0, n_fields, n_read, n_expected_fields, polar;
-	
-	BOOLEAN	error = FALSE, done, nofile = TRUE, pixel = FALSE, draw_arc = TRUE, greenwich;
-	BOOLEAN	do_coverage = FALSE, map_units = FALSE, periodic = FALSE, resample = FALSE;
-	
-	double	*x, *y, xx, yy, xmin, xmax, ymin, ymax, offset, dx, dy, r, ca, sa, lon_sum, dlon;
-	double xinc2, yinc2, distance, radius = 0.0, shrink, km_pr_deg, idx, idy, x0, y0, *in, step = 1.0;
+	int	one_or_zero, di, dj, i0, j0, n_fields, n_read, n_expected_fields, pos;
+	int 	distance_flag = 0, n_pol = 0;
+
+	float *data;
+
+	double	*x, *y, xx, yy, xmin, xmax, ymin, ymax, lon_sum, lat_sum, dlon;
+	double xinc2, yinc2, distance, radius = 0.0, shrink = 1.0, idx, idy, x0, y0, *in;
 
-	float out_edge_in[3];
-	
-	char *maskfile = CNULL, line[BUFSIZ], *ptr;
-	
 	FILE *fp;
-	
+
 	struct GRD_HEADER header;
+	struct GMT_LINE_SEGMENT P;
+	struct GRDMASK_CTRL *Ctrl;
+	
+	void *New_Grdmask_Ctrl (), Free_Grdmask_Ctrl (struct GRDMASK_CTRL *C);
 	
 	argc = GMT_begin (argc, argv);
-		
+
+	Ctrl = (struct GRDMASK_CTRL *)New_Grdmask_Ctrl ();	/* Allocate and initialize a new control structure */
+
 	GMT_grd_init (&header, argc, argv, FALSE);
-	
-	out_edge_in[2] = 1.0;			/* Default inside value */
-	out_edge_in[0] = out_edge_in[1] = 0.0;	/* Default outside and edge value */
 
 	/* Check command line arguments */
-	
+
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '-') {
 			switch (argv[i][1]) {
-			
+
 				/* Common parameters */
-			
+
 				case 'H':
 				case 'R':
 				case 'V':
@@ -79,46 +111,51 @@ main (int argc, char **argv)
 				case 'b':
 				case 'f':
 				case '\0':
-					error += GMT_get_common_args (argv[i], &header.x_min, &header.x_max, &header.y_min, &header.y_max);
+					error += GMT_parse_common_options (argv[i], &header.x_min, &header.x_max, &header.y_min, &header.y_max);
 					break;
-				
+
 				/* Supplemental parameters */
-				
+
 				case 'A':	/* Turn off draw_arc mode */
 					if (argv[i][2])
-						step = atof (&argv[i][2]);	/* Undocumented test feature */
+						Ctrl->A.step = atof (&argv[i][2]);	/* Undocumented test feature */
 					else
-						draw_arc = FALSE;
+						Ctrl->A.active = TRUE;
 					break;
 				case 'N':
-					strcpy (line, &argv[i][2]);
-					ptr = strtok (line, "/");
-					for (j = 0; j < 3; j++) {
-						out_edge_in[j] = (ptr[0] == 'N' || ptr[0] == 'n') ? GMT_f_NaN : (float)atof (ptr);
-						ptr = strtok (CNULL, "/");
+					Ctrl->N.active = TRUE;
+					j = pos = 0;
+					while (j < GRDMASK_N_CLASSES && (GMT_strtok (&argv[i][2], "/", &pos, ptr))) {
+						Ctrl->N.mask[j] = (ptr[0] == 'N' || ptr[0] == 'n') ? GMT_f_NaN : (float)atof (ptr);
+						j++;
 					}
 					break;
 				case 'F':
-					pixel = TRUE;
+					Ctrl->F.active = TRUE;
 					break;
 				case 'G':
-					maskfile = &argv[i][2];
+					Ctrl->G.active = TRUE;
+					Ctrl->G.file = strdup (&argv[i][2]);
 					break;
 				case 'I':
-					GMT_getinc (&argv[i][2], &header.x_inc, &header.y_inc);
+					Ctrl->I.active = TRUE;
+					if (GMT_getinc (&argv[i][2], &Ctrl->I.xinc, &Ctrl->I.yinc)) {
+						GMT_inc_syntax ('I', 1);
+						error = TRUE;
+					}
 					break;
 				case 'L':	/* Obsolete, but backward compatibility prevails [use -f instead] */
-					GMT_io.in_col_type[0] = GMT_io.out_col_type[0] = GMT_IS_LON;
-					GMT_io.in_col_type[1] = GMT_io.out_col_type[1] = GMT_IS_LAT;
+					GMT_io.in_col_type[GMT_X] = GMT_io.out_col_type[GMT_X] = GMT_IS_LON;
+					GMT_io.in_col_type[GMT_Y] = GMT_io.out_col_type[GMT_Y] = GMT_IS_LAT;
 					fprintf (stderr, "%s: Option -L is obsolete (but is processed correctly).  Please use -f instead\n", GMT_program);
 					break;
 				case 'M':
 					GMT_multisegment (&argv[i][2]);
 					break;
 				case 'S':
-					GMT_getinc (&argv[i][2], &radius, &radius);
-					do_coverage = TRUE;
-					map_units = (argv[i][strlen(argv[i])-1] == 'k');
+					Ctrl->S.active = TRUE;
+					Ctrl->S.radius = GMT_getradius (&argv[i][2]);
+					Ctrl->S.unit = argv[i][strlen(argv[i])-1];
 					break;
 				default:
 					error = TRUE;
@@ -130,18 +167,17 @@ main (int argc, char **argv)
 			n_files++;
 	}
 
-	if (argc == 1 || GMT_quick) {
+	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "grdmask %s - Create mask grdfile from polygons or point coverage\n\n", GMT_VERSION);
-		fprintf (stderr, "usage: grdmask [<xyfiles>] -G<mask_grd_file> -I<xinc[m|c]>[/<yinc>[m|c]]\n");
-		fprintf (stderr, "\t-R<west/east/south/north> [-A] [-F] [-H[<nrec>]] [-N<out>/<edge>/<in>] [-M[<flag>]]\n");
-		fprintf (stderr, "\t[-S<radius>[k] [-V] [-:] [-bi[s][<n>]] [-f[i|o]<colinfo>]\n\n");
-		
-		if (GMT_quick) exit (EXIT_FAILURE);
-		
+		fprintf (stderr, "usage: grdmask [<xyfiles>] -G<mask_grd_file> %s\n", GMT_I_OPT);
+		fprintf (stderr, "\t%s [-A] [-F] [%s] [-N<out>/<edge>/<in>]\n", GMT_Rgeo_OPT, GMT_H_OPT);
+		fprintf (stderr, "\t[%s] [-S<radius>[m|c|k|K] [-V] [%s] [%s] [%s]\n\n", GMT_Mo_OPT, GMT_t_OPT, GMT_bi_OPT, GMT_f_OPT);
+
+		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
+
 		fprintf (stderr, "\txyfiles is one or more polygon [or point] files\n");
 		fprintf (stderr, "\t-G Specify file name for output mask grd file.\n");
-		fprintf (stderr, "\t-I sets Increment of the grid; enter xinc, optionally xinc/yinc.\n");
-		fprintf (stderr, "\t   Default is yinc = xinc.  Append an m [or c] to indicate minutes [or seconds].\n");
+		GMT_inc_syntax ('I', 0);
 		GMT_explain_option ('R');
 		fprintf (stderr, "\n\tOPTIONS:\n");
 		fprintf (stderr, "\t-A Suppress connecting points using great circle arcs for geographic data\n");
@@ -150,9 +186,12 @@ main (int argc, char **argv)
 		GMT_explain_option ('M');
 		fprintf (stderr, "\t-N sets values to use if point is outside, on the path, or inside.\n");
 		fprintf (stderr, "\t   NaN is a valid entry.  Default values are 0/0/1.\n");
-		fprintf (stderr, "\t-S Input data is points and the mask nodes are set to <in> or <out> depending on\n");
-		fprintf (stderr, "\t    whether they are inside a circle of specified radius [0] from the nearest data point.\n");
-		fprintf (stderr, "\t    Append k to indicated km (implying -R -I are in degrees)\n");
+		fprintf (stderr, "\t-S sets search radius in -R, -I units; append m or c for minutes or seconds.\n");
+		fprintf (stderr, "\t   This means input data are points and the mask nodes are set to <in> or <out> depending on\n");
+		fprintf (stderr, "\t   whether they are inside a circle of specified radius [0] from the nearest data point.\n");
+		fprintf (stderr, "\t   Append k for km (implies -R,-I in degrees), use flat Earth approximation.\n");
+		fprintf (stderr, "\t   Append K for km (implies -R,-I in degrees), use exact geodesic distances.\n");
+		fprintf (stderr, "\t   If the current ELLIPSOID is Sphere then spherical great circle distances are used.\n");
 		fprintf (stderr, "\t    [Default is to treat xyfiles as polygons and use inside/outside searching].\n");
 		GMT_explain_option ('V');
 		GMT_explain_option (':');
@@ -168,15 +207,15 @@ main (int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify -R option\n", GMT_program);
 		error++;
 	}
-	if (header.x_inc <= 0.0 || header.y_inc <= 0.0) {
+	if (Ctrl->I.xinc <= 0.0 || Ctrl->I.yinc <= 0.0) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -I option.  Must specify positive increment(s)\n", GMT_program);
 		error = TRUE;
 	}
-	if (!maskfile) {
+	if (!Ctrl->G.file) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -G:  Must specify output file\n", GMT_program);
 		error = TRUE;
 	}
-	if (GMT_io.binary[GMT_IN] && gmtdefs.io_header[GMT_IN]) {
+	if (GMT_io.binary[GMT_IN] && GMT_io.io_header[GMT_IN]) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data cannot have header -H\n", GMT_program);
 		error++;
 	}
@@ -185,7 +224,7 @@ main (int argc, char **argv)
                 fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data (-bi) must have at least 2 columns\n", GMT_program);
 		error++;
 	}
-	
+
 	if (error) exit (EXIT_FAILURE);
 
 	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
@@ -195,22 +234,19 @@ main (int argc, char **argv)
 		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
+	if (Ctrl->S.unit == 'k') distance_flag = 1;
+	if (Ctrl->S.unit == 'K') distance_flag = 2;
+
+	header.node_offset = Ctrl->F.active;
+	header.x_inc = Ctrl->I.xinc;
+	header.y_inc = Ctrl->I.yinc;
+	GMT_RI_prepare (&header);	/* Ensure -R -I consistency and set nx, ny */
 	GMT_grd_RI_verify (&header, 1);
 
-	if (pixel) {
-		one_or_zero = 0;
-		offset = 0.0;
-		xinc2 = 0.5 * header.x_inc;
-		yinc2 = 0.5 * header.y_inc;
-	}
-	else {
-		one_or_zero = 1;
-		offset = 0.5;
-		xinc2 = yinc2 = 0.0;
-	}
-	header.nx = irint ((header.x_max - header.x_min) / header.x_inc) + one_or_zero;
-	header.ny = irint ((header.y_max - header.y_min) / header.y_inc) + one_or_zero;
-	header.node_offset = pixel;
+	header.xy_off = 0.5 * header.node_offset;
+	xinc2 = header.xy_off * header.x_inc;
+	yinc2 = header.xy_off * header.y_inc;
+	one_or_zero = 1 - header.node_offset;
 	idx = 1.0 / header.x_inc;
 	idy = 1.0 / header.y_inc;
 	nm = header.nx * header.ny;
@@ -221,40 +257,60 @@ main (int argc, char **argv)
 	sprintf (line, "%s\n", gmtdefs.d_format);
 	if (gmtdefs.verbose) {
 		fprintf (stderr, "%s: Nodes completely outside the polygons will be set to ", GMT_program);
-		(GMT_is_fnan (out_edge_in[0])) ? fprintf (stderr, "NaN\n") : fprintf (stderr, line, out_edge_in[0]);
+		(GMT_is_fnan (Ctrl->N.mask[GRDMASK_OUTSIDE])) ? fprintf (stderr, "NaN\n") : fprintf (stderr, line, Ctrl->N.mask[GRDMASK_OUTSIDE]);
 		fprintf (stderr, "%s: Nodes completely inside the polygons will be set to ", GMT_program);
-		(GMT_is_fnan (out_edge_in[2])) ? fprintf (stderr, "NaN\n") : fprintf (stderr, line, out_edge_in[2]);
+		(GMT_is_fnan (Ctrl->N.mask[GRDMASK_INSIDE])) ? fprintf (stderr, "NaN\n") : fprintf (stderr, line, Ctrl->N.mask[GRDMASK_INSIDE]);
 		fprintf (stderr, "%s: Nodes on the polygons boundary will be set to ", GMT_program);
-		(GMT_is_fnan (out_edge_in[1])) ? fprintf (stderr, "NaN\n") : fprintf (stderr, line, out_edge_in[1]);
+		(GMT_is_fnan (Ctrl->N.mask[GRDMASK_ONEDGE])) ? fprintf (stderr, "NaN\n") : fprintf (stderr, line, Ctrl->N.mask[GRDMASK_ONEDGE]);
 	}
-	
-	if (map_units) {
-		km_pr_deg = 0.001 * 2.0 * M_PI * gmtdefs.ref_ellipsoid[gmtdefs.ellipsoid].eq_radius / 360.0;
+
+	if (distance_flag > 0) {
 		shrink = cos (0.5 * (header.y_min + header.y_max) * D2R);
-		di = (int)ceil (radius / (km_pr_deg * header.x_inc * shrink));
-		dj = (int)ceil (radius / (km_pr_deg * header.y_inc));
+		di = (int)ceil (radius / (project_info.KM_PR_DEG * header.x_inc * shrink));
+		dj = (int)ceil (radius / (project_info.KM_PR_DEG * header.y_inc));
 	}
 	else {
 		di = (int)ceil (radius * idx);
 		dj = (int)ceil (radius * idy);
 	}
-	periodic = (GMT_io.in_col_type[0] == GMT_IS_LON);	/* Dealing with geographic coordinates */
-	resample = (draw_arc && periodic);
+	periodic = (GMT_io.in_col_type[GMT_X] == GMT_IS_LON);	/* Dealing with geographic coordinates */
+	resample = (!Ctrl->A.active && periodic);
 	greenwich = (header.x_min <= 0.0 || header.x_max <= 0.0);
+	if (distance_flag == 2 && !GMT_IS_SPHERICAL) distance_flag = 3;	/* Use geodesics */
+
+	switch (distance_flag) {	/* Take different action depending on how we want distances calculated */
+
+		case 0:		/* Cartesian distance */
+			GMT_distance_func = GMT_cartesian_dist;
+			break;
+		case 1:		/* Flat Earth Approximation */
+			GMT_distance_func = GMT_flatearth_dist_km;
+			break;
+		case 2:		/* Full spherical calculation */
+			GMT_distance_func = GMT_great_circle_dist_km;
+			break;
+		case 3:		/* Full Ellipsoidal calculation */
+			GMT_distance_func = GMT_geodesic_dist_km;
+		break;
+	}
+
+	P.coord = (double **) GMT_memory (VNULL, 2, sizeof (double *), GMT_program);	/* Needed as pointers below */
+	P.min = (double *) GMT_memory (VNULL, 2, sizeof (double), GMT_program);		/* Needed to hold min lon/lat */
+	P.max = (double *) GMT_memory (VNULL, 2, sizeof (double), GMT_program);		/* Needed to hold max lon/lat */
 	
 	/* Initialize all nodes to the 'outside' value */
-	
-	for (ij = 0; ij < nm; ij++) data[ij] = out_edge_in[0];
-	
+
+	for (ij = 0; ij < nm; ij++) data[ij] = (float)Ctrl->N.mask[GRDMASK_OUTSIDE];
+
 	if (n_files > 0)
 		nofile = FALSE;
 	else
 		n_files = 1;
 	n_args = (argc > 1) ? argc : 2;
-	
+
 	done = FALSE;
 	n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : 2;
-	
+
 	for (fno = 1; !done && fno < n_args; fno++) {	/* Loop over all input files */
 		if (!nofile && argv[fno][0] == '-') continue;
 		if (nofile) {
@@ -270,29 +326,29 @@ main (int argc, char **argv)
 		}
 
 		if (!nofile && gmtdefs.verbose) fprintf (stderr, "%s: Working on file %s\n", GMT_program, argv[fno]);
-		
-		if (gmtdefs.io_header[GMT_IN]) for (i = 0; i < gmtdefs.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
-		
+
+		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
+
 		n_read = 0;
 		n_fields = GMT_input (fp,  &n_expected_fields, &in);
 		while (! (GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
-			
+
 			while (GMT_io.status & GMT_IO_SEGMENT_HEADER && !(GMT_io.status & GMT_IO_EOF)) n_fields = GMT_input (fp, &n_expected_fields, &in);
 			if ((GMT_io.status & GMT_IO_EOF)) continue;	/* At EOF */
 
-			n_path = polar = 0;
-			xmin = ymin = 1.0e+100;
-			xmax = ymax = -1.0e+100;
-			lon_sum = 0.0;
-	
+			n_path = 0;
+			xmin = ymin = +DBL_MAX;
+			xmax = ymax = -DBL_MAX;
+			lon_sum = lat_sum = 0.0;
+
 			while (!(GMT_io.status & (GMT_IO_SEGMENT_HEADER | GMT_IO_EOF))) {	/* Keep going until FALSE  */
 				if (GMT_io.status & GMT_IO_MISMATCH) {
 					fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields,  n_expected_fields, n_read);
 					exit (EXIT_FAILURE);
 				}
-				x[n_path] = in[0];
-				y[n_path] = in[1];
-	
+				x[n_path] = in[GMT_X];
+				y[n_path] = in[GMT_Y];
+
 				if (x[n_path] > xmax) xmax = x[n_path];
 				if (x[n_path] < xmin) xmin = x[n_path];
 				if (y[n_path] > ymax) ymax = y[n_path];
@@ -301,9 +357,10 @@ main (int argc, char **argv)
 					dlon = x[n_path] - x[n_path-1];
 					if (fabs (dlon) > 180.0) dlon = copysign (360.0 - fabs (dlon), -dlon);
 					lon_sum += dlon;
+					lat_sum += y[n_path];
 				}
 				n_path++;
-		
+
 				if (n_path == (n_alloc-1)) {	/* n_alloc-1 since we may need 1 more to close polygon */
 					n_alloc += GMT_CHUNK;
 					x = (double *) GMT_memory ((void *)x, (size_t)n_alloc, sizeof(double), GMT_program);
@@ -311,8 +368,8 @@ main (int argc, char **argv)
 				}
 				n_fields = GMT_input (fp, &n_expected_fields, &in);
 			}
-
-			if (do_coverage) {	/* Assign 'inside' to nodes within given distance of data constrains */
+			n_pol++;
+			if (Ctrl->S.active) {	/* Assign 'inside' to nodes within given distance of data constrains */
 				for (k = 0; k < n_path; k++) {
 
 					if (y[k] < header.y_min || y[k] > header.y_max) continue;
@@ -322,14 +379,14 @@ main (int argc, char **argv)
 						if (x[k] > header.x_max) continue;
 					}
 					else if (x[k] < header.x_min || x[k] > header.x_max) continue;
-					
+
 					/* OK, this point is within bounds, but may be exactly on the border */
-					
-					i0 = (int)floor (((x[k] - header.x_min) * idx) + offset);
+
+					i0 = GMT_x_to_i (x[k], header.x_min, header.x_inc, header.xy_off, header.nx);
 					if (i0 == header.nx) i0--;	/* Was exactly on the xmax edge */
-					j0 = (int)floor (((header.y_max - y[k]) * idy) + offset);
+					j0 = GMT_y_to_j (y[k], header.y_min, header.y_inc, header.xy_off, header.ny);
 					if (j0 == header.ny) j0--;	/* Was exactly on the ymin edge */
-					data[j0 * header.nx + i0] = out_edge_in[2];	/* This is the nearest node */
+					data[j0 * header.nx + i0] = (float)Ctrl->N.mask[GRDMASK_INSIDE];	/* This is the nearest node */
 					if (radius == 0.0) continue;
 
 					for (j = j0 - dj; j <= (j0 + dj); j++) {
@@ -337,94 +394,110 @@ main (int argc, char **argv)
 						for (i = i0 - di; i <= (i0 + di); i++) {
 							if (i < 0 || i >= header.nx) continue;
 							ij = j * header.nx + i;
-							x0 = header.x_min + i * header.x_inc + xinc2;
-							y0 = header.y_max - j * header.y_inc - yinc2;
-							dx = x[k] - x0;	dy = y[k] - y0;
-							distance = (map_units) ? km_pr_deg * hypot (dx * shrink, dy) : hypot (dx, dy);
+							x0 = GMT_i_to_x (i, header.x_min, header.x_max, header.x_inc, header.xy_off, header.nx);
+							y0 = GMT_j_to_y (j, header.y_min, header.y_max, header.y_inc, header.xy_off, header.ny);
+							distance = (GMT_distance_func) (x[k], y[k], x0, y0);
 							if (distance > radius) continue;
-							data[ij] = out_edge_in[2];	/* The in value */
+							data[ij] = (float)Ctrl->N.mask[GRDMASK_INSIDE];	/* The in value */
 						}
 					}
 				}
 			}
 			else {	/* assign 'inside' to nodes if they are inside given polygon */
-
-				if (x[n_path-1] != x[0] || y[n_path-1] != y[0]) {	/* Explicitly close the polygon */
+				dlon = x[n_path-1] - x[0];
+				if (periodic && fabs(dlon) == 360.0) dlon = 0.0;
+				if (!(y[n_path-1] == y[0] && dlon == 0.0)) {	/* Explicitly close the polygon */
 					x[n_path] = x[0];
 					y[n_path] = y[0];
 					dlon = x[n_path] - x[n_path-1];
+					lat_sum += y[n_path];
 					n_path++;
 					if (periodic) {
 						if (fabs (dlon) > 180.0) dlon = copysign (360.0 - fabs (dlon), -dlon);
 						lon_sum += dlon;
 					}
 				}
-				
-				if (resample) n_path = GMT_fix_up_path (&x, &y, n_path, greenwich, step);
-				
-				if (periodic && fabs (fabs (lon_sum) - 360.0) < GMT_CONV_LIMIT) {	/* Contains a pole, convert to polar x,y */
-					polar = (y[0] < 0.0) ? -1 : +1;	/* S or N pole */
-					for (j = 0; j < n_path; j++) {
-						r = 90.0 - fabs (y[j]);
-						sincos (x[j] * D2R, &sa, &ca);
-						x[j] = r * ca;
-						y[j] = r * sa;
+
+				if (resample) n_path = GMT_fix_up_path (&x, &y, n_path, greenwich, Ctrl->A.step);
+
+				if (periodic) {	/* Make polygon structure so we can use spherical polygon machinery */
+					P.coord[GMT_X] = x;
+					P.coord[GMT_Y] = y;
+					P.n_rows = n_path;
+					P.min[GMT_Y] = ymin;	P.max[GMT_Y] = ymax;
+					P.min[GMT_X] = xmin;	P.max[GMT_X] = xmax;
+					if (fabs (fabs (lon_sum) - 360.0) < GMT_CONV_LIMIT) {	/* Contains a pole, convert to polar x,y */
+						P.pole = (lat_sum < 0.0) ? -1 : +1;	/* S or N pole */
 					}
 				}
 
 				for (j = 0; j < header.ny; j++) {
-		
-					yy = header.y_max - j * header.y_inc;
-					if (pixel) yy -= yinc2;
-					if (!polar && (yy < ymin || yy > ymax)) continue;	/* Point outside, no need to assign value */
-			
-					for (i = 0; i < header.nx; i++) {
-						xx = header.x_min + i * header.x_inc;
-						if (pixel) xx += xinc2;
-						if (periodic) {	/* Make sure we are not off by 360 somewhere */
-							xx -= 360.0;
-							while (xx < xmin) xx += 360.0;
-						}
-						if (!polar && (xx < xmin || xx > xmax)) continue;	/* Point outside, no need to assign value */
-				
-						if (polar == -1 && yy <= 0.0) {	/* Point on S hemisphere, check */
-							r = 90.0 + yy;
-							sincos (xx * D2R, &sa, &ca);
-							side = GMT_non_zero_winding (r * ca, r * sa, x, y, n_path);
-						}
-						else if (polar == +1 && yy >= 0.0) {	/* Point on N hemisphere, check */
-							r = 90.0 - yy;
-							sincos (xx * D2R, &sa, &ca);
-							side = GMT_non_zero_winding (r * ca, r * sa, x, y, n_path);
-						}
-						else if (polar)
-							side = 0;
-						else
-							side = GMT_non_zero_winding (xx, yy, x, y, n_path);
-							
-						if (side == 0) continue;	/* Outside */
-				
-						/* Here, point is inside or on edge, we must assign value */
-				
-						data[j*header.nx+i] = out_edge_in[side];
+
+					yy = GMT_j_to_y (j, header.y_min, header.y_max, header.y_inc, header.xy_off, header.ny);
+					
+					/* First check if point is outside, then there is no need to assign value */
+					
+					if (periodic) {	/* Containing annulus test */
+						if (P.pole == -1 && yy > ymax) continue;
+						if (P.pole == +1 && yy < ymin) continue;
 					}
-					if (gmtdefs.verbose) fprintf (stderr, "%s: scanning row %5.5d\r", GMT_program, j);
+					else if (yy < ymin || yy > ymax)	/* Cartesian case */
+						continue;
+
+					for (i = 0; i < header.nx; i++) {
+						xx = GMT_i_to_x (i, header.x_min, header.x_max, header.x_inc, header.xy_off, header.nx);
+						if (periodic) {
+							side = GMT_inonout_sphpol (xx, yy, &P);
+						}
+						else {
+							if (xx < xmin || xx > xmax) continue;	/* Point outside, no need to assign value */
+							side = GMT_non_zero_winding (xx, yy, x, y, n_path);
+						}
+
+						if (side == 0) continue;	/* Outside */
+
+						/* Here, point is inside or on edge, we must assign value */
+
+						data[j*header.nx+i] = (float)Ctrl->N.mask[side];
+					}
+					if (gmtdefs.verbose) fprintf (stderr, "%s: Polygon %d scanning row %5.5d\r", GMT_program, n_pol, j);
 				}
 			}
 		}
 		if (gmtdefs.verbose) fprintf (stderr, "\n");
-		
+
 		if (fp != GMT_stdin) GMT_fclose (fp);
 	}
-	
-	if (GMT_write_grd (maskfile, &header, data, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE)) {
-		fprintf (stderr, "%s: Error writing file %s\n", GMT_program, maskfile);
+
+	if (GMT_write_grd (Ctrl->G.file, &header, data, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE)) {
+		fprintf (stderr, "%s: Error writing file %s\n", GMT_program, Ctrl->G.file);
 		exit (EXIT_FAILURE);
 	}
 
 	GMT_free ((void *)data);
 	GMT_free ((void *)x);
 	GMT_free ((void *)y);
-	
+	GMT_free ((void *)P.coord);
+
+	Free_Grdmask_Ctrl (Ctrl);	/* Deallocate control structure */
+
 	GMT_end (argc, argv);
+
+	exit (EXIT_SUCCESS);
+}
+
+void *New_Grdmask_Ctrl () {	/* Allocate and initialize a new control structure */
+	struct GRDMASK_CTRL *C;
+	
+	C = (struct GRDMASK_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GRDMASK_CTRL), "New_Grdmask_Ctrl");
+	
+	/* Initialize values whose defaults are not 0/FALSE/NULL */
+	C->A.step = gmtdefs.line_step;
+	C->N.mask[GRDMASK_INSIDE] = 1.0;			/* Default inside value */
+	return ((void *)C);
+}
+
+void Free_Grdmask_Ctrl (struct GRDMASK_CTRL *C) {	/* Deallocate control structure */
+	if (C->G.file) GMT_free ((void *)C->G.file);	
+	GMT_free ((void *)C);	
 }
