@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdreformat.c,v 1.8 2004/01/02 22:45:13 pwessel Exp $
+ *	$Id: grdreformat.c,v 1.27 2006/04/13 06:20:35 pwessel Exp $
  *
- *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -19,47 +19,59 @@
  * grdreformat.c reads a grd file in one format and outputs it in another
  *
  * Author:	Paul Wessel
- * Date:	3-JAN-1991-2004
+ * Date:	3-JAN-1991-2006
  * Version:	4
  */
 
 #include "gmt.h"
 
-main (int argc, char **argv)
+struct GRDREFORMAT_CTRL {
+	struct N {	/* -N */
+		BOOLEAN active;
+	} N;
+};
+
+int main (int argc, char **argv)
 {
 	BOOLEAN error = FALSE, global = FALSE;
 
-	int i, nfiles = 0, nm, nx, ny, one_or_zero, no_header = 0;
+	int i, nfiles = 0, nx, ny, no_header, type[2];
 
-	double w, e, s, n, scale = 1.0, offset = 0.0;
+	size_t nm;
+	
+	double w, e, s, n;
 
 	float *z;
 
 	char *file[2], fname[2][BUFSIZ], line[BUFSIZ];
 
 	struct GRD_HEADER grd;
+	struct GRDREFORMAT_CTRL *Ctrl;
 
 	FILE *fp;
-	
-	file[0] = file[1] = CNULL;
+
+	void *New_Grdreformat_Ctrl (), Free_Grdreformat_Ctrl (struct GRDREFORMAT_CTRL *C);
 
 	argc = GMT_begin (argc, argv);
 
-	w = e = s = n = 0.0;
+	Ctrl = (struct GRDREFORMAT_CTRL *)New_Grdreformat_Ctrl ();	/* Allocate and initialize a new control structure */
 	
+	file[0] = file[1] = CNULL;
+	w = e = s = n = 0.0;
+
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '-') {
 			switch (argv[i][1]) {
 				/* Common parameters */
-			
+
 				case 'R':
 				case 'V':
 				case '\0':
-					error += GMT_get_common_args (argv[i], &w, &e, &s, &n);
+					error += GMT_parse_common_options (argv[i], &w, &e, &s, &n);
 					break;
-					
+
 				case 'N':
-					no_header = 64;
+					Ctrl->N.active = TRUE;
 					break;
 				default:
 					error = TRUE;
@@ -74,13 +86,13 @@ main (int argc, char **argv)
 		else
 			nfiles++;
 	}
-	
-	if (argc == 1 || GMT_quick) {	/* Display usage */
-		fprintf (stderr, "grdreformat %s - Converting between different grdfile formats\n\n", GMT_VERSION);
-		fprintf( stderr, "usage: grdreformat ingrdfile[=id[/scale/offset]] outgrdfile[=id[/scale/offset]] [-N] [-Rw/e/s/n] [-V]\n");
 
-		if (GMT_quick) exit (EXIT_FAILURE);
-		
+	if (argc == 1 || GMT_give_synopsis_and_exit) {	/* Display usage */
+		fprintf (stderr, "grdreformat %s - Converting between different grdfile formats\n\n", GMT_VERSION);
+		fprintf( stderr, "usage: grdreformat ingrdfile[=id[/scale/offset]] outgrdfile[=id[/scale/offset]] [-N]\n\t[%s] [-V]\n", GMT_Rgeo_OPT);
+
+		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
+
 		fprintf (stderr, "	ingrdfile is the grd file to convert\n");
 		fprintf (stderr, "	outgrdfile is the new converted grd file\n");
 		fprintf( stderr, "	   scale and offset, if given, will multiply data by scale and add offset.\n");
@@ -89,49 +101,53 @@ main (int argc, char **argv)
 		fprintf (stderr, "\t\t  Useful when creating files to be used by grdraster.\n");
 		GMT_explain_option ('r');
 		GMT_explain_option ('V');
-		
-		fprintf (stderr, "	The following formats are supported\n\n");
+
+		fprintf (stderr, "\n	The following grdfile formats are supported:\n\n");
 		sprintf (line, "%s%cshare%cgmtformats.d", GMTHOME, DIR_DELIM, DIR_DELIM);
 		if ((fp = fopen (line, "r")) == NULL) {
 			fprintf (stderr, "%s: GMT ERROR: Cannot read file %s\n", GMT_program, line);
 			exit (EXIT_FAILURE);
 		}
-		while (fgets (line, BUFSIZ, fp)) if (line[0] != '#') fprintf (stderr, "\t%s", line);
+		while (fgets (line, BUFSIZ, fp)) {
+			if (line[0] == '#') continue;
+			fprintf (stderr, "\t%s", &line[4]);
+		}
 		fclose (fp);
 		exit (EXIT_FAILURE);
 	}
-	
+
 	if (nfiles != 2) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR: Must specify both input and output file names\n", GMT_program);
 		error++;
 	}
-	
+
 	if (error) exit (EXIT_FAILURE);
 
 	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
-	
-	GMT_grd_i_format = GMT_grd_get_i_format (file[0], fname[0], &scale, &offset);
-	GMT_grd_o_format = GMT_grd_get_o_format (file[1], fname[1], &scale, &offset);
-	
-	if (GMT_grd_i_format == GMT_grd_o_format) {
-		fprintf (stderr, "%s: Formats are identical - aborts\n", GMT_program);
+
+	no_header = (Ctrl->N.active) ? 64 : 0;
+	type[0] = GMT_grd_get_format (file[0], &grd);
+	strcpy (fname[0], grd.name);
+	type[1] = GMT_grd_get_format (file[1], &grd);
+	strcpy (fname[1], grd.name);
+
+	if (type[1] == 20) {	/* Golden Surfer format 7 (double) is read-only */
+		fprintf (stderr, "%s: Grid format sd (Golden Software Surfer foramt 7 (double) is read-only!\n", GMT_program);
 		exit (EXIT_FAILURE);
 	}
 	
 	if (gmtdefs.verbose) {
 		if (file[0][0] == '=') strcpy (fname[0], "<stdin>");
 		if (file[1][0] == '=') strcpy (fname[1], "<stdout>");
-		fprintf (stderr, "%s: Translating file %s (format = %d) to file %s (format = %d)\n", GMT_program, fname[0], GMT_grd_i_format, fname[1], GMT_grd_o_format);
-		if (no_header && !GRD_IS_CDF(GMT_grd_o_format)) fprintf (stderr, "%s: No grd header will be written\n", GMT_program);
+		fprintf (stderr, "%s: Translating file %s (format = %d) to file %s (format = %d)\n", GMT_program, fname[0], type[0], fname[1], type[1]);
+		if (no_header && GMT_grdformats[type[1]][0] != 'c' && GMT_grdformats[type[1]][0] != 'n') fprintf (stderr, "%s: No grd header will be written\n", GMT_program);
 	}
-	
+
 	if (GMT_read_grd_info (file[0], &grd)) {
 		fprintf (stderr, "%s: Error opening file %s\n", GMT_program, fname[0]);
 		exit (EXIT_FAILURE);
 	}
-	
-	nm = grd.nx * grd.ny;
-	
+
 	if (e > w && n > s) {
 		global = (fabs (grd.x_max - grd.x_min) == 360.0);
 		if (!global && (w < grd.x_min || e > grd.x_max)) error = TRUE;
@@ -140,19 +156,20 @@ main (int argc, char **argv)
 			fprintf (stderr, "%s: Subset exceeds data domain!\n", GMT_program);
 			exit (EXIT_FAILURE);
 		}
-		one_or_zero = (grd.node_offset) ? 0 : 1;
-		nx = irint ((e - w) / grd.x_inc) + one_or_zero;
-		ny = irint ((n - s) / grd.y_inc) + one_or_zero;
-		
-		z = (float *) GMT_memory (VNULL, (size_t)(nx * ny), sizeof (float), GMT_program);
-		
+		nx = GMT_get_n (w, e, grd.x_inc, grd.node_offset);
+		ny = GMT_get_n (s, n, grd.y_inc, grd.node_offset);
+		nm = ((size_t)nx) * ((size_t)ny);
+
+		z = (float *) GMT_memory (VNULL, nm, sizeof (float), GMT_program);
+
 		if (GMT_read_grd (file[0], &grd, z, w, e, s, n, GMT_pad, FALSE)) {
 			fprintf (stderr, "%s: Error reading file %s\n", GMT_program, fname[0]);
 			exit (EXIT_FAILURE);
 		}
 	}
 	else {
-		z = (float *) GMT_memory (VNULL, (size_t)nm, sizeof (float), GMT_program);
+		nm = ((size_t)grd.nx) * ((size_t)grd.ny);
+		z = (float *) GMT_memory (VNULL, nm, sizeof (float), GMT_program);
 
 		if (GMT_read_grd (file[0], &grd, z, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE)) {
 			fprintf (stderr, "%s: Error reading file %s\n", GMT_program, fname[0]);
@@ -160,12 +177,34 @@ main (int argc, char **argv)
 		}
 	}
 
+	grd.type = type[1];
+
+	GMT_grd_init (&grd, argc, argv, TRUE);
+
 	if (GMT_write_grd (file[1], &grd, z, 0.0, 0.0, 0.0, 0.0, GMT_pad, no_header)) {
 		fprintf (stderr, "%s: Error writing file %s\n", GMT_program, fname[1]);
 		exit (EXIT_FAILURE);
 	}
-	
+
 	GMT_free ((void *)z);
-		
+
+	Free_Grdreformat_Ctrl (Ctrl);	/* Deallocate control structure */
+
 	GMT_end (argc, argv);
+
+	exit (EXIT_SUCCESS);
+}
+
+void *New_Grdreformat_Ctrl () {	/* Allocate and initialize a new control structure */
+	struct GRDREFORMAT_CTRL *C;
+	
+	C = (struct GRDREFORMAT_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GRDREFORMAT_CTRL), "New_Grdreformat_Ctrl");
+	
+	/* Initialize values whose defaults are not 0/FALSE/NULL */
+	
+	return ((void *)C);
+}
+
+void Free_Grdreformat_Ctrl (struct GRDREFORMAT_CTRL *C) {	/* Deallocate control structure */
+	GMT_free ((void *)C);	
 }
