@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdtrend.c,v 1.3.4.8 2004/01/02 22:23:11 pwessel Exp $
+ *	$Id: grdtrend.c,v 1.19 2006/04/13 01:44:31 pwessel Exp $
  *
- *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -15,7 +15,7 @@
  *
  *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
-/* grdtrend <input.grd> -N<n_model>[r] [-T<trend.grd>] [-V]
+/* grdtrend <input.grd> -N[r]<n_model> [-T<trend.grd>] [-V]
 	[-W<weight.grd] [-D<differences.grd]
 
 Reads a grdfile and fits a trend surface.  Trend surface
@@ -44,16 +44,16 @@ A robust fit may be achieved by iterative fitting of
 a weighted least squares problem, where the weights
 are set according to a scale length based on the 
 Median absolute deviation (MAD: Huber, 1982).  The
--N<n>r option acheives this.
+-N<n>r option achieves this.
 
 Author:		W. H. F. Smith
 Date:		21 May, 1991.
+Version:	4
 Calls:		uses the QR solution of the Normal
 		equations furnished by Wm. Menke's
 		C routine "gauss".  We gratefully
 		acknowledge this contribution.
 Revised:	12-JUN-1998 PW, for GMT 3.1
-Version:	3.4.3
 
 Remarks:
 
@@ -82,9 +82,52 @@ are conveniently sized near 0 or 1.
 
 #include "gmt.h"
 
-#define MAX_TABLE_COLS 10	/* Used by Menke routine gauss  */
+struct GRDTREND_CTRL {	/* All control options for this program (except common args) */
+	/* ctive is TRUE if the option has been activated */
+	struct D {	/* -D<diffgrid> */
+		BOOLEAN active;
+		char *file;
+	} D;
+	struct N {	/* -N[r]<n_model> */
+		BOOLEAN active;
+		BOOLEAN robust;
+		int value;
+	} N;
+	struct T {	/* -T<trend.grd> */
+		BOOLEAN active;
+		char *file;
+	} T;
+	struct W {	/* -W<weight.grd> */
+		BOOLEAN active;
+		char *file;
+	} W;
+};
 
-char format[BUFSIZ];
+#define GRDTREND_MAX_TABLE_COLS 10	/* Used by Menke routine gauss  */
+
+int main(int argc, char **argv)
+{
+	BOOLEAN	error = FALSE, trivial, weighted;
+
+	char	*i_filename = NULL, format[BUFSIZ];
+
+	int	i, j, k, ierror = 0, iterations, nxy;
+
+	float	*data;		/* Pointer for array from input grdfile  */
+	float	*trend;		/* Pointer for array containing fitted surface  */
+	float	*resid;		/* Pointer for array containing residual surface  */
+	float	*weight = VNULL;/* Pointer for array containing data weights  */
+
+	double	chisq, old_chisq, zero_test = 1.0e-08, scale = 1.0;
+	double	*xval;		/* Pointer for array of change of variable:  x[i]  */
+	double	*yval;		/* Pointer for array of change of variable:  y[j]  */
+	double	*gtg;		/* Pointer for array for matrix G'G normal equations  */
+	double	*gtd;		/* Pointer for array for vector G'd normal equations  */
+	double	*old;		/* Pointer for array for old model, used for robust sol'n  */
+	double	*pstuff;	/* Pointer for array for Legendre polynomials of x[i],y[j]  */
+
+	struct GRD_HEADER head_d, head_w;
+	struct GRDTREND_CTRL *Ctrl;
 
 	void	gauss(double *a, double *vec, int n, int nstore, double test, int *ierror, int itriag);		/* QR solution of the Normal equations  */
 	void	set_up_vals(double *val, int nval, double vmin, double vmax, double dv, int pixel_reg);		/* Store x[i], y[j] once for all to save time  */
@@ -96,73 +139,57 @@ char format[BUFSIZ];
 	void	compute_robust_weight(float *resid, float *weight, int nxy, double *scale);	/* Find weights from residuals  */
 	void	write_model_parameters(double *gtd, int n_model);	/* Do reports if gmtdefs.verbose == TRUE  */
 	void	load_gtg_and_gtd(float *data, int nx, int ny, double *xval, double *yval, double *pstuff, double *gtg, double *gtd, int n_model, float *weight, int weighted);		/* Fill normal equations matrices  */
+	void	*New_Grdtrend_Ctrl (), Free_Grdtrend_Ctrl (struct GRDTREND_CTRL *C);
 
-main(int argc, char **argv)
-{
-	int	i, j, k, ierror = 0, iterations, nxy, n_model = 0;
-	
-	BOOLEAN	error = FALSE, robust = FALSE, trivial, weighted;
-
-	double	chisq, old_chisq, zero_test = 1.0e-08, scale = 1.0;
-
-	char	*i_filename = NULL, *d_filename = NULL, *t_filename = NULL, *w_filename = NULL;
-
-	float	*data;		/* Pointer for array from input grdfile  */
-	float	*trend;		/* Pointer for array containing fitted surface  */
-	float	*resid;		/* Pointer for array containing residual surface  */
-	float	*weight;	/* Pointer for array containing data weights  */
-
-	double	*xval;		/* Pointer for array of change of variable:  x[i]  */
-	double	*yval;		/* Pointer for array of change of variable:  y[j]  */
-	double	*gtg;		/* Pointer for array for matrix G'G normal equations  */
-	double	*gtd;		/* Pointer for array for vector G'd normal equations  */
-	double	*old;		/* Pointer for array for old model, used for robust sol'n  */
-	double	*pstuff;	/* Pointer for array for Legendre polynomials of x[i],y[j]  */
-
-	struct GRD_HEADER head_d, head_w;
-	
 /* Execution begins here with loop over arguments:  */
 
 	argc = GMT_begin (argc, argv);
-	
+
+	Ctrl = (struct GRDTREND_CTRL *) New_Grdtrend_Ctrl ();		/* Allocate and initialize defaults in a new control structure */
+
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '-') {
 			switch (argv[i][1]) {
 
 				/* Common parameters */
-			
+
 				case 'V':
 				case '\0':
-					error += GMT_get_common_args (argv[i], 0, 0, 0, 0);
+					error += GMT_parse_common_options (argv[i], 0, 0, 0, 0);
 					break;
-				
+
 				/* Supplemental parameters */
-			
+
 				case 'D':
-					d_filename = &argv[i][2];
-					if (d_filename == NULL) {
+					Ctrl->D.active = TRUE;
+					if (argv[i][2])
+						Ctrl->D.file = strdup (&argv[i][2]);
+					else {
 						fprintf (stderr, "%s: GMT SYNTAX ERROR -D option:  Must specify file name\n", GMT_program);
 						error = TRUE;
 					}
 					break;
 				case 'N':
-					j = 2;
-					if (argv[i][j] && (argv[i][j] == 'r' || argv[i][j] == 'r') ){
-						robust = TRUE;
-						j++;
-					}
-					if (argv[i][j]) n_model = atoi(&argv[i][j]);
+					/* Must check for both -N[r]<n_model> and -N<n_model>[r] due to confusion */
+					Ctrl->N.active = TRUE;
+					if (strchr (argv[i], 'r')) Ctrl->N.robust = TRUE;
+					j = (argv[i][2] == 'r') ? 3 : 2;
+					if (argv[i][j]) Ctrl->N.value = atoi(&argv[i][j]);
 					break;
 				case 'T':
-					t_filename = &argv[i][2];
-					if (t_filename == NULL) {
+					Ctrl->T.active = TRUE;
+					if (argv[i][2])
+						Ctrl->T.file = strdup (&argv[i][2]);
+					else {
 						fprintf (stderr, "%s: GMT SYNTAX ERROR -T option:  Must specify file name\n", GMT_program);
 						error = TRUE;
 					}
 					break;
 				case 'W':
-					w_filename = &argv[i][2];
-					if (w_filename == NULL) {
+					Ctrl->W.active = TRUE;
+					if (argv[i][2])
+						Ctrl->W.file = strdup (&argv[i][2]);
+					else {
 						fprintf (stderr, "%s: GMT SYNTAX ERROR -W option:  Must specify file name\n", GMT_program);
 						error = TRUE;
 					}
@@ -178,15 +205,15 @@ main(int argc, char **argv)
 			i_filename = argv[i];
 	}
 
-	if (argc == 1 || GMT_quick) {
+	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "grdtrend %s - Fit trend surface to gridded data\n\n", GMT_VERSION);
-		fprintf (stderr,"usage:  grdtrend <input.grd> -N[r]<n_model> [-D<diff.grd>]\n");
+		fprintf (stderr,"usage:  grdtrend <input.grd> -N<n_model>[r] [-D<diff.grd>]\n");
 		fprintf (stderr,"\t[-T<trend.grd>] [-V] [-W<weight.grd>]\n\n");
-		
-		if (GMT_quick) exit (EXIT_FAILURE);
-		
+
+		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
+
 		fprintf (stderr,"\t<input.grd> is name of grdfile to fit trend to.\n");
-		fprintf (stderr,"\t-N # model parameters to fit; integer in [1,10].  Insert r for robust fit.\n");
+		fprintf (stderr,"\t-N # model parameters to fit; integer in [1,10].  Append r for robust fit.\n");
 		fprintf (stderr,"\n\tOPTIONS:\n");
 		fprintf (stderr,"\t-D Supply filename to write grdfile of differences (input - trend).\n");
 		fprintf (stderr,"\t-T Supply filename to write grdfile of trend.\n");
@@ -202,18 +229,18 @@ main(int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify input file\n", GMT_program);
 		error++;
 	}
-	if (n_model <= 0 || n_model > 10) {
+	if (Ctrl->N.value <= 0 || Ctrl->N.value > 10) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -N option:  Specify 1-10 model parameters\n", GMT_program);
 		error++;
 	}
 
 	if (error) exit (EXIT_FAILURE);
 
-	GMT_put_history (argc, argv);	/* Update .gmtcommands */
+	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
 
 /* End of argument parsing.  */
 
-	trivial = ( (n_model < 5) && (!(robust)) && (!w_filename) );
+	trivial = ( (Ctrl->N.value < 5) && (!(Ctrl->N.robust)) && (!Ctrl->W.file) );
 
 /* Read the input file:  */
 
@@ -242,21 +269,21 @@ main(int argc, char **argv)
 	resid = (float *) GMT_memory (VNULL, (size_t)nxy, sizeof (float), GMT_program);
 	xval = (double *) GMT_memory (VNULL, (size_t)head_d.nx, sizeof (double), GMT_program);
 	yval = (double *) GMT_memory (VNULL, (size_t)head_d.ny, sizeof (double), GMT_program);
-	gtg = (double *) GMT_memory (VNULL, (size_t)(n_model*n_model), sizeof (double), GMT_program);
-	gtd = (double *) GMT_memory (VNULL, (size_t)n_model, sizeof (double), GMT_program);
-	old = (double *) GMT_memory (VNULL, (size_t)n_model, sizeof (double), GMT_program);
-	pstuff = (double *) GMT_memory (VNULL, (size_t)n_model, sizeof (double), GMT_program);
-	
+	gtg = (double *) GMT_memory (VNULL, (size_t)(Ctrl->N.value*Ctrl->N.value), sizeof (double), GMT_program);
+	gtd = (double *) GMT_memory (VNULL, (size_t)Ctrl->N.value, sizeof (double), GMT_program);
+	old = (double *) GMT_memory (VNULL, (size_t)Ctrl->N.value, sizeof (double), GMT_program);
+	pstuff = (double *) GMT_memory (VNULL, (size_t)Ctrl->N.value, sizeof (double), GMT_program);
+
 
 /* If a weight array is needed, get one:  */
 
-	weighted = (robust || w_filename);
+	weighted = (Ctrl->N.robust || Ctrl->W.file);
 	if (weighted) {
 		weight = (float *) GMT_memory (VNULL, (size_t)nxy, sizeof (float), GMT_program);
-		if (!access (w_filename, R_OK)) {	/* We have weights on input  */
+		if (!GMT_access (Ctrl->W.file, R_OK)) {	/* We have weights on input  */
 			GMT_grd_init (&head_w, argc, argv, FALSE);
-			if (GMT_read_grd_info (w_filename, &head_w)) {
-				fprintf (stderr, "%s: GMT SYNTAX ERROR:  File %s not found\n", GMT_program, w_filename);
+			if (GMT_read_grd_info (Ctrl->W.file, &head_w)) {
+				fprintf (stderr, "%s: GMT SYNTAX ERROR:  File %s not found\n", GMT_program, Ctrl->W.file);
 				exit (EXIT_FAILURE);
 			}
 			if (head_w.nx != head_d.nx || head_w.ny != head_d.ny) {
@@ -264,7 +291,7 @@ main(int argc, char **argv)
 				for (i = 0; i < nxy; i++) weight[i] = 1.0;
 			}
 			else
-				GMT_read_grd (w_filename, &head_w, weight, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE);
+				GMT_read_grd (Ctrl->W.file, &head_w, weight, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE);
 		}
 		else {
 			for (i = 0; i < nxy; i++) weight[i] = 1.0;
@@ -285,36 +312,36 @@ main(int argc, char **argv)
 /* Do the problem:  */
 
 	if (trivial) {
-		grd_trivial_model(data, head_d.nx, head_d.ny, xval, yval, gtd, n_model);
-		compute_trend(trend, head_d.nx, head_d.ny, xval, yval, gtd, n_model, pstuff);
+		grd_trivial_model(data, head_d.nx, head_d.ny, xval, yval, gtd, Ctrl->N.value);
+		compute_trend(trend, head_d.nx, head_d.ny, xval, yval, gtd, Ctrl->N.value, pstuff);
 		compute_resid(data, trend, resid, nxy);
 	}
 	else {	/* Problem is not trivial  !!  */
 
-		load_gtg_and_gtd(data, head_d.nx, head_d.ny, xval, yval, pstuff, gtg, gtd, n_model, weight, weighted);
-		gauss(gtg, gtd, n_model, n_model, zero_test, &ierror, 1);
+		load_gtg_and_gtd(data, head_d.nx, head_d.ny, xval, yval, pstuff, gtg, gtd, Ctrl->N.value, weight, weighted);
+		gauss(gtg, gtd, Ctrl->N.value, Ctrl->N.value, zero_test, &ierror, 1);
 		if (ierror) {
 			fprintf (stderr,"%s:  Gauss returns error code %d\n", GMT_program, ierror);
 			exit (EXIT_FAILURE);
 		}
-		compute_trend(trend, head_d.nx, head_d.ny, xval, yval, gtd, n_model, pstuff);
+		compute_trend(trend, head_d.nx, head_d.ny, xval, yval, gtd, Ctrl->N.value, pstuff);
 		compute_resid(data, trend, resid, nxy);
 
-		if (robust) {
+		if (Ctrl->N.robust) {
 			compute_chisq(resid, weight, nxy, &chisq, scale);
 			iterations = 1;
 			sprintf(format, "%%s Robust iteration %%d:  Old Chi Squared:  %s  New Chi Squared %s\n", gmtdefs.d_format, gmtdefs.d_format);
 			do {
 				old_chisq = chisq;
-				for(k = 0; k < n_model; k++) old[k] = gtd[k];
+				for(k = 0; k < Ctrl->N.value; k++) old[k] = gtd[k];
 				compute_robust_weight(resid, weight, nxy, &scale);
-				load_gtg_and_gtd(data, head_d.nx, head_d.ny, xval, yval, pstuff, gtg, gtd, n_model, weight, weighted);
-				gauss(gtg, gtd, n_model, n_model, zero_test, &ierror, 1);
+				load_gtg_and_gtd(data, head_d.nx, head_d.ny, xval, yval, pstuff, gtg, gtd, Ctrl->N.value, weight, weighted);
+				gauss(gtg, gtd, Ctrl->N.value, Ctrl->N.value, zero_test, &ierror, 1);
 				if (ierror) {
 					fprintf (stderr,"%s:  Gauss returns error code %d\n", GMT_program, ierror);
 					exit (EXIT_FAILURE);
 				}
-				compute_trend(trend, head_d.nx, head_d.ny, xval, yval, gtd, n_model, pstuff);
+				compute_trend(trend, head_d.nx, head_d.ny, xval, yval, gtd, Ctrl->N.value, pstuff);
 				compute_resid(data, trend, resid, nxy);
 				compute_chisq(resid, weight, nxy, &chisq, scale);
 				if (gmtdefs.verbose) fprintf (stderr, format, GMT_program, iterations, old_chisq, chisq);
@@ -323,8 +350,8 @@ main(int argc, char **argv)
 
 			/* Get here when new model not significantly better; use old one:  */
 
-			for(k = 0; k < n_model; k++) gtd[k] = old[k];
-			compute_trend(trend, head_d.nx, head_d.ny, xval, yval, gtd, n_model, pstuff);
+			for(k = 0; k < Ctrl->N.value; k++) gtd[k] = old[k];
+			compute_trend(trend, head_d.nx, head_d.ny, xval, yval, gtd, Ctrl->N.value, pstuff);
 			compute_resid(data, trend, resid, nxy);
 		}
 	}
@@ -332,19 +359,19 @@ main(int argc, char **argv)
 /* End of do the problem section.  */
 
 /* Get here when ready to do output:  */
-	
-	if (gmtdefs.verbose) write_model_parameters(gtd, n_model);
-	if (t_filename) {
+
+	if (gmtdefs.verbose) write_model_parameters(gtd, Ctrl->N.value);
+	if (Ctrl->T.file) {
 		strcpy (head_d.title, "trend surface");
-		GMT_write_grd (t_filename, &head_d, trend, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE);
+		GMT_write_grd (Ctrl->T.file, &head_d, trend, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE);
 	}
-	if (d_filename) {
+	if (Ctrl->D.file) {
 		strcpy (head_d.title, "trend residuals");
-		GMT_write_grd (d_filename, &head_d, resid, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE);
+		GMT_write_grd (Ctrl->D.file, &head_d, resid, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE);
 	}
-	if (w_filename && robust) {
+	if (Ctrl->W.file && Ctrl->N.robust) {
 		strcpy (head_d.title, "trend weights");
-		GMT_write_grd (d_filename, &head_d, weight, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE);
+		GMT_write_grd (Ctrl->W.file, &head_d, weight, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE);
 	}
 
 /* That's all, folks!  */
@@ -359,10 +386,14 @@ main(int argc, char **argv)
 	GMT_free ((void *)trend);
 	GMT_free ((void *)data);
 
-	GMT_end (argc, argv);
-}		
+	Free_Grdtrend_Ctrl (Ctrl);	/* Deallocate control structure */
 
-void  set_up_vals(double *val, int nval, double vmin, double vmax, double dv, int pixel_reg)
+	GMT_end (argc, argv);
+
+	exit (EXIT_SUCCESS);
+}
+
+void set_up_vals (double *val, int nval, double vmin, double vmax, double dv, int pixel_reg)
 {
 	int	i;
 	double  v, middle, drange, true_min, true_max;
@@ -382,7 +413,7 @@ void  set_up_vals(double *val, int nval, double vmin, double vmax, double dv, in
 	return;
 }
 
-void	load_pstuff(double *pstuff, int n_model, double x, double y, int newx, int newy)
+void load_pstuff (double *pstuff, int n_model, double x, double y, int newx, int newy)
 {
 	/* If either x or y has changed, compute new Legendre polynomials as needed  */
 
@@ -405,7 +436,7 @@ void	load_pstuff(double *pstuff, int n_model, double x, double y, int newx, int
 	return;
 }
 
-void	compute_trend(float *trend, int nx, int ny, double *xval, double *yval, double *gtd, int n_model, double *pstuff)
+void compute_trend (float *trend, int nx, int ny, double *xval, double *yval, double *gtd, int n_model, double *pstuff)
 {
 	int	i, j, k, ij;
 
@@ -420,7 +451,7 @@ void	compute_trend(float *trend, int nx, int ny, double *xval, double *yval, dou
 	}
 }
 
-void	compute_resid(float *data, float *trend, float *resid, int nxy)
+void compute_resid (float *data, float *trend, float *resid, int nxy)
 {
 	int	i;
 
@@ -435,7 +466,7 @@ void	compute_resid(float *data, float *trend, float *resid, int nxy)
 	return;
 }
 
-void	grd_trivial_model(float *data, int nx, int ny, double *xval, double *yval, double *gtd, int n_model)
+void grd_trivial_model (float *data, int nx, int ny, double *xval, double *yval, double *gtd, int n_model)
 {
 	/* Routine to fit up elementary polynomial model of grd data, 
 	model = gtd[0] + gtd[1]*x + gtd[2]*y + gtd[3] * x * y,
@@ -475,7 +506,7 @@ void	grd_trivial_model(float *data, int nx, int ny, double *xval, double *yval,
 	return;
 }
 
-void	compute_chisq(float *resid, float *weight, int nxy, double *chisq, double scale)
+void compute_chisq (float *resid, float *weight, int nxy, double *chisq, double scale)
 {
 	int	i;
 	double	tmp;
@@ -498,7 +529,7 @@ void	compute_chisq(float *resid, float *weight, int nxy, double *chisq, double s
 	return;
 }
 
-void	compute_robust_weight(float *resid, float *weight, int nxy, double *scale)
+void compute_robust_weight (float *resid, float *weight, int nxy, double *scale)
 {
 	int	i, j, j2;
 	double	r, mad;
@@ -531,16 +562,17 @@ void	compute_robust_weight(float *resid, float *weight, int nxy, double *scale)
 			continue;
 		}
 		r = fabs(resid[i]) / (*scale);
-		
+
 		weight[i] = (float)((r <= 1.5) ? 1.0 : (3.0 - 2.25/r) / r);
 	}
 	return;
 }
 
-void	write_model_parameters(double *gtd, int n_model)
+void write_model_parameters (double *gtd, int n_model)
 {
 	int	i;
 	char	pbasis[10][16];
+	char format[BUFSIZ];
 
 	sprintf(pbasis[0], "Mean");
 	sprintf(pbasis[1], "X");
@@ -559,7 +591,7 @@ void	write_model_parameters(double *gtd, int n_model)
 	return;
 }
 
-void	load_gtg_and_gtd(float *data, int nx, int ny, double *xval, double *yval, double *pstuff, double *gtg, double *gtd, int n_model, float *weight, int weighted)
+void load_gtg_and_gtd (float *data, int nx, int ny, double *xval, double *yval, double *pstuff, double *gtg, double *gtd, int n_model, float *weight, int weighted)
 {
 	/* Routine to load the matrix G'G (gtg) and vector G'd (gtd)
 	for the normal equations.  Routine uses indices i,j to refer
@@ -634,7 +666,7 @@ void	load_gtg_and_gtd(float *data, int nx, int ny, double *xval, double *yval, d
 
 	return;
 }
-				
+
 void gauss (double *a, double *vec, int n, int nstore, double test, int *ierror, int itriag)
 {
  
@@ -642,7 +674,7 @@ void gauss (double *a, double *vec, int n, int nstore, double test, int *ierror,
 /* july 1978 (modified feb 1983, nov 85) */
  
 /* a subroutine to solve a system of n linear equations in n unknowns*/
-/* where n doesn't exceed MAX_TABLE_COLS */
+/* where n doesn't exceed GRDTREND_MAX_TABLE_COLS */
 /* gaussian reduction with partial pivoting is used */
 /*      a               (sent, destroyed)       n by n matrix           */
 /*      vec             (sent, overwritten)     n vector, replaced w/ solution*/
@@ -652,10 +684,9 @@ void gauss (double *a, double *vec, int n, int nstore, double test, int *ierror,
 /*      itriag          (sent)                  matrix triangularized only*/
 /*                                               on TRUE useful when solving*/
 /*                                               multiple systems with same a */
-        static int isub[MAX_TABLE_COLS], l1;
-        int line[MAX_TABLE_COLS], iet, ieb, i, j, k, l, j2;
+        static int isub[GRDTREND_MAX_TABLE_COLS], l1;
+        int line[GRDTREND_MAX_TABLE_COLS], iet, ieb, i = 0, j, k, l, j2;
         double big, testa, b, sum;
-        
 
         iet=0;  /* initial error flags, one for triagularization*/
         ieb=0;  /* one for backsolving */
@@ -678,8 +709,7 @@ void gauss (double *a, double *vec, int n, int nstore, double test, int *ierror,
                                   /* of unused portion of matrix*/
                        for( l1=0; l1<n; l1++ ) {
                                if( line[l1]==0 ) {
-                                       testa=(double) fabs(
-                                                (double) (*(a+l1*nstore+j)) );
+                                       testa=(double) fabs((double) (*(a+l1*nstore+j)));
                                        if (testa>big) {
                                                 i=l1;
                                                 big=testa;
@@ -699,9 +729,7 @@ void gauss (double *a, double *vec, int n, int nstore, double test, int *ierror,
                                 if( line[k]==0 ) {
                                         b=(*(a+k*nstore+j))*sum;
                                         for( l=j+1; l<n; l++ ) {
-                                               *(a+k*nstore+l)=
-                                                        (*(a+k*nstore+l))
-                                                        -b*(*(a+i*nstore+l));
+                                               *(a+k*nstore+l)= (*(a+k*nstore+l)) -b*(*(a+i*nstore+l));
                                                } /*end for l*/
                                        *(a+k*nstore+j)=b;
                                         } /*end if*/
@@ -710,7 +738,7 @@ void gauss (double *a, double *vec, int n, int nstore, double test, int *ierror,
  
                for( j=0; j<n; j++ ) {
                         /*find last unused row and set its pointer*/
-                        /*  this row contians the apex of the triangle*/
+                        /*  this row contains the apex of the triangle*/
                         if( line[j]==0) {
                                 l1=j;   /*apex of triangle*/
                                 isub[n-1]=j;
@@ -775,5 +803,19 @@ void gauss (double *a, double *vec, int n, int nstore, double test, int *ierror,
       *ierror = iet + ieb;   /* set final error flag*/
 }
 
+void *New_Grdtrend_Ctrl () {	/* Allocate and initialize a new control structure */
+	struct GRDTREND_CTRL *C;
+	
+	C = (struct GRDTREND_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GRDTREND_CTRL), "New_Grdtrend_Ctrl");
+	
+	/* Initialize values whose defaults are not 0/FALSE/NULL */
+		
+	return ((void *)C);
+}
 
-
+void Free_Grdtrend_Ctrl (struct GRDTREND_CTRL *C) {	/* Deallocate control structure */
+	if (C->D.file) GMT_free ((void *)C->D.file);	
+	if (C->T.file) GMT_free ((void *)C->T.file);	
+	if (C->W.file) GMT_free ((void *)C->W.file);	
+	GMT_free ((void *)C);	
+}
