@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdvolume.c,v 1.4.4.8 2004/01/02 22:23:11 pwessel Exp $
+ *	$Id: grdvolume.c,v 1.28 2006/05/08 08:56:52 pwessel Exp $
  *
- *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -24,78 +24,119 @@
  * Author:	Paul Wessel
  * Date:	23-SEP-1997
  * Revised:	02-JUN-1999
- * Version:	3.4.3
+ * Version:	4
  */
 
 #include "gmt.h"
 
-void SW_triangle (float f[], int ij, int nx, BOOLEAN triangle, double *dv, double *da);
-void NE_triangle (float f[], int ij, int nx, BOOLEAN triangle, double *dv, double *da);
-void SE_triangle (float f[], int ij, int nx, BOOLEAN triangle, double *dv, double *da);
-void NW_triangle (float f[], int ij, int nx, BOOLEAN triangle, double *dv, double *da);
-void NS_trapezoid (float f[], int ij, int nx, BOOLEAN right, double *dv, double *da);
-void EW_trapezoid (float f[], int ij, int nx, BOOLEAN top, double *dv, double *da);
-int ors_find_kink (double y[], int n, int mode);
-double vol_prism_frac_x (float *z, int ij, int nx, double x0, double x1, double a, double b, double c, double d);
-double vol_prism_frac_y (float *z, int ij, int nx, double y0, double y1, double a, double b, double c, double d);
-double median3 (double x[]);
+struct GRDVOLUME_CTRL {
+	struct C {	/* -C */
+		BOOLEAN active;
+		double low, high, inc;
+	} C;
+	struct L {	/* -L<base> */
+		BOOLEAN active;
+		double value;
+	} L;
+	struct S {	/* -S */
+		BOOLEAN active;
+		char unit;
+	} S;
+	struct T {	/* -T */
+		BOOLEAN active;
+	} T;
+	struct Z {	/* Z<fact>[/<shift>] */
+		BOOLEAN active;
+		double scale, offset;
+	} Z;
+};
 
-main (int argc, char **argv)
+double vol_prism_frac_x (float *z, size_t ij, size_t nx, double x0, double x1, double a, double b, double c, double d);
+double vol_prism_frac_y (float *z, size_t ij, size_t nx, double y0, double y1, double a, double b, double c, double d);
+
+int main (int argc, char **argv)
 {
-	int i, j, n = 0, c, ij, nx, ny, ij_inc[4], one_or_zero, k, pos, neg, nc, n_contours, mode = 0, nz = 0, nm; 
+	BOOLEAN error = FALSE, full = FALSE, bad, cut[4];
+
+	char *grdfile = CNULL, format[BUFSIZ];
+
+	int i, j, n = 0, c, k, pos, neg, nc, n_contours, mode = 0, nz = 0; 
+
+	size_t ij, nm, nx, ny;
 	
-	BOOLEAN error = FALSE, mapping = FALSE, contour = FALSE, full = FALSE, bad, cut[4];
-	BOOLEAN set_base = FALSE;
-	
-	double take_out, west, east, south, north, dv, da, cval = 0.0, x_size, y_size, m_pr_deg, x_fact, shift = 0.0, fact = 1.0;
-	double *area, *vol, *height, clow, chigh, deltac, base, this_base, small;
+	long ij_inc[4];
 	
 	float *f;
 
-	char *grdfile = CNULL, format[BUFSIZ], map_units = '\0';
-	
+	double take_out, west, east, south, north, dv, da, cval = 0.0, x_size, y_size, m_pr_deg, x_fact;
+	double *area, *vol, *height, this_base, small;
+
+	struct GRDVOLUME_CTRL *Ctrl;
+
 	struct GRD_HEADER grd;
+
+	void SW_triangle (float f[], size_t ij, size_t nx, BOOLEAN triangle, double *dv, double *da);
+	void NE_triangle (float f[], size_t ij, size_t nx, BOOLEAN triangle, double *dv, double *da);
+	void SE_triangle (float f[], size_t ij, size_t nx, BOOLEAN triangle, double *dv, double *da);
+	void NW_triangle (float f[], size_t ij, size_t nx, BOOLEAN triangle, double *dv, double *da);
+	void NS_trapezoid (float f[], size_t ij, size_t nx, BOOLEAN right, double *dv, double *da);
+	void EW_trapezoid (float f[], size_t ij, size_t nx, BOOLEAN top, double *dv, double *da);
+	int ors_find_kink (double y[], int n, int mode);
+	void *New_Grdvolume_Ctrl (), Free_Grdvolume_Ctrl (struct GRDVOLUME_CTRL *C);
+
+	argc = GMT_begin (argc, argv);
+
+	Ctrl = (struct GRDVOLUME_CTRL *) New_Grdvolume_Ctrl ();	/* Allocate and initialize a new control structure */
 	
 	west = east = south = north = 0.0;
-	chigh = deltac = clow = 0.0;
-	
-	argc = GMT_begin (argc, argv);
 
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '-') {
 			switch (argv[i][1]) {
-			
+
 				/* Common parameters */
-			
+
 				case 'V':
 					if (argv[i][2] == 'L' || argv[i][2] == 'l') full = TRUE;
 				case 'R':
 				case ':':
 				case '\0':
-					error += GMT_get_common_args (argv[i], &west, &east, &south, &north);
+					error += GMT_parse_common_options (argv[i], &west, &east, &south, &north);
 					break;
 
 
 				/* Supplemental parameters */
-				
+
 				case 'C':
-					contour = TRUE;
-					n = sscanf (&argv[i][2], "%lf/%lf/%lf", &clow, &chigh, &deltac);
-					if (n == 1) chigh = clow, deltac = 1.0;	/* So calculation of ncontours will yield 1 */
+					Ctrl->C.active = TRUE;
+					n = sscanf (&argv[i][2], "%lf/%lf/%lf", &Ctrl->C.low, &Ctrl->C.high, &Ctrl->C.inc);
+					if (n == 3) {
+						if (Ctrl->C.low >= Ctrl->C.high || Ctrl->C.inc <= 0.0) {
+							fprintf (stderr, "%s: GMT SYNTAX ERROR -C:  high must exceed low and delta must be positive\n", GMT_program);
+							error++;
+						}
+					}
+					else
+						Ctrl->C.high = Ctrl->C.low, Ctrl->C.inc = 1.0;	/* So calculation of ncontours will yield 1 */
 					break;
 				case 'L':
-					base = (argv[i][2]) ? atof (&argv[i][2]) : GMT_d_NaN;
-					set_base = TRUE;
+					Ctrl->L.active = TRUE;
+					Ctrl->L.value = (argv[i][2]) ? atof (&argv[i][2]) : GMT_d_NaN;
 					break;
 				case 'S':
-					mapping = TRUE;
-                                        if (argv[i][2]) map_units = argv[i][2];
+					Ctrl->S.active = TRUE;
+                                        if (argv[i][2]) Ctrl->S.unit = argv[i][2];
 					break;
 				case 'T':
- 					mode = 1;
+ 					Ctrl->T.active = TRUE;
 					break;
 				case 'Z':
-					nz = (argv[i][2]) ? sscanf (&argv[i][2], "%lf/%lf", &fact, &shift) : -1;
+ 					Ctrl->Z.active = TRUE;
+					nz = (argv[i][2]) ? sscanf (&argv[i][2], "%lf/%lf", &Ctrl->Z.scale, &Ctrl->Z.offset) : -1;
+					if (nz < 0 || nz > 2) {
+						fprintf (stderr, "%s: GMT SYNTAX ERROR option -Z: Must specify <fact> and optionally <shift>\n", GMT_program);
+						error++;
+					}
 					break;
 				default:
 					error = TRUE;
@@ -106,13 +147,13 @@ main (int argc, char **argv)
 		else
 			grdfile = argv[i];
 	}
-	
-	if (argc == 1 || GMT_quick) {
+
+	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr,"grdvolume %s - Calculating volume under a surface within a contour\n\n", GMT_VERSION);
-		fprintf (stderr, "usage: grdvolume <grdfile> [-C<cval> or -C<low/high/delta>] [-L<base>] [-S[k]] [-T] [-Rw/s/e/n[r]] [-V] [-Z<fact>[/<shift>]]\n");
-		
-		if (GMT_quick) exit (EXIT_FAILURE);
-		
+		fprintf (stderr, "usage: grdvolume <grdfile> [-C<cval> or -C<low/high/delta>] [-L<base>] [-S[k]] [-T]\n\t[%s] [-V] [-Z<fact>[/<shift>]]\n", GMT_Rgeo_OPT);
+
+		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
+
 		fprintf (stderr, "\t<grdfile> is the name of the 2-D binary data set\n");
 		fprintf (stderr, "\n\tOPTIONS:\n");
 		fprintf (stderr, "\t-C find area and volume inside the <cval> contour\n");
@@ -124,41 +165,37 @@ main (int argc, char **argv)
 		GMT_explain_option ('R');
 		GMT_explain_option ('V');
 		fprintf (stderr, "\t   Append l for listing of all results (when contour search is selected)\n");
-		fprintf (stderr, "\t-Z Subract <shift> and then multiply data by <fact> before processing [1/0]\n");
+		fprintf (stderr, "\t-Z Subtract <shift> and then multiply data by <fact> before processing [1/0]\n");
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
-	
+
 	if (!grdfile) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify input grd file\n", GMT_program);
 		error++;
 	}
-	if (contour && !(n == 1 || n == 3)) {
+	if (Ctrl->C.active && !(n == 1 || n == 3)) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR option -C: Must specify 1 or 3 arguments\n", GMT_program);
 		error++;
 	}
-	if (nz < 0 || nz > 2) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR option -Z: Must specify <fact> and optionally <shift>\n", GMT_program);
-		error++;
-	}
-	if (mapping && !(map_units == '\0' || map_units == 'k')) {
+	if (Ctrl->S.active && !(Ctrl->S.unit == '\0' || Ctrl->S.unit == 'k')) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR option -S: May append k only\n", GMT_program);
 		error++;
 	}
-	if (set_base && GMT_is_dnan (base)) {
+	if (Ctrl->L.active && GMT_is_dnan (Ctrl->L.value)) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR option -L: Must specify base\n", GMT_program);
 		error++;
 	}
 
 	if (error) exit (EXIT_FAILURE);
 
-	GMT_put_history (argc, argv);	/* Update .gmtcommands */
+	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
 
 	if (GMT_read_grd_info (grdfile, &grd)) {
 		fprintf (stderr, "%s: Error opening file %s\n", GMT_program, grdfile);
 		exit (EXIT_FAILURE);
 	}
-	
+
 	if (!project_info.region_supplied) {	/* No subset asked for */
 		west = grd.x_min;
 		east = grd.x_max;
@@ -168,23 +205,23 @@ main (int argc, char **argv)
 	else if (!project_info.region)	/* Got w/s/e/n, make into w/e/s/n */
 		d_swap (south, east);
 
-	one_or_zero = (grd.node_offset) ? 0 : 1;
-	nx = irint ( (east - west) / grd.x_inc) + one_or_zero;
-	ny = irint ( (north - south) / grd.y_inc) + one_or_zero;
+	nx = (size_t)GMT_get_n (west, east, grd.x_inc, grd.node_offset);
+	ny = (size_t)GMT_get_n (south, north, grd.y_inc, grd.node_offset);
 	nm = nx * ny;
 
-	f = (float *) GMT_memory (VNULL, (size_t)nm, sizeof (float), GMT_program);
+	f = (float *) GMT_memory (VNULL, nm, sizeof (float), GMT_program);
 
 	if (GMT_read_grd (grdfile, &grd, f, west, east, south, north, GMT_pad, FALSE)) {
 		fprintf (stderr, "%s: Error reading file %s\n", GMT_program, grdfile);
 		exit (EXIT_FAILURE);
 	}
 
-	ij_inc[0] = 0;	ij_inc[1] = 1;	ij_inc[2] = 1 - nx;	ij_inc[3] = -nx;
+	ij_inc[0] = 0;	ij_inc[1] = 1;	ij_inc[2] = 1 - nx;	ij_inc[3] = -(long)nx;
 	x_fact = grd.x_inc;
-	if (mapping) {
-		m_pr_deg = TWO_PI * gmtdefs.ellipse[gmtdefs.ellipsoid].eq_radius / 360.0;
-		if (map_units == 'k') m_pr_deg *= 0.001;	/* Use km instead */
+	mode = (Ctrl->T.active) ? 1 : 0;
+	if (Ctrl->S.active) {
+		m_pr_deg = TWO_PI * gmtdefs.ref_ellipsoid[gmtdefs.ellipsoid].eq_radius / 360.0;
+		if (Ctrl->S.unit == 'k') m_pr_deg *= 0.001;	/* Use km instead */
 		x_size = m_pr_deg * grd.x_inc;
 		y_size = m_pr_deg * grd.y_inc;
 	}
@@ -193,44 +230,44 @@ main (int argc, char **argv)
 		y_size = grd.y_inc;
 	}
 
-	n_contours = (contour) ? irint ((chigh - clow) / deltac) + 1 : 1;
+	n_contours = (Ctrl->C.active) ? irint ((Ctrl->C.high - Ctrl->C.low) / Ctrl->C.inc) + 1 : 1;
 
 	height = (double *) GMT_memory (VNULL, (size_t)n_contours, sizeof (double), GMT_program);
 	vol = (double *) GMT_memory (VNULL, (size_t)n_contours, sizeof (double), GMT_program);
 	area = (double *) GMT_memory (VNULL, (size_t)n_contours, sizeof (double), GMT_program);
 
-	if (!(fact == 1.0 && shift == 0.0)) {
-		if (gmtdefs.verbose) fprintf (stderr, "%s: Subtracting %lg and multiplying by %lg\n", GMT_program, shift, fact);
-		for (k = 0; k < nm; k++) f[k] = (float)((f[k] - shift) * fact);
-		grd.z_min = (grd.z_min - shift) * fact;
-		grd.z_max = (grd.z_max - shift) * fact;
-		if (fact < 0.0) d_swap (grd.z_min, grd.z_max);
+	if (!(Ctrl->Z.scale == 1.0 && Ctrl->Z.offset == 0.0)) {
+		if (gmtdefs.verbose) fprintf (stderr, "%s: Subtracting %g and multiplying by %g\n", GMT_program, Ctrl->Z.offset, Ctrl->Z.scale);
+		for (ij = 0; ij < nm; ij++) f[ij] = (float)((f[ij] - Ctrl->Z.offset) * Ctrl->Z.scale);
+		grd.z_min = (grd.z_min - Ctrl->Z.offset) * Ctrl->Z.scale;
+		grd.z_max = (grd.z_max - Ctrl->Z.offset) * Ctrl->Z.scale;
+		if (Ctrl->Z.scale < 0.0) d_swap (grd.z_min, grd.z_max);
 	}
 
-	this_base = (set_base) ? base : 0.0;
-	small = deltac * 1.0e-6;
+	this_base = (Ctrl->L.active) ? Ctrl->L.value : 0.0;
+	small = Ctrl->C.inc * 1.0e-6;
 
-	for (c = 0; contour && c < n_contours; c++) {	/* Trace contour, only count volumes inside contours */
+	for (c = 0; Ctrl->C.active && c < n_contours; c++) {	/* Trace contour, only count volumes inside contours */
 
-		cval = clow + c * deltac;
-		take_out = (c == 0) ? cval : deltac;	/* Take out start contour the first time and just the increment subsequent times */
+		cval = Ctrl->C.low + c * Ctrl->C.inc;
+		take_out = (c == 0) ? cval : Ctrl->C.inc;	/* Take out start contour the first time and just the increment subsequent times */
 
-		for (k = 0; k < nm; k++) {
-			f[k] -= (float)take_out;		/* Take out the zero value */
-			if (f[k] == 0.0) f[k] = (float)small;	/* But we dont want exactly zero, just + or - */
+		for (ij = 0; ij < nm; ij++) {
+			f[ij] -= (float)take_out;		/* Take out the zero value */
+			if (f[ij] == 0.0) f[ij] = (float)small;	/* But we dont want exactly zero, just + or - */
 		}
-		if (set_base) this_base -= take_out;
+		if (Ctrl->L.active) this_base -= take_out;
 
-		if (set_base && this_base >= 0.0) {
+		if (Ctrl->L.active && this_base >= 0.0) {
 			fprintf (stderr, "%s: Base is > than contour - exiting\n", GMT_program);
 			exit (EXIT_FAILURE);
 		}
 
-		for (j = 1, ij = grd.nx; j < grd.ny; j++) {
+		for (j = 1, ij = nx; j < (int)ny; j++) {
 
 			dv = da = 0.0;	/* Reset these for each row */
 
-			for (i = 0; i < grd.nx-1; i++, ij++) {
+			for (i = 0; i < (int)(nx-1); i++, ij++) {
 
 				/* Find if a contour goes through this bin */
 
@@ -253,7 +290,7 @@ main (int argc, char **argv)
 					if ((f[ij-nx] * f[ij+1-nx]) < 0.0) nc++, cut[2] = TRUE;	/* Crossing the N border */
 					if ((f[ij-nx] * f[ij]) < 0.0)      nc++, cut[3] = TRUE;	/* Crossing the W border */
 
-					if (nc < 2) continue;	/* Can happen if some nodes were 0 and then reset to smal, thus passing the test */
+					if (nc < 2) continue;	/* Can happen if some nodes were 0 and then reset to small, thus passing the test */
 
 					if (nc == 4) {	/* Saddle scenario */
 						if (f[ij] > 0) {	/* Need both SW and NE triangles */
@@ -264,7 +301,7 @@ main (int argc, char **argv)
 							SE_triangle (f, ij, nx, TRUE, &dv, &da);
 							NW_triangle (f, ij, nx, TRUE, &dv, &da);
 						}
-						
+
 					}
 					else if (cut[0]) {	/* Contour enters at S border ... */
 						if (cut[1])	/* and exits at E border */
@@ -286,10 +323,10 @@ main (int argc, char **argv)
 			}
 			ij++;
 
-			if (set_base) dv -= this_base;	/* Adjust for lower starting base */
+			if (Ctrl->L.active) dv -= this_base;	/* Adjust for lower starting base */
 
 			/* Allow for shrinking of longitudes with latitude */
-			if (mapping) x_fact = x_size * cosd (grd.y_max - (j+0.5) * grd.y_inc);
+			if (Ctrl->S.active) x_fact = x_size * cosd (grd.y_max - (j+0.5) * grd.y_inc);
 
 			dv *= x_fact;
 			da *= x_fact;
@@ -297,8 +334,8 @@ main (int argc, char **argv)
 			area[c] += da;
 		}
 	}
-	if (!contour) {	/* Columns with bilinear tops */
-		if (set_base && base >= grd.z_min) {
+	if (!Ctrl->C.active) {	/* Columns with bilinear tops */
+		if (Ctrl->L.active && Ctrl->L.value >= grd.z_min) {
 			fprintf (stderr, "%s: Base is > than minimum z - exiting\n", GMT_program);
 			exit (EXIT_FAILURE);
 		}
@@ -307,15 +344,15 @@ main (int argc, char **argv)
 			for (i = 0; i < grd.nx-1; i++, ij++) {
 				for (k = 0, bad = FALSE; !bad && k < 4; k++) if (GMT_is_fnan (f[ij+ij_inc[k]])) bad = TRUE;
                                 if (bad) continue;
-				
+
 				dv += 0.25 * (f[ij] + f[ij+1] + f[ij-nx] + f[ij-nx+1]);
 				da += 1.0;
-				if (set_base) dv -= this_base;	/* Adjust for lower starting base */
+				if (Ctrl->L.active) dv -= this_base;	/* Adjust for lower starting base */
 			}
 			ij++;
 
 			/* Allow for shrinking of longitudes with latitude */
-			if (mapping) x_fact = x_size * cosd (grd.y_max - (j+0.5) * grd.y_inc);
+			if (Ctrl->S.active) x_fact = x_size * cosd (grd.y_max - (j+0.5) * grd.y_inc);
 
 			dv *= x_fact;
 			da *= x_fact;
@@ -323,11 +360,10 @@ main (int argc, char **argv)
 			area[0] += da;
 		}
 	}
-	
+
 	/* Adjust values for actual y dimension */
 
 	for (c = 0; c < n_contours; c++) {
-
 		vol[c]  *= y_size;
 		area[c] *= y_size;
 		height[c] = (area[c] > 0.0) ? vol[c] / area[c] : GMT_d_NaN;
@@ -336,7 +372,7 @@ main (int argc, char **argv)
 
 	/* Find the best contour that gives largest height */
 
-	c = (contour) ? ors_find_kink (height, n_contours, mode) : 0;
+	c = (Ctrl->C.active) ? ors_find_kink (height, n_contours, mode) : 0;
 
 	/* Print out final estimates */
 
@@ -344,11 +380,11 @@ main (int argc, char **argv)
 
 	if (full) {
 		sprintf (format, "%s\t%s\t%s\t%s\n", gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
-		for (c = 0; c < n_contours; c++) fprintf (GMT_stdout, format, clow + c * deltac, area[c], vol[c], height[c]);
+		for (c = 0; c < n_contours; c++) fprintf (GMT_stdout, format, Ctrl->C.low + c * Ctrl->C.inc, area[c], vol[c], height[c]);
 	}
-	else if (contour) {
+	else if (Ctrl->C.active) {
         	sprintf (format, "%s\t%s\t%s\t%s\n", gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
-		fprintf (GMT_stdout, format, clow + c * deltac, area[c], vol[c], height[c]);
+		fprintf (GMT_stdout, format, Ctrl->C.low + c * Ctrl->C.inc, area[c], vol[c], height[c]);
 	}
 	else {
         	sprintf (format, "%s\t%s\t%s\n", gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
@@ -359,85 +395,97 @@ main (int argc, char **argv)
 	GMT_free ((void *)area);
 	GMT_free ((void *)vol);
 	GMT_free ((void *)height);
-	
+
 	GMT_end (argc, argv);
+
+	Free_Grdvolume_Ctrl (Ctrl);	/* Deallocate control structure */
+
+	exit (EXIT_SUCCESS);
 }
 
-void SW_triangle (float f[], int ij, int nx, BOOLEAN triangle, double *dv, double *da)
+void SW_triangle (float f[], size_t ij, size_t nx, BOOLEAN triangle, double *dv, double *da)
 {	/* Calculates area of a SW-corner triangle */
 	/* triangle = TRUE gets triangle, FALSE gives the complementary area */
-	double x1, y0;
+	double x1, y0, frac;
 
 	x1 = f[ij] / (f[ij] - f[ij+1]);
 	y0 = f[ij] / (f[ij] - f[ij-nx]);
+	frac = (x1 == 0.0) ? 0.0 : vol_prism_frac_x (f, ij, nx, 0.0, x1, 0.0, 0.0, -y0 / x1, y0);
 	if (triangle) {
-		*dv += vol_prism_frac_x (f, ij, nx, 0.0, x1, 0.0, 0.0, -y0 / x1, y0);
+		*dv += frac;
 		*da += 0.5 * x1 * y0;
 	}
 	else {
-		*dv += 0.25 * (f[ij] + f[ij+1] + f[ij-nx] + f[ij-nx+1]) - vol_prism_frac_x (f, ij, nx, 0.0, x1, 0.0, 0.0, -y0 / x1, y0);
+		*dv += 0.25 * (f[ij] + f[ij+1] + f[ij-nx] + f[ij-nx+1]) - frac;
 		*da += 1.0 - 0.5 * x1 * y0;
 	}
 }
 
-void NE_triangle (float f[], int ij, int nx, BOOLEAN triangle, double *dv, double *da)
+void NE_triangle (float f[], size_t ij, size_t nx, BOOLEAN triangle, double *dv, double *da)
 {	/* Calculates area of a NE-corner triangle */
 	/* triangle = TRUE gets triangle, FALSE gives the complementary area */
-	double x0, y1, a, x0_1, y1_1;
+	double x0, y1, a, x0_1, y1_1, frac = 0.0;
 
 	x0 = f[ij-nx] / (f[ij-nx] - f[ij+1-nx]);
 	y1 = f[ij+1] / (f[ij+1] - f[ij+1-nx]);
 	x0_1 = 1.0 - x0;
 	y1_1 = y1 - 1.0;
-	a = y1_1 / x0_1;
+	if (x0_1 != 0.0) {
+		a = y1_1 / x0_1;
+		frac = vol_prism_frac_x (f, ij, nx, x0, 1.0, a, 1.0 - a * x0, 0.0, 0.0);
+	}
 	if (triangle) {
-		*dv += vol_prism_frac_x (f, ij, nx, x0, 1.0, a, 1.0 - a * x0, 0.0, 0.0);
+		*dv += frac;
 		*da -= 0.5 * x0_1 * y1_1;	/* -ve because we need 1 - y1 */
 	}
 	else {
-		*dv += 0.25 * (f[ij] + f[ij+1] + f[ij-nx] + f[ij-nx+1]) - vol_prism_frac_x (f, ij, nx, x0, 1.0, a, 1.0 - a * x0, 0.0, 0.0);
+		*dv += 0.25 * (f[ij] + f[ij+1] + f[ij-nx] + f[ij-nx+1]) - frac;
 		*da += 1.0 + 0.5 * x0_1 * y1_1;	/* +ve because we need 1 - y1 */
 	}
 }
 
-void SE_triangle (float f[], int ij, int nx, BOOLEAN triangle, double *dv, double *da)
+void SE_triangle (float f[], size_t ij, size_t nx, BOOLEAN triangle, double *dv, double *da)
 {	/* Calculates area of a SE-corner triangle */
 	/* triangle = TRUE gets triangle, FALSE gives the complementary area */
-	double x0, y1, c, x0_1;
+	double x0, y1, c, x0_1, frac = 0.0;
 
 	x0 = f[ij] / (f[ij] - f[ij+1]);
 	y1 = f[ij+1] / (f[ij+1] - f[ij+1-nx]);
 	x0_1 = 1.0 - x0;
-	c = y1 / x0_1;
+	if (x0_1 != 0.0) {
+		c = y1 / x0_1;
+		frac = vol_prism_frac_x (f, ij, nx, x0, 1.0, 0.0, 0.0, c, -c * x0);
+	}
 	if (triangle) {
-		*dv += vol_prism_frac_x (f, ij, nx, x0, 1.0, 0.0, 0.0, c, -c * x0);
+		*dv += frac;
 		*da += 0.5 * x0_1 * y1;
 	}
 	else {
-		*dv += 0.25 * (f[ij] + f[ij+1] + f[ij-nx] + f[ij-nx+1]) - vol_prism_frac_x (f, ij, nx, x0, 1.0, 0.0, 0.0, c, -c * x0);
+		*dv += 0.25 * (f[ij] + f[ij+1] + f[ij-nx] + f[ij-nx+1]) - frac;
 		*da += 1.0 - 0.5 * x0_1 * y1;
 	}
 }
 
-void NW_triangle (float f[], int ij, int nx, BOOLEAN triangle, double *dv, double *da)
+void NW_triangle (float f[], size_t ij, size_t nx, BOOLEAN triangle, double *dv, double *da)
 {	/* Calculates area of a NW-corner triangle */
 	/* triangle = TRUE gets triangle, FALSE gives the complementary area */
-	double x1, y0, y0_1;
+	double x1, y0, y0_1, frac;
 
 	x1 = f[ij-nx] / (f[ij-nx] - f[ij+1-nx]);
 	y0 = f[ij] / (f[ij] - f[ij-nx]);
 	y0_1 = 1.0 - y0;
+	frac = (x1 == 0.0) ? 0.0 : vol_prism_frac_x (f, ij, nx, 0.0, x1, y0_1 / x1, y0, 0.0, 1.0);
 	if (triangle) {
-		*dv += vol_prism_frac_x (f, ij, nx, 0.0, x1, y0_1 / x1, y0, 0.0, 1.0);
+		*dv += frac;
 		*da += 0.5 * x1 * y0_1;
 	}
 	else {
-		*dv += 0.25 * (f[ij] + f[ij+1] + f[ij-nx] + f[ij-nx+1]) - vol_prism_frac_x (f, ij, nx, 0.0, x1, y0_1 / x1, y0, 0.0, 1.0);
+		*dv += 0.25 * (f[ij] + f[ij+1] + f[ij-nx] + f[ij-nx+1]) - frac;
 		*da += 1.0 - 0.5 * x1 * y0_1;
 	}
 }
 
-void NS_trapezoid (float f[], int ij, int nx, BOOLEAN right, double *dv, double *da)
+void NS_trapezoid (float f[], size_t ij, size_t nx, BOOLEAN right, double *dv, double *da)
 {	/* Calculates area of a NS trapezoid */
 	/* right = TRUE gets the right trapezoid, FALSE gets the left */
 	double x0, x1;
@@ -454,7 +502,7 @@ void NS_trapezoid (float f[], int ij, int nx, BOOLEAN right, double *dv, double
 	}
 }
 
-void EW_trapezoid (float f[], int ij, int nx, BOOLEAN top, double *dv, double *da)
+void EW_trapezoid (float f[], size_t ij, size_t nx, BOOLEAN top, double *dv, double *da)
 {	/* Calculates area of a EW trapezoid */
 	/* top = TRUE gets the top trapezoid, FALSE gets the bottom */
 	double y0, y1;
@@ -475,7 +523,7 @@ void EW_trapezoid (float f[], int ij, int nx, BOOLEAN top, double *dv, double *d
  * lines x0 and x1 and two horizontal lines y0 = ax +b and y1 = cx + d
  */
 
-double vol_prism_frac_x (float *z, int ij, int nx, double x0, double x1, double a, double b, double c, double d)
+double vol_prism_frac_x (float *z, size_t ij, size_t nx, double x0, double x1, double a, double b, double c, double d)
 {
 	double dzdx, dzdy, dzdxy, ca, db, c2a2, d2b2, cdab, v, x02, x12, x03, x04, x13, x14;
 
@@ -503,7 +551,7 @@ double vol_prism_frac_x (float *z, int ij, int nx, double x0, double x1, double
  * lines y0 and y1 and two vertical lines x0 = ay +b and x1 = cy + d
  */
 
-double vol_prism_frac_y (float *z, int ij, int nx, double y0, double y1, double a, double b, double c, double d)
+double vol_prism_frac_y (float *z, size_t ij, size_t nx, double y0, double y1, double a, double b, double c, double d)
 {
 	double dzdx, dzdy, dzdxy, ca, db, c2a2, d2b2, cdab, v, y02, y03, y04, y12, y13, y14;
 
@@ -531,8 +579,9 @@ int ors_find_kink (double y[], int n, int mode)
 {	/* mode: 0 = find maximum, 1 = find curvature kink */
 	int i, ic, im;
 	double *c, *f;
+	double median3 (double x[]);
 
-	if (mode == 0) {	/* Find maximum value */	
+	if (mode == 0) {	/* Find maximum value */
 
 		for (i = im = 0; i < n; i++) if (y[i] > y[im]) im = i;
 		return (im);
@@ -575,3 +624,18 @@ double median3 (double x[])
 		return (x[2]);
 	}
 }
+
+void *New_Grdvolume_Ctrl () {	/* Allocate and initialize a new control structure */
+	struct GRDVOLUME_CTRL *C;
+	
+	C = (struct GRDVOLUME_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GRDVOLUME_CTRL), "New_Grdvolume_Ctrl");
+	
+	/* Initialize values whose defaults are not 0/FALSE/NULL */
+	C->L.value = GMT_d_NaN;
+	C->Z.scale = 1.0;
+	return ((void *)C);
+}
+
+void  Free_Grdvolume_Ctrl (struct GRDVOLUME_CTRL *C) {	/* Deallocate control structure */
+	GMT_free ((void *)C);	
+}
