@@ -1,10 +1,10 @@
-/*	$Id: gshhs.h,v 1.6 2004/09/14 19:27:30 pwessel Exp $
+/*	$Id: gshhs.h,v 1.10 2006/05/08 00:24:15 pwessel Exp $
  *
  * Include file defining structures used in gshhs.c
  *
  * Paul Wessel, SOEST
  *
- *	Copyright (c) 1996-2004 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1996-2006 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -20,6 +20,8 @@
  *
  *	14-SEP-2004.  PW: Version 1.3.  Header is now n * 8 bytes (n = 5)
  *			  For use with version 1.3 of GSHHS
+ *	2-MAY-2006.  PW: Version 1.4.  Header is now 32 bytes (all int 4)
+ *			  For use with version 1.4 of GSHHS
  */
 
 #ifndef _GSHHS
@@ -38,20 +40,24 @@
 #define SEEK_CUR 1
 #endif
 
-/* For byte swapping on little-endian systems (GSHHS is bigendian) */
+#define GSHHS_DATA_VERSION	4	/* For v1.4 */
+
+/* For byte swapping on little-endian systems (GSHHS is defined to be bigendian) */
 
-#define swabi2(i2) (((i2) >> 8) + (((i2) & 255) << 8))
 #define swabi4(i4) (((i4) >> 24) + (((i4) >> 8) & 65280) + (((i4) & 65280) << 8) + (((i4) & 255) << 24))
 
 struct GSHHS {	/* Global Self-consistent Hierarchical High-resolution Shorelines */
 	int id;				/* Unique polygon id number, starting at 0 */
 	int n;				/* Number of points in this polygon */
-	int level;			/* 1 land, 2 lake, 3 island_in_lake, 4 pond_in_island_in_lake */
+	int flag;			/* = level + version << 8 + greenwich << 16 + source << 24 */
+	/* flag contains 4 items, one in each byte, as follows:
+	 * low byte:	level = flag & 255: Values: 1 land, 2 lake, 3 island_in_lake, 4 pond_in_island_in_lake
+	 * 2nd byte:	version = (flag >> 8) & 255: Values: Should be 4 for GSHHS version 1.4
+	 * 3rd byte:	greenwich = (flag >> 16) & 255: Values: Greenwich is 1 if Greenwich is crossed
+	 * 4th byte:	source = (flag >> 24) & 255: Values: 0 = CIA WDBII, 1 = WVS
+	 */
 	int west, east, south, north;	/* min/max extent in micro-degrees */
 	int area;			/* Area of polygon in 1/10 km^2 */
-	int version;			/* Version of GSHHS polygon (3 is latest and first with this item) */
-	short int greenwich;		/* Greenwich is 1 if Greenwich is crossed */
-	short int source;		/* 0 = CIA WDBII, 1 = WVS */
 };
 
 struct	POINT {	/* Each lon, lat pair is stored in micro-degrees in 4-byte integer format */
