@@ -1,4 +1,4 @@
-/* $Id: img2mercgrd.c,v 1.4 2002/11/10 03:13:43 lloyd Exp $
+/* $Id: img2mercgrd.c,v 1.10 2006/04/11 22:33:41 pwessel Exp $
  *
  * img2mercgrd.c
  *
@@ -15,7 +15,7 @@
  * The coordinates in the img file are determined by the latitude and
  * longitude span of the img file and the width of an img pixel in
  * minutes of longitude.  These may be changed from their default values
- * using the lower-case [-x<maxlon>] [-y<minlat>/<maxlat>] [-m<minutes>]
+ * using the lower-case [-W<maxlon>] [-D<minlat>/<maxlat>] [-m<minutes>]
  * but must be set to match the coverage of the input img file.
  *
  * The user-supplied -R<w>/<e>/<s>/<n> will be adjusted by rounding
@@ -35,7 +35,7 @@
  *
  * This is a complete rebuild (for v3.1) of the old program by this name.
  * New functionality added here is the averaging option [-N] and the
- * options to define -m, -x, -y, which permit handling very early versions
+ * options to define -m, -W, -y, which permit handling very early versions
  * of these files and anticipate higher-resolutions to come down the road.
  * Also added is the option to look for the img file in GMT_IMGDIR
  *
@@ -44,13 +44,14 @@
  * 
  **WHFS 27 Nov 1998 fixed bug so that -T0 gives correct results
  **  PW 18 Oct 1999 Use WORDS_BIGENDIAN macro set by configure.
+ *   PW 12 Apr 2006 Replaced -x -y with -W -D
  *
  */
 
 
 #include "gmt_imgsubs.h"
 
-main (int argc, char **argv)
+int main (int argc, char **argv)
 {
 	int	i, ij, navgsq, error, irarg = -1, pad[4], tempint;
 	int	navg = 1;	/* navg by navg pixels are averaged if navg > 1; else if navg == 1 do nothing */
@@ -91,7 +92,7 @@ main (int argc, char **argv)
 				case 'R':
 				case 'V':
 				case '\0':
-					error += GMT_get_common_args (argv[i], &west, &east, &south, &north);
+					error += GMT_parse_common_options (argv[i], &west, &east, &south, &north);
 					if (argv[i][1] == 'R') irarg = i;
 					break;
 
@@ -124,16 +125,20 @@ main (int argc, char **argv)
 						fprintf(stderr,"%s:  GMT SYNTAX ERROR -m requires a positive value.\n", GMT_program);
 					}
 					break;
-				case 'x':
+				case 'W':
 					if ((sscanf(&argv[i][2], "%lf", &imgrange.maxlon)) != 1 || imgrange.maxlon < 360.0) {
 						error++;
-						fprintf(stderr,"%s:  GMT SYNTAX ERROR -x requires a number >= 360.0.\n", GMT_program);
+						fprintf(stderr,"%s:  GMT SYNTAX ERROR -W requires a number >= 360.0.\n", GMT_program);
 					}
 					break;
-				case 'y':
-					if ((sscanf(&argv[i][2], "%lf/%lf", &imgrange.minlat, &imgrange.maxlat)) != 2) {
+				case 'D':
+					if (argv[i][2] && (sscanf(&argv[i][2], "%lf/%lf", &imgrange.minlat, &imgrange.maxlat)) != 2) {
 						error++;
-						fprintf(stderr,"%s:  GMT SYNTAX ERROR -y requires <minlat>/<maxlat>.\n", GMT_program);
+						fprintf(stderr,"%s:  GMT SYNTAX ERROR -D: Failed to decode <minlat>/<maxlat>.\n", GMT_program);
+					}
+					else {
+						imgrange.minlat = GMT_IMG_MINLAT_80;
+						imgrange.maxlat = GMT_IMG_MAXLAT_80;
 					}
 					break;
 				default:
@@ -147,12 +152,12 @@ main (int argc, char **argv)
 		}
 	}
 
-	if (GMT_quick || argc == 1) {
+	if (GMT_give_synopsis_and_exit || argc == 1) {
 		fprintf(stderr,"img2mercgrd %s - Extract a grd file from an img file, preserving Mercator projection.\n\n", GMT_VERSION);
-		fprintf(stderr,"usage:  img2mercgrd <world_image_filename> -R<w>/<e>/<s>/<n> -G<grdfile> -T<type> \n");
-		fprintf(stderr,"\t\t[-N<navg>] [-S<scale>] [-V] [-m<minutes>] [-x<maxlon>] [-y<minlat>/<maxlat>]\n\n");
+		fprintf(stderr,"usage:  img2mercgrd <world_image_filename> %s -G<grdfile> -T<type>\n", GMT_Rgeo_OPT);
+		fprintf(stderr,"\t\t[-D[<minlat>/<maxlat>]] [-N<navg>] [-S<scale>] [-V] [-W<maxlon>] [-m<minutes>]\n\n");
 
-		if (GMT_quick) exit (-1);
+		if (GMT_give_synopsis_and_exit) exit (-1);
 		
 		fprintf (stderr, "\tREQUIRED ARGUMENTS:\n");
 		fprintf(stderr,"\t<world_image_filename> gives location of img file.\n");
@@ -171,8 +176,9 @@ main (int argc, char **argv)
 		
 		fprintf (stderr, "\n\tOPTIONAL ARGUMENTS WHICH DEPEND ON THE img FILE VERSION:\n");
 		fprintf (stderr, "\t-m<minutes> input img pixels are <minutes> minutes of longitude wide. [2.0]\n");
-		fprintf (stderr, "\t-x<maxlon> input img file runs from 0 to <maxlon> longitude. [360.0]\n");
-		fprintf (stderr, "\t-y<minlon>/<maxlon> input img file bottom and top latitudes. [-72.006/72.006]\n");
+		fprintf (stderr, "\t-D[<minlon>/<maxlon>] input img file bottom and top latitudes. [%.3f/%.3f]\n", GMT_IMG_MINLAT, GMT_IMG_MAXLAT);
+		fprintf (stderr, "\t   If no latitudes are given it is taken to mean %.3f/%.3f\n", GMT_IMG_MINLAT_80, GMT_IMG_MAXLAT_80);
+		fprintf (stderr, "\t-W<maxlon> input img file runs from 0 to <maxlon> longitude. [360.0]\n");
 		exit(-1);
 	}
 
@@ -195,7 +201,7 @@ main (int argc, char **argv)
 	if (error) exit (EXIT_FAILURE);
 
 	if (GMT_img_setup_coord ( &imgrange, &imgcoord) ) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Error in img coordinate specification [-m -x or -y]\n", GMT_program);
+		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Error in img coordinate specification [-m -W or -D]\n", GMT_program);
 		error++;
 	}
 	else if (navg && (imgcoord.nx360%navg != 0 || imgcoord.nyrow%navg != 0) ) {
@@ -359,15 +365,12 @@ main (int argc, char **argv)
 				}
 				else {
 					tempint = (int)row[ix[iout]];
-					if (output_type) {
-						if ( ( (abs(tempint))%2) != 0) {
-							csum = 1.0;
-							tempint--;
-						}
-						else {
-							csum = 0.0;
-						}
+					if (output_type && abs(tempint)%2 != 0) {
+						csum = 1.0;
+						tempint--;
 					}
+					else
+						csum = 0.0;
 					dsum = (double) tempint;
 				}
 				
@@ -409,4 +412,6 @@ main (int argc, char **argv)
 	GMT_free((void *)row);
 	
         GMT_end (argc, argv);
+
+	exit (EXIT_SUCCESS);
 }
