@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: makecpt.c,v 1.11 2004/07/19 02:36:13 pwessel Exp $
+ *	$Id: makecpt.c,v 1.23 2006/04/12 21:55:46 pwessel Exp $
  *
- *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,6 @@
  *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
- * makecpt [-C<color_table>] [-I] [-Tz0/z1/dz | -T<table>] [-V] [-Z]
  *
  * Read an existing cpt table and desired z grid and produce
  * a GMT cpt file.  Can be inverted [-I] or made continuous [-Z].
@@ -30,23 +29,55 @@
 
 #include "gmt.h"
 
-main(int argc, char **argv)
+struct MAKECPT_CTRL {
+	struct C {	/* -C<cpt> */
+		BOOLEAN active;
+		char *file;
+	} C;
+	struct D {	/* -D */
+		BOOLEAN active;
+	} D;
+	struct I {	/* -I */
+		BOOLEAN active;
+	} I;
+	struct N {	/* -N */
+		BOOLEAN active;
+	} N;
+	struct T {	/* -T<z0/z1/dz> */
+		BOOLEAN active;
+		BOOLEAN cpt;
+		double low, high, inc;
+		char *file;
+	} T;
+	struct Q {	/* -Q[i|o */
+		BOOLEAN active;
+		int mode;
+	} Q;
+	struct Z {	/* -Z */
+		BOOLEAN active;
+	} Z;
+};
+
+int main(int argc, char **argv)
 {
-	int	i, nz, log_mode = 0;
+	int	i, nz, cpt_flags = 0;
 
-	BOOLEAN	error = FALSE, continuous = FALSE, reverse = FALSE, got_t = FALSE;
-	BOOLEAN got_tfile = FALSE, no_BFN = FALSE, ok = FALSE;
+	BOOLEAN	error = FALSE;
 
-	double	z_start, z_stop, z_inc, *z;
+	double	*z;
 
 	FILE	*fpc = NULL, *fpl = NULL;
 
-	char	*table = CNULL, *tfile = CNULL, buffer[BUFSIZ], CPT_lis[BUFSIZ], CPT_file[BUFSIZ];
+	char	buffer[BUFSIZ], CPT_lis[BUFSIZ], CPT_file[BUFSIZ];
+
+	struct MAKECPT_CTRL *Ctrl;
+
+	void *New_Makecpt_Ctrl (), Free_Makecpt_Ctrl (struct MAKECPT_CTRL *C);
 
 	argc = GMT_begin(argc, argv);
 
-	z_start = z_stop = z_inc = 0.0;
-
+	Ctrl = (struct MAKECPT_CTRL *)New_Makecpt_Ctrl ();	/* Allocate and initialize a new control structure */
+	
 	/* Get list of available color tables in $GMTHOME/share/cpt */
 
 	sprintf (CPT_lis, "%s%cshare%cGMT_CPT.lis", GMTHOME, DIR_DELIM, DIR_DELIM);
@@ -61,35 +92,37 @@ main(int argc, char **argv)
 			switch(argv[i][1]) {
 				case 'V':
 				case '\0':
-					error += GMT_get_common_args(argv[i], 0, 0, 0, 0);
+					error += GMT_parse_common_options(argv[i], 0, 0, 0, 0);
 					break;
 				case 'C':
-					table = &argv[i][2];
+					Ctrl->C.active = TRUE;
+					Ctrl->C.file = strdup (&argv[i][2]);
+					break;
+				case 'D':
+					Ctrl->D.active = TRUE;
 					break;
 				case 'I':
-					reverse = TRUE;
+					Ctrl->I.active = TRUE;
 					break;
 				case 'N':
-					no_BFN = TRUE;
+					Ctrl->N.active = TRUE;
 					break;
 				case 'T':
-					if (!access (&argv[i][2], R_OK)) {
-						got_tfile = TRUE;
-						tfile = &argv[i][2];
-					}
-					else {
-						got_t = TRUE;
-						sscanf (&argv[i][2], "%lf/%lf/%lf", &z_start, &z_stop, &z_inc);
-					}
+					Ctrl->T.active = TRUE;
+					if (!access (&argv[i][2], R_OK))
+						Ctrl->T.file = strdup (&argv[i][2]);
+					else
+						sscanf (&argv[i][2], "%lf/%lf/%lf", &Ctrl->T.low, &Ctrl->T.high, &Ctrl->T.inc);
 					break;
 				case 'Q':
+					Ctrl->Q.active = TRUE;
 					if (argv[i][2] == 'o')	/* Input data is z, but take log10(z) before interpolation colors */
-						log_mode = 2;
+						Ctrl->Q.mode = 2;
 					else			/* Input is log10(z) */
-						log_mode = 1;
+						Ctrl->Q.mode = 1;
 					break;
 				case 'Z':
-					continuous = TRUE;
+					Ctrl->Z.active = TRUE;
 					break;
 				default:
 					error = TRUE;
@@ -99,10 +132,10 @@ main(int argc, char **argv)
 		}
 	}
 
-	if (argc == 1 || GMT_quick) {
+	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "makecpt %s - Make GMT color palette tables\n\n", GMT_VERSION);
-		fprintf (stderr, "usage:  makecpt [-C<table>] [-I] [-N] [-Q[i|o]] [-T<z0/z1/dz> | -T<file>] [-V] [-Z]\n\n");
-		if (GMT_quick) exit (EXIT_FAILURE);
+		fprintf (stderr, "usage:  makecpt [-C<table>] [-D] [-I] [-N] [-Q[i|o]] [-T<z0/z1/dz> | -T<file>] [-V] [-Z]\n\n");
+		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 		fprintf (stderr, "\n\tOPTIONS:\n");
 		fprintf (stderr, "\t-C Specify a colortable [Default is rainbow]:\n");
 		fprintf (stderr, "\t   [Default min/max values for -T are given in brackets]\n");
@@ -110,6 +143,7 @@ main(int argc, char **argv)
 		while (fgets (buffer, BUFSIZ, fpc)) if (!(buffer[0] == '#' || buffer[0] == 0)) fprintf (stderr, "\t   %s", buffer);
 		fclose (fpc);
 		fprintf (stderr, "\t   ---------------------------------\n");
+		fprintf (stderr, "\t-D Set back- and foreground color to match the bottom/top limits in the cpt file [Default uses GMT defaults]:\n");
 		fprintf (stderr, "\t-I reverses the sense of the color table\n");
 		fprintf (stderr, "\t-N Do Not write back-, fore-, and nan colors [Default will].\n");
 		fprintf (stderr, "\t-Q assign a logarithmic colortable [Default is linear]\n");
@@ -126,42 +160,15 @@ main(int argc, char **argv)
 
 	fclose (fpc);
 
-	if (got_t && (z_start >= z_stop || z_inc <= 0.0)) {
+	if (Ctrl->T.active && !Ctrl->T.file && (Ctrl->T.low >= Ctrl->T.high || Ctrl->T.inc <= 0.0)) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -T option:  Give start < stop and inc > 0.0\n", GMT_program);
 		error++;
 	}
 
-	/* Open the specified master color table */
+	error += GMT_set_cpt_path (CPT_file, Ctrl->C.file);
 
-	/* First try current directory */
-
-	if (table) {
-		if (strstr (table, ".cpt"))
-			strcpy (CPT_file, table);
-		else
-			sprintf (CPT_file, "%s.cpt", table);
-
-		ok = !access (CPT_file, R_OK);
-		if (ok && gmtdefs.verbose) fprintf (stderr, "%s: Reading %s in current directory\n", GMT_program, CPT_file);
-	}
-
-	if (!ok)	{	/* No table in current dir, try /share */
-		if (table)
-			sprintf (CPT_file, "%s%cshare%ccpt%cGMT_%s.cpt", GMTHOME, DIR_DELIM, DIR_DELIM, DIR_DELIM, table);
-		else	/* Default to rainbow colors */
-			sprintf (CPT_file, "%s%cshare%ccpt%cGMT_rainbow.cpt", GMTHOME, DIR_DELIM, DIR_DELIM, DIR_DELIM);
-
-		ok = !access (CPT_file, R_OK);
-		if (!ok) {
-			fprintf (stderr, "%s: ERROR: Cannot find colortable %s\n", GMT_program, CPT_file);
-			exit (EXIT_FAILURE);
-		}
-		else if (gmtdefs.verbose)
-			fprintf (stderr, "%s: Reading %s\n", GMT_program, CPT_file);
-	}
-
-	if (got_tfile && (fpl = GMT_fopen (tfile, "r")) == NULL) {
-		fprintf (stderr, "%s: Error: Could not open file %s\n", GMT_program, tfile);
+	if (Ctrl->T.file && (fpl = GMT_fopen (Ctrl->T.file, "r")) == NULL) {
+		fprintf (stderr, "%s: Error: Could not open file %s\n", GMT_program, Ctrl->T.file);
 		error++;
 	}
 
@@ -173,9 +180,12 @@ main(int argc, char **argv)
 
 	GMT_read_cpt (CPT_file);
 
+	if (Ctrl->N.active) cpt_flags |= 1;	/* 1st bit controls if BFN will be written out */
+	if (Ctrl->D.active) cpt_flags |= 2;	/* 2nd bit controls if BF will be set to equal bottom/top rgb value */
+	
 	/* Set up arrays */
 
-	if (got_tfile) {
+	if (Ctrl->T.file) {
 		int n_alloc = GMT_SMALL_CHUNK;
 		z = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof(double), GMT_program);
 		nz = 0;
@@ -190,27 +200,27 @@ main(int argc, char **argv)
 		}
 		GMT_fclose (fpl);
 		if (nz == 0) {
-			fprintf (stderr, "%s: Error: No intervals in file %s\n", GMT_program, tfile);
+			fprintf (stderr, "%s: Error: No intervals in file %s\n", GMT_program, Ctrl->T.file);
 			exit (EXIT_FAILURE);
 		}
 		z = (double *) GMT_memory ((void *)z, (size_t)nz, sizeof(double), GMT_program);
 	}
-	else if (got_t && log_mode == 2) {	/* Establish a log10 grid */
-		if (!(z_inc == 1.0 || z_inc == 2.0 || z_inc == 3.0)) {
+	else if (Ctrl->T.active && Ctrl->Q.mode == 2) {	/* Establish a log10 grid */
+		if (!(Ctrl->T.inc == 1.0 || Ctrl->T.inc == 2.0 || Ctrl->T.inc == 3.0)) {
 			fprintf (stderr, "%s: Error: For -Qo logarithmic spacing, dz must be 1, 2, or 3\n", GMT_program);
 			exit (EXIT_FAILURE);
 		}
-		if (z_start <= 0.0) {
+		if (Ctrl->T.low <= 0.0) {
 			fprintf (stderr, "%s: Error: For -Qo logarithmic spacing, z_start must be > 0\n", GMT_program);
 			exit (EXIT_FAILURE);
 		}
-		nz = GMT_log_array (z_start, z_stop, z_inc, &z);
+		nz = GMT_log_array (Ctrl->T.low, Ctrl->T.high, Ctrl->T.inc, &z);
 	}
-	else if (got_t) {	/* Establish linear grid */
-		nz = irint ((z_stop - z_start) / z_inc) + 1;
+	else if (Ctrl->T.active) {	/* Establish linear grid */
+		nz = irint ((Ctrl->T.high - Ctrl->T.low) / Ctrl->T.inc) + 1;
 		z = (double *) GMT_memory (VNULL, (size_t)nz, sizeof(double), GMT_program);
 
-		for (i = 0; i < nz; i++) z[i] = z_start + i * z_inc;	/* Desired z values */
+		for (i = 0; i < nz; i++) z[i] = Ctrl->T.low + i * Ctrl->T.inc;	/* Desired z values */
 	}
 	else {	/* Just copy what was in the cpt file */
 		nz = GMT_n_colors + 1;
@@ -218,18 +228,37 @@ main(int argc, char **argv)
 		for (i = 0; i < nz-1; i++) z[i] = GMT_lut[i].z_low;
 		z[i] = GMT_lut[i-1].z_high;
 	}
-	
-	if (log_mode == 2) for (i = 0; i < nz; i++) z[i] = d_log10 (z[i]);	/* Make log10(z) values for interpolation step */
-	
+
+	if (Ctrl->Q.mode == 2) for (i = 0; i < nz; i++) z[i] = d_log10 (z[i]);	/* Make log10(z) values for interpolation step */
+
 	/* Write to GMT_stdout.  */
 
 	fprintf (GMT_stdout, "#\tcpt file created by: %s", GMT_program);
 	for (i = 1; i < argc; i++) fprintf (GMT_stdout, " %s", argv[i]);
 	fprintf (GMT_stdout, "\n");
 
-	GMT_sample_cpt (z, nz, continuous, reverse, log_mode, no_BFN);
+	GMT_sample_cpt (z, nz, Ctrl->Z.active, Ctrl->I.active, Ctrl->Q.mode, cpt_flags);
 
 	GMT_free ((void *)z);
 
+	Free_Makecpt_Ctrl (Ctrl);	/* Deallocate control structure */
+
 	GMT_end (argc, argv);
+
+	exit (EXIT_SUCCESS);
+}
+
+void *New_Makecpt_Ctrl () {	/* Allocate and initialize a new control structure */
+	struct MAKECPT_CTRL *C;
+	
+	C = (struct MAKECPT_CTRL *) GMT_memory (VNULL, 1, sizeof (struct MAKECPT_CTRL), "New_Makecpt_Ctrl");
+	
+	/* Initialize values whose defaults are not 0/FALSE/NULL */
+	return ((void *)C);
+}
+
+void Free_Makecpt_Ctrl (struct MAKECPT_CTRL *C) {	/* Deallocate control structure */
+	if (C->C.file) GMT_free ((void *)C->C.file);	
+	if (C->T.file) GMT_free ((void *)C->T.file);	
+	GMT_free ((void *)C);	
 }
