@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *    $Id: pscoupe.c,v 1.13 2004/09/10 23:16:15 pwessel Exp $
+ *    $Id: pscoupe.c,v 1.23 2006/04/10 05:47:30 pwessel Exp $
  *
  *    Copyright (c) 1996-2001 by G. Patau
  *    See README file for copying and redistribution conditions.
@@ -20,12 +20,12 @@ PostScript code is written to stdout.
 
  */
 
-#include "gmt.h"        /* to have gmt environment */
+#include "gmt.h"	/* to have gmt environment */
+#include "pslib.h"	/* to have pslib environment */
 
 #include "meca.h"
 #include "utilmeca.h"
 #include "submeca.h"
-void distaz(double lat1,double lon1,double lat2,double lon2,double *distrad,double *distdeg,double *distkm,double *az12rad,double *az12deg,double *az21rad,double *az21deg,int syscoord);
 
      /* symbols for plotting seismicity and/or axis */
 #define CROSS       1
@@ -37,10 +37,7 @@ void distaz(double lat1,double lon1,double lat2,double lon2,double *distrad,doub
 #define HEXAGON     7
 #define ITRIANGLE   8
 
-
-int *plot_pen;
-
-main (int argc, char **argv)
+int main (int argc, char **argv)
 {
 
     int     i, j, symbol = 0, n, ix = 0, iy = 1, n_files = 0, fno;
@@ -58,18 +55,18 @@ main (int argc, char **argv)
     BOOLEAN    transparence = FALSE, one_size = FALSE, no_label = FALSE;
     BOOLEAN    frame_coupe = FALSE;
     BOOLEAN    old_GMT_world_map, skip_if_outside = TRUE;
-    BOOLEAN    transparence_old, not_defined = FALSE;
+    BOOLEAN    transparence_old = FALSE, not_defined = FALSE;
     
     double xy[2], west = 0.0, east = 0.0, south = 0.0, north = 0.0;
     double plot_x, plot_y, scale = 0.0;
 
-    char event_title[BUFSIZ], txt_a[32];
+    char event_title[BUFSIZ], txt_a[GMT_TEXT_LEN];
 
-    char line[BUFSIZ], symbol_type, col[15][40], *cpt;
-    char newfile[120], extracted_file[120];
+    char line[BUFSIZ], symbol_type, col[15][GMT_TEXT_LEN], *cpt = CNULL;
+    char newfile[GMT_LONG_TEXT], extracted_file[GMT_LONG_TEXT];
     
     FILE *fp = NULL;
-    FILE *pnew, *pextract; 
+    FILE *pnew = NULL, *pextract = NULL; 
 
     struct GMT_PEN pen, lpen, tr_pen, tz_pen;
     struct GMT_PEN ppen, tpen;
@@ -82,7 +79,7 @@ main (int argc, char **argv)
     struct M_TENSOR mt, mtr;
     struct AXIS T, N, P, Tr, Nr, Pr;
 
-    int n_plane = 0, n_plane_old;
+    int n_plane = 0, n_plane_old = 0;
     int default_justify = 2;
     int justify = default_justify, form = 0;
 
@@ -90,7 +87,7 @@ main (int argc, char **argv)
     double fontsize = default_fontsize;
     double default_offset = gmtdefs.measure_unit == 0 ? 0.1 : 0.04;
     double offset = default_offset, angle = 0.0;
-    double size;
+    double size = 0.0;
     double tmp1, tmp2, tmp3, tmp4, tmp5;
     double n_dep, distance;
     double xlonref, ylatref;
@@ -99,9 +96,10 @@ main (int argc, char **argv)
     char project_type;
     int syscoord, fuseau = 0;
 
-    double P_x, P_y, T_x, T_y, a_size;
-    char P_sym_type, T_sym_type;
-    int P_sym, T_sym;
+    double P_x, P_y, T_x, T_y, a_size = GMT_d_NaN;
+    char P_sym_type = 0, T_sym_type = 0;
+    int P_sym = 0, T_sym = 0;
+	void distaz (double lat1,double lon1,double lat2,double lon2,double *distrad,double *distdeg,double *distkm,double *az12rad,double *az12deg,double *az21rad,double *az21deg,int syscoord);
 
     argc = GMT_begin (argc, argv);
     
@@ -117,6 +115,9 @@ main (int argc, char **argv)
     tz_pen = pen;
     pfill = fill;
     tfill = efill;
+    event_title[0] = 0;
+    memset ((void *)&meca, 0, sizeof (meca));
+    memset ((void *)&moment, 0, sizeof (moment));
     
     /* Check and interpret the command line arguments */
     
@@ -142,7 +143,7 @@ main (int argc, char **argv)
                 case 'c':
                 case ':':
                 case '\0':
-                    error += GMT_get_common_args (argv[i], &west, &east, &south, &north);
+                    error += GMT_parse_common_options (argv[i], &west, &east, &south, &north);
                     break;
                 
                 /* Supplemental parameters */
@@ -159,16 +160,16 @@ main (int argc, char **argv)
                         syscoord = project_type == 'a' ? 0 : 2;
                         distaz(lat1, lon1, lat2, lon2, &tmp1, &tmp2, &p_length, &tmp3, &PREF.str,
                             &tmp4, &tmp5, syscoord);
-                        sprintf(newfile, "A%c%.1lf_%.1lf_%.1lf_%.1lf_%.0lf_%.0lf_%.0lf_%.0lf",
+                        sprintf(newfile, "A%c%.1f_%.1f_%.1f_%.1f_%.0f_%.0f_%.0f_%.0f",
                             project_type, lon1, lat1, lon2, lat2, PREF.dip, p_width, dmin, dmax);
-                        sprintf(extracted_file,"A%c%.1lf_%.1lf_%.1lf_%.1lf_%.0lf_%.0lf_%.0lf_%.0lf_map",
+                        sprintf(extracted_file,"A%c%.1f_%.1f_%.1f_%.1f_%.0f_%.0f_%.0f_%.0f_map",
                             project_type, lon1, lat1, lon2, lat2, PREF.dip, p_width, dmin, dmax);
                     } else if(project_type == 'b' || project_type == 'd') {
                         sscanf (&argv[i][3], "%lf/%lf/%lf/%lf/%lf/%lf/%lf/%lf",
                             &lon1, &lat1, &PREF.str, &p_length, &PREF.dip, &p_width, &dmin, &dmax);
-                        sprintf(newfile, "A%c%.1lf_%.1lf_%.0lf_%.0lf_%.0lf_%.0lf_%.0lf_%.0lf",
+                        sprintf(newfile, "A%c%.1f_%.1f_%.0f_%.0f_%.0f_%.0f_%.0f_%.0f",
                             project_type, lon1, lat1, PREF.str, p_length, PREF.dip, p_width, dmin, dmax);
-                        sprintf(extracted_file,"A%c%.1lf_%.1lf_%.0lf_%.0lf_%.0lf_%.0lf_%.0lf_%.0lf_map",
+                        sprintf(extracted_file,"A%c%.1f_%.1f_%.0f_%.0f_%.0f_%.0f_%.0f_%.0f_map",
                             project_type, lon1, lat1, PREF.str, p_length, PREF.dip, p_width, dmin, dmax);
                     }
                     PREF.rake = 0.;
@@ -496,20 +497,20 @@ main (int argc, char **argv)
 
     if (!no_size_needed && (symbol > 0 && scale < 0.0)) error++;
 
-    if (argc == 1 || GMT_quick || error) {    /* Display usage */
+    if (argc == 1 || GMT_give_synopsis_and_exit || error) {    /* Display usage */
         fprintf (stderr,"pscoupe %s - Plot seismological symbols on cross-sections\n\n", GMT_VERSION);
-        fprintf (stderr,"usage: pscoupe <infiles> -A<params> -J<params> -R<west/east/south/north> \n");
-        fprintf (stderr, "   [-B<tickinfo>] [-E<fill>] [-G<fill>]\n");
-        fprintf (stderr, "   [-H] [-K] [-L<pen>] [-M] [-N] [-O] [-P]\n");
+        fprintf (stderr,"usage: pscoupe <infiles> -A<params> %s %s\n", GMT_J_OPT, GMT_Rgeo_OPT);
+        fprintf (stderr, "   [%s] [-E<fill>] [-G<fill>]\n", GMT_B_OPT);
+        fprintf (stderr, "   [%s] [-K] [-L<pen>] [-M] [-N] [-O] [-P]\n", GMT_Ho_OPT);
         fprintf (stderr, "   [-S<format><scale>[/fontsize[/justify/offset/angle/form]]]\n");
         fprintf (stderr, "   [-s<symbol><scale>[/fontsize[/justify/offset/angle/form]]]\n");
-        fprintf (stderr, "   [-Tnplane[/<pen>]] [-U[/dx/dy/][<label>]]\n");
-        fprintf (stderr, "   [-V] [-W<pen>] [-X<x_shift>] [-Y<y_shift>]\n");
+        fprintf (stderr, "   [-Tnplane[/<pen>]] [%s]\n", GMT_U_OPT);
+        fprintf (stderr, "   [-V] [-W<pen>] [%s] [%s]\n", GMT_X_OPT, GMT_Y_OPT);
         fprintf (stderr, "   [-Z<cpt>] [-a[size[Psymbol[Tsymbol]]]\n\n");
         fprintf (stderr, "   [-p<pen>] [-t<pen>] [-e<fill>] -g<fill>]\n\n");
-        fprintf (stderr, "   [-c<ncopies>] [-:]\n");
+        fprintf (stderr, "   [%s] [%s]\n", GMT_c_OPT, GMT_t_OPT);
         
-        if (GMT_quick) exit (EXIT_FAILURE);
+        if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
         
         fprintf (stderr, "    <infiles> is one or more files.  If no, read standard input\n");
         fprintf (stderr, "        -A Specify cross-section parameters. Choose between\n");
@@ -557,7 +558,7 @@ main (int argc, char **argv)
         fprintf (stderr, "             X,Y,depth,T_value,T_azimuth,T_plunge,N_value,N_azimuth,N_plunge,\n");
         fprintf (stderr, "             P_value,P_azimuth,P_plunge,exp,event_title\n");
         fprintf (stderr, "         Optionally add /fontsize[/offset][u]\n");
-        fprintf (stderr, "      Default values are /%lg/%lf\n", default_fontsize, default_offset);
+        fprintf (stderr, "      Default values are /%g/%f\n", default_fontsize, default_offset);
         fprintf (stderr, "      fontsize < 0 : no label written;\n");
         fprintf (stderr, "      offset is from the limit of the beach ball.\n");
         fprintf (stderr, "      By default label is above the beach ball. Add u to plot it under.\n");
@@ -571,7 +572,7 @@ main (int argc, char **argv)
         fprintf (stderr, "         If moment tensor is required, nodal planes overlay moment tensor.\n");
         GMT_explain_option ('U');
         GMT_explain_option ('V');
-        fprintf (stderr, "        -W sets pen attributes [width = %lgp, color = (%d/%d/%d), texture = solid line].\n", 
+        fprintf (stderr, "        -W sets pen attributes [width = %gp, color = (%d/%d/%d), texture = solid line].\n", 
             pen.width, pen.rgb[0], pen.rgb[1], pen.rgb[2]);
         fprintf (stderr, "        -Z Use cpt-file to assign colors based on depth-value in 3rd column\n");
         fprintf (stderr, "        -a plots axis. Default symbols are circles.\n");
@@ -608,14 +609,7 @@ main (int argc, char **argv)
     
     GMT_map_setup (west, east, south, north);
 
-    ps_plotinit (CNULL, gmtdefs.overlay, gmtdefs.page_orientation, 
-        gmtdefs.x_origin, gmtdefs.y_origin, gmtdefs.global_x_scale, 
-        gmtdefs.global_y_scale, gmtdefs.n_copies, gmtdefs.dpi, 
-        GMT_INCH, gmtdefs.paper_width, gmtdefs.page_rgb, gmtdefs.encoding.name, 
-        GMT_epsinfo (argv[0]));
-    GMT_echo_command (argc, argv);
-
-    if (gmtdefs.unix_time) GMT_timestamp (argc, argv);
+    GMT_plotinit (argc, argv);
     
     GMT_setpen (&pen);
     ps_setfont (gmtdefs.annot_font[0]);
@@ -642,11 +636,11 @@ main (int argc, char **argv)
 
         if (!nofile && gmtdefs.verbose) {
             fprintf (stderr, "pscoupe: Working on file %s\n", argv[fno]);
-            sprintf (line, "File: %s\0", argv[fno]);
+            sprintf (line, "File: %s", argv[fno]);
             ps_comment (line);
         }
-        if (gmtdefs.io_header[GMT_IN]) 
-            for (i = 0; i < gmtdefs.n_header_recs; i++) fgets (line, BUFSIZ, fp);
+        if (GMT_io.io_header[GMT_IN]) 
+            for (i = 0; i < GMT_io.n_header_recs; i++) fgets (line, BUFSIZ, fp);
         
 
         while (fgets (line, BUFSIZ, fp)) {
@@ -717,7 +711,7 @@ main (int argc, char **argv)
                 if (read_size) {
                     size = GMT_convert_units (col[3], GMT_INCH);
                 }
-                fprintf(pnew, "%lf %lf %lf %s\n", distance, n_dep, depth, col[3]);
+                fprintf(pnew, "%f %f %f %s\n", distance, n_dep, depth, col[3]);
                 fprintf(pextract, "%s", line);
 
                 switch (symbol) {
@@ -897,7 +891,7 @@ Definition of scalar moment.
 
                 fprintf(pextract, "%s", line);
                 if(read_axis) {
-                    fprintf(pnew, "%lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %d 0 0 %s\n", 
+                    fprintf(pnew, "%f %f %f %f %f %f %f %f %f %f %f %f %d 0 0 %s\n", 
                         xy[0], xy[1], depth, Tr.val, Tr.str, Tr.dip, Nr.val, Nr.str, Nr.dip, 
                         Pr.val, Pr.str, Pr.dip, moment.exponent, event_title);
                     T = Tr;
@@ -905,13 +899,13 @@ Definition of scalar moment.
                     P = Pr;
                 }
                 else if(read_tensor) {
-                    fprintf(pnew, "%lf %lf %lf %lf %lf %lf %lf %lf %lf %d 0 0 %s\n",
+                    fprintf(pnew, "%f %f %f %f %f %f %f %f %f %d 0 0 %s\n",
                         xy[0], xy[1], depth, mtr.f[0], mtr.f[1], mtr.f[2], mtr.f[3], mtr.f[4], mtr.f[5],
                         moment.exponent, event_title);
                     mt = mtr;
                 }
                 else {
-                    fprintf(pnew, "%lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %d 0 0 %s\n",
+                    fprintf(pnew, "%f %f %f %f %f %f %f %f %f %f %d 0 0 %s\n",
                         xy[0], xy[1], depth, mecar.NP1.str, mecar.NP1.dip, mecar.NP1.rake, 
                         mecar.NP2.str, mecar.NP2.dip, mecar.NP2.rake,
                         moment.mant, moment.exponent, event_title);
@@ -1036,7 +1030,9 @@ Definition of scalar moment.
 
     GMT_map_basemap ();
 
-    ps_plotend (gmtdefs.last_page);
+    GMT_plotend ();
     
     GMT_end (argc, argv);
+
+    exit (EXIT_SUCCESS);
 }
