@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *    $Id: psmeca.c,v 1.15 2004/09/10 23:16:15 pwessel Exp $
+ *    $Id: psmeca.c,v 1.27 2006/04/10 05:47:30 pwessel Exp $
  *
  *    Copyright (c) 1996-2001 by G. Patau
  *    See README file for copying and redistribution conditions.
@@ -21,6 +21,7 @@ PostScript code is written to stdout.
  */
 
 #include "gmt.h"        /* to have gmt environment */
+#include "pslib.h"      /* to have pslib environment */
 
 #include "meca.h"
 #include "utilmeca.h"
@@ -35,10 +36,7 @@ PostScript code is written to stdout.
 #define HEXAGON     7
 #define ITRIANGLE   8
 
-
-int *plot_pen;
-
-main (int argc, char **argv)
+int main (int argc, char **argv)
 {
 
     int     i, symbol = 0, n, ix = 0, iy = 1, n_files = 0, fno;
@@ -59,7 +57,7 @@ main (int argc, char **argv)
     BOOLEAN    def_ppen = FALSE, def_tpen = FALSE, def_tz_pen = FALSE;
     BOOLEAN    def_pfill = FALSE, def_tfill = FALSE;
     BOOLEAN    draw_box = FALSE;
-    BOOLEAN    transparence_old, not_defined = FALSE;
+    BOOLEAN    transparence_old = FALSE, not_defined = FALSE;
     
     double xy[2], xynew[2], west = 0.0, east = 0.0, south = 0.0, north = 0.0;
     double plot_x, plot_y, scale = 0.0;
@@ -67,9 +65,9 @@ main (int argc, char **argv)
     double t11 = 1.0, t12 = 0.0, t21 = 0.0, t22 = 1.0;
     double delaz;
 
-    char string[BUFSIZ], event_title[BUFSIZ], txt_a[32];
+    char string[BUFSIZ], event_title[BUFSIZ], txt_a[GMT_TEXT_LEN];
 
-    char line[BUFSIZ], symbol_type, col[15][40], *cpt;
+    char line[BUFSIZ], symbol_type, col[15][GMT_TEXT_LEN], *cpt = CNULL;
     
     FILE *fp = NULL;
     
@@ -85,7 +83,7 @@ main (int argc, char **argv)
     struct M_TENSOR mt;
     struct AXIS T, N, P;
 
-    int n_plane = 0, n_plane_old;
+    int n_plane = 0, n_plane_old = 0;
     int default_justify = 2;
     int justify = default_justify, form = 0;
     int new = 1;
@@ -96,10 +94,10 @@ main (int argc, char **argv)
     double default_pointsize = 0.005;
     double pointsize = default_pointsize, offset = default_offset, angle = 0.0;
     double fault, depth, depmin = 0.0, depmax = 900.0;
-    double size, a_size;
+    double size, a_size = GMT_d_NaN;
     double P_x, P_y, T_x, T_y;
-    char P_sym_type, T_sym_type;
-    int P_sym, T_sym;
+    char P_sym_type = 0, T_sym_type = 0;
+    int P_sym = 0, T_sym = 0;
 
     argc = GMT_begin (argc, argv);
     
@@ -117,7 +115,8 @@ main (int argc, char **argv)
     GMT_init_fill (&bfill, 255, 255, 255);
     GMT_init_fill (&pfill, 255, 255, 255);
     GMT_init_fill (&tfill, 255, 255, 255);
-
+    event_title[0] = 0;
+    memset ((void *)&meca, 0, sizeof (meca));
     
     /* Check and interpret the command line arguments */
     
@@ -143,7 +142,7 @@ main (int argc, char **argv)
                 case 'c':
                 case ':':
                 case '\0':
-                    error += GMT_get_common_args (argv[i], &west, &east, &south, &north);
+                    error += GMT_parse_common_options (argv[i], &west, &east, &south, &north);
                     break;
                 
                 /* Supplemental parameters */
@@ -444,20 +443,20 @@ main (int argc, char **argv)
 
     if(get_rgb && !new) error++;
 
-    if (argc == 1 || GMT_quick || error) {   /* Display usage */
+    if (argc == 1 || GMT_give_synopsis_and_exit || error) {   /* Display usage */
         fprintf (stderr,"psmeca %s - Plot seismological symbols on maps\n\n", GMT_VERSION);
-        fprintf (stderr,"usage: psmeca <infiles> -J<params> -R<west/east/south/north> \n");
+        fprintf (stderr,"usage: psmeca <infiles> %s %s\n", GMT_J_OPT, GMT_Rgeo_OPT);
         fprintf (stderr, "   -S<symbol><scale>[/fontsize[/justify/offset/angle/form]]\n");
-        fprintf (stderr, "   [-B<tickinfo>] [-C<pen>[P<pointsize>]] \n");
+        fprintf (stderr, "   [%s] [-C<pen>[P<pointsize>]]\n", GMT_B_OPT);
         fprintf (stderr, "   [-Ddepmin/depmax] [-E<fill>] [-G<fill>]\n");
-        fprintf (stderr, "   [-H] [-K] [-L<pen>] [-M] [-N] [-O] [-P] [-r]\n");
-        fprintf (stderr, "   [-Tnplane[/<pen>]] [-U[/dx/dy/][<label>]]\n");
-        fprintf (stderr, "   [-V] [-W<pen>] [-X<x_shift>] [-Y<y_shift>]\n");
+        fprintf (stderr, "   [%s] [-K] [-L<pen>] [-M] [-N] [-O] [-P] [-r]\n", GMT_Ho_OPT);
+        fprintf (stderr, "   [-Tnplane[/<pen>]] [%s]\n", GMT_U_OPT);
+        fprintf (stderr, "   [-V] [-W<pen>] [%s] [%s]\n", GMT_X_OPT, GMT_Y_OPT);
         fprintf (stderr, "   [-Z<cpt>] [-z] [-a[size[/Psymbol[Tsymbol]]]\n\n");
         fprintf (stderr, "   [-p<pen>] [-t<pen>] [-e<fill>] -g<fill>]\n\n");
-        fprintf (stderr, "   [-o] [-c<ncopies>] [-:]\n");
+        fprintf (stderr, "   [-o] [%s] [%s]\n", GMT_c_OPT, GMT_t_OPT);
         
-        if (GMT_quick) exit (EXIT_FAILURE);
+        if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
         
         fprintf (stderr, "    <infiles> is one or more files.  If no, read standard input\n");
         GMT_explain_option ('j');
@@ -556,14 +555,7 @@ main (int argc, char **argv)
     
     GMT_map_setup (west, east, south, north);
 
-    ps_plotinit (CNULL, gmtdefs.overlay, gmtdefs.page_orientation, 
-        gmtdefs.x_origin, gmtdefs.y_origin, gmtdefs.global_x_scale, 
-        gmtdefs.global_y_scale, gmtdefs.n_copies, gmtdefs.dpi, 
-	GMT_INCH, gmtdefs.paper_width, gmtdefs.page_rgb, gmtdefs.encoding.name,
-        GMT_epsinfo (argv[0]));
-    GMT_echo_command (argc, argv);
-
-    if (gmtdefs.unix_time) GMT_timestamp (argc, argv);
+    GMT_plotinit (argc, argv);
     
     GMT_setpen (&pen);
     ps_setfont (gmtdefs.annot_font[0]);
@@ -591,8 +583,8 @@ main (int argc, char **argv)
             sprintf (line, "File: %s", argv[fno]);
             ps_comment (line);
         }
-        if (gmtdefs.io_header[GMT_IN]) 
-            for (i = 0; i < gmtdefs.n_header_recs; i++) fgets (line, BUFSIZ, fp);
+        if (GMT_io.io_header[GMT_IN]) 
+            for (i = 0; i < GMT_io.n_header_recs; i++) fgets (line, BUFSIZ, fp);
         
 
         while (fgets (line, BUFSIZ, fp)) {
@@ -980,7 +972,9 @@ Definition of scalar moment.
     
     GMT_map_basemap ();
 
-    ps_plotend (gmtdefs.last_page);
+    GMT_plotend ();
     
     GMT_end (argc, argv);
+
+    exit (EXIT_SUCCESS);
 }
