@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *    $Id: pspolar.c,v 1.11 2004/09/10 23:16:15 pwessel Exp $ 
+ *    $Id: pspolar.c,v 1.18 2006/04/10 05:47:30 pwessel Exp $ 
  *
  *    Copyright (c) 1996-2001 by G. Patau
  *    See README file for copying and redistribution conditions.
@@ -19,7 +19,8 @@
  *
  */
 
-#include "gmt.h"          /* to have gmt environment */
+#include "gmt.h"	/* to have gmt environment */
+#include "pslib.h"	/* to have pslib environment */
 #include <string.h>
 
 #define CROSS       2
@@ -34,7 +35,7 @@
 
 #define POINTSIZE 0.005
 
-main (int argc,char **argv)
+int main (int argc,char **argv)
 {
     int     i, symbol = 0, n, n_files = 0, fno;
     int     n_args;
@@ -58,14 +59,14 @@ main (int argc,char **argv)
     double radius, ech = 0., azimut, ih, plongement;
     double c_pointsize = 0.015, fontsize_s = 12.0;
     double angle_s = 0.0;
-    double azS, sizeS;
-    double v_width, h_length, h_width, shape;
+    double azS, sizeS = GMT_d_NaN;
+    double v_width = GMT_d_NaN, h_length = GMT_d_NaN, h_width = GMT_d_NaN, shape;
     double si, co;
 
-    char line[BUFSIZ], symbol_type, col[4][50];
+    char line[BUFSIZ], symbol_type, col[4][GMT_TEXT_LEN];
     char pol;
-    char txt_a[64],txt_b[64],txt_c[64], txt_d[64];
-    char stacode[32];
+    char txt_a[GMT_TEXT_LEN],txt_b[GMT_TEXT_LEN],txt_c[GMT_TEXT_LEN], txt_d[GMT_TEXT_LEN];
+    char stacode[GMT_TEXT_LEN];
     
     FILE *fp = NULL;
     
@@ -108,7 +109,7 @@ main (int argc,char **argv)
                 case 'y':
                 case 'c':
                 case '\0':
-                    error += GMT_get_common_args (argv[i], &west, &east, &south, &north);
+                    error += GMT_parse_common_options (argv[i], &west, &east, &south, &north);
                     break;
                 
                 /* Supplemental parameters */
@@ -285,17 +286,17 @@ main (int argc,char **argv)
         error++;
     }
     
-    if (argc == 1 || GMT_quick || error) {    /* Display usage */
+    if (argc == 1 || GMT_give_synopsis_and_exit || error) {    /* Display usage */
         fprintf (stderr,"%s %s - Plot polarities on the inferior focal half-sphere on maps\n\n",argv[0], GMT_VERSION);
-        fprintf (stderr,"usage: argv[0] <infiles> -J<params> -R<west/east/south/north>\n");
+        fprintf (stderr,"usage: argv[0] <infiles> %s %s\n", GMT_J_OPT, GMT_Rgeo_OPT);
         fprintf (stderr, " -Dlongitude/latitude -Msize[i/c] -S<symbol><size>[i/c]\n");
-        fprintf (stderr, " [-A] [-B<tickinfo>] [-Clongitude/latitude[W<pen>][Ppointsize]] [-E<fill>]\n");
-        fprintf (stderr, " [-E<fill>] [-e[<pen>]] [-F<fill>] [-f[<pen>]] [-G<fill>] [-g[<pen>]] [-H] [-K] [-N] [-O] [-P]\n");
+        fprintf (stderr, " [-A] [%s] [-Clongitude/latitude[W<pen>][Ppointsize]] [-E<fill>]\n", GMT_B_OPT);
+        fprintf (stderr, " [-e[<pen>]] [-F<fill>] [-f[<pen>]] [-G<fill>] [-g[<pen>]] [%s] [-K] [-N] [-O] [-P]\n", GMT_Ho_OPT);
         fprintf (stderr, " [-s<half-size>/[V[<v_width/h_length/h_width/shape]][G<r/g/b>][L]\n");
-        fprintf (stderr, " [-T[<labelinfo>]] [-t<pen>] [-U[<label>]] [-V] [-W<pen>]\n");
-        fprintf (stderr, " [-X<x_shift>] [-Y<y_shift>] [-c<ncopies>]\n");
+        fprintf (stderr, " [-T[<labelinfo>]] [-t<pen>] [%s] [-V] [-W<pen>]\n", GMT_U_OPT);
+        fprintf (stderr, " [%s] [%s] [%s]\n", GMT_X_OPT, GMT_Y_OPT, GMT_c_OPT);
         
-        if (GMT_quick) exit (EXIT_FAILURE);
+        if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
         
         fprintf (stderr, "    <infiles> is one or more files.  If no, read standard input\n");
         GMT_explain_option ('j');
@@ -349,7 +350,7 @@ main (int argc,char **argv)
         fprintf (stderr, "        -t sets pen attributes to write station codes [default is current pen]\n");
         GMT_explain_option ('U');
         GMT_explain_option ('V');
-        fprintf (stderr, "        -W sets current pen attributes [width = %lgp, color = (%d/%d/%d), texture = solid line].\n", pen.width, pen.rgb[0], pen.rgb[1], pen.rgb[2]);
+        fprintf (stderr, "        -W sets current pen attributes [width = %gp, color = (%d/%d/%d), texture = solid line].\n", pen.width, pen.rgb[0], pen.rgb[1], pen.rgb[2]);
         GMT_explain_option ('X');
         GMT_explain_option ('c');
         GMT_explain_option ('.');
@@ -377,14 +378,7 @@ main (int argc,char **argv)
     
     GMT_map_setup (west, east, south, north);
 
-    ps_plotinit (CNULL, gmtdefs.overlay, gmtdefs.page_orientation,
-        gmtdefs.x_origin, gmtdefs.y_origin, gmtdefs.global_x_scale,
-        gmtdefs.global_y_scale, gmtdefs.n_copies, gmtdefs.dpi,
-        GMT_INCH, gmtdefs.paper_width, gmtdefs.page_rgb, gmtdefs.encoding.name,
-        GMT_epsinfo (argv[0]));
-    GMT_echo_command (argc, argv);
-
-    if (gmtdefs.unix_time) GMT_timestamp (argc, argv);
+    GMT_plotinit (argc, argv);
     
     if (label_s) ps_setfont (gmtdefs.annot_font[0]);
 
@@ -408,10 +402,10 @@ main (int argc,char **argv)
 
         if (!nofile && gmtdefs.verbose) {
             fprintf (stderr, "%s: Working on file %s\n", argv[0], argv[fno]);
-            sprintf (line, "File: %s\0", argv[fno]);
+            sprintf (line, "File: %s", argv[fno]);
             ps_comment (line);
         }
-        if (gmtdefs.io_header[GMT_IN]) for (i = 0; i < gmtdefs.n_header_recs; i++) fgets (line, 512, fp);
+        if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) fgets (line, 512, fp);
         
         GMT_world_map = TRUE;
         
@@ -643,7 +637,9 @@ main (int argc,char **argv)
 
     GMT_map_basemap ();
 
-    ps_plotend (gmtdefs.last_page);
+    GMT_plotend ();
     
     GMT_end (argc, argv);
+
+    exit (EXIT_SUCCESS);
 }
