@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *    $Id: psvelo.c,v 1.12 2004/09/10 23:16:15 pwessel Exp $
+ *    $Id: psvelo.c,v 1.20 2006/04/10 05:47:30 pwessel Exp $
  *
  *    Copyright (c) 1996-2001 by G. Patau
  *    See README file for copying and redistribution conditions.
@@ -16,13 +16,14 @@ PostScript code is written to stdout.
  Author:     Kurt Feigl
  Date:	      7 July 1998
  Version:     4
- Roots:       based on psxy.c	
+ Roots:       based on psxy.c
  Adapted to version 3.3 by Genevieve Patau (25 June 1999)
  Last modified : 18 February 2000
 
  */
 
-#include "gmt.h"        /* to have gmt environment */
+#include "gmt.h"	/* to have gmt environment */
+#include "pslib.h"	/* to have pslib environment */
 #include "utilvelo.h"
 #include "utilstrain.h"
 
@@ -43,12 +44,8 @@ PostScript code is written to stdout.
 #define TEXT		14
 #define POINTSIZE 0.005
 
-
-int *plot_pen;
-
-main (int argc, char **argv)
+int main (int argc, char **argv)
 {
-
 	int 	i, symbol = 0, n, ix = 0, iy = 1, n_files = 0, fno;
 	int	n_args;
         int     n_rec = 0;
@@ -64,11 +61,11 @@ main (int argc, char **argv)
         BOOLEAN read_wedge = FALSE;
         BOOLEAN read_cross = FALSE;
 	BOOLEAN old_GMT_world_map, skip_if_outside = TRUE;
-	
+
 	double xy[2], west = 0.0, east = 0.0, south = 0.0, north = 0.0;
 	double plot_x, plot_y, scale = 0.0;
         double vxy[2], plot_vx, plot_vy;
-        double eps1,eps2,spin,spinsig,theta;
+        double eps1 = 0.0, eps2 = 0.0, spin = 0.0, spinsig = 0.0, theta = 0.0;
 	double direction, small_axis, great_axis; 
         double sigma_x, sigma_y, corr_xy; 
         double confidence = 0., sigma_scale = 1.0, conrad = 1.0;
@@ -78,11 +75,11 @@ main (int argc, char **argv)
         double hl,hw,vw, fontsize = 0.0;
 
         char station_name[20];
-        char txt_a[32], txt_b[32], txt_c[32];
-	char line[BUFSIZ], symbol_type, col[12][40];
-	
+        char txt_a[GMT_TEXT_LEN], txt_b[GMT_TEXT_LEN], txt_c[GMT_TEXT_LEN];
+	char line[BUFSIZ], symbol_type, col[12][GMT_TEXT_LEN];
+
 	FILE *fp = NULL;
-	
+
 
 	struct GMT_PEN pen;
 	struct GMT_FILL fill, efill; /* efill is for uncertainty wedge */
@@ -90,21 +87,21 @@ main (int argc, char **argv)
         int justify;
 
 	argc = GMT_begin (argc, argv);
-	
+
 	GMT_init_pen (&pen, GMT_PENWIDTH);
 	GMT_init_fill (&fill, 0, 0, 0);
 	GMT_init_fill (&efill, 255, 255, 255);
-	
-	
-	
+
+
+
 	/* Check and interpret the command line arguments */
-	
+
 	for (i = 1; !error && i < argc; i++) {
 		if (argv[i][0] == '-') {
 			switch(argv[i][1]) {
-		
+
 				/* Common parameters */
-			
+
 				case 'B':
 				case 'H':
 				case 'J':
@@ -121,11 +118,11 @@ main (int argc, char **argv)
 				case 'c':
 				case ':':
 				case '\0':
-					error += GMT_get_common_args (argv[i], &west, &east, &south, &north);
+					error += GMT_parse_common_options (argv[i], &west, &east, &south, &north);
 					break;
-				
+
 				/* Supplemental parameters */
-			
+
                                 case 'A':       /* Change size of arrow head */
                                         strcpy(txt_a, &argv[i][3]);
                                         strcpy(txt_b, strchr(txt_a+1, '/')+1);
@@ -201,9 +198,9 @@ main (int argc, char **argv)
 				case 'W':		/* Set line attributes */
 					GMT_getpen (&argv[i][2], &pen);
 					break;
-					
+
 				/* Illegal options */
-			
+
 				default:		/* Options not recognized */
 					error = TRUE;
 					break;
@@ -212,28 +209,28 @@ main (int argc, char **argv)
 		else
 			n_files++;
 	}
-	
+
 	/* Check that the options selected are mutually consistent */
-	
+
 	no_size_needed = (read_ellipse || read_rotated_ellipse || read_anisotropy || read_cross || read_wedge );
 	error += GMT_check_rgb (pen.rgb)
 	      + GMT_check_rgb (fill.rgb)
 	      + GMT_check_rgb (efill.rgb)
 	      + GMT_check_rgb (gmtdefs.basemap_frame_rgb);
         /* Only one allowed */
-	if ((read_ellipse + read_rotated_ellipse + read_anisotropy + read_cross + read_wedge ) > 1) error = TRUE;	
+	if ((read_ellipse + read_rotated_ellipse + read_anisotropy + read_cross + read_wedge ) > 1) error = TRUE;
 	if (!no_size_needed && (symbol > 1 && scale <= 0.0)) error = TRUE;
         if (rescale_sigma && ! (read_ellipse || read_wedge)) error = TRUE;
 
-	if (argc == 1 || GMT_quick || error) {	/* Display usage */
+	if (argc == 1 || GMT_give_synopsis_and_exit || error) {	/* Display usage */
 		fprintf (stderr,"psvelo %s - Plot symbols on maps\n\n", GMT_VERSION);
-		fprintf (stderr,"usage: psvelo <infiles> -J<params> -R<west/east/south/north> [-B<tickinfo>]\n");
-		fprintf (stderr, "	[-G<fill>] [-H] [-K] [-L] [-N] [-O]\n");
-		fprintf (stderr, "	[-P] [-S<symbol><scale><fontsize>] [-U[/dx/dy/][<label>]] [-V] [-W<pen>] [-c<ncopies>]\n");
-		fprintf (stderr, "	[-X<x_shift>] [-Y<y_shift>] [-:]\n\n");
-		
-		if (GMT_quick) exit (EXIT_FAILURE);
-		
+		fprintf (stderr,"usage: psvelo <infiles> %s %s [%s]\n", GMT_J_OPT, GMT_Rgeo_OPT, GMT_B_OPT);
+		fprintf (stderr, "	[-G<fill>] [%s] [-K] [-L] [-N] [-O]\n", GMT_H_OPT);
+		fprintf (stderr, "	[-P] [-S<symbol><scale><fontsize>] [%s] [-V] [-W<pen>] [%s]\n", GMT_U_OPT, GMT_c_OPT);
+		fprintf (stderr, "	[%s] [%s] [%s]\n\n", GMT_X_OPT, GMT_Y_OPT, GMT_t_OPT);
+
+		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
+
 		fprintf (stderr, "	<infiles> is one or more files.  If no, read standard input\n");
 		GMT_explain_option ('j');
 		GMT_explain_option ('R');
@@ -259,7 +256,7 @@ main (int argc, char **argv)
                 fprintf (stderr, "         (x) Strain crosses : in X,Y,Eps1,Eps2,Theta \n");
 		GMT_explain_option ('U');
 		GMT_explain_option ('V');
-		fprintf (stderr, "	-W sets pen attributes [width = %lgp, color = (%d/%d/%d), texture = solid line].\n", 
+		fprintf (stderr, "	-W sets pen attributes [width = %gp, color = (%d/%d/%d), texture = solid line].\n", 
 			pen.width, pen.rgb[0], pen.rgb[1], pen.rgb[2]);
 		GMT_explain_option ('X');
 		GMT_explain_option ('c');
@@ -273,29 +270,23 @@ main (int argc, char **argv)
 	else
 		n_files = 1;
 	n_args = (argc > 1) ? argc : 2;
-	
+
         GMT_put_history (argc, argv);   /* Update .gmtcommands4 */
 
 	greenwich = (project_info.w < 0.0 || project_info.e <= 0.0);
-	
+
 	GMT_map_setup (west, east, south, north);
 
-	ps_plotinit (CNULL, gmtdefs.overlay, gmtdefs.page_orientation, gmtdefs.x_origin, gmtdefs.y_origin,
-		gmtdefs.global_x_scale, gmtdefs.global_y_scale, gmtdefs.n_copies,
-		gmtdefs.dpi, GMT_INCH, gmtdefs.paper_width, gmtdefs.page_rgb,
-		     gmtdefs.encoding.name, GMT_epsinfo (argv[0]));
-	GMT_echo_command (argc, argv);
+	GMT_plotinit (argc, argv);
 
-	if (gmtdefs.unix_time) GMT_timestamp (argc, argv);
-	
         GMT_setpen (&pen);
 	ps_setfont (gmtdefs.annot_font[0]);
         if (shade_uncert) outline = TRUE;
-	
+
 	if (skip_if_outside) GMT_map_clip_on (GMT_no_rgb, 3);
-	
+
 	old_GMT_world_map = GMT_world_map;
-	
+
 	ix = (gmtdefs.xy_toggle[0]);	iy = 1 - ix;
 
 	done = FALSE;
@@ -314,11 +305,11 @@ main (int argc, char **argv)
 
 		if (!nofile && gmtdefs.verbose) {
 			fprintf (stderr, "psvelo: Working on file %s\n", argv[fno]);
-			sprintf (line, "File: %s\0", argv[fno]);
+			sprintf (line, "File: %s", argv[fno]);
 			ps_comment (line);
 		}
-		if (gmtdefs.io_header[GMT_IN]) for (i = 0; i < gmtdefs.n_header_recs; i++) fgets (line, 512, fp);
-		
+		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) fgets (line, 512, fp);
+
 		if (gmtdefs.verbose && (read_ellipse || read_rotated_ellipse))	 
 		  fprintf (stderr, "psvelo: 2-D confidence interval and scaling factor %f %f\n",confidence, conrad);
 
@@ -337,7 +328,7 @@ main (int argc, char **argv)
 
 			xy[ix] = atof (col[0]);
 			xy[iy] = atof (col[1]);
-			
+
                         if(read_ellipse) {
                                     vxy[ix] = atof(col[2]);
                                     vxy[iy] = atof(col[3]);
@@ -378,7 +369,7 @@ main (int argc, char **argv)
                                     eps1 = atof(col[2]);
                                     eps2 = atof(col[3]);
                                     theta = atof(col[4]);
-				  }
+			}
                         else if(read_wedge) {
                                     spin = atof(col[2]);
                                     spinsig = atof(col[3]);
@@ -389,8 +380,8 @@ main (int argc, char **argv)
 			if ( abs (GMT_x_status_new) > 1 || abs (GMT_y_status_new) > 1) continue;
 
 			GMT_geo_to_xy (xy[0], xy[1], &plot_x, &plot_y);
-			
-				
+
+
 			switch (symbol) {
                                 case CINE:
                                         des_arrow = veclen((vxy[0]), (vxy[1])) < 1.e-8 ? FALSE : TRUE; 
@@ -467,7 +458,7 @@ main (int argc, char **argv)
 		}
 	if (fp != stdin) fclose (fp);
 	}
-	
+
 	if (gmtdefs.verbose) 
 			fprintf (stderr, "psvelo: Number of records read: %i\n", n_rec);
 
@@ -476,12 +467,14 @@ main (int argc, char **argv)
 
 
 	if (skip_if_outside) GMT_map_clip_off ();
-	
+
 	if (pen.texture) ps_setdash (CNULL, 0);
-	
+
 	GMT_map_basemap ();
 
-	ps_plotend (gmtdefs.last_page);
-	
+	GMT_plotend ();
+
 	GMT_end (argc, argv);
+
+	exit (EXIT_SUCCESS);
 }
