@@ -1,5 +1,6 @@
-/*	$Id: utilmeca.c,v 1.3 2001/04/02 15:51:23 pwessel Exp $	*/
-#include <gmt.h>
+/*	$Id: utilmeca.c,v 1.5 2005/03/04 21:00:54 remko Exp $	*/
+#include "gmt.h"	/* to have gmt environment */
+#include "pslib.h"	/* to have pslib environment */
 #include "meca.h"
 #include "nrutil.h"
 
@@ -467,7 +468,7 @@ double datan2(double y,double x)
     if(fabs(x) < EPSIL) {
         if(fabs(y) < EPSIL) {
 /*
-            fprintf(stderr, "indetermined form 0. / 0.");
+            fprintf(stderr, "undetermined form 0. / 0.");
             exit();
 */
             arctg = 0.;
@@ -508,7 +509,7 @@ double computed_strike1(struct nodal_plane NP1)
     if(cd1 < EPSIL && fabs(cr) < EPSIL) {
 /*
         fprintf(stderr, "\nThe second plane is horizontal;");
-        fprintf(stderr, "\nStrike is indetermined.");
+        fprintf(stderr, "\nStrike is undetermined.");
         fprintf(stderr, "\nstr2 = NP1.str + 180. is taken to define");
         fprintf(stderr, "\nrake in the second plane.");
 */
@@ -591,7 +592,7 @@ double computed_dip2(double str1,double dip1,double str2)
    the second plane.
    Angles are in degrees.
    Warning : if dip1 == 90 and cos(str1 - str2) == 0
-             the second plane dip is indetermined
+             the second plane dip is undetermined
              and the only first plane will be plotted.
 */
  
@@ -910,7 +911,7 @@ double ps_tensor(double x0,double y0,double size,struct AXIS T,struct AXIS N,str
     double cpd, spd, cpb, spb, cpm, spm;
     double cad, sad, cab, sab, cam, sam;
     double xz, xn, xe;
-    double az, azp = 0., takeoff, r;
+    double az = 0., azp = 0., takeoff, r;
     double azi[3][2];
     double x[400], y[400], x2[400], y2[400], x3[400], y3[400];
     double xp1[800], yp1[800], xp2[400], yp2[400];
@@ -1218,7 +1219,7 @@ from nodal plane strikes, dips and rakes.
 */
 
 {
-        int im;
+        int im = 0;
         int pure_strike_slip = 0;
         double cd1, sd1, cd2, sd2;
         double cp1, sp1, cp2, sp2;
@@ -1310,7 +1311,7 @@ from nodal plane strikes, dips and rakes.
 */
 
 {
-        int im;
+        int im = 0;
         int pure_strike_slip = 0;
         double cd1, sd1, cd2, sd2;
         double cp1, sp1, cp2, sp2;
