@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt2bin.c,v 1.2 2001/12/21 03:50:38 ben Exp $
+ *	$Id: gmt2bin.c,v 1.6 2006/03/25 03:56:52 pwessel Exp $
  *
- *    Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
+ *    Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
  *    See README file for copying and redistribution conditions.
  *--------------------------------------------------------------------*/
 /*
@@ -36,9 +36,8 @@
 #define DEG2MDEG   1000000
 #define MIL_90   90000000
 #define MIL_360 360000000
-struct GMTMGG_REC *record;
 
-main (int argc, char **argv)
+int main (int argc, char **argv)
 {
 	int bindex, error = FALSE;
 	int year, n_records, i, lastb, llat1, llat2, llon1, llon2, newbin, flag;
@@ -46,7 +45,8 @@ main (int argc, char **argv)
 	char legid[16], agency[10], gmtfile[80], binfile[80];
 	int inboxn[MBINS], inboxgmt[MBINS], totalgmt;
 	char line[BUFSIZ], path[80];
-	FILE *fpl, *fpo, *fp;
+	FILE *fpl = NULL, *fpo, *fp;
+	struct GMTMGG_REC *record;
 	
 	path[0] = 0;
 	for (i = 1; i < argc; i++) {
@@ -83,9 +83,9 @@ main (int argc, char **argv)
 		sscanf (line, "%s", legid);
 		if (gmtdefs.verbose) fprintf (stderr, "Processing leg %s\n", legid);
 		if (path[0])
-			sprintf (gmtfile, "%s%c%s.gmt\0", path, DIR_DELIM, legid);
+			sprintf (gmtfile, "%s%c%s.gmt", path, DIR_DELIM, legid);
 		else
-			sprintf (gmtfile, "%s.gmt\0", legid);
+			sprintf (gmtfile, "%s.gmt", legid);
 		if ((fp = fopen (gmtfile, "rb")) == NULL) {
 			fprintf (stderr, "gmt2bin: Cannot open file %s\n", gmtfile);
 			continue;
@@ -200,7 +200,7 @@ main (int argc, char **argv)
 		if (totalgmt == 0)
 			fprintf (stderr, "%s has empty data fields\n", gmtfile);
 		else {
-			sprintf (binfile, "%s.bix\0", legid);
+			sprintf (binfile, "%s.bix", legid);
 			fpo = fopen (binfile, "w");
 			fprintf (fpo, "%d %d\n", year, totalgmt);
 			for (i = 0; i < MBINS; i++) if (inboxn[i]) fprintf (fpo, "%d %d\n", i, inboxgmt[i]);
