@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt_mgg.c,v 1.7 2004/08/19 04:34:25 pwessel Exp $
+ *	$Id: gmt_mgg.c,v 1.10 2005/12/27 04:28:12 pwessel Exp $
  *
- *    Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
+ *    Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
  *    See README file for copying and redistribution conditions.
  *--------------------------------------------------------------------*/
 /*
@@ -363,6 +363,8 @@ int gmtmgg_decode_MGD77 (char *string, int tflag, struct GMTMGG_REC *record, str
 	return (0);
 }
 
+/* CARTER TABLE ROUTINES */
+
 int carter_setup (void)
 {
 	/* This routine must be called once before using carter table stuff.
@@ -457,11 +459,11 @@ int carter_get_bin (int lat, int lon, int *bin)
 	int latdeg, londeg;
 
 	if (lat < -90000000 || lat > 90000000) {
-		fprintf (stderr, "GMT ERROR: in carter_get_bin:  Latitude domain error (%lg)\n", 1e-6 * lat);
+		fprintf (stderr, "GMT ERROR: in carter_get_bin:  Latitude domain error (%g)\n", 1e-6 * lat);
 		return (-1);
 	}
 	if (lon < 0 || lon > 360000000) {
-		fprintf (stderr, "GMT ERROR: in carter_get_bin:  Longitude domain error (%lg)\n", 1e-6 * lon);
+		fprintf (stderr, "GMT ERROR: in carter_get_bin:  Longitude domain error (%g)\n", 1e-6 * lon);
 		return (-1);
 	}
 	latdeg = (lat + 90000000)/1000000;
