@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmtlist.c,v 1.1.1.1.4.2 2002/02/27 17:41:10 pwessel Exp $
+ *	$Id: gmtlist.c,v 1.14 2006/04/10 05:47:30 pwessel Exp $
  *
- *    Copyright (c) 1991-2002 by P. Wessel and W. H. F. Smith
+ *    Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
  *    See README file for copying and redistribution conditions.
  *--------------------------------------------------------------------*/
 /*
@@ -42,19 +42,12 @@
 #define MAXLEGS 5000
 #define S_PR_DAY 86400
 
-struct CORR **bin;
-
-int binsize = sizeof(struct CORR);
-int nlegs = 0;
-
-int get_id (char *name);
-
-main (int argc, char **argv)
+int main (int argc, char **argv)
 {
 	int leg_year, rec, i, j, n_records, argno, id, no[10], nval = 0;
-	int start_time = 0, stop_time = 2000000000, dlon, last_lon;
-	int mon1, day1 = 0, year1, hour1, min1, time, last_lat, mon2, day2 = 0, year2, hour2, min2;
-	int yy, mm, dd, hh, mi, ss, jd, dt, last_time, n_cruises = 0;
+	int start_time = 0, stop_time = 2000000000, dlon, last_lon = 0;
+	int mon1, day1 = 0, year1, hour1, min1, time, last_lat = 0, mon2, day2 = 0, year2, hour2, min2;
+	int yy, mm, dd, hh, mi, ss, jd, dt, last_time = 0, n_cruises = 0;
 	
 	int error = FALSE, wantgmt, want_all = FALSE, geodetic = TRUE;
 	int correct = FALSE, tsec = FALSE, calender = FALSE, do_heading = FALSE;
@@ -66,6 +59,13 @@ main (int argc, char **argv)
 	double ds, dx, dy, west = 0.0, east = 360.0, south = -90.0, north = 90.0;
 	double heading, speed, weight = 1.0;
 	
+	struct CORR **bin = NULL;
+
+	int binsize = sizeof (struct CORR);
+	int nlegs = 0;
+
+	int get_id (char *name, int nlegs, struct CORR **bin);
+
 	struct GMTMGG_TIME *gmt;
 	
 	FILE *fp, *fpc;
@@ -81,6 +81,8 @@ main (int argc, char **argv)
 	
 	/* Check and interpret the command line arguments */
 	
+	argc = GMT_begin (argc, argv);
+
 	for (i =1; !error && i < argc; i++) {
 		if (argv[i][0] == '-') {
 			switch(argv[i][1]) {
@@ -89,7 +91,7 @@ main (int argc, char **argv)
 				case 'R':
 				case 'V':
 				case '\0':
-					error += GMT_get_common_args (argv[i], &west, &east, &south, &north);
+					error += GMT_parse_common_options (argv[i], &west, &east, &south, &north);
 					break;
 					
 				case 'b':	/* Binary output */
@@ -201,7 +203,7 @@ main (int argc, char **argv)
 			n_cruises++;
 	}
 	
-	/* Check that the options selected are mutually consistant */
+	/* Check that the options selected are mutually consistent */
 	
 	if (nval > 10) error = TRUE;
 	if (start_dist > stop_dist || start_time > stop_time) error = TRUE;
@@ -212,9 +214,9 @@ main (int argc, char **argv)
 
 	if (error || argc == 1) {	/* Display usage */
 		fprintf(stderr,"usage: gmtlist <cruise(s)> [-C<corrfile>] [-Da<startdate>] [-Db<stopdate>] [-F<dataflags>]\n");
-		fprintf(stderr,"	[-G] [-H] [-R<west>/<east>/<south>/<north>] [-Sa<startdist>] [-Sb<stopdist>] [-V] [-W<Weight>] [-b]\n\n");
+		fprintf(stderr,"	[-G] [-H] [%s] [-Sa<startdist>] [-Sb<stopdist>] [-V] [-W<Weight>] [-b]\n\n", GMT_Rgeo_OPT);
          
-		if (GMT_quick) exit (EXIT_FAILURE);
+		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
               
 		fprintf(stderr,"	<cruises> is one or more legnames, e.g. c2104 v3206 etc.\n");
 		fprintf(stderr,"	OPTIONS:\n\n");
@@ -280,7 +282,7 @@ main (int argc, char **argv)
 		do_heading = do_speed = TRUE;
 	}
 	 
-	if (!binary && gmtdefs.io_header) {
+	if (!binary && GMT_io.io_header[GMT_OUT]) {
 		/* Write out header record */
 		for (i = 0; i < nval; i++) {
 			switch(no[i]) {
@@ -355,7 +357,7 @@ main (int argc, char **argv)
 	
 		gmt = gmtmgg_init (leg_year);	/* Initialize gmt_structure */
 		if (correct)
-			id = get_id(argv[argno]);
+			id = get_id(argv[argno], nlegs, bin);
 		else
 			id = -1;
 	
@@ -426,6 +428,7 @@ main (int argc, char **argv)
 			/* Check is lat/lon is outside specified area */
 		
 			if (lat < south || lat > north) continue;
+			while (lon > east) lon -= 360.0;
 			while (lon < west) lon += 360.0;
 			if (lon > east) continue;
 			while (lon > 360.0) lon -= 360.0;
@@ -466,13 +469,13 @@ main (int argc, char **argv)
 						if (binary)
 							fwrite ((void *)&lon, sizeof (double), (size_t)1, stdout);
 						else
-							(gmtdefs.io_header) ? printf("%.5lf", lon) : printf("%9.5lf", lon);
+							(GMT_io.io_header[GMT_OUT]) ? printf("%.5f", lon) : printf("%9.5f", lon);
 						break;
 					case 2:	/* Print out latitude */
 						if (binary)
 							fwrite ((void *)&lat, sizeof (double), (size_t)1, stdout);
 						else
-							(gmtdefs.io_header) ? printf("%.5lf", lat) : printf("%8.5lf", lat);
+							(GMT_io.io_header[GMT_OUT]) ? printf("%.5f", lat) : printf("%8.5f", lat);
 						break;
 					case 3:	/* Print out gravity */
 						if (record.gmt[0] == GMTMGG_NODATA)
@@ -482,7 +485,7 @@ main (int argc, char **argv)
 							if (binary)
 								fwrite ((void *)&grv, sizeof (double), (size_t)1, stdout);
 							else
-		 						(gmtdefs.io_header) ? printf("%.2lf", grv) : printf("%9.2lf", grv);
+		 						(GMT_io.io_header[GMT_OUT]) ? printf("%.2f", grv) : printf("%9.2f", grv);
 						}
 		 				break;
 					case 4:	/* Print out magnetics */
@@ -493,7 +496,7 @@ main (int argc, char **argv)
 							if (binary)
 								fwrite ((void *)&mag, sizeof (double), (size_t)1, stdout);
 							else
-		 						(gmtdefs.io_header) ? printf("%.1lf", mag) : printf("%8.1lf", mag);
+		 						(GMT_io.io_header[GMT_OUT]) ? printf("%.1f", mag) : printf("%8.1f", mag);
 						}
 						break;
 					case 5:	/* Print out bathymetry */
@@ -504,14 +507,14 @@ main (int argc, char **argv)
 							if (binary)
 								fwrite ((void *)&top, sizeof (double), (size_t)1, stdout);
 							else
-								(gmtdefs.io_header) ? printf("%.1lf", top) : printf("%8.1lf", top);
+								(GMT_io.io_header[GMT_OUT]) ? printf("%.1f", top) : printf("%8.1f", top);
 						}
 						break;
 					case 6:	/* Print out distance */
 						if (binary)
 							fwrite ((void *)&dist, sizeof (double), (size_t)1, stdout);
 						else
-							(gmtdefs.io_header) ? printf("%.3lf", dist) : printf("%9.3lf", dist);
+							(GMT_io.io_header[GMT_OUT]) ? printf("%.3f", dist) : printf("%9.3f", dist);
 						break;
 					case 7:	/* Print out heading */
 						if (heading == GMTMGG_NODATA)
@@ -520,7 +523,7 @@ main (int argc, char **argv)
 							if (binary)
 								fwrite ((void *)&heading, sizeof (double), (size_t)1, stdout);
 							else
-								(gmtdefs.io_header) ? printf("%.1lf", heading) : printf("%6.1lf", heading);
+								(GMT_io.io_header[GMT_OUT]) ? printf("%.1f", heading) : printf("%6.1f", heading);
 						}
 						break;
 					case 8:	/* Print out velocity */
@@ -530,7 +533,7 @@ main (int argc, char **argv)
 							if (binary)
 								fwrite ((void *)&speed, sizeof (double), (size_t)1, stdout);
 							else
-								(gmtdefs.io_header) ? printf("%.2lf", speed) : printf("%6.2lf", speed);
+								(GMT_io.io_header[GMT_OUT]) ? printf("%.2f", speed) : printf("%6.2f", speed);
 						}
 						break;
 					case 9:	/* Print out weight */
@@ -538,7 +541,7 @@ main (int argc, char **argv)
 							fwrite ((void *)&weight, sizeof (double), (size_t)1, stdout);
 
 						else
-							(gmtdefs.io_header) ? printf("%lg", weight) : printf("%6.2lf", weight);
+							(GMT_io.io_header[GMT_OUT]) ? printf("%g", weight) : printf("%6.2f", weight);
 						break;
 				}
 				if (!binary) ((i+1) < nval) ? printf("\t") : printf("\n");
@@ -551,7 +554,7 @@ main (int argc, char **argv)
 	exit (EXIT_SUCCESS);
 }
 
-int get_id (char *name)
+int get_id (char *name, int nlegs, struct CORR **bin)
 {
 	int left, right, mid, cmp;
 	
