@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmttrack.c,v 1.8 2004/09/10 23:16:15 pwessel Exp $
+ *	$Id: gmttrack.c,v 1.17 2006/04/10 05:47:30 pwessel Exp $
  *
- *    Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
+ *    Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
  *    See README file for copying and redistribution conditions.
  *--------------------------------------------------------------------*/
 /*
@@ -24,21 +24,19 @@
  */
  
 #include "gmt.h"
+#include "pslib.h"
 #include "gmt_mgg.h"
 
 #define DST 0.25
 #define ANSIZE 0.125
 #define MPRDEG 111194.9
 
-double heading (int rec, int n_records);
-struct GMTMGG_REC *record;
-int *distance;
 struct ANOT {
 	int annot_int_dist;
 	int annot_int_time;
 	int tick_int_dist;
 	int tick_int_time;
-} info;
+};
 
 struct LEG_ANOT {	/* Structure used to annotate legs after clipping is terminated */
 	double x, y;
@@ -46,19 +44,13 @@ struct LEG_ANOT {	/* Structure used to annotate legs after clipping is terminate
 	char text[16];
 };
 
-struct GMT_PEN pen;
-
-int get_annotinfo (char *args, struct ANOT *info);
-int gmttrack_outside (int lon, int lat, int w, int e, int s, int n);
-void annot_legname (double x, double y, int rec, int nrecs, char *text, double size, int w, int e, int s, int n);
-
-main (int argc, char **argv)
+int main (int argc, char **argv)
 {
 	double west = 0.0, east = 0.0, south = 0.0, north = 0.0, lat, lon, angle;
-	double last_lon, last_lat, annotsize = ANSIZE, factor = 0.001, dlon;
-	double dx, dy, x, y, x0, y0, dist, start_dist = 0., stop_dist = 1.0E100; 
+	double last_lon = 0.0, last_lat = 0.0, annotsize = ANSIZE, factor = 0.001, dlon;
+	double dx, dy, x, y, x0, y0, dist = 0.0, start_dist = 0., stop_dist = 1.0E100; 
 	
-	int i, iw, ie, is, in, annot_dist, tick_dist, annot_time, tick_time;
+	int i, iw, ie, is, in, annot_dist = 0, tick_dist = 0, annot_time = 0, tick_time = 0;
 	int nfiles = 0, n_records, leg_year, rec, pen_type = 0, k, white[2], black[3];
 	int error = FALSE, first, last = FALSE, annot_name = FALSE, both = FALSE, no_clip = FALSE;
 	int  annot_tick = FALSE, draw_tick = FALSE, start_time = 0, stop_time = 2000000000;
@@ -67,12 +59,21 @@ main (int argc, char **argv)
         
        	char agency[10], gmtfile[BUFSIZ], label[50], comment[BUFSIZ];
 	
-	struct LEG_ANOT *cruise_id;
-       	
+	struct LEG_ANOT *cruise_id = VNULL;
+	struct GMT_PEN pen;
+	struct ANOT info;
+struct GMTMGG_REC *record;
+int *distance;
 	FILE *fp;
 	
 	struct GMTMGG_TIME *gmt;
 	
+double heading (int rec, int n_records, struct GMTMGG_REC *record);
+int get_annotinfo (char *args, struct ANOT *info);
+int gmttrack_outside (int lon, int lat, int w, int e, int s, int n);
+void annot_legname (double x, double y, int rec, int nrecs, char *text, double size, int w, int e, int s, int n, struct GMTMGG_REC *record, struct GMT_PEN *pen);
+
+
 	GMT_init_pen (&pen, GMT_PENWIDTH);
 	for (i = 0; i < 3; i++) black[i] = 0, white[i] = 255;
 	
@@ -95,7 +96,7 @@ main (int argc, char **argv)
 				case 'y':
 				case 'c':
 				case '\0':
-					error += GMT_get_common_args (argv[i], &west, &east, &south, &north);
+					error += GMT_parse_common_options (argv[i], &west, &east, &south, &north);
 					break;
 				case 'W':
 					GMT_getpen (&argv[i][2], &pen);
@@ -140,11 +141,11 @@ main (int argc, char **argv)
 	if (GMT_check_rgb (pen.rgb)) error = TRUE;
 		 
 	if (error) {
-		fprintf(stderr,"usage: gmttrack leg(s) -R<west>/<east>/<south>/<north> -J<params> [-A[size]] [-B<tickinfo>]\n");
+		fprintf(stderr,"usage: gmttrack leg(s) %s %s [-A[size]] [%s]\n", GMT_Rgeo_OPT, GMT_J_OPT, GMT_B_OPT);
 		fprintf(stderr,"	[-Da<startdate>] [-Db<stopdate>] [-K] [-M<trackticks>] [-N] [-O] [-P] [-Sa<startdist>]\n");
-		fprintf(stderr,"	[-Sb<stopdist>] [-U[dx/dy/][label]] [-V] [-W<pen>] [-X<x_shift>] [-Y<y_shift>] [-c<ncopies>\n\n");
+		fprintf(stderr,"	[-Sb<stopdist>] [%s] [-V] [-W<pen>] [%s] [%s] [%s]\n\n", GMT_U_OPT, GMT_X_OPT, GMT_Y_OPT, GMT_c_OPT);
 
-		if (GMT_quick) exit (EXIT_FAILURE);
+		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
 		fprintf (stderr, "	Give one or more names of .gmt files\n");
 		GMT_explain_option ('J');
@@ -164,7 +165,7 @@ main (int argc, char **argv)
 		fprintf(stderr,"        -Sb<dist> only plots up to dist (in km)\n");
 		GMT_explain_option ('U');
 		GMT_explain_option ('V');
-		fprintf (stderr, "	-W sets track pen attributes [width = %lgp, color = (%d/%d/%d), texture = solid line].\n", 
+		fprintf (stderr, "	-W sets track pen attributes [width = %gp, color = (%d/%d/%d), texture = solid line].\n", 
 			pen.width, pen.rgb[0], pen.rgb[1], pen.rgb[2]);
 		GMT_explain_option ('X');
 		GMT_explain_option ('c');
@@ -181,13 +182,7 @@ main (int argc, char **argv)
 		
 	GMT_map_setup (west, east, south, north);
 	
-	ps_plotinit (NULL, gmtdefs.overlay, gmtdefs.page_orientation, gmtdefs.x_origin, gmtdefs.y_origin,
-		gmtdefs.global_x_scale, gmtdefs.global_y_scale, gmtdefs.n_copies,
-		gmtdefs.dpi, GMT_INCH, gmtdefs.paper_width, gmtdefs.page_rgb,
-		     gmtdefs.encoding.name, GMT_epsinfo (argv[0]));
-		
-	GMT_echo_command (argc, argv);
-	if (gmtdefs.unix_time) GMT_timestamp (argc, argv);
+	GMT_plotinit (argc, argv);
 	
 	ps_setpaint (gmtdefs.basemap_frame_rgb);
 	
@@ -273,7 +268,7 @@ main (int argc, char **argv)
 		n_records = k;
 		
 		if (gmtdefs.verbose) fprintf (stderr, "gmttrack: Plotting %s\n", agency);
-		sprintf (comment, "Tracking %s\0", agency);
+		sprintf (comment, "Tracking %s", agency);
 		ps_comment (comment);
 		
 		/* First draw the track line, clip segments outside the area */
@@ -314,7 +309,7 @@ main (int argc, char **argv)
 						}
 					}
 					else
-						annot_legname (x, y, rec, n_records, argv[i], GMT_u2u[GMT_INCH][GMT_PT] * 1.25 * annotsize, iw ,ie, is, in);
+						annot_legname (x, y, rec, n_records, argv[i], GMT_u2u[GMT_INCH][GMT_PT] * 1.25 * annotsize, iw ,ie, is, in, record, &pen);
 				}
 				first = FALSE;
 				if (info.annot_int_dist > 0) annot_dist = (distance[rec] / info.annot_int_dist + 1) * info.annot_int_dist;
@@ -344,7 +339,7 @@ main (int argc, char **argv)
 				draw_tick += 2;
 			}
 			if (annot_tick) {
-				angle = heading (rec, n_records);
+				angle = heading (rec, n_records, record);
 				if (angle < 0.0)
 					angle += 90.0;
 				else
@@ -353,12 +348,12 @@ main (int argc, char **argv)
 					this_julian = gmtmgg_date (annot_time - info.annot_int_time,&year,&month,&day,&hour,&minute,&second,gmt);
 					ps_command ("S");
 					if (this_julian != last_julian) {
-						sprintf (label, " %.2d:%.2d %d/%d\0", hour, minute, month, day);
+						sprintf (label, " %.2d:%.2d %d/%d", hour, minute, month, day);
 						ps_circle (x, y, 0.2*annotsize, black, TRUE);
 						ps_setfont (7);
 					}
 					else {
-						sprintf (label, " %.2d:%.2d\0", hour, minute);
+						sprintf (label, " %.2d:%.2d", hour, minute);
 						ps_circle (x, y, 0.2*annotsize, white, TRUE);
 					}
 					ps_setpaint (gmtdefs.basemap_frame_rgb);
@@ -368,7 +363,7 @@ main (int argc, char **argv)
 				}
 				if (annot_tick & 2) {	/* Distance mark */
 					ps_command ("S");
-					sprintf (label, "%d km  \0", (int)((annot_dist - info.annot_int_dist) * factor));
+					sprintf (label, "%d km  ", (int)((annot_dist - info.annot_int_dist) * factor));
 					ps_square (x, y, 0.4*annotsize, gmtdefs.basemap_frame_rgb, FALSE);
 					ps_setpaint (gmtdefs.basemap_frame_rgb);
 					ps_text (x, y, GMT_u2u[GMT_INCH][GMT_PT] * annotsize, label, angle, 7, 0);
@@ -403,19 +398,21 @@ main (int argc, char **argv)
 		int id;
 		double size;
 		size = GMT_u2u[GMT_INCH][GMT_PT] * 1.25 * annotsize;
-		for (id = 0; id < n_id; id++) annot_legname (cruise_id[id].x, cruise_id[id].y, cruise_id[id].rec, n_records, cruise_id[id].text, size, iw ,ie, is, in);
+		for (id = 0; id < n_id; id++) annot_legname (cruise_id[id].x, cruise_id[id].y, cruise_id[id].rec, n_records, cruise_id[id].text, size, iw ,ie, is, in, record, &pen);
 		GMT_free ((void *)cruise_id);
 	}
 
-	ps_plotend (gmtdefs.last_page);
+	GMT_plotend ();
 	GMT_free ((void *)record);
 	GMT_free ((void *)distance);
 	
 	GMT_end (argc, argv);
+
+	exit (EXIT_SUCCESS);
 }
 
 
-double heading (int rec, int n_records)
+double heading (int rec, int n_records, struct GMTMGG_REC *record)
 {
 	int i1, i2;
 	double angle, lon, lat, x1, x2, y1, y2;
@@ -509,7 +506,7 @@ int gmttrack_outside (int lon, int lat, int w, int e, int s, int n)
 	return (FALSE);
 }
 
-void annot_legname (double x, double y, int rec, int nrecs, char *text, double size, int w, int e, int s, int n)
+void annot_legname (double x, double y, int rec, int nrecs, char *text, double size, int w, int e, int s, int n, struct GMTMGG_REC *record, struct GMT_PEN *pen)
 {
 	int just;
 	double angle;
@@ -519,7 +516,7 @@ void annot_legname (double x, double y, int rec, int nrecs, char *text, double s
 		just = 6;
 	}
 	else {
-		angle = heading (rec, nrecs);
+		angle = heading (rec, nrecs, record);
 		if (record[rec-1].lat < s)
 			just = (angle >= 0.0) ? 1 : 3;
 		else if (record[rec-1].lat > n)
@@ -532,7 +529,7 @@ void annot_legname (double x, double y, int rec, int nrecs, char *text, double s
 	ps_setfont (5);
 	ps_setpaint (gmtdefs.basemap_frame_rgb);
 	ps_text (x, y, size, text, angle, just, 0);
-	ps_setpaint (pen.rgb);
+	ps_setpaint (pen->rgb);
 	ps_plot (x, y, 3);
 	ps_setfont (7);
 }
