@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: mgd77togmt.c,v 1.2 2001/12/21 03:50:38 ben Exp $
+ *	$Id: mgd77togmt.c,v 1.6 2006/03/25 03:56:52 pwessel Exp $
  *
- *    Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
+ *    Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
  *    See README file for copying and redistribution conditions.
  *--------------------------------------------------------------------*/
 /*
@@ -39,18 +39,16 @@
 #include "gmt.h"
 #include "gmt_mgg.h"
 
-struct GMTMGG_TIME *gmt;
-struct GMTMGG_REC *record;
-
-char *LIBDIR;
-
-main (int argc, char **argv)
+int main (int argc, char **argv)
 {
 	int i, rec, n_records, n_read, n_files = 0, n_alloc = GMT_CHUNK, leg_year = 0, len, *year;
 	int T_INC = 60, fake_running_time = 0;
 	BOOLEAN use_list = FALSE, set_agency = FALSE;
 	int t_flag = FALSE, error = FALSE;
 	char file[80], *mfile, *list, agency[10], *legfname, line[BUFSIZ], **mgd77, **prefix;
+	char *LIBDIR;
+	struct GMTMGG_TIME *gmt;
+	struct GMTMGG_REC *record;
 	FILE *fp = NULL, *fpo;
 	
 	if ((LIBDIR = getenv ("GMTHOME")) == (char *)NULL) {
@@ -211,7 +209,7 @@ main (int argc, char **argv)
 		GMT_free ((void *)gmt);
 		n_records = rec;
 	
-		sprintf (file, "%s.gmt\0", prefix[i]);
+		sprintf (file, "%s.gmt", prefix[i]);
 		if ((fpo = fopen (file, "wb")) == NULL) {
 			fprintf (stderr, "mgd77togmt: Could not create file %s\n", file);
 			exit (EXIT_FAILURE);
