@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: makepattern.c,v 1.1.1.1 2000/12/28 01:23:45 gmt Exp $
+ *	$Id: makepattern.c,v 1.11 2006/04/10 05:47:30 pwessel Exp $
  *
- *      Copyright (c) 1999-2001 by P. Wessel
+ *      Copyright (c) 1999-2006 by P. Wessel
  *      See COPYING file for copying and redistribution conditions.
  *
  *      This program is free software; you can redistribute it and/or modify
@@ -31,19 +31,12 @@
 
 */
 
-#include "../gmt.h"
+#include "gmt.h"
+#include "pslib.h"
 
 /* Structure for Sun rasterfile */
 
-struct rasterfile h;
-
-#define	RAS_MAGIC	0x59a66a95
-#define RT_STANDARD	1
-#define RMT_EQUAL_RGB	1	/* red[ras_maplength/3],green[],blue[] */
-
-unsigned short int *get_icon (char *file, int *nx, int *ny, int invert);
-
-main (int argc, char **argv)
+int main (int argc, char **argv)
 {
 	BOOLEAN pad = FALSE, icon = FALSE;
 
@@ -55,14 +48,18 @@ main (int argc, char **argv)
 #endif
 
 	unsigned int p;
-	unsigned short int *pattern;
+	unsigned short int *pattern = VNULL;
 
-	char *fname;
+	char *fname = CNULL;
 
 	struct GMT_FILL fg, bg;
 
+	struct imageinfo h;
+
+	unsigned short int *get_icon (char *file, int *nx, int *ny, int invert);
+
 	FILE *fp = NULL, *fpo;
-	
+
 	GMT_init_fill (&fg, 0, 0, 0);		/* Default foreground is black */
 	GMT_init_fill (&bg, 255, 255, 255);	/* Default background is white */
 
@@ -71,16 +68,16 @@ main (int argc, char **argv)
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '-') {
 			switch (argv[i][1]) {
-			
-		
+
+
 				/* Common parameters */
-			
+
 				case '\0':
-					error += GMT_get_common_args (argv[i], 0, 0, 0, 0);
+					error += GMT_parse_common_options (argv[i], 0, 0, 0, 0);
 					break;
-				
+
 				/* Supplemental parameters */
-				
+
 				case 'C':
 					switch (argv[i][2]) {
 						case 'f':
@@ -97,7 +94,7 @@ main (int argc, char **argv)
 							break;
 						default:
 							error++;
-					}	
+					}
 					break;
 #ifdef _WIN32
 				case 'G':
@@ -118,7 +115,7 @@ main (int argc, char **argv)
 			}
 		}
 	}
-	
+
 #ifdef _WIN32
 	if (!ofile) {
 		fprintf(stderr,"%s: No output file given\n", GMT_program);
@@ -131,7 +128,7 @@ main (int argc, char **argv)
 #else
 	fpo = GMT_stdout;
 #endif
-	if (argc == 1 || GMT_quick) {
+	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "makepattern %s - make color pattern from b/w pattern\n\n", GMT_VERSION);
 #ifdef _WIN32
 		fprintf(stderr,"usage:	makepattern 1bit.ras OR iconfile -Cf<rgb> -Cb<rgb> -Gimage.ras\n\n");
@@ -139,7 +136,7 @@ main (int argc, char **argv)
 		fprintf(stderr,"usage:	makepattern 1bit.ras OR iconfile -Cf<rgb> -Cb<rgb> > image.ras\n\n");
 #endif
 
-		if (GMT_quick) exit (EXIT_FAILURE);
+		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
 			fprintf(stderr,"\tGive the name of a 1-bit standard Sun rasterfile or iconfile\n");
 		fprintf (stderr, "\n\tOPTIONS:\n");
@@ -150,7 +147,7 @@ main (int argc, char **argv)
 #endif
 		exit(EXIT_FAILURE);
 	}
-	
+
 	if (!fp) {
 		fprintf(stderr,"%s: Specify either a 1-bit rasterfile or an iconfile\n", GMT_program);
 		exit (EXIT_FAILURE);
@@ -160,25 +157,25 @@ main (int argc, char **argv)
 
 	if (fp) {	/* Got a 1-bit Sun rasterfile */
 
-		if (fread ((void *)&h, sizeof (struct rasterfile), (size_t)1, fp) != 1) {
+		if (ps_read_rasheader (fp, &h, 0, 7)) {
 			fprintf (stderr, "%s: Trouble reading Sun rasterfile header!\n", GMT_program);
 			exit (EXIT_FAILURE);
 		}
-		if (h.ras_magic != RAS_MAGIC) {	/* Try to read as icon instead */
+		if (h.magic != RAS_MAGIC) {	/* Try to read as icon instead */
 			fclose (fp);
 			pattern = get_icon (fname, &nx, &ny, FALSE);
 			icon = TRUE;
 		}
 		else {
 
-			if (h.ras_depth != 1) {
+			if (h.depth != 1) {
 				fprintf (stderr, "%s: File is not a 1-bit Sun rasterfile!\n", GMT_program);
 				exit (EXIT_FAILURE);
 			}
 
-			nx = h.ras_width;
-			ny = h.ras_height;
-			mx = h.ras_length/2;
+			nx = h.width;
+			ny = h.height;
+			mx = h.length/2;
 			pattern = (unsigned short int *) GMT_memory (VNULL, (size_t)mx, sizeof (unsigned short int), GMT_program);
 			if (fread ((void *)pattern, sizeof (unsigned short int), (size_t)mx, fp) != (size_t)mx) {
 				fprintf (stderr, "%s: Trouble reading 1-bit Sun rasterfile image!\n", GMT_program);
@@ -233,26 +230,26 @@ main (int argc, char **argv)
 	mx = nx;
 	if (mx%2) mx++;	/* Must be even number of bytes */
 
-	h.ras_magic = RAS_MAGIC;
-	h.ras_width = nx;
-	h.ras_height = ny;
-	h.ras_depth = 8;
-	h.ras_length = mx * ny;
-	h.ras_type  = RT_STANDARD;
-	h.ras_maptype = RMT_EQUAL_RGB;
-	h.ras_maplength = 6;
+	h.magic = RAS_MAGIC;
+	h.width = nx;
+	h.height = ny;
+	h.depth = 8;
+	h.length = mx * ny;
+	h.type  = RT_STANDARD;
+	h.maptype = RMT_EQUAL_RGB;
+	h.maplength = 6;
 
-	if (fwrite ((void *)&h, sizeof (struct rasterfile), (size_t)1, fpo) != 1) {
+	if (ps_write_rasheader (fp, &h, 0, 7)) {
 		fprintf (stderr, "%s: Trouble writing Sun rasterfile header!\n", GMT_program);
 		exit (EXIT_FAILURE);
 	}
 
-	if (fwrite ((void *)map, sizeof (unsigned char), (size_t)h.ras_maplength, fpo) != (size_t)h.ras_maplength) {
+	if (fwrite ((void *)map, sizeof (unsigned char), (size_t)h.maplength, fpo) != (size_t)h.maplength) {
 		fprintf (stderr, "%s: Trouble writing Sun rasterfile colormap!\n", GMT_program);
 		exit (EXIT_FAILURE);
 	}
 
-	if (fwrite ((void *)rgb, sizeof (unsigned char), (size_t)h.ras_length, fpo) != (size_t)h.ras_length) {
+	if (fwrite ((void *)rgb, sizeof (unsigned char), (size_t)h.length, fpo) != (size_t)h.length) {
 		fprintf (stderr, "%s: Trouble writing Sun rasterfile image!\n", GMT_program);
 		exit (EXIT_FAILURE);
 	}
@@ -268,21 +265,24 @@ main (int argc, char **argv)
 		GMT_free ((void *)pattern);
 
 	GMT_end (argc, argv);
+
+	exit (EXIT_SUCCESS);
 }
 
 
 unsigned short int *get_icon (char *file, int *nx, int *ny, int invert)
 {
-	int i, n_items, n_int, j, k, n_read, integer, n_alloc, n_lines, last;
+	int i, n_items, n_int, j, k, n_read, n_alloc, n_lines, last;
+	unsigned int integer;
 	unsigned short int *pattern;
 	char t[8][7], line[80], width[12], height[12];
 	FILE *fp;
-	
+
 	if ((fp = fopen (file, "r")) == NULL) {
 		fprintf (stderr, "%s: Cannot open file %s\n", GMT_program, file);
 		return ((unsigned short int *)NULL);
 	}
-	
+
 	fgets (line, 80, fp);
 	line[strlen(line)-1] = 0;
 	sscanf (&line[3], "%*[^,], %[^,], %[^,]", width, height);
