@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: project.c,v 1.24 2004/04/25 09:10:45 pwessel Exp $
+ *	$Id: project.c,v 1.41 2006/04/13 08:34:41 pwessel Exp $
  *
- *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -32,66 +32,114 @@
 		03-NOV-1998	PW: Can read any number of data columns; z in -Fz refers to
 				all these columns in the output.
    Version:	3.4		PW: Fixed problem with small circle distances
-   Version:	4		
+   Version:	4
 */
 
 #include "gmt.h"
 
-struct DATA {
+#define PROJECT_N_FARGS	7
+
+struct PROJECT_CTRL {	/* All control options for this program (except common args) */
+	/* active is TRUE if the option has been activated */
+	struct A {	/* -A<azimuth> */
+		BOOLEAN active;
+		double azimuth;
+	} A;
+	struct C {	/* -C<ox>/<oy> */
+		BOOLEAN active;
+		double x, y;
+	} C;
+	struct D {	/* -D<d_or_g> */
+		BOOLEAN active;
+		char set;
+	} D;
+	struct E {	/* -E<bx>/<by> */
+		BOOLEAN active;
+		double x, y;
+	} E;
+	struct F {	/* -F<flags> */
+		BOOLEAN active;
+		char col[PROJECT_N_FARGS];	/* Character codes for desired output in the right order */
+	} F;
+	struct G {	/* -G<inc> */
+		BOOLEAN active;
+		double inc;
+	} G;
+	struct L {	/* -L[w][<l_min>/<l_max>] */
+		BOOLEAN active;
+		BOOLEAN constrain;
+		double min, max;
+	} L;
+	struct N {	/* -N */
+		BOOLEAN active;
+	} N;
+	struct Q {	/* -Q */
+		BOOLEAN active;
+	} Q;
+	struct S {	/* -S */
+		BOOLEAN active;
+	} S;
+	struct T {	/* -T<px>/<py> */
+		BOOLEAN active;
+		double x, y;
+	} T;
+	struct W {	/* -W<w_min>/<w_max> */
+		BOOLEAN active;
+		double min, max;
+	} W;
+};
+
+struct PROJECT_DATA {
         double  a[6];
 	double *z;
 	char *t;
 };
 
-int	compare_distances(const void *point_1, const void *point_2);
-int	do_flat_earth();
-int	solve_right_spherical_triangle();
-int	sphere_azim_dist();
-double	oblique_setup(double plat, double plon, double *p, double clat, double clon, double *c, int c_given, int rads);
-void	oblique_transform(double xlat, double xlon, double *x_t_lat, double *x_t_lon, double *p, double *c, int rads);
-void	make_euler_matrix(double *p, double *e, double *theta, int rads);
-void	matrix_3v(double *a, double *x, double *b);
-void	matrix_2v(double *a, double *x, double *b);
-void	sphere_project_setup(double alat, double alon, double *a, double blat, double blon, double *b, double *azim, double *p, double *c, int two_pts, int rads);
-void	flat_project_setup(double alat, double alon, double blat, double blon, double plat, double plon, double *azim, double *e, int two_pts, BOOLEAN pole_set);
-void	copy_text_from_col3 (char *line, char *z_cols);
+int main (int argc, char **argv)
+{
+	int	i, j, k, n_outputs, n_used, n_total_read, n_alloc = GMT_CHUNK;
+	int	n_files = 0, fno, n_args, n_fields, n_expected_fields;
+	int	n_total_used = 0, n_z = 0, n_items, output_choice[PROJECT_N_FARGS];
 
-main (int argc, char **argv)
-{	
-	int	i, j, k, n_definitions, n_outputs, n_used, n_total_read, n_alloc = GMT_CHUNK;
-	int	nc, ne, np, nl, nw, n_files = 0, fno, n_args, n_fields, n_expected_fields, n_total_used = 0, n_z = 0, n_items;
-	int     output_choice[7];
+	double	xx, yy, cos_theta, sin_theta, sin_lat_to_pole = 1.0;
+	double	theta, d_along, *in, *out = (double *)NULL;
 
-	double	xx, yy, x_a, y_a, x_b, y_b, x_p, y_p, cos_theta, sin_theta, sin_lat_to_pole = 1.0;
-	double	theta, d_inc = 0.0, d_along, *in, *out = (double *)NULL;
-	
-	double	azimuth = 0.0, l_min = 0.0, l_max = 0.0, w_min = 0.0, w_max = 0.0;
 	double	a[3], b[3], x[3], xt[3], pole[3], center[3], e[9];
 
-	BOOLEAN	check_length, check_width, convert_units, dateline, error, find_new_point, flat_earth, first = TRUE;
-	BOOLEAN generate, greenwich, origin_set, pole_set, rads, sort_output, stay_within, two_points;
-	BOOLEAN nofile = TRUE, done = FALSE, want_z_output = FALSE, pure_ascii, skip;
-	
+	BOOLEAN	dateline = FALSE, error = FALSE, find_new_point = FALSE, first = TRUE, pure_ascii, skip;
+	BOOLEAN greenwich = FALSE, rads = TRUE, nofile = TRUE, done = FALSE, want_z_output = FALSE;
+
 	FILE *fp = NULL;
-	
-	char	modifier, record_str[BUFSIZ], heading[7][64], txt_a[32], txt_b[32];
-	
-	struct DATA *p_data;
-	
+
+	char	modifier, record_str[BUFSIZ], heading[PROJECT_N_FARGS][GMT_TEXT_LEN];
+	char	txt_a[GMT_LONG_TEXT], txt_b[GMT_LONG_TEXT];
+
+	struct PROJECT_DATA *p_data;
+	struct PROJECT_CTRL *Ctrl;
+
+	int	compare_distances (const void *point_1, const void *point_2);
+	int	do_flat_earth ();
+	int	solve_right_spherical_triangle ();
+	int	sphere_azim_dist ();
+	double	oblique_setup (double plat, double plon, double *p, double clat, double clon, double *c, int c_given, int rads);
+	void	oblique_transform (double xlat, double xlon, double *x_t_lat, double *x_t_lon, double *p, double *c, int rads);
+	void	make_euler_matrix (double *p, double *e, double *theta, int rads);
+	void	matrix_3v (double *a, double *x, double *b);
+	void	matrix_2v (double *a, double *x, double *b);
+	void	sphere_project_setup (double alat, double alon, double *a, double blat, double blon, double *b, double *azim, double *p, double *c, int two_pts, int rads);
+	void	flat_project_setup (double alat, double alon, double blat, double blon, double plat, double plon, double *azim, double *e, int two_pts, BOOLEAN pole_set);
+	void	copy_text_from_col3 (char *line, char *z_cols);
+	void	*New_Project_Ctrl (), Free_Project_Ctrl (struct PROJECT_CTRL *C);
+
 	argc = GMT_begin (argc, argv);
 
-	check_length = check_width = dateline = error = find_new_point = flat_earth = generate = greenwich = FALSE;
-	pole_set = sort_output = stay_within = two_points = convert_units = FALSE;
-	rads = TRUE;
-	x_a = x_b = x_p = y_a = y_b = y_p = 0.0;
-	n_definitions = 0;
-	n_outputs = 0;
-	j = 1;
-	for (i = 0; i < 7; i++) output_choice[i] = 0;
-	
+	Ctrl = (struct PROJECT_CTRL *) New_Project_Ctrl ();		/* Allocate and initialize defaults in a new control structure */
+
+	memset ((void *)output_choice, 0, PROJECT_N_FARGS * sizeof (int));
+
 	/* New in GMT 4: Must process -N before the rest to ensure col_types are set properly (there is no -H here)  */
-	for (i = 1; i < argc; i++) if (!strcmp (argv[i], "-N")) flat_earth = TRUE;
-	if (!flat_earth) {
+	for (i = 1; i < argc; i++) if (!strcmp (argv[i], "-N")) Ctrl->N.active = TRUE;
+	if (!Ctrl->N.active) {
 		GMT_io.in_col_type[0] = GMT_IS_LON;
 		GMT_io.in_col_type[1] = GMT_IS_LAT;
 	}
@@ -99,123 +147,112 @@ main (int argc, char **argv)
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '-') {
 			switch (argv[i][1]) {
-			
-		
+
+
 				/* Common parameters */
-			
+
 				case 'H':
 				case 'V':
 				case ':':
 				case 'b':
 				case 'f':
 				case '\0':
-					error += GMT_get_common_args (argv[i], 0, 0, 0, 0);
+					error += GMT_parse_common_options (argv[i], 0, 0, 0, 0);
 					break;
-				
+
 				/* Supplemental parameters */
-				
+
 				case 'F':
+					Ctrl->F.active = TRUE;
 					for (j = 2, k = 0; argv[i][j]; j++, k++) {
-						switch (argv[i][j]) {
-							case 'z':	/* Special flag, can mean any number of z columns */
-								output_choice[k] = -1;
-								want_z_output = TRUE;
-								break;
-							case 'x':
-								output_choice[k] = 0;
-								break;
-							case 'y':
-								output_choice[k] = 1;
-								break;
-							case 'p':
-								output_choice[k] = 2;
-								break;
-							case 'q':
-								output_choice[k] = 3;
-								break;
-							case 'r':
-								output_choice[k] = 4;
-								find_new_point = TRUE;
-								break;
-							case 's':
-								output_choice[k] = 5;
-								find_new_point = TRUE;
-								break;
-							default:
-								fprintf (stderr, "%s: GMT SYNTAX ERROR -F option:  Unrecognized choice %c\n", GMT_program, argv[i][j]);
-								error = TRUE;
+						if (k < PROJECT_N_FARGS)
+							Ctrl->F.col[k] = argv[i][j];
+						else {
+							error++;
+							fprintf (stderr, "%s: GMT SYNTAX ERROR -F option: Too many output columns selected\n", GMT_program);
+							fprintf (stderr, "%s: GMT SYNTAX ERROR -F option: Choose from -Fxyzpqrs\n", GMT_program);
 						}
-						n_outputs++;
 					}
 					break;
 				case 'A':
-					azimuth = atof(&argv[i][2]);
-					n_definitions++;
+					Ctrl->A.active = TRUE;
+					Ctrl->A.azimuth = atof(&argv[i][2]);
 					break;
 				case 'C':
-					nc = sscanf(&argv[i][2], "%[^/]/%s", txt_a, txt_b);
-					error += GMT_verify_expectations (GMT_io.in_col_type[0], GMT_scanf_arg (txt_a, GMT_io.in_col_type[0], &x_a), txt_a);
-					error += GMT_verify_expectations (GMT_io.in_col_type[1], GMT_scanf_arg (txt_b, GMT_io.in_col_type[1], &y_a), txt_b);
-					if (error) fprintf (stderr, "%s: GMT SYNTAX ERROR -C option:  Undecipherable argument %s\n", GMT_program, &argv[i][2]);
-					origin_set = TRUE;
-					break;
-				case 'D':
-					modifier = argv[i][2];
-					if (modifier == 'D' || modifier == 'd') {
-						dateline = TRUE;
-					}
-					else if (modifier == 'g' || modifier == 'G') {
-						greenwich = TRUE;
-					}
-					else if (modifier) {
-						fprintf (stderr, "%s: GMT SYNTAX ERROR -D option:  Unrecognized modifier %c\n", GMT_program, modifier);
-						error = TRUE;
-					}
-					break;
-				case 'E':
-					ne = sscanf(&argv[i][2], "%[^/]/%s", txt_a, txt_b);
-					error += GMT_verify_expectations (GMT_io.in_col_type[0], GMT_scanf_arg (txt_a, GMT_io.in_col_type[0], &x_b), txt_a);
-					error += GMT_verify_expectations (GMT_io.in_col_type[1], GMT_scanf_arg (txt_b, GMT_io.in_col_type[1], &y_b), txt_b);
-					if (error) fprintf (stderr, "%s: GMT SYNTAX ERROR -E option:  Undecipherable argument %s\n", GMT_program, &argv[i][2]);
-					two_points = TRUE;
-					n_definitions++;
-					break;
-				case 'G':
-					generate = TRUE;
-					d_inc = atof(&argv[i][2]);
-					break;
-				case 'L':
-					check_length = TRUE;
-					modifier = argv[i][2]; 
-					if (modifier == 'W' || modifier == 'w') {
-						stay_within = TRUE;
+					Ctrl->C.active = TRUE;
+					if (sscanf(&argv[i][2], "%[^/]/%s", txt_a, txt_b) != 2) {
+						fprintf (stderr, "%s: GMT SYNTAX ERROR -C option.  Correct syntax: -C<lon0>/<lat0>\n", GMT_program);
+						error++;
 					}
 					else {
-						nl = sscanf(&argv[i][2], "%lf/%lf", &l_min, &l_max);
+						error += GMT_verify_expectations (GMT_io.in_col_type[0], GMT_scanf_arg (txt_a, GMT_io.in_col_type[0], &Ctrl->C.x), txt_a);
+						error += GMT_verify_expectations (GMT_io.in_col_type[1], GMT_scanf_arg (txt_b, GMT_io.in_col_type[1], &Ctrl->C.y), txt_b);
+						if (error) fprintf (stderr, "%s: GMT SYNTAX ERROR -C option:  Undecipherable argument %s\n", GMT_program, &argv[i][2]);
+					}
+					break;
+				case 'D':
+					Ctrl->D.active = TRUE;
+					Ctrl->D.set = argv[i][2];
+					break;
+				case 'E':
+					Ctrl->E.active = TRUE;
+					if (sscanf(&argv[i][2], "%[^/]/%s", txt_a, txt_b) != 2) {
+						fprintf (stderr, "%s: GMT SYNTAX ERROR -E option.  Correct syntax: -E<lon1>/<lat1>\n", GMT_program);
+						error++;
+					}
+					else {
+						error += GMT_verify_expectations (GMT_io.in_col_type[0], GMT_scanf_arg (txt_a, GMT_io.in_col_type[0], &Ctrl->E.x), txt_a);
+						error += GMT_verify_expectations (GMT_io.in_col_type[1], GMT_scanf_arg (txt_b, GMT_io.in_col_type[1], &Ctrl->E.y), txt_b);
+						if (error) fprintf (stderr, "%s: GMT SYNTAX ERROR -E option:  Undecipherable argument %s\n", GMT_program, &argv[i][2]);
+					}
+					break;
+				case 'G':
+					Ctrl->G.active = TRUE;
+					Ctrl->G.inc = atof(&argv[i][2]);
+					break;
+				case 'L':
+					Ctrl->L.active = TRUE;
+					modifier = argv[i][2]; 
+					if (modifier == 'W' || modifier == 'w') {
+						Ctrl->L.constrain = TRUE;
+					}
+					else {
+						if (sscanf(&argv[i][2], "%lf/%lf", &Ctrl->L.min, &Ctrl->L.max) != 2) {
+							fprintf (stderr, "%s: GMT SYNTAX ERROR -L option.  Correct syntax: -L[w | <min>/<max>]\n", GMT_program);
+							error++;
+						}
 					}
 					break;
 				case 'M':
 					GMT_multisegment (&argv[i][2]);
 					break;
 				case 'N': /* Handled above but still in argv */
+					Ctrl->N.active = TRUE;
 					break;
 				case 'Q':
-					convert_units = TRUE;
+					Ctrl->Q.active = TRUE;
 					break;
 				case 'S':
-					sort_output = TRUE;
+					Ctrl->S.active = TRUE;
 					break;
 				case 'T':
-					np = sscanf(&argv[i][2], "%[^/]/%s", txt_a, txt_b);
-					error += GMT_verify_expectations (GMT_io.in_col_type[0], GMT_scanf_arg (txt_a, GMT_io.in_col_type[0], &x_p), txt_a);
-					error += GMT_verify_expectations (GMT_io.in_col_type[1], GMT_scanf_arg (txt_b, GMT_io.in_col_type[1], &y_p), txt_b);
-					if (error) fprintf (stderr, "%s: GMT SYNTAX ERROR -T option:  Undecipherable argument %s\n", GMT_program, &argv[i][2]);
-					pole_set = TRUE;
-					n_definitions++;
+					Ctrl->T.active = TRUE;
+					if (sscanf(&argv[i][2], "%[^/]/%s", txt_a, txt_b) != 2) {
+						fprintf (stderr, "%s: GMT SYNTAX ERROR -T option.  Correct syntax: -T<lonp>/<latp>\n", GMT_program);
+						error++;
+					}
+					else {
+						error += GMT_verify_expectations (GMT_io.in_col_type[0], GMT_scanf_arg (txt_a, GMT_io.in_col_type[0], &Ctrl->T.x), txt_a);
+						error += GMT_verify_expectations (GMT_io.in_col_type[1], GMT_scanf_arg (txt_b, GMT_io.in_col_type[1], &Ctrl->T.y), txt_b);
+						if (error) fprintf (stderr, "%s: GMT SYNTAX ERROR -T option:  Undecipherable argument %s\n", GMT_program, &argv[i][2]);
+					}
 					break;
 				case 'W':
-					nw = sscanf(&argv[i][2], "%lf/%lf", &w_min, &w_max);
-					check_width = TRUE;
+					Ctrl->W.active = TRUE;
+					if (sscanf(&argv[i][2], "%lf/%lf", &Ctrl->W.min, &Ctrl->W.max) != 2) {
+						fprintf (stderr, "%s: GMT SYNTAX ERROR -W option.  Correct syntax: -W<min>/<max>\n", GMT_program);
+						error++;
+					}
 					break;
 				default:
 					error = TRUE;
@@ -226,15 +263,15 @@ main (int argc, char **argv)
 		else
 			n_files++;
 	}
-	
-	if (argc == 1 || GMT_quick) {
+
+	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "project %s - project data onto line or great circle, generate track, or translate coordiantes\n\n", GMT_VERSION);
 		fprintf(stderr,"usage:	project [files] -C<ox>/<oy> [-A<azimuth>] [-D<d_or_g>] [-E<bx>/<by>]\n");
-		fprintf(stderr,"\t[-F<flags>] [-G<dist>] [-H[<nrec>]] [-L[w][<l_min>/<l_max>]]\n");
-		fprintf(stderr,"\t[-M[<flag>]] [-N] [-Q] [-S] [-T<px>/<py>] [-V] [-W<w_min>/<w_max>]\n");
-		fprintf(stderr,"\t[-:] [-bi[s][<n>]] [-bo[s][<n>]] [-f[i|o]<colinfo>]\n\n");
-		
-		if (GMT_quick) exit (EXIT_FAILURE);
+		fprintf(stderr,"\t[-F<flags>] [-G<dist>] [%s] [-L[w][<l_min>/<l_max>]]\n", GMT_H_OPT);
+		fprintf(stderr,"\t[%s] [-N] [-Q] [-S] [-T<px>/<py>] [-V] [-W<w_min>/<w_max>]\n", GMT_M_OPT);
+		fprintf(stderr,"\t[%s] [%s] [%s]\n\n", GMT_t_OPT, GMT_b_OPT, GMT_f_OPT);
+
+		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
 		fprintf(stderr,"\tproject will read stdin or file, and does not want input if -G option.\n");
 		fprintf(stderr,"\tThe projection may be defined in (only) one of three ways:\n");
@@ -291,60 +328,78 @@ main (int argc, char **argv)
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
-	
-	if ( !(origin_set && nc == 2) ) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR -C option.  Correct syntax: -C<lon0>/<lat0>\n", GMT_program);
+
+	if (Ctrl->L.active && !Ctrl->L.constrain && Ctrl->L.min >= Ctrl->L.max) {
+		fprintf (stderr, "%s: GMT SYNTAX ERROR -L option.  w_min must be < w_max\n", GMT_program);
 		error++;
 	}
-	if (two_points && ne != 2) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR -E option.  Correct syntax: -E<lon1>/<lat1>\n", GMT_program);
+	if (Ctrl->W.active && Ctrl->W.min >= Ctrl->W.max) {
+		fprintf (stderr, "%s: GMT SYNTAX ERROR -W option.  w_min must be < w_max\n", GMT_program);
 		error++;
 	}
-	if (pole_set && np != 2) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR -T option.  Correct syntax: -T<lonp>/<latp>\n", GMT_program);
-		error++;
-	}
-	if (check_length && !stay_within && nl != 2) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR -L option.  Correct syntax: -L[w | <min>/<max>]\n", GMT_program);
-		error++;
-	}
-	if (check_width && (nw != 2 || w_min >= w_max)) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR -L option.  Correct syntax: -L[w | <min>/<max>]\n", GMT_program);
-		error++;
-	}
-	if (azimuth < 0.0 || azimuth >= 360.0) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR -A option.  Must specify azimuth in 0-360 degree range\n", GMT_program);
-		error++;
-	}
-	if ( n_definitions != 1) {
+	if ((Ctrl->A.active + Ctrl->E.active + Ctrl->T.active) > 1) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR: Specify only one of -A, -E, and -T\n", GMT_program);
 		error++;
 	}
-	if ( two_points && (x_a == x_b) && (y_a == y_b) ) {
+	if (Ctrl->E.active && (Ctrl->C.x == Ctrl->E.x) && (Ctrl->C.y == Ctrl->E.y)) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -E option: Second point must differ from origin!\n", GMT_program);
 		error++;
 	}
-	if ( generate && l_min == l_max && !(two_points)) {	/* We don't know how long to generate  */
+	if (Ctrl->G.active && Ctrl->L.min == Ctrl->L.max && !Ctrl->E.active) {	/* We don't know how long to generate  */
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -G option: Must also specify -Lmin/max or use -E instead\n", GMT_program);
 		error++;
 	}
-	if ( generate && n_outputs > 0) {	/* -F not allowed with -G  */
+	if (Ctrl->G.active && Ctrl->F.active) {	/* -F not allowed with -G  */
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -G option: -F not allowed [Defaults to rsp]\n", GMT_program);
 		error++;
 	}
-	if ( generate && d_inc <= 0.0) {	/* No increment given  */
+	if (Ctrl->G.active && Ctrl->G.inc <= 0.0) {	/* No increment given  */
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -G option: Must specify a positive increment\n", GMT_program);
 		error++;
 	}
-	if (stay_within && !(two_points) ) {	/* Same problem.  */
+	if (Ctrl->L.constrain && !Ctrl->E.active) {	/* Same problem.  */
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -L option: Must specify -Lmin/max or use -E instead\n", GMT_program);
 		error++;
 	}
-	if (n_outputs > 7) {
+	/* Convert user's -F choices to internal parameters */
+	for (k = n_outputs = 0; k < PROJECT_N_FARGS && Ctrl->F.col[k]; k++) {
+		switch (Ctrl->F.col[k]) {
+			case 'z':	/* Special flag, can mean any number of z columns */
+				output_choice[k] = -1;
+				want_z_output = TRUE;
+				break;
+			case 'x':
+				output_choice[k] = 0;
+				break;
+			case 'y':
+				output_choice[k] = 1;
+				break;
+			case 'p':
+				output_choice[k] = 2;
+				break;
+			case 'q':
+				output_choice[k] = 3;
+				break;
+			case 'r':
+				output_choice[k] = 4;
+				find_new_point = TRUE;
+				break;
+			case 's':
+				output_choice[k] = 5;
+				find_new_point = TRUE;
+				break;
+			default:
+				fprintf (stderr, "%s: GMT SYNTAX ERROR -F option:  Unrecognized choice %c\n", GMT_program, Ctrl->F.col[k]);
+				error = TRUE;
+		}
+		n_outputs++;
+	}
+
+	if (n_outputs > PROJECT_N_FARGS) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -F option: Too many output columns selected (%d)\n", GMT_program, n_outputs);
 		error++;
 	}
-	if (GMT_io.binary[GMT_IN] && gmtdefs.io_header[GMT_IN]) {
+	if (GMT_io.binary[GMT_IN] && GMT_io.io_header[GMT_IN]) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data cannot have header -H\n", GMT_program);
 		error++;
 	}
@@ -353,6 +408,17 @@ main (int argc, char **argv)
                 fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data (-bi) must have at least 2 columns\n", GMT_program);
 		error++;
 	}
+	if (Ctrl->D.active && Ctrl->D.set) {
+		if (Ctrl->D.set == 'D' || Ctrl->D.set == 'd')
+			dateline = TRUE;
+		else if (Ctrl->D.set == 'g' || Ctrl->D.set == 'G')
+			greenwich = TRUE;
+		else {
+			fprintf (stderr, "%s: GMT SYNTAX ERROR -D option:  Unrecognized modifier %c\n", GMT_program, Ctrl->D.set);
+			error = TRUE;
+		}
+	}
+	
 	if (error) exit (EXIT_FAILURE);
 
 	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
@@ -368,55 +434,55 @@ main (int argc, char **argv)
 
 	pure_ascii = !(GMT_io.binary[GMT_IN] || GMT_io.binary[GMT_OUT]);
 
-	if (n_outputs == 0 && !(generate) ) {	/* Generate default -F setting (all) */
-		n_outputs = 7;
+	if (n_outputs == 0 && !(Ctrl->G.active) ) {	/* Generate default -F setting (all) */
+		n_outputs = PROJECT_N_FARGS;
 		for (i = 0; i < 2; i++) output_choice[i] = i;
 		output_choice[2] = -1;
 		for (i = 3; i < n_outputs; i++) output_choice[i] = i-1;
 		find_new_point = TRUE;
 	}
 
-	p_data = (struct DATA *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (struct DATA), GMT_program);
-	
-	if (generate && two_points && (l_min == l_max) ) stay_within = TRUE;	/* Default generate from A to B  */
+	p_data = (struct PROJECT_DATA *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (struct PROJECT_DATA), GMT_program);
 
-	/* Set up rotation matrix e for flat earth, or pole and center for spherical; get l_min, l_max if stay_within  */
+	if (Ctrl->G.active && Ctrl->E.active && (Ctrl->L.min == Ctrl->L.max) ) Ctrl->L.constrain = TRUE;	/* Default generate from A to B  */
 
-	if (flat_earth) {
-		flat_project_setup(y_a, x_a, y_b, x_b, y_p, x_p, &azimuth, e, two_points, pole_set);
+	/* Set up rotation matrix e for flat earth, or pole and center for spherical; get Ctrl->L.min, Ctrl->L.max if stay_within  */
+
+	if (Ctrl->N.active) {
+		flat_project_setup(Ctrl->C.y, Ctrl->C.x, Ctrl->E.y, Ctrl->E.x, Ctrl->T.y, Ctrl->T.x, &Ctrl->A.azimuth, e, Ctrl->E.active, Ctrl->T.active);
 		/* Azimuth is now changed to cartesian theta in radians */
-		if (stay_within) {
-			l_min = 0.0;
-			xx = x_b - x_a;
-			yy = y_b - y_a;
-			l_max = d_sqrt(xx*xx + yy*yy);
-			if (convert_units) l_max *= DEG2KM;
+		if (Ctrl->L.constrain) {
+			Ctrl->L.min = 0.0;
+			xx = Ctrl->E.x - Ctrl->C.x;
+			yy = Ctrl->E.y - Ctrl->C.y;
+			Ctrl->L.max = d_sqrt(xx*xx + yy*yy);
+			if (Ctrl->Q.active) Ctrl->L.max *= project_info.KM_PR_DEG;
 		}
 	}
 	else {
-		if (pole_set) {
-			sin_lat_to_pole = oblique_setup(y_p, x_p, pole, y_a, x_a, center, pole_set, rads);
+		if (Ctrl->T.active) {
+			sin_lat_to_pole = oblique_setup(Ctrl->T.y, Ctrl->T.x, pole, Ctrl->C.y, Ctrl->C.x, center, Ctrl->T.active, rads);
 		}
 		else {
-			sphere_project_setup(y_a, x_a, a, y_b, x_b, b, &azimuth, pole, center, two_points, rads);
+			sphere_project_setup(Ctrl->C.y, Ctrl->C.x, a, Ctrl->E.y, Ctrl->E.x, b, &Ctrl->A.azimuth, pole, center, Ctrl->E.active, rads);
 		}
 		/* Azimuth is now changed to radians  */
-		if (stay_within) {
-			l_min = 0.0;
-			l_max = GMT_dot3v(a,b);
-			l_max = d_acos(l_max) * R2D;
-			if (convert_units) l_max *= DEG2KM;
+		if (Ctrl->L.constrain) {
+			Ctrl->L.min = 0.0;
+			Ctrl->L.max = GMT_dot3v(a,b);
+			Ctrl->L.max = d_acos(Ctrl->L.max) * R2D;
+			if (Ctrl->Q.active) Ctrl->L.max *= project_info.KM_PR_DEG;
 		}
 	}
 
 	/* Now things are initialized.  We will work in degrees for awhile, so we convert things:  */
 
-	if (convert_units) {
-		d_inc /= DEG2KM;
-		l_min /= DEG2KM;
-		l_max /= DEG2KM;
-		w_min /= DEG2KM;
-		w_max /= DEG2KM;
+	if (Ctrl->Q.active) {
+		Ctrl->G.inc /= project_info.KM_PR_DEG;
+		Ctrl->L.min /= project_info.KM_PR_DEG;
+		Ctrl->L.max /= project_info.KM_PR_DEG;
+		Ctrl->W.min /= project_info.KM_PR_DEG;
+		Ctrl->W.max /= project_info.KM_PR_DEG;
 	}
 
 	/*  Now we are ready to work  */
@@ -424,40 +490,40 @@ main (int argc, char **argv)
 	n_used = 0;
 	n_total_read = 0;
 
-	if (generate) {	/* Not input data expected, just generate track from arguments given */
+	if (Ctrl->G.active) {	/* Not input data expected, just generate track from arguments given */
 		n_outputs = 3;
 		output_choice[0] = 4;
 		output_choice[1] = 5;
 		output_choice[2] = 2;
 		out = (double *) GMT_memory (VNULL, (size_t)n_outputs, sizeof (double), GMT_program);
 
-		d_along = l_min;
-		while (d_along < l_max) {
+		d_along = Ctrl->L.min;
+		while ((Ctrl->L.max - d_along) > (GMT_CONV_LIMIT*Ctrl->G.inc)) {
 			p_data[n_used].a[2] = d_along;
 			n_used++;
-			d_along = l_min + n_used * d_inc;
+			d_along = Ctrl->L.min + n_used * Ctrl->G.inc;
 			if (n_used == (n_alloc-1)) {
 				n_alloc += GMT_CHUNK;
-				p_data = (struct DATA *) GMT_memory ((void *)p_data, (size_t)n_alloc, sizeof (struct DATA), GMT_program);
+				p_data = (struct PROJECT_DATA *) GMT_memory ((void *)p_data, (size_t)n_alloc, sizeof (struct PROJECT_DATA), GMT_program);
 			}
 		}
-		p_data[n_used].a[2] = l_max;
+		p_data[n_used].a[2] = Ctrl->L.max;
 		n_used ++;
 
 		/* We need to find r,s  */
-	
-		if (flat_earth) {
-			sincos (azimuth, &sin_theta, &cos_theta);
+
+		if (Ctrl->N.active) {
+			sincos (Ctrl->A.azimuth, &sin_theta, &cos_theta);
 			for (i = 0; i < n_used; i++) {
-				p_data[i].a[4] = x_a + p_data[i].a[2] * cos_theta;
-				p_data[i].a[5] = y_a + p_data[i].a[2] * sin_theta;
+				p_data[i].a[4] = Ctrl->C.x + p_data[i].a[2] * cos_theta;
+				p_data[i].a[5] = Ctrl->C.y + p_data[i].a[2] * sin_theta;
 				while (greenwich && p_data[i].a[4] < 0.0) p_data[i].a[4] += 360.0;
 				while (dateline && p_data[i].a[4] > 180.0) p_data[i].a[4] -= 360.0;
 			}
 		}
 		else {
-			xx = x_a;
-			yy = y_a;
+			xx = Ctrl->C.x;
+			yy = Ctrl->C.y;
 			GMT_geo_to_cart(&yy, &xx, x, rads);
 			for (i = 0; i < n_used; i++) {
 				theta = p_data[i].a[2] / sin_lat_to_pole;
@@ -470,28 +536,28 @@ main (int argc, char **argv)
 				while (dateline && p_data[i].a[4] > 180.0) p_data[i].a[4] -= 360.0;
 			}
 		}
-	
+
 		/* At this stage, all values are still in degrees.  */
 
-		if (convert_units) {
+		if (Ctrl->Q.active) {
 			for (i = 0; i < n_used; i++) {
-				p_data[i].a[2] *= DEG2KM;
-				p_data[i].a[3] *= DEG2KM;
+				p_data[i].a[2] *= project_info.KM_PR_DEG;
+				p_data[i].a[3] *= project_info.KM_PR_DEG;
 			}
 		}
 
 		/* Now output generated track */
 
 		if (!GMT_io.binary[GMT_OUT]) {
-			if (gmtdefs.io_header[GMT_OUT]) fprintf (GMT_stdout, "lon\tlat\tdist\n");
-	
+			if (GMT_io.io_header[GMT_OUT]) fprintf (GMT_stdout, "lon\tlat\tdist\n");
+
 			for (i = 0; i < n_used; i++) {
 				for (j = 0; j < n_outputs; j++) out[j] = p_data[i].a[output_choice[j]];
 				GMT_output (GMT_stdout, n_outputs, out);
 			}
 		}
 	}
-	
+
 	else {	/* Must read input file */
 
 		n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : BUFSIZ;
@@ -505,7 +571,7 @@ main (int argc, char **argv)
 
 		for (fno = 1; !done && fno < n_args; fno++) {	/* Loop over input files, if any */
 			if (!nofile && argv[fno][0] == '-') continue;
-		
+
 			if (nofile) {	/* Just read standard input */
 				fp = GMT_stdin;
 				done = TRUE;
@@ -519,8 +585,8 @@ main (int argc, char **argv)
 			}
 
 			if (!nofile && gmtdefs.verbose) fprintf (stderr, "%s: Working on file %s\n", GMT_program, argv[fno]);
-		
-			if (gmtdefs.io_header[GMT_IN]) {
+
+			if (GMT_io.io_header[GMT_IN]) {
 				GMT_fgets (record_str, BUFSIZ, fp);
 				sscanf (record_str, "%s %s %s", heading[0], heading[1], heading[6]);
 				if (! (heading[6]) ) strcpy (heading[6],"Z");
@@ -528,9 +594,9 @@ main (int argc, char **argv)
 				strcpy (heading[3],"q");
 				strcpy (heading[4],"r");
 				strcpy (heading[5],"s");
-				for (i = 1; i < gmtdefs.n_header_recs; i++) GMT_fgets (record_str, BUFSIZ, fp);
+				for (i = 1; i < GMT_io.n_header_recs; i++) GMT_fgets (record_str, BUFSIZ, fp);
 			}
-	
+
 			n_fields = GMT_input (fp, &n_expected_fields, &in);
 			n_z = n_expected_fields - 2;
 			if (n_z == 0 && want_z_output) {
@@ -540,7 +606,7 @@ main (int argc, char **argv)
 			n_used = 0;
 
 			while (! (GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
-			
+
 				while (GMT_io.status & GMT_IO_SEGMENT_HEADER) {
 					GMT_write_segmentheader (GMT_stdout, n_expected_fields);
 					n_fields = GMT_input (fp, &n_expected_fields, &in);
@@ -554,19 +620,19 @@ main (int argc, char **argv)
 
 					xx = in[0];
 					yy = in[1];
-	
+
 					n_total_read ++;
-			
-					if (flat_earth) {
-						x[0] = xx - x_a;
-						x[1] = yy - y_a;
+
+					if (Ctrl->N.active) {
+						x[0] = xx - Ctrl->C.x;
+						x[1] = yy - Ctrl->C.y;
 						matrix_2v (e,x,xt);
 					}
 					else {
 						oblique_transform(yy, xx, &xt[1], &xt[0], pole, center, rads);
 					}
 
-					skip = ((check_length && (xt[0] < l_min || xt[0] > l_max)) || (check_width && (xt[1] < w_min || xt[1] > w_max)));
+					skip = ((Ctrl->L.active && (xt[0] < Ctrl->L.min || xt[0] > Ctrl->L.max)) || (Ctrl->W.active && (xt[1] < Ctrl->W.min || xt[1] > Ctrl->W.max)));
 
 					if (skip) {
 						n_fields = GMT_input (fp, &n_expected_fields, &in);
@@ -590,30 +656,30 @@ main (int argc, char **argv)
 					n_used++;
 					if (n_used == n_alloc) {
 						n_alloc += GMT_CHUNK;
-						p_data = (struct DATA *) GMT_memory ((void *)p_data, (size_t)n_alloc, sizeof (struct DATA), GMT_program);
+						p_data = (struct PROJECT_DATA *) GMT_memory ((void *)p_data, (size_t)n_alloc, sizeof (struct PROJECT_DATA), GMT_program);
 					}
 
 					n_fields = GMT_input (fp, &n_expected_fields, &in);
 				}
 
-				if (sort_output) qsort ((void *)p_data, (size_t)n_used, sizeof (struct DATA), compare_distances);
+				if (Ctrl->S.active) qsort ((void *)p_data, (size_t)n_used, sizeof (struct PROJECT_DATA), compare_distances);
 
 /*				Get here when all data are loaded with p,q and p is in increasing order if desired.  */
 
 				if (find_new_point) {	/* We need to find r,s  */
-	
-					if (flat_earth) {
-						sincos (azimuth, &sin_theta, &cos_theta);
+
+					if (Ctrl->N.active) {
+						sincos (Ctrl->A.azimuth, &sin_theta, &cos_theta);
 						for (i = 0; i < n_used; i++) {
-							p_data[i].a[4] = x_a + p_data[i].a[2] * cos_theta;
-							p_data[i].a[5] = y_a + p_data[i].a[2] * sin_theta;
+							p_data[i].a[4] = Ctrl->C.x + p_data[i].a[2] * cos_theta;
+							p_data[i].a[5] = Ctrl->C.y + p_data[i].a[2] * sin_theta;
 							while (greenwich && p_data[i].a[4] < 0.0) p_data[i].a[4] += 360.0;
 							while (dateline && p_data[i].a[4] > 180.0) p_data[i].a[4] -= 360.0;
 						}
 					}
 					else {
-						xx = x_a;
-						yy = y_a;
+						xx = Ctrl->C.x;
+						yy = Ctrl->C.y;
 						GMT_geo_to_cart(&yy, &xx, x, rads);
 						for (i = 0; i < n_used; i++) {
 							theta = p_data[i].a[2];
@@ -627,20 +693,20 @@ main (int argc, char **argv)
 						}
 					}
 				}
-	
+
 				/* At this stage, all values are still in degrees.  */
 
-				if (convert_units) {
+				if (Ctrl->Q.active) {
 					for (i = 0; i < n_used; i++) {
-						p_data[i].a[2] *= DEG2KM;
-						p_data[i].a[3] *= DEG2KM;
+						p_data[i].a[2] *= project_info.KM_PR_DEG;
+						p_data[i].a[3] *= project_info.KM_PR_DEG;
 					}
 				}
 
 				/* Now output  */
 
 				if (!GMT_io.binary[GMT_OUT]) {	/* First do header */
-					if (first && gmtdefs.io_header[GMT_OUT]) {
+					if (first && GMT_io.io_header[GMT_OUT]) {
 						for (j = 0; j < n_outputs; j++) {
 							if (output_choice[j] == -1)
 								fprintf (GMT_stdout, "%s", heading[6]);
@@ -651,7 +717,7 @@ main (int argc, char **argv)
 						first = FALSE;
 					}
 				}
-	
+
 				n_items = n_outputs + ((want_z_output && n_z) ? n_z - 1 : 0);
 				if (!out) out = (double *) GMT_memory (VNULL, (size_t)n_items, sizeof (double), GMT_program);
 
@@ -695,19 +761,23 @@ main (int argc, char **argv)
 	}
 
 	if (gmtdefs.verbose) fprintf(stderr, "%s: %d read, %d used\n", GMT_program, n_total_read, n_total_used);
-	
+
 	GMT_free ((void *)p_data);
 	GMT_free ((void *)out);
-	
+
+	Free_Project_Ctrl (Ctrl);	/* Deallocate control structure */
+
 	GMT_end (argc, argv);
+
+	exit (EXIT_SUCCESS);
 }
 
-int	compare_distances(const void *point_1, const void *point_2)
+int compare_distances (const void *point_1, const void *point_2)
 {
 	double	d_1, d_2;
 
-	d_1 = ((struct DATA *)point_1)->a[2];
-	d_2 = ((struct DATA *)point_2)->a[2];
+	d_1 = ((struct PROJECT_DATA *)point_1)->a[2];
+	d_2 = ((struct PROJECT_DATA *)point_2)->a[2];
 
 	if (d_1 < d_2)
 		return (-1);
@@ -717,7 +787,7 @@ int	compare_distances(const void *point_1, const void *point_2)
 		return (0);
 }
 
-double	oblique_setup(double plat, double plon, double *p, double clat, double clon, double *c, int c_given, int rads)
+double oblique_setup (double plat, double plon, double *p, double clat, double clon, double *c, int c_given, int rads)
 {
 	/* routine sets up a unit 3-vector p, the pole of an 
 	   oblique projection, given plat, plon, the position 
@@ -732,7 +802,7 @@ double	oblique_setup(double plat, double plon, double *p, double clat, double cl
 	   meridian, pointing in the positive normal (east) sense.
 	   rads = TRUE if we need to convert plat, plon, clat, clon
 	   from degrees to radians.  */
-	
+
 	double	s[3];  /* s points to the south pole  */
 	double cp, sin_lat_to_pole;
 
@@ -740,7 +810,7 @@ double	oblique_setup(double plat, double plon, double *p, double clat, double cl
 	s[2] = -1.0;
 
 	GMT_geo_to_cart(&plat, &plon, p, rads);
-	
+
 	if (c_given) {	/* s points to user's clat, clon  */
 		GMT_geo_to_cart(&clat, &clon, s, rads);
 	}
@@ -751,7 +821,7 @@ double	oblique_setup(double plat, double plon, double *p, double clat, double cl
 	return (sin_lat_to_pole);
 }
 
-void	oblique_transform(double xlat, double xlon, double *x_t_lat, double *x_t_lon, double *p, double *c, int rads)
+void oblique_transform (double xlat, double xlon, double *x_t_lat, double *x_t_lon, double *p, double *c, int rads)
 {
 	/* routine takes the point x at conventional (xlat, xlon) and
 	   computes the transformed coordinates (x_t_lat, x_t_lon) in
@@ -762,7 +832,7 @@ void	oblique_transform(double xlat, double xlon, double *x_t_lat, double *x_t_lo
 	   are in degrees.  */
 
 	double	x[3], p_cross_x[3], temp1, temp2;
-	
+
 	GMT_geo_to_cart(&xlat, &xlon, x, rads);
 
 	temp1 = GMT_dot3v(x,p);
@@ -770,7 +840,7 @@ void	oblique_transform(double xlat, double xlon, double *x_t_lat, double *x_t_lo
 
 	GMT_cross3v(p,x,p_cross_x);
 	GMT_normalize3v(p_cross_x);
-	
+
 	temp1 = GMT_dot3v(p_cross_x, c);
 	temp2 = GMT_dot3v(x, c);
 	*x_t_lon = copysign( d_acos(temp1), temp2);
@@ -781,33 +851,33 @@ void	oblique_transform(double xlat, double xlon, double *x_t_lat, double *x_t_lo
 	}
 }
 
-void	make_euler_matrix(double *p, double *e, double *theta, int rads)
+void make_euler_matrix (double *p, double *e, double *theta, int rads)
 {
 	/* Routine to fill an euler matrix e with the elements
 	   needed to rotate a 3-vector about the pole p through
 	   an angle theta.  p is a unit 3-vector.  If rads = TRUE,
 	   we have to convert theta from degrees into radians before
 	   we use it.  */
-	
+
 	double	cos_theta, sin_theta, one_minus_cos_theta;
 	double	pxsin, pysin, pzsin, temp;
-	
+
 	if (rads) {
 		*theta *= D2R;
 	}
 	cos_theta = cos(*theta);
 	sin_theta = sin(*theta);
 	one_minus_cos_theta = 1.0 - cos_theta;
-	
+
 	pxsin = p[0] * sin_theta;
 	pysin = p[1] * sin_theta;
 	pzsin = p[2] * sin_theta;
-	
+
 	temp = p[0] * one_minus_cos_theta;
 	e[0] = temp * p[0] + cos_theta;
 	e[1] = temp * p[1] - pzsin;
 	e[2] = temp * p[2] + pysin;
-	
+
 	temp = p[1] * one_minus_cos_theta;
 	e[3] = temp * p[0] + pzsin;
 	e[4] = temp * p[1] + cos_theta;
@@ -819,30 +889,30 @@ void	make_euler_matrix(double *p, double *e, double *theta, int rads)
 	e[8] = temp * p[2] + cos_theta;
 }
 
-void	matrix_3v(double *a, double *x, double *b)
+void matrix_3v (double *a, double *x, double *b)
 {
 	/* routine to find b, where Ax = b, A is a 3 by 3 square matrix,
 	   and x and b are 3-vectors.  A is stored row wise, that is:
 	   
 	   A = { a11, a12, a13, a21, a22, a23, a31, a32, a33 }  */
-	
+
 	b[0] = x[0]*a[0] + x[1]*a[1] + x[2]*a[2];
 	b[1] = x[0]*a[3] + x[1]*a[4] + x[2]*a[5];
 	b[2] = x[0]*a[6] + x[1]*a[7] + x[2]*a[8];
 }
 
-void	matrix_2v(double *a, double *x, double *b)
+void matrix_2v (double *a, double *x, double *b)
 {
 	/* routine to find b, where Ax = b, A is a 2 by 2 square matrix,
 	   and x and b are 2-vectors.  A is stored row wise, that is:
 	   
 	   A = { a11, a12, a21, a22 }  */
-	
+
 	b[0] = x[0]*a[0] + x[1]*a[1];
 	b[1] = x[0]*a[2] + x[1]*a[3];
 }
 
-void sphere_project_setup(double alat, double alon, double *a, double blat, double blon, double *b, double *azim, double *p, double *c, int two_pts, int rads)
+void sphere_project_setup (double alat, double alon, double *a, double blat, double blon, double *b, double *azim, double *p, double *c, int two_pts, int rads)
 {
 	/* routine to initialize a pole vector, p, and a central meridian 
 	   normal vector, c, for use in projecting points onto a great circle.
@@ -883,7 +953,6 @@ void sphere_project_setup(double alat, double alon, double *a, double blat, doub
 		b[2] = 1.0;
 		GMT_cross3v(a, b, c);	/* use c for p_temp  */
 		GMT_normalize3v(c);
-/*		make_euler_matrix(a, e, azim, rads);	*/
 		neg_azim = -(*azim);
 		make_euler_matrix(a, e, &neg_azim, rads);
 		if (rads) *azim *= D2R;
@@ -891,17 +960,17 @@ void sphere_project_setup(double alat, double alon, double *a, double blat, doub
 	}
 
 	/* Now set c vector  */
-	
+
 	GMT_cross3v(p, a, c);
 	GMT_normalize3v(c);
 }
 
-void	flat_project_setup(double alat, double alon, double blat, double blon, double plat, double plon, double *azim, double *e, int two_pts, BOOLEAN pole_set)
+void flat_project_setup (double alat, double alon, double blat, double blon, double plat, double plon, double *azim, double *e, int two_pts, BOOLEAN pole_set)
 {
 	/* Sets up stuff for rotation of cartesian 2-vectors, analogous
 	   to the spherical three vector stuff above.  Also change azim
-	   to the cartesian theta, counterclockwise from the x axis.  */		
-	
+	   to the cartesian theta, counterclockwise from the x axis.  */
+
 	if (two_pts) {
 		*azim = d_atan2((blat - alat), (blon - alon));
 	}
@@ -909,12 +978,13 @@ void	flat_project_setup(double alat, double alon, double blat, double blon, doub
 		*azim = d_atan2((plat - alat), (plon - alon)) - 0.5 * M_PI;
 	}
 	else {
-		*azim = D2R * (90.0 - *azim);
+		/* *azim = D2R * (90.0 - *azim); */
+		*azim = -(*azim) * D2R;
 	}
-	
+
 	e[0] = e[3] = cos(*azim);
-	e[2] = sin(*azim);
-	e[1] = -e[2];
+	e[1] = sin(*azim);
+	e[2] = -e[1];
 }
 
 void copy_text_from_col3 (char *line, char *z_cols)
@@ -928,3 +998,17 @@ void copy_text_from_col3 (char *line, char *z_cols)
 
 	sscanf (line, "%*s %*s %[^\n]", z_cols);
 }
+
+void *New_Project_Ctrl () {	/* Allocate and initialize a new control structure */
+	struct PROJECT_CTRL *C;
+	
+	C = (struct PROJECT_CTRL *) GMT_memory (VNULL, 1, sizeof (struct PROJECT_CTRL), "New_Project_Ctrl");
+	
+	/* Initialize values whose defaults are not 0/FALSE/NULL */
+	
+	return ((void *)C);
+}
+
+void Free_Project_Ctrl (struct PROJECT_CTRL *C) {	/* Deallocate control structure */
+	GMT_free ((void *)C);	
+}
