@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: psrose.c,v 1.30 2004/08/17 01:48:06 pwessel Exp $
+ *	$Id: psrose.c,v 1.43 2006/04/12 21:55:46 pwessel Exp $
  *
- *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -32,67 +32,100 @@
  */
  
 #include "gmt.h"
+#include "pslib.h"
 
-int n_alloc = GMT_CHUNK;
+struct PSROSE_CTRL {	/* All control options for this program (except common args) */
+	/* active is TRUE if the option has been activated */
+	struct A {	/* -A<sector_angle>[r] */
+		BOOLEAN active;
+		BOOLEAN rose;
+		double inc;
+	} A;
+	struct C {	/* -C[<modefile>] */
+		BOOLEAN active;
+		char *file;
+	} C;
+	struct E {	/* -Eazimuth/elevation */
+		BOOLEAN active;
+		double azimuth, elevation;
+	} E;
+	struct G {	/* -G<fill> */
+		BOOLEAN active;
+		struct GMT_FILL fill;
+	} G;
+	struct I {	/* -I */
+		BOOLEAN active;
+	} I;
+	struct M {	/* -M<params> */
+		BOOLEAN active;
+		double width, length, thickness;
+		int rgb[3];
+	} M;
+	struct N {	/* -N */
+		BOOLEAN active;
+	} N;
+	struct S {	/* -Sscale[n] */
+		BOOLEAN active;
+		BOOLEAN normalize;
+		double scale;
+	} S;
+	struct T {	/* -T */
+		BOOLEAN active;
+	} T;
+	struct W {	/* -W<pen> */
+		BOOLEAN active;
+		struct GMT_PEN pen;
+	} W;
+	struct Z {	/* -Zscale */
+		BOOLEAN active;
+		double scale;
+	} Z;
+};
 
-double *xx, *yy, *sum;
-double *azimuth, *length;
-double *mode_direction;
-double *mode_length;
-
-void plot3d(double x, double y, int pen);
-void pie3d(double x, double y, double rad, double a1, double a2, int rgb[], BOOLEAN outline);
-void arc3d(double x, double y, double r, double a1, double a2, int status);
-
-main (int argc, char **argv)
+int main (int argc, char **argv)
 {
-	int i, n_bins, bin, n_annot, n = 0, a_rgb[3];
+	BOOLEAN error = FALSE, find_mean = FALSE, half_only = FALSE, windrose = TRUE;
+	BOOLEAN automatic = FALSE, sector_plot = FALSE;
+
+	char text[BUFSIZ], lw[10], le[10], ln[10], *file = CNULL, format[BUFSIZ], *choice[2] = {"OFF", "ON"};
+	char txt_a[GMT_LONG_TEXT], txt_b[GMT_LONG_TEXT], txt_c[GMT_LONG_TEXT], txt_d[GMT_LONG_TEXT];
+
+	int i, n_bins, bin, n_annot, n = 0, n_alloc = GMT_CHUNK;
 	int status, n_alpha, n_modes, n_fields, n_expected_fields;
-	
-	BOOLEAN error = FALSE, inquire = FALSE, outline = FALSE, find_mean = FALSE, normalize = FALSE;
-	BOOLEAN half_only = FALSE, rose = FALSE, windrose = TRUE, sector_plot = FALSE;
-	BOOLEAN eq_area = FALSE, draw_modes = FALSE, automatic = FALSE, oriented = FALSE;
-	
-	double scale = 3.0, max = 0.0, sector = 0.0, radius, az, x_origin, y_origin, tmp;
+
+	double max = 0.0, radius, az, x_origin, y_origin, tmp, one_or_two = 1.0, s, c;
 	double angle1, angle2, angle, x, y, mean_theta, mean_radius, xr = 0.0, yr = 0.0;
 	double x1, x2, y1, y2, total = 0.0, total_arc, off, max_radius, az_offset, *in;
-	double tailwidth = 0.03, headlength = 0.12, headwidth = 0.1, one_or_two = 1.0, s, c;
-	double west = 0.0, east = 0.0, south = 0.0, north = 0.0, asize, lsize, zscale = 1.0;
-	
-	char text[BUFSIZ], lw[10], le[10], ln[10], *file = CNULL, format[BUFSIZ];
-	char *choice[2] = {"OFF", "ON"}, txt_a[32], txt_b[32], txt_c[32], txt_d[64];
-	
+	double west = 0.0, east = 0.0, south = 0.0, north = 0.0, asize, lsize;
+	double *xx, *yy, *sum, *azimuth, *length, *mode_direction, *mode_length;
+
 	FILE *fp = NULL, *fpm = NULL;
-	
-	struct GMT_PEN pen;
-	struct GMT_FILL fill;
-	
+
+	struct PSROSE_CTRL *Ctrl;
+
+	void plot3d (double x, double y, int pen);
+	void pie3d (double x, double y, double rad, double a1, double a2, int rgb[], BOOLEAN outline);
+	void arc3d (double x, double y, double r, double a1, double a2, int status);
+	void *New_Psrose_Ctrl (), Free_Psrose_Ctrl (struct PSROSE_CTRL *C);
+
 	argc = GMT_begin (argc, argv);
-	
-	if (gmtdefs.measure_unit == GMT_CM) {
-		scale = 7.5 / 2.54;
-		tailwidth = 0.075 / 2.54;
-		headlength = 0.3 / 2.54;
-		headwidth = 0.25 / 2.54;
-	}
+
+	Ctrl = (struct PSROSE_CTRL *) New_Psrose_Ctrl ();	/* Allocate and initialize a new control structure */
 
-	a_rgb[0] = a_rgb[1] = a_rgb[2] = 0;
-	GMT_init_fill (&fill, -1, -1, -1);
-	GMT_init_pen (&pen, GMT_PENWIDTH);
 	asize = gmtdefs.annot_font_size[0] * GMT_u2u[GMT_PT][GMT_INCH];
 	lsize = gmtdefs.annot_font_size[0] * GMT_u2u[GMT_PT][GMT_INCH];
 	strcpy (lw, "WEST");	strcpy (le, "EAST");	strcpy (ln, "NORTH");
 	mode_direction = (double *) GMT_memory (VNULL, (size_t)1, sizeof (double), "psrose");
 	mode_length = (double *) GMT_memory (VNULL, (size_t)1, sizeof (double), "psrose");
-	
+
 	/* Check and interpret the command line arguments */
-	
+
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '-') {
 			switch(argv[i][1]) {
-		
+
 				/* Common parameters */
-			
+
 				case 'B':
 				case 'H':
 				case 'K':
@@ -109,70 +142,76 @@ main (int argc, char **argv)
 				case 'c':
 				case ':':
 				case '\0':
-					error += GMT_get_common_args (argv[i], &west, &east, &south, &north);
+					error += GMT_parse_common_options (argv[i], &west, &east, &south, &north);
 					west = 0.0;
 					break;
-					
+
 				/* Supplemental options */
-				
+
 				case 'A':		/* Get Sector angle in degrees */
-					sector = atof (&argv[i][2]);
-					if (argv[i][strlen (argv[i])-1] == 'r') rose = TRUE;
+					Ctrl->A.active = TRUE;
+					Ctrl->A.inc = atof (&argv[i][2]);
+					if (argv[i][strlen (argv[i])-1] == 'r') Ctrl->A.rose = TRUE;
 					break;
 				case 'C':		/* Read mode file and plot directions */
-					draw_modes = TRUE;
-					if ((fpm = fopen (&argv[i][2], "r")) == NULL) find_mean = TRUE;
+					Ctrl->C.active = TRUE;
+					if (argv[i][2]) Ctrl->C.file = strdup (&argv[i][2]);
 					break;
 				case 'E':
-					sscanf (&argv[i][2], "%lf/%lf", &z_project.view_azimuth, &z_project.view_elevation);
+					Ctrl->E.active = TRUE;
+					sscanf (&argv[i][2], "%lf/%lf", &Ctrl->E.azimuth, &Ctrl->E.elevation);
 					break;
 				case 'G':		/* Set Gray shade */
-					if (GMT_getfill (&argv[i][2], &fill)) {
+					Ctrl->G.active = TRUE;
+					if (GMT_getfill (&argv[i][2], &Ctrl->G.fill)) {
 						GMT_fill_syntax ('G');
 						error++;
 					}
 					break;
 				case 'I':		/* Compute statistics only - no plot */
-					inquire = TRUE;
+					Ctrl->I.active = TRUE;
 					break;
 				case 'M':		/* Get arrow parameters */
+					Ctrl->M.active = TRUE;
 					if (argv[i][2]) {
 						n = sscanf (&argv[i][2], "%[^/]/%[^/]/%[^/]/%s", txt_a, txt_b, txt_c, txt_d);
-						if (n != 4 || GMT_getrgb (txt_d, a_rgb)) {
+						if (n != 4 || GMT_getrgb (txt_d, Ctrl->M.rgb)) {
 							fprintf (stderr, "%s: GMT SYNTAX ERROR -M option.  Correct syntax:\n", GMT_program);
 							fprintf (stderr, "\t-M<tailwidth/headlength/headwidth/<color>>\n");
 							error = TRUE;
 						}
 						else {
-							tailwidth  = GMT_convert_units (txt_a, GMT_INCH);
-							headlength = GMT_convert_units (txt_b, GMT_INCH);
-							headwidth  = GMT_convert_units (txt_c, GMT_INCH);
+							Ctrl->M.thickness = GMT_convert_units (txt_a, GMT_INCH);
+							Ctrl->M.length = GMT_convert_units (txt_b, GMT_INCH);
+							Ctrl->M.width = GMT_convert_units (txt_c, GMT_INCH);
 						}
 					}
 					break;
 				case 'N':		/* Make sectors area be proportional to frequency instead of radius */
-					eq_area = TRUE;
+					Ctrl->N.active = TRUE;
 					break;
 				case 'S':		/* Get radius of unit circle in inches */
+					Ctrl->S.active = TRUE;
 					n = strlen (argv[i]) - 1;
 					if (argv[i][n] == 'n') {
-						normalize = TRUE;
+						Ctrl->S.normalize = TRUE;
 						argv[i][n] = 0;
 					}
-					scale = GMT_convert_units (&argv[i][2], GMT_INCH);
+					Ctrl->S.scale = GMT_convert_units (&argv[i][2], GMT_INCH);
 					break;
 				case 'T':		/* Oriented instead of directed data */
-					oriented = TRUE;
+					Ctrl->T.active = TRUE;
 					break;
 				case 'W':		/* Get pen width for outline */
-					if (GMT_getpen (&argv[i][2], &pen)) {
+					Ctrl->W.active = TRUE;
+					if (GMT_getpen (&argv[i][2], &Ctrl->W.pen)) {
 						GMT_pen_syntax ('W');
 						error++;
 					}
-					outline = TRUE;
 					break;
 				case 'Z':		/* Scale radii before using data */
-					zscale = atof (&argv[i][2]);
+					Ctrl->Z.active = TRUE;
+					Ctrl->Z.scale = atof (&argv[i][2]);
 					break;
 				default:		/* Options not recognized */
 					error = TRUE;
@@ -183,16 +222,16 @@ main (int argc, char **argv)
 		else
 			file = argv[i];
 	}
-	
-	if (argc == 1 || GMT_quick) {
+
+	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr,"psrose %s - Polar histogram (rose diagram) plotter\n\n", GMT_VERSION);
-		fprintf (stderr, "usage: psrose <infile> [-A<sector_angle>[r]] [-B<tickinfo>] \n");
-		fprintf (stderr, "\t[-C[<modes>]] [-Eaz/el] [-G<fill>] [-H[<nrec>]] [-I] [-K] [-M<parameters>]\n");
-		fprintf (stderr, "\t[-N] [-O] [-P] [-R<r0/r1/theta0/theta1>] [-Sscale[n]] [-T] [-U[label]] [-V] [-:]\n");
-		fprintf (stderr, "\t[-Wpen] [-Xx_shift] [-Yy_shift] [-Zscale] [-cncopies] [-bi[s][<n>]]\n\n");
-		
-		if (GMT_quick) exit (EXIT_FAILURE);
-		
+		fprintf (stderr, "usage: psrose <infile> [-A<sector_angle>[r]] [%s]\n", GMT_B_OPT);
+		fprintf (stderr, "\t[-C[<modes>]] [-E<azim>/<elev>] [-G<fill>] [%s] [-I] [-K] [-M<parameters>]\n", GMT_H_OPT);
+		fprintf (stderr, "\t[-N] [-O] [-P] [-R<r0/r1/theta0/theta1>] [-S<scale>[n]] [-T] [%s] [-V] [%s]\n", GMT_U_OPT, GMT_t_OPT);
+		fprintf (stderr, "\t[-W<pen>] [%s] [%s] [-Z<scale>] [%s] [%s]\n\n", GMT_X_OPT, GMT_Y_OPT, GMT_c_OPT, GMT_bi_OPT);
+
+		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
+
 		fprintf (stderr, "\t<infile> (in ASCII or binary) has (length,azimuth) pairs.  If not given read standard input\n");
 		fprintf (stderr, "\n\tOPTIONS:\n");
 		fprintf (stderr, "\t-A sector width in degrees for sector diagram [Default is windrose]\n");
@@ -213,7 +252,7 @@ main (int argc, char **argv)
 		fprintf (stderr, "\t-R specifies the region.  (r0 = 0, r1 = max_radius.  For azimuth:\n");
 		fprintf (stderr, "\t   Specify theta0/theta1 = -90/90 (half-circle) or 0/360 only)\n");
 		fprintf (stderr, "\t   If r0 = r1 = 0, psrose will compute reasonable a r1 value.\n");
-		fprintf (stderr, "\t-S specifies the radius of the unit circle in %s [%g]. Normalize r if n is appended\n", GMT_unit_names[gmtdefs.measure_unit], scale);
+		fprintf (stderr, "\t-S specifies the radius of the unit circle in %s [%g]. Normalize r if n is appended\n", GMT_unit_names[gmtdefs.measure_unit], Ctrl->S.scale);
 		fprintf (stderr, "\t-T indicates that the vectors are oriented (two-headed), not directed [Default]\n");
 		GMT_explain_option ('U');
 		GMT_explain_option ('V');
@@ -230,20 +269,20 @@ main (int argc, char **argv)
 	}
 
 	/* Check that the options selected are mutually consistent */
-	
-	if (z_project.view_azimuth > 360.0 || z_project.view_elevation <= 0.0 || z_project.view_elevation > 90.0) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR -E option:  Enter azimuth in 0-360 range, elevation in 0-90 range\n", GMT_program);
+
+	if (Ctrl->E.elevation <= 0.0 || Ctrl->E.elevation > 90.0) {
+		fprintf (stderr, "%s: GMT SYNTAX ERROR -E option:  Elevation must be in 0-90 range\n", GMT_program);
 		error++;
 	}
-	if (scale <= 0.0) {
+	if (Ctrl->S.scale <= 0.0) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -S option:  radius must be nonzero\n", GMT_program);
 		error++;
 	}
-	if (fabs (zscale) < GMT_CONV_LIMIT) {
+	if (fabs (Ctrl->Z.scale) < GMT_CONV_LIMIT) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -Z option:  factor must be nonzero\n", GMT_program);
 		error++;
 	}
-	if (sector < 0.0) {
+	if (Ctrl->A.inc < 0.0) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -A option:  sector width must be positive\n", GMT_program);
 		error++;
 	}
@@ -251,7 +290,7 @@ main (int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -R option:  theta0/theta1 must be either -90/90 or 0/360\n", GMT_program);
 		error++;
 	}
-	if (GMT_io.binary[GMT_IN] && gmtdefs.io_header[GMT_IN]) {
+	if (GMT_io.binary[GMT_IN] && GMT_io.io_header[GMT_IN]) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data cannot have header -H\n", GMT_program);
 		error++;
 	}
@@ -260,7 +299,7 @@ main (int argc, char **argv)
                 fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data (-bi) must have at least 2 columns\n", GMT_program);
 		error++;
 	}
-	
+
 	if (error) exit (EXIT_FAILURE);
 
 	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
@@ -270,6 +309,9 @@ main (int argc, char **argv)
 		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
+	z_project.view_azimuth = Ctrl->E.azimuth;
+	z_project.view_elevation = Ctrl->E.elevation;
+	if (Ctrl->C.file && (fpm = fopen (Ctrl->C.file, "r")) == NULL) find_mean = TRUE;
 	if (file && (fp = GMT_fopen (file, GMT_io.r_mode)) == NULL) {
 		fprintf (stderr, "%s:  Cannot open file %s\n", GMT_program, file);
 		exit (EXIT_FAILURE);
@@ -281,16 +323,16 @@ main (int argc, char **argv)
 		GMT_setmode (GMT_IN);
 #endif
 	}
-	
+
 	max_radius = east;
 	half_only = (fabs (south + 90.0) < GMT_CONV_LIMIT);
-	if (rose) windrose = FALSE;
-	sector_plot = (sector > 0.0);
+	if (Ctrl->A.rose) windrose = FALSE;
+	sector_plot = (Ctrl->A.inc > 0.0);
 	if (sector_plot) windrose = FALSE;	/* Draw rose diagram instead of sector diagram */
-	if (!normalize) eq_area = FALSE;	/* Only do this is data is normalized for length also */
-	if (!inquire && !project_info.region_supplied) automatic = TRUE;
-	if (oriented) one_or_two = 2.0;
-	
+	if (!Ctrl->S.normalize) Ctrl->N.active = FALSE;	/* Only do this is data is normalized for length also */
+	if (!Ctrl->I.active && !project_info.region_supplied) automatic = TRUE;
+	if (Ctrl->T.active) one_or_two = 2.0;
+
 	if (half_only) {
 		total_arc = 180.0;
 		az_offset = 90.0;
@@ -299,21 +341,21 @@ main (int argc, char **argv)
 		total_arc = 360.0;
 		az_offset = 0.0;
 	}
-	n_bins = (sector <= 0.0) ? 1 : irint (total_arc / sector);
-	
+	n_bins = (Ctrl->A.inc <= 0.0) ? 1 : irint (total_arc / Ctrl->A.inc);
+
 	sum = (double *) GMT_memory (VNULL, (size_t)n_bins, sizeof (double), GMT_program);
 	xx = (double *) GMT_memory (VNULL, (size_t)(n_bins+2), sizeof (double), GMT_program);
 	yy = (double *) GMT_memory (VNULL, (size_t)(n_bins+2), sizeof (double), GMT_program);
 	azimuth = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
 	length = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
-	
-	if (gmtdefs.io_header[GMT_IN]) for (i = 0; i < gmtdefs.n_header_recs; i++) GMT_fgets (text, BUFSIZ, fp);
-		
+
+	if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (text, BUFSIZ, fp);
+
 	/* Read data and do some stats */
-	
+
 	n = 0;
 	n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : 2;
-	
+
 	n_fields = GMT_input (fp,  &n_expected_fields, &in);
 
 	while (! (GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
@@ -325,27 +367,27 @@ main (int argc, char **argv)
 
 		length[n] = in[0];
 		azimuth[n] = in[1];
-		
-		if (zscale != 1.0) length[n] *= zscale;
-		
+
+		if (Ctrl->Z.scale != 1.0) length[n] *= Ctrl->Z.scale;
+
 		/* Make sure azimuth is in 0 <= az < 360 range */
-		
+
 		while (azimuth[n] < 0.0) azimuth[n] += 360.0;
 		while (azimuth[n] >= 360.0) azimuth[n] -= 360.0;
-		
+
 		if (half_only) {	/* Flip azimuths about E-W line i.e. -90 < az <= 90 */
 			if (azimuth[n] > 90.0 && azimuth[n] <= 270.0) azimuth[n] -= 180.0;
 			if (azimuth[n] > 270.0) azimuth[n] -= 360.0;
 		}
-		
+
 		/* Double angle to find mean azimuth */
-		
+
 		sincos (one_or_two * azimuth[n] * D2R, &s, &c);
 		xr += length[n] * c;
 		yr += length[n] * s;
-		
+
 		total += length[n];
-		
+
 		n++;
 		if (n == n_alloc) {	/* Get more memory */
 			n_alloc += GMT_CHUNK;
@@ -355,38 +397,38 @@ main (int argc, char **argv)
 		n_fields = GMT_input (fp, &n_expected_fields, &in);
 	}
 	if (fp != GMT_stdin) GMT_fclose (fp);
-	
-	if (sector > 0.0) {	/* Sum up sector diagram info */
+
+	if (Ctrl->A.inc > 0.0) {	/* Sum up sector diagram info */
 		for (i = 0; i < n; i++) {
-			bin = (int) ((azimuth[i] + az_offset) / sector);
+			bin = (int) ((azimuth[i] + az_offset) / Ctrl->A.inc);
 			sum[bin] += length[i];
 		}
 	}
-		
+
 	mean_theta = R2D * d_atan2 (yr, xr) / one_or_two;
 	if (mean_theta < 0.0) mean_theta += 360.0;
 	mean_radius = hypot (xr, yr) / total;
-	if (!normalize) mean_radius *= max_radius;
-	
-	if (sector > 0.0) {	/* Find max of the bins */
+	if (!Ctrl->S.normalize) mean_radius *= max_radius;
+
+	if (Ctrl->A.inc > 0.0) {	/* Find max of the bins */
 		for (bin = 0; bin < n_bins; bin++) if (sum[bin] > max) max = sum[bin];
-		if (normalize) for (bin = 0; bin < n_bins; bin++) sum[bin] /= max;
-		if (eq_area) for (bin = 0; bin < n_bins; bin++) sum[bin] = sqrt (sum[bin]);
+		if (Ctrl->S.normalize) for (bin = 0; bin < n_bins; bin++) sum[bin] /= max;
+		if (Ctrl->N.active) for (bin = 0; bin < n_bins; bin++) sum[bin] = sqrt (sum[bin]);
 	}
 	else {
 		for (i = 0; i < n; i++) if (length[i] > max) max = length[i];
-		if (normalize) {
+		if (Ctrl->S.normalize) {
 			max = 1.0 / max;
 			for (i = 0; i < n; i++) length[i] *= max;
 			max = 1.0 / max;
 		}
 	}
-	
-	if (inquire || gmtdefs.verbose) {
+
+	if (Ctrl->I.active || gmtdefs.verbose) {
 		if (file) strcpy (text, file); else strcpy (text, "<stdin>");
 		sprintf (format, "%%s: Info for %%s: n = %%d rmax = %s mean r/az = (%s/%s) totlength = %s\n", gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
 		fprintf (stderr, format, GMT_program, text, n, max, mean_radius, mean_theta, total);
-		if (inquire) {
+		if (Ctrl->I.active) {
 			GMT_free ((void *)sum);
 			GMT_free ((void *)xx);
 			GMT_free ((void *)yy);
@@ -395,7 +437,7 @@ main (int argc, char **argv)
 			exit (EXIT_FAILURE);
 		}
 	}
-	
+
 	if (automatic) {
 		if (frame_info.axis[0].item[GMT_ANNOT_UPPER].interval < GMT_CONV_LIMIT) {
 			tmp = pow (10.0, floor (d_log10 (max)));
@@ -409,58 +451,54 @@ main (int argc, char **argv)
 			frame_info.plot = TRUE;
 		}
 	}
-	
+
 	if (frame_info.plot && frame_info.axis[1].item[GMT_ANNOT_UPPER].interval < GMT_CONV_LIMIT) frame_info.axis[1].item[GMT_ANNOT_UPPER].interval = frame_info.axis[1].item[GMT_GRID_UPPER].interval = 30.0;
-	
+
 	/* Ready to plot */
-	
-	project_info.projection = LINEAR;
+
+	project_info.projection = GMT_LINEAR;
 	project_info.region = TRUE;
-	project_info.pars[0] = project_info.pars[1] = (normalize) ? 1.0 : scale / max_radius;
-	GMT_map_setup (-scale, scale, -scale, scale);
-	
-	ps_plotinit (NULL, gmtdefs.overlay, gmtdefs.page_orientation, gmtdefs.x_origin, gmtdefs.y_origin,
-		gmtdefs.global_x_scale, gmtdefs.global_y_scale, gmtdefs.n_copies,
-		gmtdefs.dpi, GMT_INCH, gmtdefs.paper_width, gmtdefs.page_rgb, gmtdefs.encoding.name, GMT_epsinfo (argv[0]));
-	GMT_echo_command (argc, argv);
-	if (gmtdefs.unix_time) GMT_timestamp (argc, argv);
-	
-	x_origin = scale;	y_origin = ((half_only) ? 0.0 : scale);
+	project_info.pars[0] = project_info.pars[1] = (Ctrl->S.normalize) ? 1.0 : Ctrl->S.scale / max_radius;
+	GMT_map_setup (-Ctrl->S.scale, Ctrl->S.scale, -Ctrl->S.scale, Ctrl->S.scale);
+
+	GMT_plotinit (argc, argv);
+
+	x_origin = Ctrl->S.scale;	y_origin = ((half_only) ? 0.0 : Ctrl->S.scale);
 	ps_transrotate (x_origin, y_origin, 0.0);
 	if (project_info.three_D) ps_transrotate (-z_project.xmin, -z_project.ymin - y_origin, 0.0);
-	if (!normalize) scale /= max_radius;
-	
-	GMT_setpen (&pen);
+	if (!Ctrl->S.normalize) Ctrl->S.scale /= max_radius;
+
+	GMT_setpen (&Ctrl->W.pen);
 	if (windrose) {
 		for (i = 0; i < n; i++) {
 			angle = (90.0 - azimuth[i]) * D2R;
-			radius = length[i] * scale;
+			radius = length[i] * Ctrl->S.scale;
 			x = radius * cos (angle);
 			y = radius * sin (angle);
 			plot3d (0.0, 0.0, 3);
 			plot3d (x, y, -2);
 		}
 	}
-	
-	if (sector_plot && !rose) {	/* Draw pie slices for sector plot */
-			
+
+	if (sector_plot && !Ctrl->A.rose) {	/* Draw pie slices for sector plot */
+
 		for (bin = 0; bin < n_bins; bin++) {
-			az = bin * sector - az_offset;
-			angle1 = (90.0 - az - sector);
-			angle2 = angle1 + sector;
-			pie3d (0.0, 0.0, sum[bin] * scale, angle1, angle2, fill.rgb, FALSE);
+			az = bin * Ctrl->A.inc - az_offset;
+			angle1 = (90.0 - az - Ctrl->A.inc);
+			angle2 = angle1 + Ctrl->A.inc;
+			pie3d (0.0, 0.0, sum[bin] * Ctrl->S.scale, angle1, angle2, Ctrl->G.fill.rgb, FALSE);
 		}
 	}
-	else if (rose) {	/* Draw rose diagram */
-	
+	else if (Ctrl->A.rose) {	/* Draw rose diagram */
+
 		for (bin = i = 0; bin < n_bins; bin++, i++) {
-			az = bin * sector - az_offset + 0.5 * sector;
+			az = bin * Ctrl->A.inc - az_offset + 0.5 * Ctrl->A.inc;
 			angle = 90.0 - az;
-			xx[i] = scale * sum[bin] * cos (D2R * angle);
-			yy[i] = scale * sum[bin] * sin (D2R * angle);
+			xx[i] = Ctrl->S.scale * sum[bin] * cos (D2R * angle);
+			yy[i] = Ctrl->S.scale * sum[bin] * sin (D2R * angle);
 		}
 		if (half_only) {
-			xx[i] = scale * 0.5 * (sum[0] + sum[n_bins-1]);
+			xx[i] = Ctrl->S.scale * 0.5 * (sum[0] + sum[n_bins-1]);
 			yy[i] = 0.0;
 			i++;
 			xx[i] = -xx[i-1];
@@ -468,34 +506,34 @@ main (int argc, char **argv)
 			i++;
 		}
 		if (project_info.three_D) GMT_2D_to_3D (xx, yy, project_info.z_level, i);
-		ps_polygon (xx, yy, i, fill.rgb, outline);
+		ps_polygon (xx, yy, i, Ctrl->G.fill.rgb, Ctrl->W.active);
 	}
-	
-	if (sector_plot && outline && !rose) {	/* Draw a line outlining the pie slices */
+
+	if (sector_plot && Ctrl->W.active && !Ctrl->A.rose) {	/* Draw a line outlining the pie slices */
 		angle1 = (half_only) ? 180.0 : 90.0;
 		angle2 = (half_only) ? 0.0 : 90.0;
 		angle1 *= D2R;
 		angle2 *= D2R;
 		sincos (angle1, &s, &c);
-		x1 = (sum[0] * scale) * c;
-		y1 = (sum[0] * scale) * s;
+		x1 = (sum[0] * Ctrl->S.scale) * c;
+		y1 = (sum[0] * Ctrl->S.scale) * s;
 		sincos (angle2, &s, &c);
-		x2 = (sum[n_bins-1] * scale) * c;
-		y2 = (sum[n_bins-1] * scale) * s;
+		x2 = (sum[n_bins-1] * Ctrl->S.scale) * c;
+		y2 = (sum[n_bins-1] * Ctrl->S.scale) * s;
 		plot3d (x1, y1, 3);
 		plot3d (x2, y2, 2);
 		for (bin = n_bins-1; bin >= 0; bin--) {
 			status = (bin == 0) ? 2 : 0;
-			az = bin * sector - az_offset;
-			angle1 = 90.0 - az - sector;
-			angle2 = angle1 + sector;
-			arc3d (0.0, 0.0, sum[bin] * scale, angle1, angle2, status);
+			az = bin * Ctrl->A.inc - az_offset;
+			angle1 = 90.0 - az - Ctrl->A.inc;
+			angle2 = angle1 + Ctrl->A.inc;
+			arc3d (0.0, 0.0, sum[bin] * Ctrl->S.scale, angle1, angle2, status);
 			angle1 *= D2R;
 		}
 	}
-	
-	if (draw_modes) {
-		
+
+	if (Ctrl->C.active) {
+
 		if (find_mean) {	/* Use the mean direction only */
 			n_modes = 1;
 			mode_direction[0] = mean_theta;
@@ -518,39 +556,39 @@ main (int argc, char **argv)
 			mode_length = (double *) GMT_memory ((void *)mode_length, (size_t)n_modes, sizeof (double), "psrose");
 		}
 		for (i = 0; i < n_modes; i++) {
-			if (eq_area) mode_length[i] = sqrt (mode_length[i]);
+			if (Ctrl->N.active) mode_length[i] = sqrt (mode_length[i]);
 			if (half_only && mode_direction[i] > 90.0 && mode_direction[i] <= 270.0) mode_direction[i] -= 180.0;
 			angle = D2R * (90.0 - mode_direction[i]);
 			sincos (angle, &s, &c);
-			xr = scale * mode_length[i] * c;
-			yr = scale * mode_length[i] * s;
-			GMT_vector3D (0.0, 0.0, xr, yr, project_info.z_level, tailwidth, headlength, headwidth, gmtdefs.vector_shape, a_rgb, TRUE);
+			xr = Ctrl->S.scale * mode_length[i] * c;
+			yr = Ctrl->S.scale * mode_length[i] * s;
+			GMT_vector3D (0.0, 0.0, xr, yr, project_info.z_level, Ctrl->M.thickness, Ctrl->M.length, Ctrl->M.width, gmtdefs.vector_shape, Ctrl->M.rgb, TRUE);
 		}
-		
+
 	}
-	
+
 	if (frame_info.plot) {	/* Draw grid lines etc */
 		GMT_setpen (&gmtdefs.grid_pen[0]);
-		off = max_radius * scale;
+		off = max_radius * Ctrl->S.scale;
 		n_alpha = (frame_info.axis[1].item[GMT_GRID_UPPER].interval > 0.0) ? irint (total_arc / frame_info.axis[1].item[GMT_GRID_UPPER].interval) : -1;
 		for (i = 0; i <= n_alpha; i++) {
 			angle = D2R * i * frame_info.axis[1].item[GMT_GRID_UPPER].interval;
 			sincos (angle, &s, &c);
-			x = max_radius * scale * c;
-			y = max_radius * scale * s;
+			x = max_radius * Ctrl->S.scale * c;
+			y = max_radius * Ctrl->S.scale * s;
 			plot3d (0.0, 0.0, 3);
 			plot3d (x, y, -2);
 		}
-		
+
 		n_bins = (frame_info.axis[0].item[GMT_GRID_UPPER].interval > 0.0) ? irint (max_radius / frame_info.axis[0].item[GMT_GRID_UPPER].interval) : -1;
 		for (i = 1; i <= n_bins; i++)
-			arc3d (0.0, 0.0, i * frame_info.axis[0].item[GMT_GRID_UPPER].interval * scale, 0.0, total_arc, 3);
+			arc3d (0.0, 0.0, i * frame_info.axis[0].item[GMT_GRID_UPPER].interval * Ctrl->S.scale, 0.0, total_arc, 3);
 		ps_setpaint (gmtdefs.basemap_frame_rgb);
 		y = lsize + 6.0 * gmtdefs.annot_offset[0];
 		GMT_text3D (0.0, off + y, project_info.z_level, gmtdefs.header_font_size, gmtdefs.header_font, frame_info.header, 0.0, 2, 0);
-		
+
 		GMT_get_format (frame_info.axis[0].item[GMT_ANNOT_UPPER].interval, frame_info.axis[0].unit, frame_info.axis[0].prefix, format);
-		
+
 		if (half_only) {
 			int which, no_degree;
 			which = (gmtdefs.degree_format >= 100) ? gmt_ring : gmt_degree;
@@ -574,7 +612,7 @@ main (int argc, char **argv)
 			for (i = 1; i <= n_annot; i++) {
 				x = i * frame_info.axis[0].item[GMT_ANNOT_UPPER].interval;
 				sprintf (text, format, x);
-				x *= scale;
+				x *= Ctrl->S.scale;
 				GMT_text3D (x, -gmtdefs.annot_offset[0], project_info.z_level, gmtdefs.annot_font_size[0], gmtdefs.annot_font[0], text, 0.0, 10, 0);
 				GMT_text3D (-x, -gmtdefs.annot_offset[0], project_info.z_level, gmtdefs.annot_font_size[0], gmtdefs.annot_font[0], text, 0.0, 10, 0);
 			}
@@ -582,11 +620,11 @@ main (int argc, char **argv)
 		else {
 			GMT_text3D (0.0, -off - 2.0 * gmtdefs.annot_offset[0], project_info.z_level, gmtdefs.label_font_size, gmtdefs.label_font, "SOUTH", 0.0, 10, 0);
 			plot3d (off, -off, 3);
-			plot3d ((max_radius - frame_info.axis[0].item[GMT_GRID_UPPER].interval) * scale, -off, 2);
+			plot3d ((max_radius - frame_info.axis[0].item[GMT_GRID_UPPER].interval) * Ctrl->S.scale, -off, 2);
 			plot3d (off, -off, 3);
 			plot3d (off, gmtdefs.tick_length - off, 2);
-			plot3d ((max_radius - frame_info.axis[0].item[GMT_GRID_UPPER].interval) * scale, -off, 3);
-			plot3d ((max_radius - frame_info.axis[0].item[GMT_GRID_UPPER].interval) * scale, gmtdefs.tick_length - off, -2);
+			plot3d ((max_radius - frame_info.axis[0].item[GMT_GRID_UPPER].interval) * Ctrl->S.scale, -off, 3);
+			plot3d ((max_radius - frame_info.axis[0].item[GMT_GRID_UPPER].interval) * Ctrl->S.scale, gmtdefs.tick_length - off, -2);
 			y = -(off + 5.0 * gmtdefs.annot_offset[0] + GMT_font[gmtdefs.annot_font[0]].height * lsize + GMT_font[gmtdefs.label_font].height * lsize);
 			if (frame_info.axis[1].label[0]) GMT_text3D (0.0, y, project_info.z_level, gmtdefs.label_font_size, gmtdefs.label_font, frame_info.axis[1].label, 0.0, 10, 0);
 			if (frame_info.axis[0].label[0]) {
@@ -595,7 +633,7 @@ main (int argc, char **argv)
 			}
 			else
 				sprintf (text, format, frame_info.axis[0].item[GMT_GRID_UPPER].interval);
-			GMT_text3D ((max_radius - 0.5 * frame_info.axis[0].item[GMT_GRID_UPPER].interval) * scale, -(off + gmtdefs.annot_offset[0]), project_info.z_level, gmtdefs.annot_font_size[0], gmtdefs.annot_font[0], text, 0.0, 10, 0);
+			GMT_text3D ((max_radius - 0.5 * frame_info.axis[0].item[GMT_GRID_UPPER].interval) * Ctrl->S.scale, -(off + gmtdefs.annot_offset[0]), project_info.z_level, gmtdefs.annot_font_size[0], gmtdefs.annot_font[0], text, 0.0, 10, 0);
 		}
 		GMT_text3D (off + 2.0 * gmtdefs.annot_offset[0], 0.0, project_info.z_level, gmtdefs.label_font_size, gmtdefs.label_font, le, 0.0, 5, 0);
 		GMT_text3D (-off - 2.0 * gmtdefs.annot_offset[0], 0.0, project_info.z_level, gmtdefs.label_font_size, gmtdefs.label_font, lw, 0.0, 7, 0);
@@ -605,16 +643,18 @@ main (int argc, char **argv)
 	if (project_info.three_D) ps_rotatetrans (z_project.xmin, z_project.ymin + y_origin, 0.0);
 	ps_rotatetrans (-x_origin, -y_origin, 0.0);
 
-	ps_plotend (gmtdefs.last_page);
-	
-	GMT_end (argc, argv);
-	
+	GMT_plotend ();
+
 	GMT_free ((void *)sum);
 	GMT_free ((void *)xx);
 	GMT_free ((void *)yy);
 	GMT_free ((void *)azimuth);
 	GMT_free ((void *)length);
-	
+
+	Free_Psrose_Ctrl (Ctrl);	/* Deallocate control structure */
+
+	GMT_end (argc, argv);
+
 	exit (EXIT_SUCCESS);
 }
 
@@ -622,13 +662,13 @@ void arc3d (double x, double y, double r, double a1, double a2, int status)
 {
 	int i, n;
 	double da, a, *xx, *yy, s, c;
-	
+
 	if (project_info.three_D) {	/* Must project and draw line segments */
 		n = (int)ceil (a2 - a1);
-		
+
 		xx = (double *) GMT_memory (VNULL, (size_t)(n + 1), sizeof (double), GMT_program);
 		yy = (double *) GMT_memory (VNULL, (size_t)(n + 1), sizeof (double), GMT_program);
-		
+
 		a1 *= D2R;
 		a2 *= D2R;
 		da = (a2 - a1) / n;
@@ -638,7 +678,7 @@ void arc3d (double x, double y, double r, double a1, double a2, int status)
 			xx[i] = x + r * c;
 			yy[i] = y + r * s;
 		}
-		n++;	
+		n++;
 		GMT_2D_to_3D (xx, yy, project_info.z_level, n);
 		ps_line (xx, yy, n, status, FALSE, TRUE);
 		GMT_free ((void *)xx);
@@ -658,13 +698,13 @@ void pie3d (double x, double y, double rad, double a1, double a2, int rgb[], BOO
 {
 	int i, n;
 	double da, a, *xx, *yy, s, c;
-	
+
 	if (project_info.three_D) {	/* Must project and draw line segments */
 		n = (int)ceil (a2 - a1);
-		
+
 		xx = (double *) GMT_memory (VNULL, (size_t)(n + 2), sizeof (double), GMT_program);
 		yy = (double *) GMT_memory (VNULL, (size_t)(n + 2), sizeof (double), GMT_program);
-		
+
 		a1 *= D2R;
 		a2 *= D2R;
 		da = (a2 - a1) / n;
@@ -675,7 +715,7 @@ void pie3d (double x, double y, double rad, double a1, double a2, int rgb[], BOO
 			yy[i] = y + rad * s;
 		}
 		xx[i] = x;	yy[i] = y;
-		n += 2;	
+		n += 2;
 		GMT_2D_to_3D (xx, yy, project_info.z_level, n);
 		ps_polygon (xx, yy, n, rgb, outline);
 		GMT_free ((void *)xx);
@@ -684,3 +724,33 @@ void pie3d (double x, double y, double rad, double a1, double a2, int rgb[], BOO
 	else
 		ps_pie (x, y, rad, a1, a2, rgb, outline);
 }
+
+void *New_Psrose_Ctrl () {	/* Allocate and initialize a new control structure */
+	struct PSROSE_CTRL *C;
+	
+	C = (struct PSROSE_CTRL *) GMT_memory (VNULL, 1, sizeof (struct PSROSE_CTRL), "New_Psrose_Ctrl");
+	
+	/* Initialize values whose defaults are not 0/FALSE/NULL */
+	C->E.azimuth = 180.0;
+	C->E.elevation = 90.0;
+	C->M.rgb[0] = C->M.rgb[1] = C->M.rgb[2] = 0;
+	GMT_init_fill (&C->G.fill, -1, -1, -1);
+	GMT_init_pen (&C->W.pen, GMT_PENWIDTH);
+	C->M.thickness = 0.03;
+	C->M.length = 0.12;
+	C->M.width = 0.1;
+	C->S.scale = 3.0;
+	C->Z.scale = 1.0;
+	if (gmtdefs.measure_unit == GMT_CM) {
+		C->S.scale = 7.5 / 2.54;
+		C->M.thickness = 0.075 / 2.54;
+		C->M.length = 0.3 / 2.54;
+		C->M.width = 0.25 / 2.54;
+	}
+	return ((void *)C);
+}
+
+void Free_Psrose_Ctrl (struct PSROSE_CTRL *C) {	/* Deallocate control structure */
+	if (C->C.file) GMT_free ((void *)C->C.file);	
+	GMT_free ((void *)C);	
+}
