@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: pstext.c,v 1.35 2004/06/09 06:23:28 pwessel Exp $
+ *	$Id: pstext.c,v 1.53 2006/04/12 21:55:46 pwessel Exp $
  *
- *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -32,18 +32,67 @@
  */
 
 #include "gmt.h"
+#include "pslib.h"
 
-struct GMT_TEXT {
-	double x_offset, y_offset;	/* Offset from reference point */
-	double line_spacing;
-	double paragraph_width;
+struct PSTEXT_CTRL {
+	struct A {	/* -A */
+		BOOLEAN active;
+	} A;
+	struct C {	/* -C<dx>/<dy> */
+		BOOLEAN active;
+		BOOLEAN percent;
+		double dx, dy;
+	} C;
+	struct D {	/* -D[j]<dx>/<dy>[v[<pen>] */
+		BOOLEAN active;
+		BOOLEAN justify;
+		BOOLEAN line;
+		double dx, dy;
+		struct GMT_PEN pen;
+	} D;
+	struct E {	/* -Eazim/elev */
+		BOOLEAN active;
+		double azimuth, elevation;
+	} E;
+	struct G {	/* -G<fill> */
+		BOOLEAN active;
+		struct GMT_FILL fill;
+	} G;
+	struct L {	/* -L */
+		BOOLEAN active;
+	} L;
+	struct N {	/* -N */
+		BOOLEAN active;
+	} N;
+	struct S {	/* -S<pen> */
+		BOOLEAN active;
+		struct GMT_PEN pen;
+	} S;
+	struct W {	/* -W[<fill>][o|O|c[<pen>] */
+		BOOLEAN active;
+		BOOLEAN outline;
+		BOOLEAN paint;
+		int mode;
+		struct GMT_PEN pen;
+		struct GMT_FILL fill;
+	} W;
+	struct Z {	/* -Z<z_level> */
+		BOOLEAN active;
+		double level;
+	} Z;
+};
+
+struct PSTEXT_INFO {
 	int text_justify;
 	int paragraph_font;
-	double font_size;
-	double paragraph_angle;
 	int block_justify;
 	int boxflag;
 	int space_flag;
+	double x_offset, y_offset;	/* Offset from reference point */
+	double line_spacing;
+	double paragraph_width;
+	double font_size;
+	double paragraph_angle;
 	double x_space, y_space;	/* Extra spacing between box and text */
 	struct GMT_PEN boxpen;
 	struct GMT_PEN vecpen;
@@ -52,49 +101,46 @@ struct GMT_TEXT {
 	struct GMT_FILL boxfill;
 };
 
-void GMT_putwords (double x, double y, char **text, int n_words, struct GMT_TEXT *info);
-
-main(int argc, char **argv)
+int main(int argc, char **argv)
 {
 	int i, j, k, last_font, fno, n_files = 0, n_args, n_words = 0, nscan;
-	int ix, iy, *use_font, n_processed = 0, n_read = 0, n_paragraphs = 0;
-	size_t n_alloc;
-	
+	int ix, iy, *use_font, n_processed = 0, n_read = 0, n_paragraphs = 0, pos;
+	size_t n_alloc = 0;
+
 	double xy[2], west = 0., east = 0.0, south = 0.0, north = 0.0, plot_x, plot_y;
-	double new_z_level = 0.0, z_level = 0.0, tmp, dx, dy;
-	
-	BOOLEAN error = FALSE, nofile = TRUE, outline = FALSE, set_z = FALSE, master_record = FALSE, offset_outward = FALSE;
-	BOOLEAN solid = TRUE, do_paragraphs = FALSE, list = FALSE, done = FALSE, clip = TRUE, old_GMT_world_map, add;
-	BOOLEAN convert_az = FALSE, skip_text_records = FALSE;
-	
-	char text[BUFSIZ], line[BUFSIZ], this_font[80], just_key[5], txt_a[32], txt_b[32], txt_x[32], txt_y[32];
-	char *p, **word, pjust;
-	
+	double z_level = 0.0, tmp, dx, dy, xx[2], yy[2];
+
+	BOOLEAN error = FALSE, nofile = TRUE, master_record = FALSE;
+	BOOLEAN do_paragraphs = FALSE, done = FALSE, old_GMT_world_map, add;
+	BOOLEAN skip_text_records = FALSE;
+
+	char text[BUFSIZ], line[BUFSIZ], this_font[GMT_LONG_TEXT], just_key[5], txt_a[GMT_LONG_TEXT], txt_b[GMT_LONG_TEXT];
+	char txt_x[GMT_LONG_TEXT], txt_y[GMT_LONG_TEXT], p[BUFSIZ];
+	char **word = (char **)NULL, pjust;
+
 	FILE *fp = NULL;
+
+	struct PSTEXT_INFO T;
+	struct PSTEXT_CTRL *Ctrl;
+
+	void GMT_putwords (double x, double y, char **text, int n_words, struct PSTEXT_INFO *info);
+	void *New_Pstext_Ctrl (), Free_Pstext_Ctrl (struct PSTEXT_CTRL *C);
+	void load_parameters (struct PSTEXT_INFO *T, struct PSTEXT_CTRL *C);
 	
-	struct GMT_TEXT T;
-
-	memset ((void *)&T, 0, sizeof (struct GMT_TEXT));
-
-	GMT_init_pen (&T.txtpen, GMT_PENWIDTH);
-	GMT_init_pen (&T.boxpen, GMT_PENWIDTH);
-	GMT_init_pen (&T.vecpen, GMT_PENWIDTH);
-	GMT_init_fill (&T.boxfill, -1, -1, -1);	/* No fill */
-	GMT_init_fill (&T.txtfill, 0, 0, 0);
-
 	argc = GMT_begin (argc, argv);
 
-	T.x_space = T.y_space = 15.0;	/* 15% of font size is default clearance */
-	T.space_flag = 1;		/* Meaning we gave percentage */
+	Ctrl = (struct PSTEXT_CTRL *)New_Pstext_Ctrl ();	/* Allocate and initialize a new control structure */
+
+	memset ((void *)&T, 0, sizeof (struct PSTEXT_INFO));
 
 	/* Check and interpret the command line arguments */
-	
+
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '-') {
 			switch(argv[i][1]) {
-		
+
 				/* Common parameters */
-			
+
 				case 'B':
 				case 'H':
 				case 'J':
@@ -111,97 +157,94 @@ main(int argc, char **argv)
 				case 'c':
 				case ':':
 				case '\0':
-					error += GMT_get_common_args (argv[i], &west, &east, &south, &north);
+					error += GMT_parse_common_options (argv[i], &west, &east, &south, &north);
 					break;
-				
+
 				/* Supplemental parameters */
 
 				case 'A':	/* Getting azimuths rather than directions, must convert via map projection */
-					convert_az = TRUE;
+					Ctrl->A.active = TRUE;
 					break;
 				case 'C':
+					Ctrl->C.active = TRUE;
 					k = sscanf (&argv[i][2], "%[^/]/%s", txt_a, txt_b);
-					T.x_space = GMT_convert_units (txt_a, GMT_INCH);
-					T.y_space = (k == 2) ? GMT_convert_units (txt_b, GMT_INCH) : T.x_space;
-					T.space_flag = (strchr (argv[i], '%')) ? 1 : 0;
+					Ctrl->C.dx = GMT_convert_units (txt_a, GMT_INCH);
+					Ctrl->C.dy = (k == 2) ? GMT_convert_units (txt_b, GMT_INCH) : Ctrl->C.dx;
+					Ctrl->C.percent = (strchr (argv[i], '%')) ? TRUE : FALSE;
 					break;
 				case 'D':
+					Ctrl->D.active = TRUE;
 					k = 2;
-					if (argv[i][k] == 'j') offset_outward = TRUE, k++;
+					if (argv[i][k] == 'j') Ctrl->D.justify = TRUE, k++;
 					for (j = k; argv[i][j] && argv[i][j] != 'v'; j++);
 					if (argv[i][j] == 'v') {
-						T.boxflag |= 32;
-						if (argv[i][j+1] && GMT_getpen (&argv[i][j+1], &T.vecpen)) {
+						Ctrl->D.line = TRUE;
+						if (argv[i][j+1] && GMT_getpen (&argv[i][j+1], &Ctrl->D.pen)) {
 							fprintf (stderr, "%s: GMT SYNTAX ERROR -D option: Give pen after c\n", GMT_program);
 							error++;
 						}
 						argv[i][j] = 0;
 					}
 					sscanf (&argv[i][k], "%[^/]/%s", txt_a, txt_b);
-					T.x_offset = GMT_convert_units (txt_a, GMT_INCH);
-					T.y_offset = GMT_convert_units (txt_b, GMT_INCH);
+					Ctrl->D.dx = GMT_convert_units (txt_a, GMT_INCH);
+					Ctrl->D.dy = GMT_convert_units (txt_b, GMT_INCH);
 					break;
 				case 'E':
-					sscanf (&argv[i][2], "%lf/%lf", &z_project.view_azimuth, &z_project.view_elevation);
+					Ctrl->E.active = TRUE;
+					sscanf (&argv[i][2], "%lf/%lf", &Ctrl->E.azimuth, &Ctrl->E.elevation);
 					break;
 				case 'G':
-					if (GMT_getfill (&argv[i][2], &T.txtfill)) {
+					Ctrl->G.active = TRUE;
+					if (GMT_getfill (&argv[i][2], &Ctrl->G.fill)) {
 						GMT_fill_syntax ('G');
 						error++;
 					}
-					solid = TRUE;
 					break;
 				case 'L':
-					list = GMT_quick = TRUE;
+					Ctrl->L.active = TRUE;
 					break;
 				case 'M':               /* Multiple line segments input: Text blocks */
 					GMT_multisegment (&argv[i][2]);
-					do_paragraphs = TRUE;
 					break;
 				case 'N':	/* Do not clip at border */
-					clip = FALSE;
+					Ctrl->N.active = TRUE;
 					break;
 				case 'S':
-					outline = TRUE;
-					if (GMT_getpen (&argv[i][2], &T.txtpen)) {
+					Ctrl->S.active = TRUE;
+					if (GMT_getpen (&argv[i][2], &Ctrl->S.pen)) {
 						GMT_pen_syntax ('S');
 						error++;
 					}
 					break;
 				case 'W':
+					Ctrl->W.active = TRUE;
 					for (j = 0; argv[i][j] && !(strchr ("cCoO", argv[i][j])); j++);
 					if (argv[i][j])  {
-						T.boxflag |= 1;	/* Want box outline */
-						if (argv[i][j] == 'O') T.boxflag |= 4;	/* Want rounded box outline */
-						if (argv[i][j] == 'c') T.boxflag |= 8;	/* Want concave box outline */
-						if (argv[i][j] == 'C') T.boxflag |= 16;	/* Want convex box outline */
-						if (argv[i][j+1] && GMT_getpen (&argv[i][j+1], &T.boxpen)) {
+						Ctrl->W.outline = TRUE;	/* Want box outline */
+						Ctrl->W.mode = argv[i][j];
+						if (argv[i][j+1] && GMT_getpen (&argv[i][j+1], &Ctrl->W.pen)) {
 							fprintf (stderr, "%s: GMT SYNTAX ERROR -W option: Give pen after o\n", GMT_program);
 							error++;
 						}
 						argv[i][j] = 0;
 					}
 					if (argv[i][2]) {	/* Also gave fill */
-						if (GMT_getfill (&argv[i][2], &T.boxfill)) {
+						if (GMT_getfill (&argv[i][2], &Ctrl->W.fill)) {
 							GMT_fill_syntax ('W');
 							error++;
 						}
-						T.boxflag |= 2;
-					}
-					if (!(T.boxflag | 31)) {
-						fprintf (stderr, "%s: GMT SYNTAX ERROR -W option: Must give arguments\n", GMT_program);
-						error++;
+						Ctrl->W.paint = TRUE;
 					}
 					break;
 				case 'Z':
 					if (argv[i][2]) {
-						new_z_level = atof (&argv[i][2]);
-						set_z = TRUE;
+						Ctrl->Z.level = atof (&argv[i][2]);
+						Ctrl->Z.active = TRUE;
 					}
 					break;
-					
+
 				/* Options not recognized */
-						
+
 				default:
 					error = TRUE;
 					GMT_default_error (argv[i][1]);
@@ -211,13 +254,14 @@ main(int argc, char **argv)
 		else
 			n_files++;
 	}
-	
-	if (argc == 1 || GMT_quick) {
+
+	if (argc == 1 || Ctrl->L.active || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "pstext %s - To plot text on maps\n\n", GMT_VERSION);
-		fprintf (stderr, "usage: pstext <txtfile> -J<params> -R<west>/<east>/<south>/<north>\n");
-		fprintf (stderr, "\t[-A] [-B<tickinfo>] [-C<dx>/<dy>] [-D[j]<dx>/<dy>[v[<pen>]] [-Eaz/el] [-G<color>]\n");
-		fprintf (stderr, "\t[-H[<nrec>]] [-K] [-L] [-M[<flag>]] [-N] [-O] [-P] [-S<pen>] [-U] [-V] [-W[<fill>][o|O|c[<pen>]]]\n");
-		fprintf (stderr, "\t[-X<x_shift>] [-Y<y_shift>] [-Z<zlevel>] [-c<ncopies>] [-:]\n\n");
+		fprintf (stderr, "usage: pstext <txtfile> %s %s\n", GMT_J_OPT, GMT_Rgeoz_OPT);
+		fprintf (stderr, "\t[-A] [%s] [-C<dx>/<dy>] [-D[j]<dx>/<dy>[v[<pen>]] [-Eaz/el] [-G<color>]\n", GMT_B_OPT);
+		fprintf (stderr, "\t[%s] [-K] [-L] [%s] [-N] [-O] [-P] [-S<pen>] [%s]\n", GMT_Ho_OPT, GMT_Mo_OPT, GMT_U_OPT);
+		fprintf (stderr, "\t[-V] [-W[<fill>][o|O|c[<pen>]]] [%s] [%s]\n", GMT_X_OPT, GMT_X_OPT);
+		fprintf (stderr, "\t[-Z<zlevel>] [%s] [%s]\n\n", GMT_t_OPT, GMT_c_OPT);
 		fprintf (stderr, "\tReads (x,y,size,angle,fontno,justify,text) from <txtfile> [or stdin]\n");
 		fprintf (stderr, "\tOR (with -M) one or more text paragraphs with formatting info in the segment header.\n");
 		fprintf (stderr, "\tjustify is of the form [T|M|B][L|C|R] (top/middle/bottom/left/center/right)\n");
@@ -235,16 +279,16 @@ main(int argc, char **argv)
 		fprintf (stderr, "\t   @;<r/g/b>; switches font color; @;; resets font color\n");
 		fprintf (stderr, "\t   @_ toggles between normal and underlined text\n");
 		fprintf (stderr, "\t(See manual page for more information)\n");
-		
-		if (list) {	/* List fonts */
+
+		if (Ctrl->L.active) {	/* List fonts */
 			fprintf (stderr, "\n\tFont #	Font Name\n");
 			fprintf (stderr, "\t------------------------------------\n");
-			for (i = 0; i < N_FONTS; i++)
+			for (i = 0; i < GMT_N_FONTS; i++)
 				fprintf (stderr, "\t%3d\t%s\n", i, GMT_font[i].name);
 		}
-			
-		if (GMT_quick) exit (EXIT_FAILURE);
-		
+
+		if (Ctrl->L.active || GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
+
 		GMT_explain_option ('j');
 		GMT_explain_option ('R');
 		fprintf (stderr, "\n\tOPTIONS:\n");
@@ -254,7 +298,7 @@ main(int argc, char **argv)
 		fprintf (stderr, "\t   if -W has been set.  Append units {c,i,m,p} or %% of fontsize [15%%]\n");
 		fprintf (stderr, "\t-D adds <add_x>,<add_y> to the text origin AFTER projecting with -J. [0/0]\n");
 		fprintf (stderr, "\t   Use -Dj to move text origin away from point (direction determined by text's justification)\n");
-		fprintf (stderr, "\t   Append v[<pen>] to draw line from text to original point (-M only)\n");
+		fprintf (stderr, "\t   Append v[<pen>] to draw line from text to original point\n");
 		fprintf (stderr, "\t-E set azimuth and elevation of viewpoint for 3-D perspective [180/90]\n");
 		fprintf (stderr, "\t-G set the color (red/green/blue (0-255)) for solid text [0/0/0]\n");
 		GMT_explain_option ('H');
@@ -284,33 +328,30 @@ main(int argc, char **argv)
 	}
 
 	/* Check that the options selected are mutually consistent */
-	
-	if (T.x_space < 0.0 || T.y_space < 0.0) {
+
+	if (GMT_io.multi_segments[GMT_IN] || GMT_io.multi_segments[GMT_OUT]) do_paragraphs = TRUE;
+	if (Ctrl->C.dx < 0.0 || Ctrl->C.dy < 0.0) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -C option:  clearances cannot be negative!\n", GMT_program);
 		error++;
 	}
-	if (T.x_space == 0.0 && T.y_space == 0.0 && (T.boxflag & 4)) {
+	if (Ctrl->C.dx == 0.0 && Ctrl->C.dy == 0.0 && (Ctrl->W.mode == 'O')) {
 		fprintf (stderr, "%s: Warning: -WO requires a nonzero -C\n", GMT_program);
 		error++;
 	}
-	if (T.x_space == 0.0 && T.y_space == 0.0 && (T.boxflag & 8 || T.boxflag & 16)) {
+	if (Ctrl->C.dx == 0.0 && Ctrl->C.dy == 0.0 && (Ctrl->W.mode == 'c' || Ctrl->W.mode == 'C')) {
 		fprintf (stderr, "%s: Warning: -Wc|C requires a nonzero -C\n", GMT_program);
 		error++;
 	}
-	if ((T.boxflag & 32) && !do_paragraphs) {
-		fprintf (stderr, "%s: Warning: -D<x/y>v only applies to -M\n", GMT_program);
-		error++;
-	}
 	if (!(z_project.view_azimuth == 180.0 && z_project.view_elevation == 90.0) && do_paragraphs) {
 		fprintf (stderr, "%s: -E not implemented with -M yet\n", GMT_program);
 		error++;
 	}
-	if (T.x_offset == 0.0 && T.y_offset == 0.0 && (T.boxflag & 32)) {
+	if (Ctrl->D.dx == 0.0 && Ctrl->D.dy == 0.0 && Ctrl->D.line) {
 		fprintf (stderr, "%s: Warning: -D<x/y>v requires one nonzero <x/y>\n", GMT_program);
 		error++;
 	}
-	if (z_project.view_azimuth > 360.0 || z_project.view_elevation <= 0.0 || z_project.view_elevation > 90.0) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR -E option:  Enter azimuth in 0-360 range, elevation in 0-90 range\n", GMT_program);
+	if (Ctrl->E.elevation <= 0.0 || Ctrl->E.elevation > 90.0) {
+		fprintf (stderr, "%s: GMT SYNTAX ERROR -E option:  Elevation must be in 0-90 range\n", GMT_program);
 		error++;
 	}
 	if (!project_info.region_supplied) {
@@ -322,43 +363,43 @@ main(int argc, char **argv)
 
 	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
 
-	use_font = (int *) GMT_memory (VNULL, (size_t)N_FONTS, sizeof (int), GMT_program);
+	load_parameters (&T, Ctrl);	/* Pass info from Ctrl to T */
+	
+	z_project.view_azimuth = Ctrl->E.azimuth;
+	z_project.view_elevation = Ctrl->E.elevation;
+	use_font = (int *) GMT_memory (VNULL, (size_t)GMT_N_FONTS, sizeof (int), GMT_program);
 	add = !(T.x_offset == 0.0 && T.y_offset == 0.0);
-	if (add && offset_outward) T.boxflag |= 64;
+	if (add && Ctrl->D.justify) T.boxflag |= 64;
 	if (n_files > 0)
 		nofile = FALSE;
 	else
 		n_files = 1;
 	n_args = (argc > 1) ? argc : 2;
-	
+
 	GMT_map_setup (west, east, south, north);
-	
-	ps_plotinit (CNULL, gmtdefs.overlay, gmtdefs.page_orientation, gmtdefs.x_origin, gmtdefs.y_origin,
-		gmtdefs.global_x_scale, gmtdefs.global_y_scale, gmtdefs.n_copies,
-		gmtdefs.dpi, GMT_INCH, gmtdefs.paper_width, gmtdefs.page_rgb, gmtdefs.encoding.name, GMT_epsinfo (argv[0]));
-		
-	GMT_echo_command (argc, argv);
-	if (gmtdefs.unix_time) GMT_timestamp (argc, argv);
+
+	GMT_plotinit (argc, argv);
+
 	if (project_info.three_D) ps_transrotate (-z_project.xmin, -z_project.ymin, 0.0);
-	
-	if (set_z) {
-		project_info.z_level = new_z_level;
-		GMT_z_to_zz (new_z_level, &z_level);
+
+	if (Ctrl->Z.active) {
+		project_info.z_level = Ctrl->Z.level;
+		GMT_z_to_zz (Ctrl->Z.level, &z_level);
 	}
 	GMT_setpen (&T.txtpen);
 
-	if (clip) GMT_map_clip_on (GMT_no_rgb, 3);
-	
+	if (!Ctrl->N.active) GMT_map_clip_on (GMT_no_rgb, 3);
+
 	last_font = 0;
 	ix = (gmtdefs.xy_toggle[0]);	iy = 1 - ix;
-	
+
 	/* if (draw_box && project_info.three_D) draw_box = FALSE; */	/* Not implemented yet */
-	
+
 	/* Mark used fonts */
-	
+
 	if (strcmp (gmtdefs.encoding.name, "Standard") != 0) {
-		memset (use_font, 0, N_FONTS * sizeof (int));
-		if (gmtdefs.unix_time) use_font[0] = use_font[1] = TRUE;
+		memset (use_font, 0, GMT_N_FONTS * sizeof (int));
+		if (GMT_ps.unix_time) use_font[0] = use_font[1] = TRUE;
 		use_font[gmtdefs.annot_font[0]] = TRUE;
 		use_font[gmtdefs.annot_font[1]] = TRUE;
 		if (frame_info.header[0]) use_font[gmtdefs.header_font] = TRUE;
@@ -385,14 +426,14 @@ main(int argc, char **argv)
 		}
 
 		if (!nofile && gmtdefs.verbose) fprintf (stderr, "%s: Working on file %s\n", GMT_program, argv[fno]);
-		if (gmtdefs.io_header[GMT_IN]) for (i = 0; i < gmtdefs.n_header_recs; i++) fgets (line, BUFSIZ, fp);
+		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) fgets (line, BUFSIZ, fp);
 
 		while (fgets (line, BUFSIZ, fp)) {
 
-			if (line[0] == '\0' || (line[0] == '#' && GMT_io.EOF_flag != '#')) continue;	/* Skip blank lines or # comments */
-			
+			if (line[0] == '\0' || (line[0] == '#' && GMT_io.EOF_flag[GMT_IN] != '#')) continue;	/* Skip blank lines or # comments */
+
 			if (do_paragraphs) {	/* Paragraph mode */
-				if (line[0] == GMT_io.EOF_flag) {
+				if (line[0] == GMT_io.EOF_flag[GMT_IN]) {
 
 					skip_text_records = FALSE;
 					if (n_processed) {	/* Must output what we got */
@@ -406,19 +447,19 @@ main(int argc, char **argv)
 						fprintf (stderr, "%s: Record %d had bad x and/or y coordinates, must exit)\n", GMT_program, n_read);
 						exit (EXIT_FAILURE);
 					}
-					
+
 					if (nscan != 9) {
 						fprintf (stderr, "%s: Record %d had incomplete paragraph information, must exit)\n", GMT_program, n_read);
 						exit (EXIT_FAILURE);
 					}
 					GMT_geo_to_xy (xy[0], xy[1], &plot_x, &plot_y);
-					if (clip) {
+					if (!Ctrl->N.active) {
 						skip_text_records = TRUE;	/* If this record should be skipped we must skip the whole paragraph */
 						GMT_map_outside (xy[0], xy[1]);
 						if ( abs (GMT_x_status_new) > 1 || abs (GMT_y_status_new) > 1) continue;
 						skip_text_records = FALSE;	/* Since we got here we do not want to skip */
 					}
-					if (convert_az) {
+					if (Ctrl->A.active) {
 						GMT_azim_to_angle (xy[0], xy[1], 0.1, T.paragraph_angle, &tmp);
 						T.paragraph_angle = fmod (tmp + 360.0 + 90.0, 180.0) - 90.0;	/* Ensure usable angles for text plotting */
 					}
@@ -431,8 +472,8 @@ main(int argc, char **argv)
 					if (this_font[0] >= '0' && this_font[0] <= '9')	/* Gave a font # */
 						T.paragraph_font = atoi (this_font);
 					else {	/* Gave a font name */
-						T.paragraph_font = GMT_font_lookup (this_font, GMT_font, N_FONTS);
-						if (T.paragraph_font == N_FONTS) {
+						T.paragraph_font = GMT_font_lookup (this_font, GMT_font, GMT_N_FONTS);
+						if (T.paragraph_font == GMT_N_FONTS) {
 							fprintf (stderr, "%s: Record %d had bad font (set to %s (0))\n", GMT_program, n_read, GMT_font[0].name);
 							T.paragraph_font = 0;
 						}
@@ -462,9 +503,8 @@ main(int argc, char **argv)
 							word = (char **) GMT_memory (VNULL, n_alloc, sizeof (char *), GMT_program);
 						}
 					}
-
-					p = strtok (line, " ");
-					while (p) {
+					pos = 0;
+					while ((GMT_strtok (line, " ", &pos, p))) {
 						word[n_words] = (char *) GMT_memory (VNULL, (size_t)(strlen(p)+1), sizeof (char), GMT_program);
 						strcpy (word[n_words], p);
 						n_words++;
@@ -472,7 +512,6 @@ main(int argc, char **argv)
 							n_alloc += GMT_CHUNK;
 							word = (char **) GMT_memory (VNULL, n_alloc, sizeof (char *), GMT_program);
 						}
-						p = strtok (NULL, " ");
 					}
 
 					n_processed++;
@@ -491,39 +530,41 @@ main(int argc, char **argv)
 				}
 				n_read++;
 				GMT_geo_to_xy (xy[0], xy[1], &plot_x, &plot_y);
-				if (clip) {
+				xx[0] = plot_x;	yy[0] = plot_y;
+				if (!Ctrl->N.active) {
 					GMT_map_outside (xy[0], xy[1]);
 					if ( abs (GMT_x_status_new) > 1 || abs (GMT_y_status_new) > 1) continue;
 				}
-			
+
 				T.block_justify = (isdigit ((int)just_key[0])) ? atoi (just_key) : GMT_just_decode (just_key, -1, -1);
 				if (T.block_justify == -99) {
 					fprintf (stderr, "%s: Record %d had bad justification info (skipped)\n", GMT_program, n_read);
 					continue;
 				}
-				if (convert_az) {
+				if (Ctrl->A.active) {
 					GMT_azim_to_angle (xy[0], xy[1], 0.1, T.paragraph_angle, &tmp);
 					T.paragraph_angle = fmod (tmp + 360.0 + 90.0, 180.0) - 90.0;	/* Ensure usable angles for text plotting */
 				}
 				if (add) {
-					if (offset_outward) {	/* Smart offset according to justification (from Dave Huang) */
+					if (Ctrl->D.justify) {	/* Smart offset according to justification (from Dave Huang) */
 						GMT_smart_justify (T.block_justify, T.paragraph_angle, T.x_offset, T.y_offset, &plot_x, &plot_y);
 					} else {	/* Default hard offset */
 						plot_x += T.x_offset;
 						plot_y += T.y_offset;
 					}
+					xx[1] = plot_x;	yy[1] = plot_y;
 				}
 				if (this_font[0] >= '0' && this_font[0] <= '9')	/* Gave a font # */
 					T.paragraph_font = atoi (this_font);
 				else {	/* Gave a font name */
-					T.paragraph_font = GMT_font_lookup (this_font, GMT_font, N_FONTS);
-					if (T.paragraph_font == N_FONTS) {
+					T.paragraph_font = GMT_font_lookup (this_font, GMT_font, GMT_N_FONTS);
+					if (T.paragraph_font == GMT_N_FONTS) {
 						fprintf (stderr, "%s: Record %d had bad font (set to %s (0))\n", GMT_program, n_read, GMT_font[0].name);
 						T.paragraph_font = 0;
 					}
 				}
 				n_paragraphs++;
-				
+
 				if (T.paragraph_font != last_font) {
 					ps_setfont (T.paragraph_font);
 					last_font = T.paragraph_font;
@@ -545,14 +586,19 @@ main(int argc, char **argv)
 					GMT_textbox3D (plot_x, plot_y, z_level, T.font_size, T.paragraph_font, text, T.paragraph_angle, T.block_justify, (T.boxflag & 5), dx, dy, T.boxfill.rgb);
 					GMT_setpen (&T.txtpen);
 				}
-				if (outline) {
+				if (Ctrl->S.active) {
 					GMT_setpen (&T.txtpen);
 					GMT_text3D (plot_x, plot_y, z_level, T.font_size, T.paragraph_font, text, T.paragraph_angle, T.block_justify, TRUE);
 				}
-				if (solid) {
+				if (Ctrl->G.active) {
 					ps_setpaint (T.txtfill.rgb);
 					GMT_text3D (plot_x, plot_y, z_level, T.font_size, T.paragraph_font, text, T.paragraph_angle, T.block_justify, FALSE);
 				}
+				if (T.boxflag & 32) {	/* Draw line from original point to shifted location */
+					GMT_setpen (&T.vecpen);
+					if (project_info.three_D) GMT_2D_to_3D (xx, yy, project_info.z_level, 2);
+					ps_segment (xx[0], yy[0], xx[1], yy[1]);
+				}
 			}
 		} /* Go read next line */
 
@@ -563,18 +609,18 @@ main(int argc, char **argv)
 		}
 		if (fp != GMT_stdin) fclose (fp);
 	}
-	
+
  	if (do_paragraphs) GMT_free ((void *)word);
 
-	if (clip) GMT_map_clip_off (); 
+	if (!Ctrl->N.active) GMT_map_clip_off (); 
 
 	GMT_world_map = old_GMT_world_map;
 
 	GMT_map_basemap ();
-		
+
 	if (project_info.three_D) ps_rotatetrans (z_project.xmin, z_project.ymin, 0.0);
 
-	ps_plotend (gmtdefs.last_page);
+	GMT_plotend ();
 
 	if (gmtdefs.verbose) {
 		if (do_paragraphs)
@@ -582,13 +628,17 @@ main(int argc, char **argv)
 		else
 			fprintf (stderr, "pstext: Plotted %d text-strings\n", n_paragraphs);
 	}
-	
+
 	GMT_free ((void *)use_font);
-	
+
+	Free_Pstext_Ctrl (Ctrl);	/* Deallocate control structure */
+
 	GMT_end (argc, argv);
+
+	exit (EXIT_SUCCESS);
 }
 
-void GMT_putwords (double x, double y, char **text, int n_words, struct GMT_TEXT *T) {
+void GMT_putwords (double x, double y, char **text, int n_words, struct PSTEXT_INFO *T) {
 	char *boxpen_texture = CNULL, *vecpen_texture = CNULL;
 	int boxpen_width, boxpen_offset, boxpen_rgb[3];
 	int vecpen_width, vecpen_offset, vecpen_rgb[3];
@@ -619,3 +669,60 @@ void GMT_putwords (double x, double y, char **text, int n_words, struct GMT_TEXT
 		GMT_free ((void *)text[i]);
 	}
 }
+
+void load_parameters (struct PSTEXT_INFO *T, struct PSTEXT_CTRL *C) {
+	T->x_space = C->C.dx;
+	T->y_space = C->C.dy;
+	T->space_flag = (C->C.percent) ? 1 : 0;
+	if (C->D.active) {
+		T->x_offset = C->D.dx;
+		T->y_offset = C->D.dy;
+		if (C->D.line) T->boxflag |= 32;
+		T->vecpen = C->D.pen;
+	}
+	if (C->G.active) {
+		T->txtfill = C->G.fill;
+	}
+	if (C->S.active) {
+		T->txtpen = C->S.pen;
+	}
+	if (C->W.active) {
+		if (C->W.outline) T->boxflag |= 1;	/* Want box outline */
+		if (C->W.mode == 'O') T->boxflag |= 4;	/* Want rounded box outline */
+		if (C->W.mode == 'c') T->boxflag |= 8;	/* Want concave box outline */
+		if (C->W.mode == 'C') T->boxflag |= 16;	/* Want convex box outline */
+		T->boxpen = C->W.pen;
+		T->boxfill = C->W.fill;
+		if (C->W.paint) T->boxflag |= 2;
+		if (!(T->boxflag | 31)) {
+			fprintf (stderr, "%s: GMT SYNTAX ERROR -W option: Must give arguments\n", GMT_program);
+			exit (EXIT_FAILURE);
+		}
+	}
+
+}
+
+void *New_Pstext_Ctrl () {	/* Allocate and initialize a new control structure */
+	struct PSTEXT_CTRL *C;
+	
+	C = (struct PSTEXT_CTRL *) GMT_memory (VNULL, 1, sizeof (struct PSTEXT_CTRL), "New_Pstext_Ctrl");
+	
+	/* Initialize values whose defaults are not 0/FALSE/NULL */
+		
+	GMT_init_pen (&C->D.pen, GMT_PENWIDTH);
+	C->C.dx = C->C.dy = 15.0;	/* 15% of font size is default clearance */
+	C->C.percent = TRUE;
+	C->E.azimuth = 180.0;
+	C->E.elevation = 90.0;
+	C->G.active = TRUE;
+	GMT_init_fill (&C->G.fill, 0, 0, 0);
+	GMT_init_fill (&C->W.fill, -1, -1, -1);	/* No fill */
+	GMT_init_pen (&C->S.pen, GMT_PENWIDTH);
+	GMT_init_pen (&C->W.pen, GMT_PENWIDTH);
+
+	return ((void *)C);
+}
+
+void Free_Pstext_Ctrl (struct PSTEXT_CTRL *C) {	/* Deallocate control structure */
+	GMT_free ((void *)C);	
+}
