@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: sample1d.c,v 1.3.4.7 2004/01/02 22:23:11 pwessel Exp $
+ *	$Id: sample1d.c,v 1.26 2006/04/13 08:34:41 pwessel Exp $
  *
- *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -24,101 +24,128 @@
  * increasing variable.
  *
  * Author:	Paul Wessel
- * Date:	05-JUL--2000
- * Version:	3.4.3
+ * Date:	05-JUL-2000
+ * Version:	4
  *
  */
  
 #include "gmt.h"
 
-main (int argc, char **argv)
+struct SAMPLE1D_CTRL {
+	struct F {	/* -Fl|a|c */
+		BOOLEAN active;
+		int mode;
+	} F;
+	struct I {	/* -I<inc> */
+		BOOLEAN active;
+		double inc;
+	} I;
+	struct T {	/* -T<time_col> */
+		BOOLEAN active;
+		int col;
+	} T;
+	struct N {	/* -N<knotfile> */
+		BOOLEAN active;
+		char *file;
+	} N;
+	struct S {	/* -S<xstart> */
+		BOOLEAN active;
+		double start;
+	} S;
+};
+
+int main (int argc, char **argv)
 {
-	int i, j, k, t = 0, n, m, n_alloc, m_alloc, result, n_col, rows = 1, n_fields, n_expected_fields;
-	int n_files = 0, fno, n_args, n_req, m_supplied;
-	
-	BOOLEAN error = FALSE, equidist = FALSE, t_supplied = FALSE, *nan_flag;
-	BOOLEAN got_t0 = FALSE, got_dt = FALSE, nofile = TRUE, done = FALSE;
-	
-	double *t_supplied_out, *t_out, *ttime, *data, **col, **out;
-	double dt = 0.0, t0 = 0.0, tt, low_t, high_t, *in, *dout;
-	
+	int i, j, k, n, m, n_alloc = 0, m_alloc, result, n_col = 0, rows = 1, n_fields, n_expected_fields;
+	int n_files = 0, fno, n_args, n_req, m_supplied = 0;
+
+	BOOLEAN error = FALSE, *nan_flag = VNULL, nofile = TRUE, done = FALSE;
+
+	double *t_supplied_out = VNULL, *t_out, *ttime, *data, **col = VNULL, **out = VNULL;
+	double tt, low_t, high_t, *in, *dout = VNULL;
+
 	char line[BUFSIZ], type[3];
-	
+
 	FILE *fp = NULL, *fpt = NULL;
-	
-	type[0] = 'l';	type[1] = 'a';	type[2] = 'c';
+
+	struct SAMPLE1D_CTRL *Ctrl;
+
+	void *New_Sample1d_Ctrl (), Free_Sample1d_Ctrl (struct SAMPLE1D_CTRL *C);
 	
 	argc = GMT_begin (argc, argv);
 
+	Ctrl = (struct SAMPLE1D_CTRL *)New_Sample1d_Ctrl ();	/* Allocate and initialize a new control structure */
+	
+	type[0] = 'l';	type[1] = 'a';	type[2] = 'c';
+
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '-') {
 			switch (argv[i][1]) {
-		
+
 				/* Common parameters */
-			
+
 				case 'H':
 				case 'V':
+				case 'b':
+				case 'f':
 				case '\0':
-					error += GMT_get_common_args (argv[i], 0, 0, 0, 0);
+					error += GMT_parse_common_options (argv[i], 0, 0, 0, 0);
 					break;
 
 				/* Supplemental parameters */
-				
-				case 'b':
-					error += GMT_io_selection (&argv[i][2]);
-					break;
+
 				case 'F':
+					Ctrl->F.active = TRUE;
 					switch (argv[i][2]) {
 						case 'l':
 						case 'L':
-							gmtdefs.interpolant = 0;
+							Ctrl->F.mode = 0;
 							break;
 						case 'a':
 						case 'A':
-							gmtdefs.interpolant = 1;
+							Ctrl->F.mode = 1;
 							break;
 						case 'c':
 						case 'C':
-							gmtdefs.interpolant = 2;
+							Ctrl->F.mode = 2;
 							break;
 						default:	/* Use GMT defaults */
 							break;
 					}
 					break;
 				case 'I':
-					dt = atof (&argv[i][2]);
-					equidist = TRUE;
-					got_dt = TRUE;
+					Ctrl->I.active = TRUE;
+					Ctrl->I.inc = atof (&argv[i][2]);
 					break;
 				case 'T':
-					t = atoi (&argv[i][2]);
+					Ctrl->T.active = TRUE;
+					Ctrl->T.col = atoi (&argv[i][2]);
 					break;
 				case 'M':               /* Multiple line segments */
 					GMT_multisegment (&argv[i][2]);
 					break;
 				case 'N':
-					if ((fpt = fopen (&argv[i][2], "r")) == NULL) {
-						fprintf (stderr, "%s: Cannot open file %s\n", GMT_program, &argv[i][2]);
-						exit (EXIT_FAILURE);
-					}
-					t_supplied = TRUE;
+					Ctrl->N.file = strdup (&argv[i][2]);
+					Ctrl->N.active = TRUE;
 					break;
 				case 'S':
-					t0 = atof (&argv[i][2]);
-					equidist = TRUE;
-					got_t0 = TRUE;
+					Ctrl->S.active = TRUE;
+					Ctrl->S.start = atof (&argv[i][2]);
 					break;
-					
+
 				/* For backward compatibility for now */
 
 				case 'L':
-					gmtdefs.interpolant = 0;
+					Ctrl->F.active = TRUE;
+					Ctrl->F.mode = 0;
 					break;
 				case 'A':
-					gmtdefs.interpolant = 1;
+					Ctrl->F.active = TRUE;
+					Ctrl->F.mode = 1;
 					break;
 				case 'C':
-					gmtdefs.interpolant = 2;
+					Ctrl->F.active = TRUE;
+					Ctrl->F.mode = 2;
 					break;
 
 				default:
@@ -129,16 +156,16 @@ main (int argc, char **argv)
 		}
 		else
 			n_files++;
-		
+
 	}
 
-	if (argc == 1 || GMT_quick) {	/* Display usage */
+	if (argc == 1 || GMT_give_synopsis_and_exit) {	/* Display usage */
 		fprintf (stderr,"sample1d %s - Resampling of 1-D data sets\n\n", GMT_VERSION);
-		fprintf (stderr, "usage: sample1d <infile(s)> [-Fl|a|c] [-H[<nrec>]] [-I<t_inc>] [-M[<flag>]] [-N<knotfile>]\n");
-		fprintf (stderr, "\t[-S<xstart>] [-T<time_col>] [-V] [-bi[s][<n>]] [-bo[s][<n>]]\n\n");
-		
-		if (GMT_quick) exit (EXIT_FAILURE);
-		
+		fprintf (stderr, "usage: sample1d <infile(s)> [-Fl|a|c] [%s] [-I<t_inc>] [%s] [-N<knotfile>]\n", GMT_H_OPT, GMT_M_OPT);
+		fprintf (stderr, "\t[-S<xstart>] [-T<time_col>] [-V] [%s] [%s]\n\n", GMT_b_OPT, GMT_f_OPT);
+
+		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
+
 		fprintf (stderr, "\t<infile> is one or more multicolumn ASCII (or binary, see -b) tables. [Default is standard input]\n");
 		fprintf (stderr, "\tThe independent variable (see -T) must be monotonically in/de-creasing\n");
 		fprintf (stderr, "\n\tOPTIONS:\n");
@@ -146,69 +173,79 @@ main (int argc, char **argv)
 		fprintf (stderr, "\t   l Linear interpolation\n");
 		fprintf (stderr, "\t   a Akima spline interpolation\n");
 		fprintf (stderr, "\t   c Cubic spline interpolation\n");
-		fprintf (stderr, "\t   [Default is -F%c]\n", type[gmtdefs.interpolant]);
+		fprintf (stderr, "\t   [Default is -F%c]\n", type[Ctrl->F.mode]);
 		GMT_explain_option ('H');
-		fprintf (stderr, "\t-I <x_inc> sets equidistant grid interval\n");
+		fprintf (stderr, "\t-I <x_inc> sets equidistant grid interval [x1 - x0]\n");
 		GMT_explain_option ('M');
 		fprintf (stderr, "\t-N <knotfile> is an ASCII table with the desired time positions in column 0\n");
 		fprintf (stderr, "\t   Overrides the -I and -S settings.  If none of -I, -S, and -N is set\n");
 		fprintf (stderr, "\t   then <tstart> = first input point, <t_inc> = (t[1] - t[0])\n");
-		fprintf (stderr, "\t-S <xstart> sets the first output point\n");
+		fprintf (stderr, "\t-S <xstart> sets the first output point [first multiple of x_inc in range]\n");
 		fprintf (stderr, "\t-T gives column number of the independent variable (time) [Default is 0 (first)]\n");
 		GMT_explain_option ('V');
 		GMT_explain_option ('i');
 		GMT_explain_option ('n');
 		fprintf (stderr, "\t   Default is 2 input columns\n");
 		GMT_explain_option ('o');
+		GMT_explain_option ('n');
+		GMT_explain_option ('f');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
-	
-	if (t < 0) {
+
+	if (Ctrl->T.col < 0) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -T option: Column number cannot be negative\n", GMT_program);
 		error++;
 	}
-	if (t_supplied && equidist) {
+	GMT_io.skip_if_NaN[0] = GMT_io.skip_if_NaN[1] = FALSE;	/* Turn off default GMT NaN-handling */
+	GMT_io.skip_if_NaN[Ctrl->T.col] = TRUE;				/* ... But disallow NaN in "time" column */
+	if (Ctrl->N.active && Ctrl->I.active) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR: Specify only one of -N and -S\n", GMT_program);
 		error++;
 	}
-	if (got_dt && dt <= 0.0) {
+	if (Ctrl->I.active && Ctrl->I.inc <= 0.0) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -I option: Must specify positive increment\n", GMT_program);
 		error++;
 	}
-	if (GMT_io.binary[0] && gmtdefs.io_header) {
+	if (GMT_io.binary[GMT_IN] && GMT_io.io_header[GMT_IN]) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data cannot have header -H\n", GMT_program);
 		error++;
 	}
-        if (GMT_io.binary[0] && GMT_io.ncol[0] == 0) GMT_io.ncol[0] = 2;
-	n_req = (t >= 2) ? t + 1 : 2;
-        if (GMT_io.binary[0] && GMT_io.ncol[0] < n_req) {
+        if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] == 0) GMT_io.ncol[GMT_IN] = 2;
+	n_req = (Ctrl->T.col >= 2) ? Ctrl->T.col + 1 : 2;
+        if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] < n_req) {
                 fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data (-bi) must have at least %d columns\n", GMT_program, n_req);
 		error++;
 	}
-	
+	if (Ctrl->N.active && (fpt = GMT_fopen (Ctrl->N.file, "r")) == NULL) {
+		fprintf (stderr, "%s: GMT SYNTAX ERROR -N. Cannot open file %s\n", GMT_program, Ctrl->N.file);
+		error++;
+	}
+
 	if (error) exit (EXIT_FAILURE);
 
-	GMT_put_history (argc, argv);	/* Update .gmtcommands */
-	
-	if (GMT_io.binary[0] && gmtdefs.verbose) {
+	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
+
+	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[0], type[GMT_io.single_precision[0]]);
+		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 #ifdef SET_IO_MODE
-	GMT_setmode (1);
+	GMT_setmode (GMT_OUT);
 #endif
 
+	gmtdefs.interpolant = Ctrl->F.mode;
+	
 	t_out = (double *)NULL;
 
-	if (t_supplied) {	/* read file with abscissae */
+	if (Ctrl->N.active) {	/* read file with abscissae */
 		m_alloc = GMT_CHUNK;
 		t_supplied_out = (double *) GMT_memory (VNULL, (size_t)m_alloc, sizeof (double), GMT_program);
 		m = 0;
-		if (gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) fgets (line, BUFSIZ, fpt);
+		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) fgets (line, BUFSIZ, fpt);
 		while (fgets (line, BUFSIZ, fpt)) {
-			if (line[0] == '#' || line[0] == GMT_io.EOF_flag) continue;
+			if (line[0] == '#' || line[0] == GMT_io.EOF_flag[GMT_IN]) continue;
 			sscanf (line, "%lf", &t_supplied_out[m]);
 			m++;
 			if (m == m_alloc) {	/* Get more memory */
@@ -216,7 +253,7 @@ main (int argc, char **argv)
 				t_supplied_out = (double *) GMT_memory ((void *)t_supplied_out, (size_t)m_alloc, sizeof (double), GMT_program);
 			}
 		}
-		fclose (fpt);
+		GMT_fclose (fpt);
 		m_supplied = m;
 		t_supplied_out = (double *) GMT_memory ((void *)t_supplied_out, (size_t)m_supplied, sizeof (double), GMT_program);
 		t_out = (double *) GMT_memory (VNULL, (size_t)m_supplied, sizeof (double), GMT_program);
@@ -232,12 +269,12 @@ main (int argc, char **argv)
 
 	for (fno = 1; !done && fno < n_args; fno++) {	/* Loop over input files, if any */
 		if (!nofile && argv[fno][0] == '-') continue;
-		
+
 		if (nofile) {	/* Just read standard input */
 			fp = GMT_stdin;
 			done = TRUE;
 #ifdef SET_IO_MODE
-			GMT_setmode (0);
+			GMT_setmode (GMT_IN);
 #endif
 		}
 		else if ((fp = GMT_fopen (argv[fno], GMT_io.r_mode)) == NULL) {
@@ -247,17 +284,17 @@ main (int argc, char **argv)
 
 		if (!nofile && gmtdefs.verbose) fprintf (stderr, "%s: Working on file %s\n", GMT_program, argv[fno]);
 
-		if (gmtdefs.io_header) {
-			for (i = 0; i < gmtdefs.n_header_recs; i++) {
+		if (GMT_io.io_header[GMT_IN]) {
+			for (i = 0; i < GMT_io.n_header_recs; i++) {
 				GMT_fgets (line, BUFSIZ, fp);
-				fprintf (GMT_stdout, "%s", line);
+				if (GMT_io.io_header[GMT_OUT]) fprintf (GMT_stdout, "%s", line);
 			}
 		}
-	
-		n_expected_fields = (GMT_io.ncol[0]) ? GMT_io.ncol[0] : BUFSIZ;
+
+		n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : BUFSIZ;
 		i = n_col = 0;
 		n_fields = GMT_input (fp, &n_expected_fields, &in);
-	
+
 		while (! (GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
 			while (GMT_io.status & GMT_IO_SEGMENT_HEADER) {
@@ -268,7 +305,7 @@ main (int argc, char **argv)
 
 			i++;
 			n = 0;
-	
+
 			while (! (GMT_io.status & (GMT_IO_SEGMENT_HEADER | GMT_IO_EOF))) {	/* Keep going until FALSE or = 2 segment header */
 				if (GMT_io.status & GMT_IO_MISMATCH) {
 					fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, n_expected_fields, i);
@@ -277,8 +314,8 @@ main (int argc, char **argv)
 
 				if (n_col == 0) {	/* Allocate memory first time around */
 					n_col = n_expected_fields;
-					if (t >= n_col) {
-						fprintf (stderr, "%s: time_col = %d exceeds range of columns (%d)!\n", GMT_program, t, n_col);
+					if (Ctrl->T.col >= n_col) {
+						fprintf (stderr, "%s: time_col = %d exceeds range of columns (%d)!\n", GMT_program, Ctrl->T.col, n_col);
 						exit (EXIT_FAILURE);
 					}
 					dout = (double *) GMT_memory (VNULL, (size_t)n_col, sizeof (double), GMT_program);
@@ -288,7 +325,7 @@ main (int argc, char **argv)
 					n_alloc = GMT_CHUNK;
 					for (j = 0; j < n_col; j++) col[j] = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
 				}
-		
+
 				for (j = 0; j < n_col; j++) {
 					if (GMT_is_dnan (in[j])) {
 						nan_flag[j] = TRUE;
@@ -298,7 +335,7 @@ main (int argc, char **argv)
 						col[j][n] = in[j];
 				}
 				n++;
-		
+
 				if (n == n_alloc) {	/* Get more memory */
 					n_alloc += GMT_CHUNK;
 					for (j = 0; j < n_col; j++) col[j] = (double *) GMT_memory ((void *)col[j], (size_t)n_alloc, sizeof (double), GMT_program);
@@ -306,100 +343,100 @@ main (int argc, char **argv)
 
 				n_fields = GMT_input (fp, &n_expected_fields, &in);
 			}
-			
+
 			/* Here we have one segment to work on */
 
 			/* If we didn't get input abscissa, now's the time to generate them */
 
-			if (t_supplied) {	/* Get relevant t_out segment */
-				low_t  = MIN (col[t][0], col[t][n-1]);
-				high_t = MAX (col[t][0], col[t][n-1]);
+			if (Ctrl->N.active) {	/* Get relevant t_out segment */
+				low_t  = MIN (col[Ctrl->T.col][0], col[Ctrl->T.col][n-1]);
+				high_t = MAX (col[Ctrl->T.col][0], col[Ctrl->T.col][n-1]);
 				for (i = m = 0; i < m_supplied; i++) {
 					if (t_supplied_out[i] < low_t || t_supplied_out[i] > high_t) continue;
 					t_out[m++] = t_supplied_out[i];
 				}
-				if (m == 0) fprintf (stderr, "%s: Warning: No output points for range %lg to %lg\n", GMT_program, col[t][0], col[t][n-1]);
+				if (m == 0) fprintf (stderr, "%s: Warning: No output points for range %g to %g\n", GMT_program, col[Ctrl->T.col][0], col[Ctrl->T.col][n-1]);
 			}
 			else {	/* Generate evenly spaced grid */
-				if (!got_dt) dt = col[t][1] - col[t][0];
-				if (got_dt && (col[t][1] - col[t][0]) < 0.0 && dt > 0.0) dt = -dt;	/* For monotonically decreasing data */
-				if (!got_t0) {
-					if (dt > 0.0) {
-						t0 = floor (col[t][0] / dt) * dt;
-						if (t0 < col[t][0]) t0 += dt;
+				if (!Ctrl->I.active) Ctrl->I.inc = col[Ctrl->T.col][1] - col[Ctrl->T.col][0];
+				if (Ctrl->I.active && (col[Ctrl->T.col][1] - col[Ctrl->T.col][0]) < 0.0 && Ctrl->I.inc > 0.0) Ctrl->I.inc = -Ctrl->I.inc;	/* For monotonically decreasing data */
+				if (!Ctrl->S.active) {
+					if (Ctrl->I.inc > 0.0) {
+						Ctrl->S.start = floor (col[Ctrl->T.col][0] / Ctrl->I.inc) * Ctrl->I.inc;
+						if (Ctrl->S.start < col[Ctrl->T.col][0]) Ctrl->S.start += Ctrl->I.inc;
 					}
 					else {
-						t0 = ceil (col[t][0] / (-dt)) * (-dt);
-						if (t0 > col[t][0]) t0 += dt;
+						Ctrl->S.start = ceil (col[Ctrl->T.col][0] / (-Ctrl->I.inc)) * (-Ctrl->I.inc);
+						if (Ctrl->S.start > col[Ctrl->T.col][0]) Ctrl->S.start += Ctrl->I.inc;
 					}
 				}
-				m = m_alloc = irint (fabs((col[t][n-1] - t0) / dt)) + 1;
+				m = m_alloc = irint (fabs((col[Ctrl->T.col][n-1] - Ctrl->S.start) / Ctrl->I.inc)) + 1;
 				t_out = (double *) GMT_memory ((void *)t_out, (size_t)m_alloc, sizeof (double), GMT_program);
-				t_out[0] = t0;
+				t_out[0] = Ctrl->S.start;
 				i = 1;
-				if (dt > 0.0) {
-					while (i < m && (tt = t0 + i * dt) <= col[t][n-1]) {
+				if (Ctrl->I.inc > 0.0) {
+					while (i < m && (tt = Ctrl->S.start + i * Ctrl->I.inc) <= col[Ctrl->T.col][n-1]) {
 						t_out[i] = tt;
 						i++;
 					}
 				}
 				else {
-					while (i < m && (tt = t0 + i * dt) >= col[t][n-1]) {
+					while (i < m && (tt = Ctrl->S.start + i * Ctrl->I.inc) >= col[Ctrl->T.col][n-1]) {
 						t_out[i] = tt;
 						i++;
 					}
 				}
 				m = i;
-				if (fabs (t_out[m-1]-col[t][n-1]) < SMALL) {	/* Fix roundoff */
-					t_out[m-1] = col[t][n-1];
+				if (fabs (t_out[m-1]-col[Ctrl->T.col][n-1]) < GMT_SMALL) {	/* Fix roundoff */
+					t_out[m-1] = col[Ctrl->T.col][n-1];
 				}
 			}
 
-			if (nan_flag[t]) {
+			if (nan_flag[Ctrl->T.col]) {
 				fprintf (stderr, "%s: Independent column has NaN's!\n", GMT_program);
 				exit (EXIT_FAILURE);
 			}
 
 			for (j = 0; m && j < n_col; j++) {
 
-				if (j == t) continue;	/* Skip the time column */
+				if (j == Ctrl->T.col) continue;	/* Skip the time column */
 
 				out[j] = (double *) GMT_memory (VNULL, (size_t)m, sizeof (double), GMT_program);
-		
+
 				if (nan_flag[j]) {	/* NaN's present, need "clean" time and data columns */
 
 					ttime = (double *) GMT_memory (VNULL, (size_t)n, sizeof (double), GMT_program);
 					data = (double *) GMT_memory (VNULL, (size_t)n, sizeof (double), GMT_program);
 					for (i = k = 0; i < n; i++) {
 						if ( GMT_is_dnan (col[j][i]) ) continue;
-						ttime[k] = col[t][i];
+						ttime[k] = col[Ctrl->T.col][i];
 						data[k] = col[j][i];
 						k++;
 					}
-					result = GMT_intpol (ttime, data, k, m, t_out, out[j], gmtdefs.interpolant);
+					result = GMT_intpol (ttime, data, k, m, t_out, out[j], Ctrl->F.mode);
 					GMT_free ((void *)ttime);
 					GMT_free ((void *)data);
 				}
 				else
-					result = GMT_intpol (col[t], col[j], n, m, t_out, out[j], gmtdefs.interpolant);
-				
+					result = GMT_intpol (col[Ctrl->T.col], col[j], n, m, t_out, out[j], Ctrl->F.mode);
+
 				if (result != 0) {
 					fprintf (stderr, "%s: Error from GMT_intpol near row %d!\n", GMT_program, rows+result+1);
 					exit (EXIT_FAILURE);
 				}
 			}
-	
-			out[t] = t_out;
+
+			out[Ctrl->T.col] = t_out;
 
 			for (i = 0; i < m; i++) {
 				for (j = 0; j < n_col; j++) dout[j] = out[j][i];
 				GMT_output (GMT_stdout, n_col, dout);
 			}
-			for (j = 0; m && j < n_col; j++) if (j != t) GMT_free ((void *)out[j]);
-		
+			for (j = 0; m && j < n_col; j++) if (j != Ctrl->T.col) GMT_free ((void *)out[j]);
+
 			rows += n + 1;
 		}
-	
+
 		if (fp != GMT_stdin) GMT_fclose(fp);
 	}
 
@@ -408,6 +445,26 @@ main (int argc, char **argv)
 	GMT_free ((void *)t_out);
 	GMT_free ((void *)out);
 	GMT_free ((void *)col);
-	
+
+	Free_Sample1d_Ctrl (Ctrl);	/* Deallocate control structure */
+
 	GMT_end (argc, argv);
+
+	exit (EXIT_SUCCESS);
+}
+
+void *New_Sample1d_Ctrl () {	/* Allocate and initialize a new control structure */
+	struct SAMPLE1D_CTRL *C;
+	
+	C = (struct SAMPLE1D_CTRL *) GMT_memory (VNULL, 1, sizeof (struct SAMPLE1D_CTRL), "New_Sample1d_Ctrl");
+	
+	/* Initialize values whose defaults are not 0/FALSE/NULL */
+	C->F.mode = gmtdefs.interpolant;
+		
+	return ((void *)C);
+}
+
+void Free_Sample1d_Ctrl (struct SAMPLE1D_CTRL *C) {	/* Deallocate control structure */
+	if (C->N.file) GMT_free ((void *)C->N.file);	
+	GMT_free ((void *)C);	
 }
