@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: pssegy.c,v 1.5 2003/04/11 22:57:16 pwessel Exp $
+ *	$Id: pssegy.c,v 1.13 2006/04/10 05:47:30 pwessel Exp $
  *
- *    Copyright (c) 1999-2001 by T. Henstock
+ *    Copyright (c) 1999-2006 by T. Henstock
  *    See README file for copying and redistribution conditions.
  *--------------------------------------------------------------------*/
 /* pssegy program to plot segy files in postscript with variable trace spacing option
@@ -10,7 +10,7 @@
  *
  * Author:	Tim Henstock (then@soc.soton.ac.uk)
  * Date:	1-July-1996
- * Version:	1.0	
+ * Version:	1.0
  *
  * Bug fixes:	1.1, 11-6-96 remove undesirable normalization introduced by -Z option. Add -U option for reduction.
  *
@@ -27,11 +27,12 @@
  */
  
 #include "gmt.h"
+#include "pslib.h"
 #include "segy_io.h"
 
 /* internal function calls */
 	double rms( float *data, int nsamp);
-	void wig_bmap(double x0, float data0, float data1, double y0, double y1);	
+	void wig_bmap(double x0, float data0, float data1, double y0, double y1);
 	void shade_bmap(double x0, float data0, float data1, double y0, double y1, int negative);
 	int paint( int ix, int iy);
 	void plot_trace(float *data, double dy, double x0, int n_samp, int do_fill, int negative, int plot_wig, float toffset); 
@@ -42,49 +43,43 @@ unsigned char *bitmap;
 int bm_nx, bm_ny;
 
 
-main (int argc, char **argv)
+int main (int argc, char **argv)
 {
 	int error = FALSE;
 	int plot_cdp = FALSE, plot_offset = FALSE, byte_x = 0, doclip = FALSE;
 	int normalize = FALSE, do_fill = FALSE, negative = FALSE, plot_wig = FALSE;
 	int no_zero = FALSE, trace_file = FALSE;
 	int swap_bytes = !WORDS_BIGENDIAN;
-	
 	int i, nm;
-	int ix, iy, n_traces=10000, n_samp=0, n_sampr=0, shade[3], n_tracelist=0;
+	int ix, iy, n_traces=10000, n_samp=0, n_sampr=0, shade[3]={0,0,0}, trans[3]={-1,-1,-1}, n_tracelist=0;
+	int check, polarity=1;
+	BOOLEAN plot_it = FALSE;
 
-	int check, plot_it, polarity=1;
-	
+	float bias = 0.0, scale = 1.0, clip = 0.0, deviation = 0.0;
+	float toffset=0.0;
 	double w, e, s, n, dx = 1.0, dy = 0.0; /* dx, dy are trace and sample interval */
 	double xlen, ylen, xpix, ypix;
 	double x0, test, err_head=0.0;
-
-
-	float bias = 0.0, scale = 1.0, clip = 0.0, deviation = 0.0;
 	double redvel=0.0;
-	double *tracelist;
-	float toffset=0.0;
-	
+	double *tracelist = VNULL;
+
 	char input[512]="";
-
-
 	char reelhead[3200];
 	float *data;
 	SEGYHEAD *header;
 	char *head;
 	long *head2;
 	SEGYREEL binhead;
-	
+
 	FILE *fpi = NULL, *fpt = NULL;
-	
+
 
 	input[0] = 0;
 	w = e = s = n = 0.0;
-	shade[0]=shade[1]=shade[2]=0;
-	
+
 	argc = GMT_begin (argc, argv);
 
-	
+
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '-') {
 			switch (argv[i][1]) {
@@ -97,7 +92,7 @@ main (int argc, char **argv)
 				case 'K':
 				case 'P':
 				case '\0':
-					error += GMT_get_common_args (argv[i], &w, &e, &s, &n);
+					error += GMT_parse_common_options (argv[i], &w, &e, &s, &n);
 					break;
 				/* parameters for wiggle mode */
 				case 'F':
@@ -121,7 +116,7 @@ main (int argc, char **argv)
 					doclip = TRUE;
 					clip = (float) atof (&argv[i][2]);
  					break;
-				case 'B':	
+				case 'B':
 					bias = (float) atof (&argv[i][2]);
 					break;
 				case 'Z':
@@ -168,7 +163,7 @@ main (int argc, char **argv)
 				/* plot traces only at listed locations */
 				case 'T':
 					strcpy (input, &argv[i][2]);
-					if (trace_file = strlen(input)){
+					if ((trace_file = strlen(input))){
 						if ((fpt = fopen (input, "r")) == NULL){
 							fprintf(stderr, "%s: Cannot find trace list file %s\n", GMT_program, input);
 							error++;
@@ -195,18 +190,18 @@ main (int argc, char **argv)
 			exit (EXIT_FAILURE);
 		}
 	}
-		
 
 
-	if (argc == 1 || GMT_quick) {
+
+	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "pssegy %s - Plot a segy file in PostScript\n\n", GMT_VERSION);
-		fprintf (stderr, "usage: pssegy [<segyfile>]  -J<params> -R<w/e/s/n> -D<dev>  \n");
+		fprintf (stderr, "usage: pssegy [<segyfile>] %s %s -D<dev> \n", GMT_Jx_OPT, GMT_Rx_OPT);
 		fprintf (stderr, "	[-C<clip>] [-B<bias>] [-N] [-Z]\n");
 		fprintf (stderr, "	[-F<gray>|<r/g/b>] [-I] [-W] [-S<header>]\n");
 		fprintf (stderr, "	[-X<dx>] [-Y<dy>] [-L<nsamp>] [-M<ntraces>] \n");
 		fprintf (stderr, "	[-U<redvel>] [-T<tracefile>] [-E<slop>] [-A] [-O] [-K] [-P]\n");
-		
-		if (GMT_quick) exit (EXIT_FAILURE);
+
+		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
 		fprintf (stderr, 
 			"\n\t-Jx for projection.  Scale in INCH/units.  Specify one:\n\t -Jx<x-scale>              Linear projection\n\t-Jx<x-scale>l             Log10 projection\n\t  -Jx<x-scale>p<power>      x^power projection\n\tUse / to specify separate x/y scaling.\n\t If -JX is used then give axes lengths rather than scales\n\t regular map projections are not allowed!\n");
@@ -243,7 +238,7 @@ main (int argc, char **argv)
 		GMT_explain_option ('P');
 		exit (EXIT_FAILURE);
 	}
-	
+
 	if (err_head < 0.0){
 		error++;
 		fprintf(stderr,"%s: SYNTAX ERROR: slop cannot be negative\n", GMT_program);
@@ -293,12 +288,8 @@ main (int argc, char **argv)
 
 /* set up map projection and PS plotting */
 	GMT_map_setup (w, e, s, n);
-        ps_plotinit (CNULL, gmtdefs.overlay, gmtdefs.page_orientation, gmtdefs.x_origin, gmtdefs.y_origin,
-                gmtdefs.global_x_scale, gmtdefs.global_y_scale, gmtdefs.n_copies,
-                gmtdefs.dpi, GMT_INCH , gmtdefs.paper_width, gmtdefs.page_rgb,
-		     gmtdefs.encoding.name, GMT_epsinfo (argv[0]));
-	GMT_echo_command (argc, argv);
-	
+	GMT_plotinit (argc, argv);
+
 /* define area for plotting and size of array for bitmap */
 	xlen = project_info.xmax-project_info.xmin;
 	xpix = xlen*gmtdefs.dpi; /* pixels in x direction */
@@ -357,14 +348,14 @@ use a few of these*/
 		fprintf(stderr,"%s: Sample interval is %f s\n", GMT_program, dy);
 	}
 	else if ((dy != binhead.sr) && (binhead.sr)) /* value in header overridden by input */
-		fprintf(stderr, "%s: Warning dy input %lf, dy in header %f\n", GMT_program, dy, (float)binhead.sr);
-	
+		fprintf(stderr, "%s: Warning dy input %f, dy in header %f\n", GMT_program, dy, (float)binhead.sr);
+
 	if (!dy){ /* still no sample interval at this point is a problem! */
 		fprintf(stderr, "%s: Error, no sample interval in reel header\n", GMT_program);
 		exit(-1);
 	}
 
-	
+
 	bitmap = (unsigned char *) GMT_memory (NULL, nm, sizeof (unsigned char), "pssegy");
 
 	ix=0;
@@ -452,16 +443,17 @@ overlaps a defined header in a strange way */
 	}
 
 	GMT_map_clip_on (GMT_no_rgb, 3); /* set a clip at the map boundary since the image space overlaps a little */
-	ps_imagemask (0.0,0.0,xlen, ylen, bitmap, 8*bm_nx, bm_ny, polarity, shade); 
+	ps_bitimage (0.0,0.0,xlen, ylen, bitmap, 8*bm_nx, bm_ny, polarity, shade, trans);
 /* have to multiply by 8 since pslib version of ps_imagemask is based on a _pixel_ count, whereas pssegy uses _byte_ count internally */
 	GMT_map_clip_off ();
 
 	if (fpi != stdin) fclose (fpi);
-	
-	ps_plotend (gmtdefs.last_page);
-		
+
+	GMT_plotend ();
+
 	GMT_end (argc, argv);
 
+	exit (EXIT_SUCCESS);
 }
 
 double rms (float *data, int n_samp)
@@ -498,8 +490,7 @@ double y0 = 0.0, y1;
 	}
 }
 
-
-void wig_bmap(double x0, float data0, float data1, double y0, double y1) /* apply current sample with all options to bitmap */ 
+void wig_bmap(double x0, float data0, float data1, double y0, double y1) /* apply current sample with all options to bitmap */
 {
 double xp0, xp1, yp0, yp1, slope;
 int px0, px1, py0, py1, ix, iy;
