@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: pssegyz.c,v 1.9 2004/01/13 02:04:36 pwessel Exp $
+ *	$Id: pssegyz.c,v 1.19 2006/04/10 05:47:30 pwessel Exp $
  *
- *    Copyright (c) 1999-2001 by T. Henstock
+ *    Copyright (c) 1999-2006 by T. Henstock
  *    See README file for copying and redistribution conditions.
  *--------------------------------------------------------------------*/
 /* pssegyz program to plot segy files in 3d in postscript with variable trace spacing option
@@ -10,7 +10,7 @@
  *
  * Author:	Tim Henstock (Timothy.J.Henstock@soc.soton.ac.uk)
  * Date:	17-Nov-97
- * Version:	1.0	
+ * Version:	1.0
  *
  * heavily modified from pssegy version 1.2
  *
@@ -33,6 +33,7 @@
 
  
 #include "gmt.h"
+#include "pslib.h"
 #include "segy_io.h"
  
 /* internal function calls */
@@ -50,7 +51,7 @@ unsigned char *bitmap;
 int bm_nx, bm_ny;
 
 
-main (int argc, char **argv)
+int main (int argc, char **argv)
 {
 	int error = FALSE;
 	int xplot_cdp = FALSE, xplot_offset = FALSE, fix_x = FALSE, byte_x = 0; /* parameters for location plot */
@@ -59,12 +60,12 @@ main (int argc, char **argv)
 	int normalize = FALSE, do_fill = FALSE, negative = FALSE, plot_wig = FALSE;
 	int no_zero = FALSE, polarity=1;
 	int swap_bytes = !WORDS_BIGENDIAN;
-	
+
 	int i, nm;
-	int ix, iz, n_traces=10000, n_samp=0, n_sampr=0, shade[3]={0,0,0};
+	int ix, iz, n_traces=10000, n_samp=0, n_sampr=0, shade[3]={0,0,0}, trans[3]={-1,-1,-1};
 
 	int check;
-	
+
 	double w, e, s, n, dx = 1.0, dz = 0.0; /* dx, dz are trace and sample interval */
 	double xlen, ylen, xpix, ypix;
 	double x0 = 0.0 , y0 = 0.0;
@@ -73,7 +74,7 @@ main (int argc, char **argv)
 	float bias = 0.0, scale = 1.0, clip = 0.0, dev_x = 0.0, dev_y = 0.0;
 	double redvel=0.0;
 	float toffset=0.0;
-	
+
 	char input[512];
 	char *text;
 	char *head;
@@ -84,16 +85,15 @@ main (int argc, char **argv)
 	float *data;
 	SEGYHEAD *header;
 	SEGYREEL binhead;
-	
+
 	FILE *fpi = NULL;
-	
+
 
 	input[0] = 0;
 	w = e = s = n = 0.0;
-	
-	GMT_begin (argc, argv);
 
-	
+	argc = GMT_begin (argc, argv);
+
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '-') {
 			switch (argv[i][1]) {
@@ -106,7 +106,7 @@ main (int argc, char **argv)
 				case 'K':
 				case 'P':
 				case '\0':
-					error += GMT_get_common_args (argv[i], &w, &e, &s, &n);
+					error += GMT_parse_common_options (argv[i], &w, &e, &s, &n);
 					break;
                                 case 'E':
 					sscanf (&argv[i][2], "%lf/%lf", &z_project.view_azimuth, &z_project.view_elevation);
@@ -133,7 +133,7 @@ main (int argc, char **argv)
 					doclip = TRUE;
 					clip = (float) atof (&argv[i][2]);
  					break;
-				case 'B':	
+				case 'B':
 					bias = (float) atof (&argv[i][2]);
 					break;
 				case 'Z':
@@ -238,20 +238,20 @@ main (int argc, char **argv)
 			exit (EXIT_FAILURE);
 		}
 	}
-		
 
-	if (argc == 1 || GMT_quick) {
+
+	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "pssegyz %s - Plot a segy file in PostScript\n\n", GMT_VERSION);
-		fprintf (stderr, "usage: pssegyz [<segyfile>]  -J<params> -R<w/e/s/n> -D<dev>  \n");
+		fprintf (stderr, "usage: pssegyz [<segyfile>] %s %s -D<dev>\n", GMT_Jx_OPT, GMT_Rx_OPT);
 		fprintf (stderr, "	[-E<az/el>] [-C<clip>] [-B<bias>] [-N] [-Z]\n");
 		fprintf (stderr, "	[-F<gray>|<r/g/b>] [-I] [-W] [-S<x>/<y>]\n");
 		fprintf (stderr, "	[-X<dx>] [-Y<dz>] [-L<nsamp>] [-M<ntraces>] \n");
 		fprintf (stderr, "	[-U<redvel>] [-A] [-O] [-K] [-P]\n");
-		
 
 
 
-		if (GMT_quick) exit (EXIT_FAILURE);
+
+		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
 		fprintf (stderr, 
 			"\n\t-Jx for projection.  Scale in INCH/units.  Specify one:\n\t -Jx<x-scale>              Linear projection\n\t-Jx<x-scale>l             Log10 projection\n\t  -Jx<x-scale>p<power>      x^power projection\n\tUse / to specify separate x/y scaling.\n\t If -JX is used then give axes lengths rather than scales\n\t regular map projections may not work!\n");
@@ -321,21 +321,18 @@ main (int argc, char **argv)
  	}
  
  	if (error) exit (EXIT_FAILURE);
-	
+
 	if (fpi == NULL) fpi = stdin;
 
 	GMT_put_history (argc, argv);   /* Update .gmtcommands4 */
 
 /* set up map projection and PS plotting */
 	GMT_map_setup (w, e, s, n);
-        ps_plotinit (CNULL, gmtdefs.overlay, gmtdefs.page_orientation, gmtdefs.x_origin, gmtdefs.y_origin,
-                gmtdefs.global_x_scale, gmtdefs.global_y_scale, gmtdefs.n_copies,
-                gmtdefs.dpi, GMT_INCH , gmtdefs.paper_width, gmtdefs.page_rgb, gmtdefs.encoding.name, GMT_epsinfo (argv[0]));
-	GMT_echo_command (argc, argv);
+	GMT_plotinit (argc, argv);
 
         /*if (project_info.three_D) ps_transrotate (-z_project.xmin, -z_project.ymin, 0.0);*/
 
-	
+
 /* define area for plotting and size of array for bitmap */
 	xlen = z_project.xmax-z_project.xmin;
 	xpix = xlen*gmtdefs.dpi; /* pixels in x direction */
@@ -394,13 +391,13 @@ use a few of these*/
 	}
 	else if ((dz != binhead.sr) && (binhead.sr)) /* value in header overridden by input */
 		fprintf(stderr, "%s: Warning dz input %f, dz in header %f\n", GMT_program, dz, (float)binhead.sr);
-	
+
 	if (!dz){ /* still no sample interval at this point is a problem! */
 		fprintf(stderr, "%s: Error, no sample interval in reel header\n", GMT_program);
 		exit (EXIT_FAILURE);
 	}
 
-	
+
 	bitmap = (unsigned char *) GMT_memory (NULL, nm, sizeof (char), "pssegyz");
 
 	ix=0;
@@ -498,21 +495,20 @@ overlaps a defined header in a strange way */
 		ix++;
 	}
 
-	/* map_clip_on (-1, -1, -1, 3); /* set a clip at the map boundary since
-the image space overlaps a little */
-	ps_imagemask (0.0,0.0,xlen, ylen, bitmap, 8*bm_nx, bm_ny, polarity, shade);
+	/* map_clip_on (-1, -1, -1, 3); */
+	/* set a clip at the map boundary since the image space overlaps a little */
+	ps_bitimage (0.0,0.0,xlen, ylen, bitmap, 8*bm_nx, bm_ny, polarity, shade, trans);
 
 	/* map_clip_off ();*/
 
 	if (fpi != stdin) fclose (fpi);
-	
-	/*ps_rotatetrans (z_project.xmin, z_project.ymin, 0.0);*/
-	ps_plotend (gmtdefs.last_page);
 
-	
-	
+	/*ps_rotatetrans (z_project.xmin, z_project.ymin, 0.0);*/
+	GMT_plotend ();
+
 	GMT_end (argc, argv);
 
+	exit (EXIT_SUCCESS);
 }
 
 
@@ -679,7 +675,7 @@ interp = 0.0;
 			shade_tri(xp[0], yp[0], yp[1], slope01, slope0);
 			slope2 = (((xp[0]<xp[1]) && (xp[2]<xp[3])) || ((xp[0]>xp[1]) && (xp[2]>xp[3])))*slope02 + (((xp[0]<xp[1]) && (xp[3]<xp[2])) || ((xp[0]>xp[1]) && (xp[3]>xp[2])))*slope12;
 			slope3 = (((xp[0]<xp[1]) && (xp[3]<xp[2])) || ((xp[0]>xp[1]) && (xp[3]>xp[2])))*slope03 + (((xp[0]<xp[1]) && (xp[2]<xp[3])) || 
-((xp[0]>xp[1]) && (xp[2]>xp[3])))*slope13;		
+((xp[0]>xp[1]) && (xp[2]>xp[3])))*slope13;
 			shade_quad(xp[2], yp[2], xp[3], yp[1], slope2, slope3);
 		}
 	}
@@ -766,9 +762,6 @@ and slopes of the two other sides */
 		}
 	}
 }
-		
-
-
 
 int paint(int ix, int iy)
 /* ix iy is pixel to paint */
