@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: segy2grd.c,v 1.6 2004/01/13 02:33:04 pwessel Exp $
+ *	$Id: segy2grd.c,v 1.16 2006/04/18 13:26:27 remko Exp $
  *
- *	Copyright (c) 1991-2002 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2006 by T. Henstock
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -28,27 +28,26 @@
 #include "gmt.h"
 #include "segy_io.h"
 
-main (int argc, char **argv)
+int main (int argc, char **argv)
 {
 	int error = FALSE, got_input = FALSE, pixel = FALSE, read_cont = FALSE, count = FALSE, average = TRUE;
 	int nodata_set = FALSE, read_cdp = FALSE, read_offset = FALSE;
 	int swap_bytes = !WORDS_BIGENDIAN;
 
 	int i, ij, ii, jj, nm, n_read = 0, n_filled = 0, n_used = 0, one_or_zero, *flag;
-	int n_empty = 0, n_stuffed = 0, n_bad = 0, n_confused = 0, n_files = 0;
+	int n_empty = 0, n_stuffed = 0, n_bad = 0, n_confused = 0;
 	int check, n_traces=10000, n_samp=0, n_sampr=0, byte_x = 0, ix, isamp, ij0;
-	
-	double w, e, s, n, dx = 0.0, dy = 0.0, off, idx, idy, no_data_d, s_int, x_coord_scale=1.0, x0, yval;
+
+	double w, e, s, n, dx = 0.0, dy = 0.0, xy_off, idy, no_data_d;
+	double s_int = 0.0, x_coord_scale=1.0, x0, yval;
 
 	float no_data_f, *a;
-	
-	char *grdfile, line[BUFSIZ], input[BUFSIZ];
-	
-	FILE *fpi = NULL;
-	
-	struct GRD_HEADER grd;
 
-	struct GMT_Z_IO r;
+	char *grdfile, line[BUFSIZ], input[BUFSIZ];
+
+	FILE *fpi = NULL;
+
+	struct GRD_HEADER grd;
 
 /* SEGY parameters */
 	char reelhead[3200];
@@ -59,7 +58,7 @@ main (int argc, char **argv)
 	SEGYREEL binhead;
 
 
-	
+
 	argc = GMT_begin (argc, argv);
 
 	grdfile = CNULL;
@@ -68,17 +67,15 @@ main (int argc, char **argv)
 	no_data_f = GMT_f_NaN;
 	no_data_d = GMT_d_NaN;
 
-	GMT_init_z_io (&r, TRUE);
-	
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '-') {
 			switch (argv[i][1]) {
 				/* Common parameters */
-			
+
 				case 'R':
 				case 'V':
 				case '\0':
-					error += GMT_get_common_args (argv[i], &w, &e, &s, &n);
+					error += GMT_parse_common_options (argv[i], &w, &e, &s, &n);
 					break;
 
 				case 'A':
@@ -155,15 +152,15 @@ main (int argc, char **argv)
 			exit (EXIT_FAILURE);
 		}
 	}
-		
-	if (argc == 1 || GMT_quick) {
+
+	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "segy2grd %s - Converting segy data to a GMT grdfile\n\n", GMT_VERSION);
-		fprintf (stderr, "usage: segy2grd <segyfile> -G<grdfile> -I<dx>[/<dy>] -R<west/east/south/north>\n");
-		fprintf (stderr, "\t[-A[n|z]] [-D<xunit/yunit/zunit/scale/offset/title/remark>] [-F] \n");
+		fprintf (stderr, "usage: segy2grd <segyfile> -G<grdfile> %s %s\n", GMT_Id_OPT, GMT_Rgeo_OPT);
+		fprintf (stderr, "\t[-A[n|z]] [%s] [-F] \n", GMT_GRDEDIT);
 		fprintf (stderr, "\t[-N<nodata>] [-X<x-scale>] [-Y<s_int>] [-V] \n");
-		
-		if (GMT_quick) exit (EXIT_FAILURE);
-		
+
+		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
+
 		fprintf (stderr, "\tsegyfile(s) is an IEEE floating point SEGY file. Traces are all assumed to start at 0 time/depth\n");
 		fprintf (stderr, "\t-G to name the output grdfile.\n");
 		fprintf (stderr, "\t-I specifies grid size(s). \n");
@@ -205,51 +202,42 @@ main (int argc, char **argv)
 		fprintf(stderr, "%s: GMT SYNTAX ERROR option -S:  Must only specify one of cdp, offset, byte\n", GMT_program);
 		error++;
 	}
-				
+
 	if (error) exit (EXIT_FAILURE);
 
 	GMT_put_history (argc, argv);	/* Update .gmtcommands */
 
 
 	GMT_grd_init (&grd, argc, argv, FALSE);
-	
+
 	/* if (!project_info.region) d_swap (s, e); */  /* Got w/s/e/n, make into w/e/s/n */
 
 	/* Decode grd information given, if any */
-	
+
 	if (got_input) GMT_decode_grd_h_info (input, &grd);
-	
-	if (pixel) {
-		grd.node_offset = 1;
-		one_or_zero = 0;
-		off = 0.0;
-	}
-	else {
-		grd.node_offset = 0;
-		one_or_zero = 1;
-		off = 0.5;
-	}
+
+	grd.node_offset = pixel;
+	one_or_zero = 1 - grd.node_offset;
+	xy_off = 0.5 * grd.node_offset;
 
 	grd.nx = irint ((e-w)/dx) + one_or_zero;
 	grd.ny = irint ((n-s)/dy) + one_or_zero;
 	grd.x_min = w;	grd.x_max = e;
 	grd.y_min = s;	grd.y_max = n;
 	grd.x_inc = dx;	grd.y_inc = dy;
-	
+
 	GMT_grd_RI_verify (&grd, 1);
-		
+
 	if (gmtdefs.verbose) fprintf (stderr, "%s: nx = %d  ny = %d\n", GMT_program, grd.nx, grd.ny);
-	
+
 	nm = grd.nx * grd.ny;
-	
+
 	a = (float *) GMT_memory (VNULL, (size_t)nm, sizeof (float), GMT_program);
 	flag = (int *) GMT_memory (VNULL, (size_t)nm, sizeof (int), GMT_program);
 
-	idx = 1.0 / dx;	idy = 1.0 / dy;
+	idy = 1.0 / dy;
 	ij = -1;	/* Will be incremented to 0 or set first time around */
 
-	GMT_set_z_io (&r, &grd);
-
 /* read in reel headers from segy file */
 	if (fpi == NULL) fpi = GMT_stdin;
 	if ((check = get_segy_reelhd (fpi, reelhead)) != TRUE) exit(1);
@@ -296,8 +284,8 @@ use a few of these*/
 		fprintf(stderr,"%s: Sample interval is %f s\n", GMT_program, s_int);
 	}
 	else if ((s_int != binhead.sr) && (binhead.sr)) /* value in header overridden by input */
-		fprintf(stderr, "%s: Warning s_int input %lf, s_int in header %f\n", GMT_program, s_int, (float)binhead.sr);
-	
+		fprintf(stderr, "%s: Warning s_int input %f, s_int in header %f\n", GMT_program, s_int, (float)binhead.sr);
+
 	if (!s_int){ /* still no sample interval at this point is a problem! */
 		fprintf(stderr, "%s: Error, no sample interval in reel header\n", GMT_program);
 		exit(-1);
@@ -313,7 +301,7 @@ use a few of these*/
 
 
 /* starts reading actual data here....... */
-		
+
 	if (read_cont) {	/* old-style segy2grd */
 		ix=0;
 		for (ij=0; ij<nm; ij++) a[ij] = no_data_f;
@@ -351,7 +339,7 @@ use a few of these*/
 		ix++;
 		}
 	}
-	else {	/* Get trace data and position by headers */		
+	else {	/* Get trace data and position by headers */
 		ix=0;
 		while ((ix<n_traces) && (header = get_segy_header(fpi))){   /* read traces one by one */
 			if (read_offset){ /* plot traces by offset, cdp, or input order */
@@ -396,13 +384,13 @@ overlaps a defined header in a strange way */
 			}
 
 			if (!(x0 < w || x0 > e)){	/* inside x-range */
-				/* find horizontal grid pos of this trace */			
-				ii = (int)floor ((x0 - w) * idx + off);
+				/* find horizontal grid pos of this trace */
+				ii = GMT_x_to_i (x0, grd.x_min, grd.x_inc, xy_off, grd.nx);
 				if (ii == grd.nx) ii--, n_confused++;
 				for (isamp = 0; isamp< n_samp; isamp++){
 					yval = isamp*s_int;
 					if (!(yval < s || yval > n)){	/* inside y-range */
-						jj = (int)floor ((n - yval) * idy + off);
+						jj = GMT_y_to_j (yval, grd.y_min, grd.y_inc, xy_off, grd.ny);
 						if (jj == grd.ny) jj--, n_confused++;
 						ij = jj * grd.nx + ii;
 						a[ij] += data[isamp];	/* Add up incase we must average */
@@ -450,9 +438,11 @@ overlaps a defined header in a strange way */
 		fprintf (stderr, "%s: Error writing file %s\n", GMT_program, grdfile);
 		exit (EXIT_FAILURE);
 	}
-	
+
 	GMT_free ((void *)a);
 	GMT_free ((void *)flag);
-	
+
 	GMT_end (argc, argv);
+
+	exit (EXIT_SUCCESS);
 }
