@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: triangulate.c,v 1.14 2004/04/25 09:10:46 pwessel Exp $
+ *	$Id: triangulate.c,v 1.31 2006/04/13 08:34:41 pwessel Exp $
  *
- *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -39,35 +39,61 @@
  
 #include "gmt.h"
 
-struct EDGE {
+struct TRIANGULATE_CTRL {
+	struct D {	/* -Dx|y */
+		BOOLEAN active;
+		int dir;
+	} D;
+	struct E {	/* -E<Ctrl->E.value> */
+		BOOLEAN active;
+		double value;
+	} E;
+	struct F {	/* -F */
+		BOOLEAN active;
+	} F;
+	struct G {	/* -G<output_grdfile> */
+		BOOLEAN active;
+		char *file;
+	} G;
+	struct I {	/* -Idx[/dy] */
+		BOOLEAN active;
+		double xinc, yinc;
+	} I;
+};
+
+struct TRIANGULATE_EDGE {
 	int begin, end;
 };
-	
-main (int argc, char **argv)
+
+int main (int argc, char **argv)
 {
 	int i, j, ij, ij1, ij2, ij3, np, k, fno, n = 0, n_alloc = 0, n_files = 0, n_edge;
 	int n_expected_fields, n_args, *link, n_fields, compare_edge(const void *p1, const void *p2);
-	int one_or_zero, i_min, i_max, j_min, j_max, p;
-	
-	BOOLEAN error = FALSE, map_them = FALSE, inc_set = FALSE, do_ddx = FALSE, do_ddy = FALSE;
-	BOOLEAN nofile = TRUE, done = FALSE, do_grid = FALSE, set_empty = FALSE;
-	
-	double west = 0.0, east = 0.0, south = 0.0, north = 0.0, zj, zk, zl, zlj, zkj, *in;
-	double *xx, *yy, *zz, vx[4], vy[4], xinc2, yinc2, idx, idy, xp, yp, a, b, c, f;
-	double xkj, xlj, ykj, ylj, out[2];
-	
-	float *grd, empty = 0.0;
-	
-	char line[BUFSIZ], *outfile = CNULL;
-	
-	FILE *fp = NULL;
-	
-	struct GRD_HEADER header;
+	int i_min, i_max, j_min, j_max, p;
 
-	struct EDGE *edge;
+	BOOLEAN error = FALSE, map_them = FALSE;
+	BOOLEAN nofile = TRUE, done = FALSE, do_grid = FALSE;
+
+	double zj, zk, zl, zlj, zkj, *in;
+	double *xx, *yy, *zz = VNULL, vx[4], vy[4], xp, yp, a, b, c, f;
+	double xkj, xlj, ykj, ylj, out[2];
+
+	float *grd;
+
+	char line[BUFSIZ];
+
+	FILE *fp = NULL;
+
+	struct GRD_HEADER header;
+	struct TRIANGULATE_EDGE *edge;
+	struct TRIANGULATE_CTRL *Ctrl;
+
+	void *New_Triangulate_Ctrl (), Free_Triangulate_Ctrl (struct TRIANGULATE_CTRL *C);
 	
 	argc = GMT_begin (argc, argv);
 
+	Ctrl = (struct TRIANGULATE_CTRL *)New_Triangulate_Ctrl ();	/* Allocate and initialize a new control structure */
+
 	GMT_grd_init (&header, argc, argv, FALSE);
 
 	for (i = 1; i < argc; i++) {
@@ -84,28 +110,39 @@ main (int argc, char **argv)
 				case 'b':
 				case 'f':
 				case '\0':
-					error += GMT_get_common_args (argv[i], &west, &east, &south, &north);
+					error += GMT_parse_common_options (argv[i], &header.x_min, &header.x_max, &header.y_min, &header.y_max);
 					break;
         
 				/* Supplemental parameters */
         
 				case 'D':
-					do_ddx = (argv[i][2] == 'x' || argv[i][2] == 'X');
-					do_ddy = (argv[i][2] == 'y' || argv[i][2] == 'Y');
+					Ctrl->D.active = TRUE;
+					if (argv[i][2] == 'x' || argv[i][2] == 'X')
+						Ctrl->D.dir = GMT_X;
+					else if (argv[i][2] == 'y' || argv[i][2] == 'Y')
+						Ctrl->D.dir = GMT_Y;
+					else {
+						fprintf (stderr, "%s: GMT SYNTAX ERROR.  Give -Dx or -Dy\n", GMT_program);
+						error++;
+					}
 					break;
 				case 'E':
-					empty = (argv[i][2] == 'N' || argv[i][2] == 'n') ? GMT_f_NaN : (float)atof (&argv[i][2]);
-					set_empty = TRUE;
+					Ctrl->E.active = TRUE;
+					Ctrl->E.value = (argv[i][2] == 'N' || argv[i][2] == 'n') ? GMT_d_NaN : atof (&argv[i][2]);
 					break;
 				case 'F':
-					header.node_offset = TRUE;
+					Ctrl->F.active = TRUE;
 					break;
 				case 'G':
-					outfile = &argv[i][2];
+					Ctrl->G.active = TRUE;
+					Ctrl->G.file = strdup (&argv[i][2]);
 					break;
 				case 'I':
-					GMT_getinc (&argv[i][2], &header.x_inc, &header.y_inc);
-					inc_set = TRUE;
+					Ctrl->I.active = TRUE;
+					if (GMT_getinc (&argv[i][2], &Ctrl->I.xinc, &Ctrl->I.yinc)) {
+						GMT_inc_syntax ('I', 1);
+						error = TRUE;
+					}
 					break;
 				case 'L':	/* Obsolete, but backward compatibility prevails [use -f instead] */
 					GMT_io.in_col_type[0] = GMT_io.out_col_type[0] = GMT_IS_LON;
@@ -125,13 +162,13 @@ main (int argc, char **argv)
 			n_files++;
 	}
 
-	if (argc == 1 || GMT_quick) {
+	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "triangulate %s - Optimal (Delaunay) triangulation of xyz-data\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: triangulate <infiles> [-Dx|y] [-E<empty>] [-F] [-G<grdfile>]\n");
-		fprintf (stderr, "\t[-H[<nrec>]] [-I<dx>[m|c][/<dy>[m|c]]] [-J<parameters>] [-M[<flag>]]\n");
-		fprintf (stderr, "\t[-R<west/east/south/north>] [-V] [-:] [-bi[s][<n>]] [-bo[s][<n>]]\n\t[-f[i|o]<colinfo>]\n\n");
+		fprintf (stderr, "\t[%s] [%s] [%s] [%s]\n", GMT_H_OPT, GMT_I_OPT, GMT_J_OPT, GMT_M_OPT);
+		fprintf (stderr, "\t[%s] [-V] [%s] [%s]\n\t[%s]\n\n", GMT_Rgeo_OPT, GMT_t_OPT, GMT_b_OPT, GMT_f_OPT);
                 
-		if (GMT_quick) exit (EXIT_FAILURE);
+		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
                 
 		fprintf (stderr, "\tinfiles (in ASCII) has 2 or more columns.  If no file(s) is given, standard input is read.\n");
 		fprintf (stderr, "\n\tOPTIONS:\n");
@@ -140,7 +177,7 @@ main (int argc, char **argv)
 		fprintf (stderr, "\t-F Force pixel registration (only with -G) [Default is gridline registration]\n");
 		fprintf (stderr, "\t-G Grid data. Give name of output gridfile and specify -R -I\n");
 		GMT_explain_option ('H');
-		fprintf (stderr, "\t-I sets the grid spacing for the grid.  Append m for minutes, c for seconds\n");
+		GMT_inc_syntax ('I', 0);
 		GMT_explain_option ('J');   
 		fprintf (stderr, "\t-M output triangle edges as multiple segments separated by a record\n");
 		fprintf (stderr, "\t   whose first character is <flag> ['>']\n");
@@ -156,8 +193,8 @@ main (int argc, char **argv)
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
-	
-	if (GMT_io.binary[GMT_IN] && gmtdefs.io_header[GMT_IN]) {
+
+	if (GMT_io.binary[GMT_IN] && GMT_io.io_header[GMT_IN]) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data cannot have header -H\n", GMT_program);
 		error++;
 	}
@@ -166,14 +203,24 @@ main (int argc, char **argv)
                 fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data (-bi) must have at least 2 columns\n", GMT_program);
 		error++;
 	}
-	
-	do_grid = (outfile && inc_set && header.x_inc > 0.0 && header.y_inc > 0.0);
-	
-	if (do_grid && !project_info.region_supplied) {
+
+	if (Ctrl->D.active && (Ctrl->D.dir < GMT_X || Ctrl->D.dir > GMT_Y)) {
+		fprintf (stderr, "%s: GMT SYNTAX ERROR -D option.  Must specify x or y\n", GMT_program);
+		error = TRUE;
+	}
+	if (Ctrl->I.active && (Ctrl->I.xinc <= 0.0 || Ctrl->I.yinc <= 0.0)) {
+		fprintf (stderr, "%s: GMT SYNTAX ERROR -I option.  Must specify positive increment(s)\n", GMT_program);
+		error = TRUE;
+	}
+	if (Ctrl->G.active && !Ctrl->G.file) {
+		fprintf (stderr, "%s: GMT SYNTAX ERROR -G option.  Must specify file name\n", GMT_program);
+		error = TRUE;
+	}
+	if ((do_grid = (Ctrl->G.active + Ctrl->I.active + project_info.region_supplied)) > 0 && do_grid != 3) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Must specify -R, -I, -G for gridding\n", GMT_program);
 		error++;
 	}
-	
+
 	if (error) exit (EXIT_FAILURE);
 
 	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
@@ -188,34 +235,37 @@ main (int argc, char **argv)
 #endif
 
 	if (do_grid) {
-		header.x_min = west;	header.x_max = east;
-		header.y_min = south;	header.y_max = north;
+		do_grid = 1;
+		header.node_offset = Ctrl->F.active;
+		header.x_inc = Ctrl->I.xinc;
+		header.y_inc = Ctrl->I.yinc;
+		GMT_RI_prepare (&header);	/* Ensure -R -I consistency and set nx, ny */
 		GMT_grd_RI_verify (&header, 1);
 	}
 
-	if (west != east && project_info.projection > 0) { /* Gave -R -J */
+	if (project_info.region_supplied && project_info.projection > 0) { /* Gave -R -J */
 		map_them = TRUE;
-		GMT_map_setup (west, east, south, north);
+		GMT_map_setup (header.x_min, header.x_max, header.y_min, header.y_max);
 	}
 
 	/* Now we are ready to take on some input values */
-	
+
 	if (n_files > 0)
 		nofile = FALSE;
 	else
 		n_files = 1;
 	n_args = (argc > 1) ? argc : 2;
 	n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : 2 + do_grid;
-	
+
 	n_alloc = GMT_CHUNK;
 	xx = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
 	yy = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
 	if (do_grid) zz = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
-		
+
 	n = 0;
 	for (fno = 1; !done && fno < n_args; fno++) {	/* Loop over input files, if any */
 		if (!nofile && argv[fno][0] == '-') continue;
-		
+
 		if (nofile) {	/* Just read standard input */
 			fp = GMT_stdin;
 			done = TRUE;
@@ -229,11 +279,11 @@ main (int argc, char **argv)
 		}
 
 		if (!nofile && gmtdefs.verbose) fprintf (stderr, "%s: Reading file %s\n", GMT_program, argv[fno]);
-		
-		if (gmtdefs.io_header[GMT_IN]) for (i = 0; i < gmtdefs.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
+
+		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
 
 		while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
-					
+
 			if (GMT_io.status & GMT_IO_MISMATCH) {
 				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, n_expected_fields, n);
 				exit (EXIT_FAILURE);
@@ -243,7 +293,7 @@ main (int argc, char **argv)
 			yy[n] = in[1];
 			if (do_grid) zz[n] = in[2];
 			n++;
-			
+
 			if (n == n_alloc) {	/* Get more memory */
 				n_alloc += GMT_CHUNK;
 				xx = (double *) GMT_memory ((void *)xx, (size_t)n_alloc, sizeof (double), GMT_program);
@@ -251,10 +301,10 @@ main (int argc, char **argv)
 				if (do_grid) zz = (double *) GMT_memory ((void *)zz, (size_t)n_alloc, sizeof (double), GMT_program);
 			}
 		}
-		
+
 		if (fp != GMT_stdin) GMT_fclose (fp);
 	}
-	
+
 	xx = (double *) GMT_memory ((void *)xx, (size_t)n, sizeof (double), GMT_program);
 	yy = (double *) GMT_memory ((void *)yy, (size_t)n, sizeof (double), GMT_program);
 	if (do_grid) zz = (double *) GMT_memory ((void *)zz, (size_t)n, sizeof (double), GMT_program);
@@ -283,36 +333,26 @@ main (int argc, char **argv)
 	if (gmtdefs.verbose) fprintf (stderr, "%s: %d Delaunay triangles found\n", GMT_program, np);
 
 	if (do_grid) {
-	
-		if (header.node_offset) {
-			one_or_zero = 0;
-			xinc2 = 0.5 * header.x_inc;
-			yinc2 = 0.5 * header.y_inc;
-		}
-		else {
-			one_or_zero = 1;
-			xinc2 = yinc2 = 0.0;
-		}
-		idx = 1.0 / header.x_inc;
-		idy = 1.0 / header.y_inc;
-		header.nx = irint ( (header.x_max - header.x_min) * idx) + one_or_zero;
-		header.ny = irint ( (header.y_max - header.y_min) * idy) + one_or_zero;
+
+		header.nx = GMT_get_n (header.x_min, header.x_max, header.x_inc, header.node_offset);
+		header.ny = GMT_get_n (header.y_min, header.y_max, header.y_inc, header.node_offset);
+		header.xy_off = 0.5 * header.node_offset;
 		grd = (float *) GMT_memory (VNULL, (size_t)(header.nx * header.ny), sizeof (float), GMT_program);
-		if (!set_empty) empty = GMT_f_NaN;
-		for (i = 0; i < header.nx * header.ny; i++) grd[i] = empty;	/* initialize grid */
-		
+		if (!Ctrl->E.active) Ctrl->E.value = GMT_d_NaN;
+		for (i = 0; i < header.nx * header.ny; i++) grd[i] = (float)Ctrl->E.value;	/* initialize grid */
+
 		for (k = ij = 0; k < np; k++) {
-		
+
 			/* Find equation for the plane as z = ax + by + c */
-			
+
 			vx[0] = vx[3] = xx[link[ij]];	vy[0] = vy[3] = yy[link[ij]];	zj = zz[link[ij++]];
 			vx[1] = xx[link[ij]];	vy[1] = yy[link[ij]];	zk = zz[link[ij++]];
 			vx[2] = xx[link[ij]];	vy[2] = yy[link[ij]];	zl = zz[link[ij++]];
-			
+
 			xkj = vx[1] - vx[0];	ykj = vy[1] - vy[0];
 			zkj = zk - zj;	xlj = vx[2] - vx[0];
 			ylj = vy[2] - vy[0];	zlj = zl - zj;
-			
+
 			f = 1.0 / (xkj * ylj - ykj * xlj);
 			a = -f * (ykj * zlj - zkj * ylj);
 			b = -f * (zkj * xlj - xkj * zlj);
@@ -322,11 +362,11 @@ main (int argc, char **argv)
 			   in the -R region (header.x_min/x_max etc.)  Always, i_min <= i_max, j_min <= j_max.
 			 */
 
-			i_min = (int)floor ((MIN (MIN (vx[0], vx[1]), vx[2]) - header.x_min + xinc2) * idx);
-			i_max = (int)ceil ((MAX (MAX (vx[0], vx[1]), vx[2]) - header.x_min + xinc2) * idx);
-			j_min = (int)floor ((header.y_max - MAX (MAX (vy[0], vy[1]), vy[2]) + yinc2) * idy);
-			j_max = (int)ceil ((header.y_max - MIN (MIN (vy[0], vy[1]), vy[2]) + yinc2) * idy);
-			
+			xp = MIN (MIN (vx[0], vx[1]), vx[2]);	i_min = GMT_x_to_i (xp, header.x_min, header.x_inc, header.xy_off, header.nx);
+			xp = MAX (MAX (vx[0], vx[1]), vx[2]);	i_max = GMT_x_to_i (xp, header.x_min, header.x_inc, header.xy_off, header.nx);
+			yp = MAX (MAX (vy[0], vy[1]), vy[2]);	j_min = GMT_y_to_j (yp, header.y_min, header.y_inc, header.xy_off, header.ny);
+			yp = MIN (MIN (vy[0], vy[1]), vy[2]);	j_max = GMT_y_to_j (yp, header.y_min, header.y_inc, header.xy_off, header.ny);
+
 			/* Adjustments for triangles outside -R region. */
 			/* Triangle to the left or right. */
 			if ((i_max < 0) || (i_min >= header.nx)) continue;
@@ -339,49 +379,49 @@ main (int argc, char **argv)
 			if (j_min < 0) j_min = 0;       if (j_max >= header.ny) j_max = header.ny - 1;
 
 			for (j = j_min; j <= j_max; j++) {
-				yp = header.y_max - j * header.y_inc - yinc2;
+				yp = GMT_j_to_y (j, header.y_min, header.y_max, header.y_inc, header.xy_off, header.ny);
 				p = j * header.nx + i_min;
 				for (i = i_min; i <= i_max; i++, p++) {
-					xp = header.x_min + i * header.x_inc + xinc2;
-					
+					xp = GMT_i_to_x (i, header.x_min, header.x_max, header.x_inc, header.xy_off, header.nx);
+
 					if (!GMT_non_zero_winding (xp, yp, vx, vy, 4)) continue;	/* Outside */
-					
-					if (do_ddx)
+
+					if (Ctrl->D.dir == GMT_X)
 						grd[p] = (float)a;
-					else if (do_ddy)
+					else if (Ctrl->D.dir == GMT_Y)
 						grd[p] = (float)b;
 					else
 						grd[p] = (float)(a * xp + b * yp + c);
 				}
 			}
 		}
-		if (GMT_write_grd (outfile, &header, grd, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE)) {
-			fprintf (stderr, "%s: Error writing file %s\n", GMT_program, outfile);
+		if (GMT_write_grd (Ctrl->G.file, &header, grd, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE)) {
+			fprintf (stderr, "%s: Error writing file %s\n", GMT_program, Ctrl->G.file);
 			exit (EXIT_FAILURE);
 		}
-					
+
 	}
-	if (GMT_io.multi_segments) {	/* Must find unique edges to output only once */
+	if (GMT_io.multi_segments[GMT_OUT]) {	/* Must find unique edges to output only once */
 		n_edge = 3 * np;
-		edge = (struct EDGE *) GMT_memory (VNULL, (size_t)n_edge, sizeof (struct EDGE), GMT_program);
+		edge = (struct TRIANGULATE_EDGE *) GMT_memory (VNULL, (size_t)n_edge, sizeof (struct TRIANGULATE_EDGE), GMT_program);
 		for (i = ij1 = 0, ij2 = 1, ij3 = 2; i < np; i++, ij1 += 3, ij2 += 3, ij3 += 3) {
 			edge[ij1].begin = link[ij1];	edge[ij1].end = link[ij2];
 			edge[ij2].begin = link[ij2];	edge[ij2].end = link[ij3];
 			edge[ij3].begin = link[ij1];	edge[ij3].end = link[ij3];
 		}
 		for (i = 0; i < n_edge; i++) if (edge[i].begin > edge[i].end) i_swap (edge[i].begin, edge[i].end);
-		
-		qsort ((void *)edge, (size_t)n_edge, sizeof (struct EDGE), compare_edge);
+
+		qsort ((void *)edge, (size_t)n_edge, sizeof (struct TRIANGULATE_EDGE), compare_edge);
 		for (i = 1, j = 0; i < n_edge; i++) {
 			if (edge[i].begin != edge[j].begin || edge[i].end != edge[j].end) j++;
 			edge[j] = edge[i];
 		}
 		n_edge = j + 1;
-	
+
 		if (gmtdefs.verbose) fprintf (stderr, "%s: %d unique triangle edges\n", GMT_program, n_edge);
-		
+
 		for (i = 0; i < n_edge; i++) {
-			fprintf (GMT_stdout, "%c Edge %d-%d\n", GMT_io.EOF_flag, edge[i].begin, edge[i].end);
+			fprintf (GMT_stdout, "%c Edge %d-%d\n", GMT_io.EOF_flag[GMT_OUT], edge[i].begin, edge[i].end);
 			out[0] = xx[edge[i].begin];	out[1] = yy[edge[i].begin];
 			GMT_output (GMT_stdout, 2, out);
 			out[0] = xx[edge[i].end];	out[1] = yy[edge[i].end];
@@ -393,22 +433,26 @@ main (int argc, char **argv)
 		fwrite ((void *)link, sizeof (int), (size_t)(3*np), GMT_stdout);
 	else
 		for (i = ij = 0; i < np; i++, ij += 3) fprintf (GMT_stdout, "%d\t%d\t%d\n", link[ij], link[ij+1], link[ij+2]);
-	
+
 	GMT_free ((void *) xx);
 	GMT_free ((void *) yy);
 	GMT_free ((void *) link);
-	
+
 	if (gmtdefs.verbose) fprintf (stderr, "%s: Done!\n", GMT_program);
-	
+
+	Free_Triangulate_Ctrl (Ctrl);	/* Deallocate control structure */
+
 	GMT_end (argc, argv);
+
+	exit (EXIT_SUCCESS);
 }
 
 int compare_edge (const void *p1, const void *p2)
 {
-	struct EDGE *a, *b;
+	struct TRIANGULATE_EDGE *a, *b;
 
-	a = (struct EDGE *)p1;
-	b = (struct EDGE *)p2;
+	a = (struct TRIANGULATE_EDGE *)p1;
+	b = (struct TRIANGULATE_EDGE *)p2;
 	if (a->begin < b->begin)
 		return (-1);
 	else if (a->begin > b->begin)
@@ -422,3 +466,18 @@ int compare_edge (const void *p1, const void *p2)
 			return (0);
 	}
 }
+
+void *New_Triangulate_Ctrl () {	/* Allocate and initialize a new control structure */
+	struct TRIANGULATE_CTRL *C;
+	
+	C = (struct TRIANGULATE_CTRL *) GMT_memory (VNULL, 1, sizeof (struct TRIANGULATE_CTRL), "New_Triangulate_Ctrl");
+	
+	/* Initialize values whose defaults are not 0/FALSE/NULL */
+	C->D.dir = -1;	/* No derivatives */
+	return ((void *)C);
+}
+
+void Free_Triangulate_Ctrl (struct TRIANGULATE_CTRL *C) {	/* Deallocate control structure */
+	if (C->G.file) GMT_free ((void *)C->G.file);	
+	GMT_free ((void *)C);	
+}
