@@ -1,7 +1,7 @@
 /*-----------------------------------------------------------------
- *	$Id: x2sys.c,v 1.17 2004/08/20 20:43:38 pwessel Exp $
+ *	$Id: x2sys.c,v 1.55 2006/04/10 05:47:30 pwessel Exp $
  *
- *      Copyright (c) 1999-2001 by P. Wessel
+ *      Copyright (c) 1999-2006 by P. Wessel
  *      See COPYING file for copying and redistribution conditions.
  *
  *      This program is free software; you can redistribute it and/or modify
@@ -27,14 +27,12 @@
  * x2sys_read_gmtfile	: Specifically reads an old .gmt file
  * x2sys_read_mgd77file : Specifically reads an MGD77 file
  * x2sys_read_list	: Read an ascii list of track names
- * x2sys_distances	: Calculate cumulative distances along track
  * x2sys_dummytimes	: Make dummy times for tracks missing times
  * x2sys_n_data_cols	: Gives number of data columns in this data set
  * x2sys_fopen		: Opening files with error message and exit
  * x2sys_fclose		: Closes files and gives error messages
  * x2sys_skip_header	: Skips the header record(s) in the open file
  * x2sys_read_record	: Reads and returns one record from the open file
- * x2sys_output_record	: Writes one data record to stdout
  * x2sys_pick_fields	: Decodes the -F<fields> flag of desired columns
  * x2sys_free_info	: Frees the information structure
  * x2sys_free_data	: Frees the data matrix
@@ -72,7 +70,6 @@ struct MGD77_CONTROL M;
 
 void x2sys_set_home (void);
 int x2sys_record_length (struct X2SYS_INFO *s);
-int MGD77_items[MGD77_DATA_COLS] = { 0, MGD77_LONGITUDE, MGD77_LATITUDE, 10, 11, 14, 15, 16, 18, 19, 20, 21, 22};
 
 #define MAX_DATA_PATHS 32
 char *x2sys_datadir[MAX_DATA_PATHS];	/* Directories where track data may live */
@@ -88,20 +85,14 @@ FILE *x2sys_fopen (char *fname, char *mode)
 	FILE *fp;
 	char file[BUFSIZ];
 
-	if (mode[0] == 'w') {	/* Writing: Do this in X2SYS_HOME */
+	if (mode[0] == 'w') {	/* Writing: Do this only in X2SYS_HOME */
 		x2sys_path (fname, file);
-		if ((fp = fopen (file, mode)) == NULL) {	/* Not in X2SYS_HOME directory either */
-			fprintf (stderr, "x2sys: Error from fopen on %s using mode %s\n", file, mode);
-			exit (EXIT_FAILURE);
-		}
+		fp = fopen (file, mode);
 	}
 	else {			/* Reading: Try both current directory and X2SYS_HOME */
-		if ((fp = fopen (fname, mode)) == NULL) {	/* Not in current directory */
+		if ((fp = fopen (fname, mode)) == NULL) {	/* Not in current directory, try $X2SYS_HOME */
 			x2sys_path (fname, file);
-			if ((fp = fopen (file, mode)) == NULL) {	/* Not in X2SYS_HOME directory either */
-				fprintf (stderr, "x2sys: Error from fopen on %s using mode %s\n", fname, mode);
-				exit (EXIT_FAILURE);
-			}
+			fp = fopen (file, mode);
 		}
 	}
 	return (fp);
@@ -149,15 +140,15 @@ void x2sys_skip_header (FILE *fp, struct X2SYS_INFO *s)
 
 int x2sys_read_record (FILE *fp, double *data, struct X2SYS_INFO *s, struct GMT_IO *G)
 {
-	int j, k, i, n_read = 0;
+	int j, k, i, n_read = 0, pos;
 	BOOLEAN error = FALSE;
-	char line[BUFSIZ], buffer[64], *p, c;
+	char line[BUFSIZ], buffer[GMT_TEXT_LEN], p[BUFSIZ], c;
 	unsigned char u;
 	short int h;
 	float f;
 	long L;
 	double NaN;
-	
+
 	GMT_make_dnan(NaN);
 
 	for (j = 0; !error && j < s->n_fields; j++) {
@@ -167,11 +158,12 @@ int x2sys_read_record (FILE *fp, double *data, struct X2SYS_INFO *s, struct GMT_
 			case 'A':	/* ASCII Card Record, must extract columns */
 				if (j == 0) {
 					s->ms_next = FALSE;
-					fgets (line, BUFSIZ, fp);	/* Get new record */
+					if (!fgets (line, BUFSIZ, fp)) return (-1);
 					while (line[0] == '#' || line[0] == s->ms_flag) {
-						fgets (line, BUFSIZ, fp);
+						if (!fgets (line, BUFSIZ, fp)) return (-1);
 						if (s->multi_segment) s->ms_next = TRUE;
 					}
+					GMT_chop (line);	/* Remove trailing CR or LF */
 				}
 				strncpy (buffer, &line[s->info[j].start_col], s->info[j].n_cols);
 				buffer[s->info[j].n_cols] = 0;
@@ -181,16 +173,16 @@ int x2sys_read_record (FILE *fp, double *data, struct X2SYS_INFO *s, struct GMT_
 			case 'a':	/* ASCII Record, get all columns directly */
 				k = 0;
 				s->ms_next = FALSE;
-				fgets (line, BUFSIZ, fp);
+				if (!fgets (line, BUFSIZ, fp)) return (-1);
 				while (line[0] == '#' || line[0] == s->ms_flag) {
-					fgets (line, BUFSIZ, fp);
+					if (!fgets (line, BUFSIZ, fp)) return (-1);
 					if (s->multi_segment) s->ms_next = TRUE;
 				}
-				p = strtok (line, " ,\t\n");
-				while (p) {
+				GMT_chop (line);	/* Remove trailing CR or LF */
+				pos = 0;
+				while ((GMT_strtok (line, " ,\t\n", &pos, p))) {
 					GMT_scanf (p, G->in_col_type[k], &data[k]);
 					k++;;
-					p = strtok (NULL, " ,\t\n");
 				}
 				return ((k != s->n_fields) ? -1 : 0);
 				break;
@@ -263,16 +255,16 @@ int x2sys_read_file (char *fname, double ***data, struct X2SYS_INFO *s, struct X
 	char path[BUFSIZ];
 
  	if (n_x2sys_paths) {
-  		if (x2sys_get_data_path (path, fname)) {
+  		if (x2sys_get_data_path (path, fname, s->suffix)) {
    			fprintf (stderr, "x2sys_read_file : Cannot find track %s\n", fname);
      			return (-1);
   		}
-  		if ((fp = fopen (path, "rb")) == NULL) {
-   			fprintf (stderr, "x2sys_read_mgd77file : Cannot open file %s\n", path);
+  		if ((fp = fopen (path, G->r_mode)) == NULL) {
+   			fprintf (stderr, "x2sys_read_file : Cannot open file %s\n", path);
      			return (-1);
   		}
 	}
-	else if ((fp = fopen (fname, "rb")) == NULL) {
+	else if ((fp = fopen (fname, G->r_mode)) == NULL) {
 		fprintf (stderr, "x2sys_read_file: Could not open %s\n", fname);
 		return (-1);
 	}
@@ -329,24 +321,32 @@ struct X2SYS_INFO *x2sys_initialize (char *fname, struct GMT_IO *G)
 	X->ascii_in = TRUE;
 	X->x_col = X->y_col = X->t_col = -1;
 	X->ms_flag = '>';	/* Default multisegment header flag */
-	sprintf (line, "%s%c%s.def\0", X2SYS_HOME, DIR_DELIM, fname);
+	sprintf (line, "%s%c%s.def", X2SYS_HOME, DIR_DELIM, fname);
 
-	fp = x2sys_fopen (line, "r");
+	if ((fp = x2sys_fopen (line, "r")) == NULL) {
+  		fprintf (stderr, "x2sys_initialize : Cannot find format definition file %s in either current or X2SYS_HOME directories\n", line);
+     		exit (EXIT_FAILURE);
+	}
 
 	if (!strcmp (fname, "gmt")) {
 		X->read_file = (PFI) x2sys_read_gmtfile;
 		X->geographic = TRUE;
 		X->geodetic = 0;
 	}
+	else if (!strcmp (fname, "mgd77+")) {
+		X->read_file = (PFI) x2sys_read_ncfile;
+		X->geographic = TRUE;
+		X->geodetic = 0;
+		MGD77_Init (&M, TRUE);			/* Initialize MGD77 Machinery */
+	}
 	else if (!strcmp (fname, "mgd77")) {
 		X->read_file = (PFI) x2sys_read_mgd77file;
 		X->geographic = TRUE;
-		X->geodetic = 1;
+		X->geodetic = 0;
 		MGD77_Init (&M, TRUE);			/* Initialize MGD77 Machinery */
 	}
 	else
 		X->read_file = (PFI) x2sys_read_file;
-
 	while (fgets (line, BUFSIZ, fp)) {
 		if (line[0] == '\0') continue;
 		if (line[0] == '#') {
@@ -359,6 +359,7 @@ struct X2SYS_INFO *x2sys_initialize (char *fname, struct GMT_IO *G)
 			}
 			continue;
 		}
+		GMT_chop (line);	/* Remove trailing CR or LF */
 
 		sscanf (line, "%s %c %c %lf %lf %lf %s %s", X->info[i].name, &X->info[i].intype, &yes_no, &X->info[i].nan_proxy, &X->info[i].scale, &X->info[i].offset, X->info[i].format, cardcol);
 		if (X->info[i].intype == 'A') {	/* ASCII Card format */
@@ -378,13 +379,18 @@ struct X2SYS_INFO *x2sys_initialize (char *fname, struct GMT_IO *G)
 			n_alloc += 10;
 			X->info = (struct X2SYS_DATA_INFO *) GMT_memory ((void *)X->info, n_alloc, sizeof (struct X2SYS_DATA_INFO), "x2sys_initialize");
 		}
-			
+
 	}
 	fclose (fp);
 
 	if (i < (int)n_alloc) X->info = (struct X2SYS_DATA_INFO *) GMT_memory ((void *)X->info, i, sizeof (struct X2SYS_DATA_INFO), "x2sys_initialize");
 	X->n_fields = X->n_out_columns = i;
 
+	if (!X->ascii_in) {	/* Binary mode needed */
+		strcpy (G->r_mode, "rb");
+		strcpy (G->w_mode, "wb");
+		strcpy (G->a_mode, "ab+");
+	}
 	X->out_order  = (int *) GMT_memory (VNULL, sizeof (int), (size_t)X->n_fields, "x2sys_initialize");
 	X->use_column = (int *) GMT_memory (VNULL, sizeof (int), (size_t)X->n_fields, "x2sys_initialize");
 	for (i = 0; i < X->n_fields; i++) {	/* Default is same order and use all columns */
@@ -430,7 +436,7 @@ int x2sys_n_data_cols (struct X2SYS_INFO *s)
 {
 	int i, n = 0;
 
-	for (i = 0; i < s->n_fields; i++) {
+	for (i = 0; i < s->n_out_columns; i++) {
 		if (i == s->x_col) continue;
 		if (i == s->y_col) continue;
 		if (i == s->t_col) continue;
@@ -441,29 +447,6 @@ int x2sys_n_data_cols (struct X2SYS_INFO *s)
 	return (n);
 }
 
-int x2sys_output_record (FILE *fp, double data[], struct X2SYS_INFO *s)
-{
-	int i, k;
-
-	if (s->ascii_out) {
-		for (i = 0; i < s->n_out_columns-1; i++) {
-			k = s->out_order[i];
-			(GMT_is_dnan (data[k])) ? fprintf (fp, "NaN") : fprintf (fp, s->info[k].format, data[k]);
-			fputc ('\t', fp);
-		}
-		k = s->out_order[i];
-		(GMT_is_dnan (data[k])) ? fprintf (fp, "NaN") : fprintf (fp, s->info[k].format, data[k]);
-		fputc ('\n', fp);
-	}
-	else {	/* Binary */
-		for (i = 0; i < s->n_out_columns; i++) {
-			k = s->out_order[i];
-			fwrite ((void *)&data[k], sizeof (double), (size_t)1, fp);
-		}
-	}
-	return (s->n_out_columns);
-}
-
 void x2sys_pick_fields (char *string, struct X2SYS_INFO *s)
 {
 	/* Scan the -Fstring and select which columns to use and which order
@@ -471,26 +454,29 @@ void x2sys_pick_fields (char *string, struct X2SYS_INFO *s)
 	 * order as on input
 	 */
 
-	char line[BUFSIZ], *p;
-	int i = 0, j;
+	char line[BUFSIZ], p[BUFSIZ];
+	int i = 0, j, pos = 0;
 
-	strncpy (line, string, BUFSIZ);
+	strncpy (s->fflags, string, BUFSIZ);
+	strncpy (line, string, BUFSIZ);	/* Make copy for later use */
 	memset ((void *)s->use_column, 0, (size_t)(s->n_fields * sizeof (int)));
 
-	p = strtok (line, ",");
-
-	while (p) {
+	s->x_col = s->y_col = s->t_col = -1;	/* Need to reset this to match data order */
+	while ((GMT_strtok (line, ",", &pos, p))) {
 		j = 0;
 		while (j < s->n_fields && strcmp (p, s->info[j].name)) j++;
 		if (j < s->n_fields) {
 			s->out_order[i] = j;
 			s->use_column[j] = 1;
+			/* Reset x,y,t indeces */
+			if (!strcmp (s->info[j].name, "x") || !strcmp (s->info[j].name, "lon"))  s->x_col = i;
+			if (!strcmp (s->info[j].name, "y") || !strcmp (s->info[j].name, "lat"))  s->y_col = i;
+			if (!strcmp (s->info[j].name, "t") || !strcmp (s->info[j].name, "time")) s->t_col = i;
 		}
 		else {
 			fprintf (stderr, "X2SYS: ERROR: Unknown column name %s\n", p);
 			exit (EXIT_FAILURE);
 		}
-		p = strtok (NULL, ",");
 		i++;
 	}
 
@@ -509,7 +495,7 @@ void x2sys_set_home (void)
 	}
 	else if ((this = getenv ("GMTHOME")) != CNULL) {	/* Assume GMT path */
 		X2SYS_HOME = (char *) GMT_memory (VNULL, (size_t)(strlen (this) + 13), 1, "x2sys_set_home");
-		sprintf (X2SYS_HOME, "%s%cshare%cx2sys\0", this, DIR_DELIM, DIR_DELIM);
+		sprintf (X2SYS_HOME, "%s%cshare%cx2sys", this, DIR_DELIM, DIR_DELIM);
 	}
 	else {	/* Set default path */
 #ifdef _WIN32
@@ -536,43 +522,6 @@ void x2sys_free_data (double **data, int n)
 	GMT_free ((void *)data);
 }
 
-double *x2sys_distances (double x[], double y[], int n, int dist_flag)
-{
-	int this, prev;
-	double *d, km_pr_deg;
-	
-	km_pr_deg = 0.001 * 2.0 * M_PI * 6371008.7714 / 360.0;
-
-	d = (double *) GMT_memory (VNULL, (size_t)n, sizeof (double), "x2sys_distances");
-
-	switch (dist_flag) {
-
-		case 0:	/* Cartesian distances */
-
-			for (this = 1, prev = 0; this < n; this++, prev++)
-				d[this] = d[prev] + hypot (x[this] - x[prev], y[this] - y[prev]);
-			break;
-
-		case 1:	/* Flat earth distances */
-
-			for (this = 1, prev = 0; this < n; this++, prev++)
-				d[this] = d[prev] + hypot ((x[this] - x[prev]) * cosd (0.5 * (y[this] + y[prev])), y[this] - y[prev]) * km_pr_deg;
-			break;
-
-		case 2:	/* Great circle distances */
-
-			for (this = 1, prev = 0; this < n; this++, prev++)
-				d[this] = d[prev] + GMT_great_circle_dist (x[this], y[this], x[prev], y[prev]) * km_pr_deg;
-			break;
-		default:
-			fprintf (stderr, "x2sys: Error: Wrong flag passed to x2sys_distances (%d)\n", dist_flag);
-			exit (EXIT_FAILURE);
-			break;
-	}
-
-	return (d);
-}
-
 double *x2sys_dummytimes (int n)
 {
 	int i;
@@ -658,7 +607,7 @@ int x2sys_read_gmtfile (char *fname, double ***data, struct X2SYS_INFO *s, struc
 	}
 
 	fclose (fp);
-	
+
 	p->ms_rec = NULL;
 	p->n_segments = 0;
 
@@ -669,58 +618,109 @@ int x2sys_read_gmtfile (char *fname, double ***data, struct X2SYS_INFO *s, struc
 
 int x2sys_read_mgd77file (char *fname, double ***data, struct X2SYS_INFO *s, struct X2SYS_FILE_INFO *p, struct GMT_IO *G)
 {
-	int i, j, n_alloc = GMT_CHUNK;
-	char path[BUFSIZ];
-	double **z;
-	FILE *fp;
-	struct MGD77_DATA_RECORD D;
-	struct MGD77_HEADER_RECORD H;
+	int i, j, col[MGD77_N_DATA_EXTENDED], n_alloc = GMT_CHUNK;
+	char path[BUFSIZ], *tvals[MGD77_N_STRING_FIELDS];
+	double **z, dvals[MGD77_N_DATA_EXTENDED];
+	struct MGD77_HEADER H;
+	struct MGD77_CONTROL M;
 	double NaN;
-	
+
 	GMT_make_dnan(NaN);
+	MGD77_Init (&M, TRUE);			/* Initialize MGD77 Machinery */
 
   	if (n_x2sys_paths) {
-  		if (x2sys_get_data_path (path, fname)) {
+  		if (x2sys_get_data_path (path, fname, s->suffix)) {
    			fprintf (stderr, "x2sys_read_mgd77file : Cannot find leg %s\n", fname);
      			return (-1);
   		}
-  		if ((fp = fopen (path, "r")) == NULL) {
+		if (MGD77_Open_File (path, &M, 0)) {
    			fprintf (stderr, "x2sys_read_mgd77file : Cannot find file %s\n", path);
      			return (-1);
   		}
 	}
-	else if ((fp = fopen (fname, "r")) == NULL) {
+	else if (MGD77_Open_File (fname, &M, 0)) {
    		fprintf (stderr, "x2sys_read_mgd77file : Cannot find file %s\n", fname);
      		return (-1);
   	}
 
-	if (!MGD77_Read_Header_Record (fp, &H)) {
+	if (MGD77_Read_Header_Record (fname, &M, &H)) {
 		fprintf (stderr, "%s: Error reading header sequence for cruise %s\n", X2SYS_program, fname);
 		exit (EXIT_FAILURE);
 	}
-	
-	z = (double **) GMT_memory (VNULL, (size_t)MGD77_DATA_COLS, sizeof (double *), "x2sys_read_mgd77file");
-	for (i = 0; i < MGD77_DATA_COLS; i++) z[i] = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), "x2sys_read_mgd77file");
+
+	for (i = 0; i < MGD77_N_STRING_FIELDS; i++) tvals[i] = (char *) GMT_memory (VNULL, 9, sizeof (char), "x2sys_read_mgd77file");
+	z = (double **) GMT_memory (VNULL, (size_t)s->n_fields, sizeof (double *), "x2sys_read_mgd77file");
+	for (i = 0; i < s->n_fields; i++) z[i] = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), "x2sys_read_mgd77file");
+	for (i = 0; i < s->n_out_columns; i++) {
+		col[i] = MGD77_Get_Column (s->info[s->out_order[i]].name, &M);
+	}
 
 	j = 0;
-	while (MGD77_Read_Data_Record (fp, &D)) {		/* While able to read a data record */
-		z[0][j] = D.time;
-		for (i = 1; i < MGD77_DATA_COLS; i++) z[i][j] = D.number[MGD77_items[i]];
+	while (!MGD77_Read_Data_Record (&M, &H, dvals, tvals)) {		/* While able to read a data record */
+		GMT_lon_range_adjust (s->geodetic, &dvals[MGD77_LONGITUDE]);
+		for (i = 0; i < s->n_out_columns; i++) z[i][j] = dvals[col[i]];
 
 		j++;
 		if (j == n_alloc) {
 			n_alloc += GMT_CHUNK;
-			for (i = 0; i < MGD77_DATA_COLS; i++) z[i] = (double *) GMT_memory ((void *)z[i], (size_t)n_alloc, sizeof (double), "x2sys_read_mgd77file");
+			for (i = 0; i < s->n_fields; i++) z[i] = (double *) GMT_memory ((void *)z[i], (size_t)n_alloc, sizeof (double), "x2sys_read_mgd77file");
 		}
 	}
-	fclose (fp);
+	MGD77_Close_File (&M);
 
 	strncpy (p->name, fname, 32);
 	p->n_rows = j;
-	for (i = 0; i < MGD77_DATA_COLS; i++) z[i] = (double *) GMT_memory ((void *)z[i], (size_t)p->n_rows, sizeof (double), "x2sys_read_mgd77file");
+	for (i = 0; i < s->n_fields; i++) z[i] = (double *) GMT_memory ((void *)z[i], (size_t)p->n_rows, sizeof (double), "x2sys_read_mgd77file");
 
 	p->ms_rec = NULL;
 	p->n_segments = 0;
+	for (i = 0; i < MGD77_N_STRING_FIELDS; i++) GMT_free ((void *)tvals[i]);
+
+	*data = z;
+
+	return (p->n_rows);
+}
+
+int x2sys_read_ncfile (char *fname, double ***data, struct X2SYS_INFO *s, struct X2SYS_FILE_INFO *p, struct GMT_IO *G)
+{
+	int i;
+	double **z;
+	struct MGD77_DATASET *S;
+	struct MGD77_CONTROL M;
+
+	MGD77_Init (&M, TRUE);		/* Initialize MGD77 Machinery */
+	M.format  = 1;			/* Set input file's format to netCDF */
+	for (i = 0; i < MGD77_N_FORMATS; i++) MGD77_format_allowed[i] = (M.format == i) ? TRUE : FALSE;	/* Only allow the specified input format */
+
+	for (i = 0; i < s->n_out_columns; i++) strcpy (M.desired_column[i], s->info[s->out_order[i]].name);	/* Set all the required fields */
+	M.n_out_columns = s->n_out_columns;
+	
+	S = MGD77_Create_Dataset ();	/* Get data structure w/header */
+
+	if (MGD77_Open_File (fname, &M, 0)) {
+   		fprintf (stderr, "x2sys_read_nc77file : Cannot find file %s\n", fname);
+     		return (-1);
+  	}
+	if (MGD77_Read_Header_Record (fname, &M, &S->H)) {	/* Returns info on all columns */
+		fprintf (stderr, "x2sys_read_nc77file: Error reading header sequence for cruise %s\n", fname);
+     		return (-1);
+	}
+
+	if (MGD77_Read_Data (fname, &M, S)) {	/* Only gets the specified columns and barfs otherwise */
+		fprintf (stderr, "x2sys_read_nc77file: Error reading data set for cruise %s\n", fname);
+     		return (-1);
+	}
+	MGD77_Close_File (&M);
+
+	z = (double **) GMT_memory (VNULL, (size_t)M.n_out_columns, sizeof (double *), "x2sys_read_nc77file");
+	for (i = 0; i < M.n_out_columns; i++) z[i] = (double *) S->values[i];
+
+	strncpy (p->name, fname, 32);
+	p->n_rows = S->H.n_records;
+	p->ms_rec = NULL;
+	p->n_segments = 0;
+	for (i = 0; i < MGD77_N_SETS; i++) if (S->flags[i]) GMT_free ((void *)S->flags[i]);
+	GMT_free ((void *)S->H.mgd77);
 
 	*data = z;
 
@@ -740,14 +740,18 @@ int x2sys_xover_output (FILE *fp, int n, double out[])
 int x2sys_read_list (char *file, char ***list)
 {
 	int n_alloc = GMT_CHUNK, n = 0;
-	char **p, line[BUFSIZ], name[64];
+	char **p, line[BUFSIZ], name[GMT_TEXT_LEN];
 	FILE *fp;
 
-	fp = x2sys_fopen (file, "r");
-
+	if ((fp = x2sys_fopen (file, "r")) == NULL) {
+  		fprintf (stderr, "x2sys_initialize : Cannot find track list file %s in either current or X2SYS_HOME directories\n", line);
+		exit (EXIT_FAILURE);
+	}
+	
 	p = (char **) GMT_memory (VNULL, n_alloc, sizeof (char *), "x2sys_read_list");
 
 	while (fgets (line, BUFSIZ, fp)) {
+		GMT_chop (line);	/* Remove trailing CR or LF */
 		sscanf (line, "%s", name);
 		p[n] = (char *) GMT_memory (VNULL, (size_t)(strlen(name)+1), sizeof (char), "x2sys_read_list");
 		strcpy (p[n], name);
@@ -768,38 +772,54 @@ int x2sys_read_list (char *file, char ***list)
 
 void x2sys_set_system (char *TAG, struct X2SYS_INFO **s, struct X2SYS_BIX *B, struct GMT_IO *G)
 {
-	char tag_file[BUFSIZ], line[BUFSIZ], *p, *sfile;
-	int geodetic, error = 0;
+	char tag_file[BUFSIZ], line[BUFSIZ], p[BUFSIZ], sfile[BUFSIZ], suffix[16];
+	int geodetic = 0, pos = 0, n;
+	double dist;
 	BOOLEAN geographic = FALSE;
 	FILE *fp;
+
+	if (!TAG) {
+		fprintf (stderr,"%s: TAG not set\n", X2SYS_program);
+		exit (EXIT_FAILURE);
+	}
 	
 	x2sys_set_home ();
 
 	memset ((void *)B, 0, sizeof (struct X2SYS_BIX));
 	B->bin_x = B->bin_y = 1.0;
 	B->x_min = 0.0;	B->x_max = 360.0;	B->y_min = -90.0;	B->y_max = +90.0;
-	
+	B->time_gap = B->dist_gap = dist = DBL_MAX;	/* Default is no data gap */
+	B->periodic = sfile[0] = suffix[0] = 0;
+
 	sprintf (tag_file, "%s.tag", TAG);
-	if ((fp = x2sys_fopen (tag_file, "r")) == NULL) {
-		fprintf (stderr,"%s: Could not open file %s\n", X2SYS_program, tag_file);
+	if ((fp = x2sys_fopen (tag_file, "r")) == NULL) {	/* Not in current directory */
+		fprintf (stderr,"%s: Could not find/open file %s either in current of X2SYS_HOME directories\n", X2SYS_program, tag_file);
 		exit (EXIT_FAILURE);
 	}
-	while (fgets (line, BUFSIZ, fp) && line[0] == '#');	/* Skip comment records */
 	
+	while (fgets (line, BUFSIZ, fp) && line[0] == '#');	/* Skip comment records */
+	GMT_chop (line);	/* Remove trailing CR or LF */
+
 	GMT_set_processed_option (8, FALSE);	/* In case -R has been processed before */
-	p = strtok (line, " \t");
-	while (p) {	/* Process the -D -I -R -G arguments from the header */
+	
+	while ((GMT_strtok (line, " \t", &pos, p))) {	/* Process the -D -I -R -G -W arguments from the header */
 		if (p[0] == '-') {
 			switch (p[1]) {
 				/* Common parameters */
 				case 'R':
-					error += GMT_get_common_args (p, &B->x_min, &B->x_max, &B->y_min, &B->y_max);
+					if (GMT_parse_common_options (p, &B->x_min, &B->x_max, &B->y_min, &B->y_max)) {
+						fprintf (stderr, "%s: Error processing %s setting in %s!\n", X2SYS_program, &p[1], tag_file);
+						exit (EXIT_FAILURE);
+					}
 					break;
-				
+
 				/* Supplemental parameters */
-				
+
 				case 'D':
-					sfile = &p[2];
+					strcpy (sfile, &p[2]);
+					break;
+				case 'E':
+					strcpy (suffix, &p[2]);
 					break;
 				case 'G':	/* Geographical coordinates, set discontinuity */
 					geographic = TRUE;
@@ -807,20 +827,61 @@ void x2sys_set_system (char *TAG, struct X2SYS_INFO **s, struct X2SYS_BIX *B, st
 					if (p[2] == 'd') geodetic = 2;
 					break;
 				case 'I':
-					if (p[2]) GMT_getinc (&p[2], &B->bin_x, &B->bin_y);
+					if (GMT_getinc (&p[2], &B->bin_x, &B->bin_y)) {
+						fprintf (stderr, "%s: Error processing %s setting in %s!\n", X2SYS_program, &p[1], tag_file);
+						exit (EXIT_FAILURE);
+					}
+					break;
+				case 'M':	/* Multisegment files */
+					GMT_multisegment (&p[2]);
+					break;
+				case 'W':
+					switch (p[2]) {
+						case 't':
+							B->time_gap = atof (&p[3]);
+							break;
+						case 'd':
+							B->dist_gap = atof (&p[3]);
+							break;
+						default:	/* Backwards compatible with old -Wtgap/dgap option */
+							n = sscanf (&p[2], "%lf/%lf", &B->time_gap, &dist);
+							if (n == 2) B->dist_gap = dist;
+						break;
+					}
+					break;
+				default:
+					fprintf (stderr, "%s: Bad arg in x2sys_set_system! (%s)\n", X2SYS_program, p);
+					exit (EXIT_FAILURE);
 					break;
 			}
 		}
-		p = strtok (NULL, " \t");
 	}
 	x2sys_fclose (tag_file, fp);
-
+	
 	*s = x2sys_initialize (sfile, G);	/* Initialize X2SYS and info structure */
 
 	if (geographic) {
+		if (geodetic == 0 && (B->x_min < 0 || B->x_max < 0)) {
+			fprintf (stderr, "%s: Your -R and -G settings are contradicting each other!\n", X2SYS_program);
+			exit (EXIT_FAILURE);
+		}
+		else if  (geodetic == 2 && (B->x_min > 0 && B->x_max > 0)) {
+			fprintf (stderr, "%s: Your -R and -G settings are contradicting each other!\n", X2SYS_program);
+			exit (EXIT_FAILURE);
+		}
 		(*s)->geographic = TRUE;
 		(*s)->geodetic = geodetic;	/* Override setting */
+		if (fabs (fabs (B->x_max - B->x_min) - 360.0) <= GMT_CONV_LIMIT) B->periodic = 1;
 	}
+	if (GMT_io.multi_segments[GMT_IN]) {	/* Files have multiple segments; make sure this is also set in s */
+		(*s)->multi_segment = TRUE;
+		(*s)->ms_flag = GMT_io.EOF_flag[GMT_IN];
+	}
+	if (suffix[0])
+		strcpy ((*s)->suffix, suffix);
+	else
+		strcpy ((*s)->suffix, sfile);
+		
 	x2sys_path_init (TAG);		/* Prepare directory paths to data */
 }
 
@@ -832,7 +893,7 @@ void x2sys_bix_init (struct X2SYS_BIX *B, BOOLEAN alloc)
 	B->ny_bin = irint ((B->y_max - B->y_min) * B->i_bin_y);
 	B->nm_bin = B->nx_bin * B->ny_bin;
 	if (alloc) B->binflag = (unsigned int *) GMT_memory (VNULL, (size_t)B->nm_bin, sizeof (unsigned int), X2SYS_program);
-}		
+}
 
 struct X2SYS_BIX_TRACK_INFO *x2sys_bix_make_entry (char *name, int id_no, int flag)
 {
@@ -863,8 +924,8 @@ int x2sys_bix_read_tracks (char *TAG, struct X2SYS_BIX *B, int mode)
 	size_t n_alloc = GMT_CHUNK;
 	char track_file[BUFSIZ], track_path[BUFSIZ], line[BUFSIZ], name[BUFSIZ];
 	FILE *ftrack;
-	struct X2SYS_BIX_TRACK_INFO *this_info;
-	
+	struct X2SYS_BIX_TRACK_INFO *this_info = VNULL;
+
 	sprintf (track_file, "%s_tracks.d", TAG);
 	x2sys_path (track_file, track_path);
 
@@ -873,18 +934,19 @@ int x2sys_bix_read_tracks (char *TAG, struct X2SYS_BIX *B, int mode)
 		exit (EXIT_FAILURE);
 	}
 
-	
+
 	if (mode == 1)
 		B->head = (struct X2SYS_BIX_TRACK_INFO *) GMT_memory (VNULL, n_alloc, sizeof (struct X2SYS_BIX_TRACK_INFO), X2SYS_program);
 	else
 		B->head = this_info = x2sys_bix_make_entry ("-", 0, 0);
-		
+
 	fgets (line, BUFSIZ, ftrack);	/* Skip header record */
 	while (fgets (line, BUFSIZ, ftrack)) {
+		GMT_chop (line);	/* Remove trailing CR or LF */
 		sscanf (line, "%s %d %d", name, &id, &flag);
 		if (mode == 1) {
 			if (id >= (int)n_alloc) {
-				n_alloc += GMT_CHUNK;
+				while (id >= (int)n_alloc) n_alloc += GMT_CHUNK;
 				B->head = (struct X2SYS_BIX_TRACK_INFO *) GMT_memory ((void *)B->head, n_alloc, sizeof (struct X2SYS_BIX_TRACK_INFO), X2SYS_program);
 			}
 			B->head[id].track_id = id;
@@ -909,11 +971,11 @@ void x2sys_bix_read_index (char *TAG, struct X2SYS_BIX *B)
 {
 	char index_file[BUFSIZ], index_path[BUFSIZ];
 	FILE *fbin;
-	int index, no_of_tracks, i, id, flag;
-	
+	int index = 0, no_of_tracks, i, id, flag;
+
 	sprintf (index_file, "%s_index.b",  TAG);
 	x2sys_path (index_file, index_path);
-	
+
 	if ((fbin = GMT_fopen (index_path, "rb")) == NULL) {
 		fprintf (stderr,"%s: Could not open %s\n", X2SYS_program, index_path);
 		exit (EXIT_FAILURE);
@@ -937,16 +999,28 @@ void x2sys_bix_read_index (char *TAG, struct X2SYS_BIX *B)
 
 int x2sys_bix_get_ij (double x, double y, int *i, int *j, struct X2SYS_BIX *B)
 {
-	int index;
-	
+	int index = 0;
+
 	*j = (y == B->y_max) ? B->ny_bin - 1 : (int)floor ((y - B->y_min) * B->i_bin_y);
+	if ((*j) < 0 || (*j) >= B->ny_bin) {
+		fprintf (stderr, "x2sys_binlist: j (%d) outside range implied by -R -I! [0-%d>\n", *j, B->ny_bin);
+		exit (EXIT_FAILURE);
+	}
 	*i = (x == B->x_max) ? B->nx_bin - 1 : (int)floor ((x - B->x_min)  * B->i_bin_x);
+	if (B->periodic) {
+		while (*i < 0) *i += B->nx_bin;
+		while (*i >= B->nx_bin) *i -= B->nx_bin;
+	}
+	if ((*i) < 0 || (*i) >= B->nx_bin) {
+		fprintf (stderr, "x2sys_binlist: i (%d) outside range implied by -R -I! [0-%d>\n", *i, B->nx_bin);
+		exit (EXIT_FAILURE);
+	}
 	index = (*j) * B->nx_bin + (*i);
-	if (index >= B->nm_bin) {
+	if (index < 0 || index >= B->nm_bin) {
 		fprintf (stderr, "x2sys_binlist: Index (%d) outside range implied by -R -I! [0-%d>\n", index, B->nm_bin);
 		exit (EXIT_FAILURE);
 	}
-	
+
 	return (index);
 }
 
@@ -959,11 +1033,11 @@ void x2sys_path_init (char *TAG)
 	int i;
 	char file[BUFSIZ], line[BUFSIZ];
 	FILE *fp;
-	
+
 	x2sys_set_home ();
 
 	sprintf (file, "%s%c%s_paths.txt", X2SYS_HOME, DIR_DELIM, TAG);
-	
+
 	n_x2sys_paths = 0;
 
 	if ((fp = fopen (file, "r")) == NULL) {
@@ -971,12 +1045,12 @@ void x2sys_path_init (char *TAG)
 		fprintf (stderr, "%s: (Will only look in current directory for such files)\n", X2SYS_program);
 		return;
 	}
-	
+
 	while (fgets (line, BUFSIZ, fp) && n_x2sys_paths < MAX_DATA_PATHS) {
 		if (line[0] == '#') continue;	/* Comments */
 		if (line[0] == ' ' || line[0] == '\0') continue;	/* Blank line */
-		x2sys_datadir[n_x2sys_paths] = GMT_memory (VNULL, (size_t)1, (size_t)(strlen (line)), "x2sys_path_init");
-		line[strlen (line)-1] = 0;
+		GMT_chop (line);	/* Remove trailing CR or LF */
+		x2sys_datadir[n_x2sys_paths] = GMT_memory (VNULL, (size_t)1, (size_t)(strlen (line)+1), "x2sys_path_init");
 #if _WIN32
 		for (i = 0; line[i]; i++) if (line[i] == '/') line[i] = DIR_DELIM;
 #else
@@ -988,31 +1062,48 @@ void x2sys_path_init (char *TAG)
 	}
 	fclose (fp);
 }
-	
+
 /* x2sys_get_data_path takes a track name as argument and returns the full path
  * to where this data file can be found.  x2sys_path_init must be called first.
  */
  
-int x2sys_get_data_path (char *track_path, char *track)
+int x2sys_get_data_path (char *track_path, char *track, char *suffix)
 {
 	int id;
+	BOOLEAN add_suffix;
 	char geo_path[BUFSIZ];
-	
-	/* First look in current directory */
-	
-	if (!access(track, R_OK)) {
+
+	if (track[0] == '/' || track[1] == ':') {	/* Full path given, just return it */
 		strcpy(track_path, track);
 		return (0);
 	}
 	
-	/* Then look elsewhere */
+	/* Check if we need to append suffix */
 	
+	add_suffix = strncmp (&track[strlen(track)-strlen(suffix)], suffix, strlen(suffix));	/* Need to add suffix? */
+
+	/* First look in current directory */
+
+	if (add_suffix)
+		sprintf (geo_path, "%s.%s", track, suffix);
+	else
+		strcpy (geo_path, track);
+	if (!access(geo_path, R_OK)) {
+		strcpy(track_path, geo_path);
+		return (0);
+	}
+
+	/* Then look elsewhere */
+
 	for (id = 0; id < n_x2sys_paths; id++) {
-		sprintf (geo_path, "%s%c%s", x2sys_datadir[id], DIR_DELIM, track);
+		if (add_suffix)
+			sprintf (geo_path, "%s%c%s.%s", x2sys_datadir[id], DIR_DELIM, track, suffix);
+		else
+			sprintf (geo_path, "%s%c%s", x2sys_datadir[id], DIR_DELIM, track);
 		if (!access (geo_path, R_OK)) {
 			strcpy (track_path, geo_path);
 			return (0);
 		}
 	}
-	return(1);
+	return(1);	/* Schwinehund! */
 }
